/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Actions;

import java.util.ArrayList;
import java.io.*;

/**
 * PrimaryAction.  CharacterActions that occur in the foreground, currently
 * consisting of Speak and Think.  Character movement could be added in 
 * future versions (or it could be considered a background activity.)
 * 
 * The most important attributes of a PrimaryAction are the source (acting
 * character), startingCol (time of action in scene) and secondary action
 * (emotion).  Several other fields are not currently used, such as 
 * minDuration and length.
 * 
 * @author Terry J Bradley
 * @version 04.7.26
 *
 */

public class PrimaryAction extends CharacterAction {
	
	protected int m_sourceGID;			// Primary actor
	protected ArrayList m_targetGID;	// List of Integers
	protected int m_startingCol;		// Chunk that this action occurs in
	protected int m_minDuration = 0;	// Minimum time in msec (0 = unlimited)
	protected int m_length = 1;			// Number of chunks (< 1 == invalid)
	
	protected SecondaryAction m_secondary;	// Secondary action (like emo)
	protected String m_dialog = "";		// Speak or Think dialog
	
//////////////////////////  CONSTRUCTORS  /////////////////////////////////////
	public PrimaryAction(int source, ArrayList target, int time, 
						int duration, int length)
	{
		m_sourceGID = source;
		m_targetGID = target;
		m_startingCol = time;
		m_minDuration = duration;
		m_length = length;	
		m_secondary = null;	
	}
	
	public PrimaryAction(int source, int target, int time) {
		init();
		m_sourceGID = source;
		m_startingCol = time;
		m_targetGID.add(new Integer(target));
	}
	
	public PrimaryAction(int time) {
		init();
		m_sourceGID = -1;
		m_startingCol = time;
	}
	

	private void init(){
		m_targetGID = new ArrayList();
		m_secondary = null;	
	}
	
	/**
	 * Returns action text.
	 *
	 */
	public String getActionText(){
		return m_dialog;
	}
	
	/**
	 * Sets action text.
	 *
	 */
	public void setActionText(String s){
		m_dialog = s;
	}
	
	/**
	 * Returns secondary action.
	 *
	 */
	public SecondaryAction getSecondaryAction(){
		return m_secondary;
	}
	
	/**
	 * Sets secondary action
	 *
	 */
	public void setSecondaryAction(SecondaryAction sa){
		m_secondary = sa;
	}
	
	/**
	 * Returns column (or time) in Action Grid
	 *
	 */
	public int getStartingCol(){
		return m_startingCol;
	}
	
	/**
	 * Sets column (or time) in Action Grid
	 *
	 */
	public void setStartingCol(int col){
		m_startingCol = col;
	}
	
	/**
	 * Get primary actor's GID
	 *
	 */
	public int getSourceGID(){
		return m_sourceGID;
	}
	
	/**
	 * Store the GID of the primary actor
	 *
	 */
	public void setSourceGID(int gid){
		m_sourceGID = gid;
	}
	
	
	public String toString(){
		return "CharacterID " + m_sourceGID + " ";
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(m_targetGID);
		oos.writeObject(m_secondary);
		oos.writeObject(m_dialog);
		oos.writeInt(m_sourceGID);
		oos.writeInt(m_startingCol);
		oos.writeInt(m_minDuration);
		oos.writeInt(m_length);
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			m_targetGID		= (ArrayList)ois.readObject();
			m_secondary		= (SecondaryAction)ois.readObject();
			m_dialog		= (String)ois.readObject();
			m_sourceGID		= ois.readInt();
			m_startingCol	= ois.readInt();
			m_minDuration	= ois.readInt();
			m_length		= ois.readInt();
		} catch (ClassNotFoundException ce){
			System.err.println(ce);
		}		
	}

}