/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover;

import java.awt.Color;
import java.util.*;

/**
 * Wrapper for Properties that provides methods like "getInt",
 * returning int instead of Object.  Saves some headaches.
 * Also sets default values when none is passed.
 * 
 * @author Terry Bradley
 * @version 04.7.26
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


public class CloverProperties extends Properties{
	
	public CloverProperties() {
		super();
	}
	
	public CloverProperties(CloverProperties p) {
		super(p);
	}
		
	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.
	 * 
	 * @param key Property key
	 * @param defVal Default value
	 * @return Value if found, else default
	 */
	public int getInt(String key, int defVal) {
		int result;

		try {
			result = Integer.parseInt(this.getProperty(key));
		}
		catch (Exception e) {
			result = defVal;
			this.setProperty(key, Integer.toString(defVal));
		}
		
		return result;
	}

	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.
	 * 
	 * @param s Property key
	 * @param d Default value
	 * @return Value if found, else default
	 */
	public double getDouble(String s, double d) {
		double result;

		try {
			result = Double.parseDouble(this.getProperty(s));
		}
		catch (Exception e) { 
			result = d;
			this.setProperty(s, Double.toString(d));
		}
		return result;
	}
	
	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.
	 * 
	 * @param s Property key
	 * @param f Default value
	 * @return Value if found, else default
	 */
	public float getFloat(String s, float f) {
		float result;

		try {
			result = Float.parseFloat(this.getProperty(s));
		}
		catch (Exception e) {
			result = f;
			this.setProperty(s, Float.toString(f));
		}
		return result;
	}

	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.
	 * 
	 * @param s Property key
	 * @param b Default value
	 * @return Value if found, else default
	 */
	public boolean getBool(String s, boolean b) {
		boolean result;

		try {
			result = Boolean.valueOf(this.getProperty(s)).booleanValue();
		}
		catch (Exception e) {
			result = b;
			this.setProperty(s, Boolean.toString(b));
		}
		return result;
	}

	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.
	 * 
	 * @param s Property key
	 * @param c Default value
	 * @return Value if found, else default
	 */
	public Color getColor(String s, Color c) {
		Color result;
		int rgb;
		
		try {
			rgb = getInt(s);	// gets 0 if no entry - no exceptions
			result = new Color(rgb, true);
		}
		catch (Exception e) {
			result = c;
			this.setProperty(s, ""+c.getRGB());
		}
		if(result.getRGB() == 0) {
			result = c;
			this.setProperty(s, ""+c.getRGB());
		}
		return result;
	}

	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.  When default is not specified, 0 is used
	 * for numbers, false for booleans.
	 * 
	 * @param key Property key
	 * @return Value if found, else default
	 */
	public int getInt(String key) {
		return getInt(key, 0);
	}

	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.  When default is not specified, 0 is used
	 * for numbers, false for booleans.
	 * 
	 * @param key Property key
	 * @return Value if found, else default
	 */
	public double getDouble(String key) {
		return getDouble(key, 0.);
	}
	
	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.  When default is not specified, 0 is used
	 * for numbers, false for booleans.
	 * 
	 * @param key Property key
	 * @return Value if found, else default
	 */
	public float getFloat(String key) {
		return getFloat(key, 0.0f);
	}
	
	/**
	 * Looks for value in properties.  If not found, default is
	 * set and returned.  When default is not specified, 0 is used
	 * for numbers, false for booleans.
	 * 
	 * @param key Property key
	 * @return Value if found, else default
	 */
	public boolean getBool(String key) {
		return getBool(key, false);
	}

}