/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Data;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import javax.imageio.*;

// CloverWindow, Constants, & BackgroundSurface
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.DrawingTools.*;

/**
 * GlobalCharacter - Defines GlobalCharacter datatype.
 * Each character is defined by one GlobalCharacter, which contains
 * appearance and voice properties, and multiple localCharacters,
 * which define how a character appears in a scene.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 10/18/03
 * 
 */

public class GlobalCharacter implements Serializable {

	// Variables
	ArrayList pict;		// Array of character image locations (Strings)
						// allows differences between scenes.
						// NOT CURRENTLY USED
	
	String drawnPict;	// image filename (no path).  Should be
						// identical to pict.get(0).

	transient ImageIcon thumbnail;	
						// Thumbnail for Gallery and DialogTable

	String name;		// Character's name
	int charID;			// Character's unique numeric ID, used 
						// throughout Clover
	boolean gender;		// Character's gender (See Constants.MALE)
	boolean record;		// True to record sounds, false to use TTS
	
	// Voice properties
	float pitch;
	float volume;
	float tone;
	float rate;
	int voiceIndex;		// Index of TTS voice to use.  Could probably remove
						// record (above) by saying voiceIndex == -1 ->
						// record sounds, > -1 -> TTS.
	
	long nextActionID;			// Used to generate action filenames
	Point rightBubbleLocation;	// Offset where bubble point should appear
	Point leftBubbleLocation;	// (Must be adjusted for scale.)

	transient static BufferedImage big;	// Single BufferedImage used as
										// a temp location.  Meant to 
										// keep down memory usage.
	
	// Prevent InvalidClassExceptions by hardcoding a version id.
	//  - "cloverGC"
	static final long serialVersionUID = 312152251873l;
	
	
	/**
	 * Constructor - Defines a character with no id and the name 
	 * "New Character"
	 * 
	 * @param size Size of character drawing canvas
	 */
	public GlobalCharacter(Dimension size){
		charID = -1;
		name = "New Character";
		init(size);
	}
	
	
	/**
	 * Constructor - Defines a character with an id and the name 
	 * "New Character"
	 * 
	 * @param id Character's unique id
	 * @param size Size of character drawing canvas
	 */
	public GlobalCharacter(int id, Dimension size) {
		charID = id;
		name = "New Character";
		init(size);
	}
	
	/**
	 * Constructor - Defines a character with an id, name, and gender
	 * 
	 * @param id Character's unique id
	 * @param n Character's name
	 * @param g Character's gender (see Constants.MALE...)
	 * @param size Size of character drawing canvas
	 */
	public GlobalCharacter(int id, String n, boolean g, Dimension size) {
		charID = id;
		name = n;
		gender = g;
		init(size);
	}
	
	/**
	 * Constructor - Defines a character with an id, name, and gender
	 * 
	 * @param id Character's unique id
	 * @param n Character's name
	 * @param g Character's gender (see Constants.MALE...)
	 * @param p Character's vocal pitch
	 * @param v Character's vocal volume
	 * @param size Size of character drawing canvas
	 */
	public GlobalCharacter(int id, String n, boolean g, float p, float v,
							Dimension size)
	{
		name = n;
		//path = storyPath;
		charID = id;		
		gender = g;
		pitch = p;
		volume = v;
		init(size);
	}
	
	/**
	 * Called as part of GlobalCharacter constructors to setup
	 * a filename for the character drawing and store a blank image.
	 * 
	 * @param size Size of character drawing canvas
	 */
	public void init(Dimension size){
		nextActionID = 0;
		record = true;
		pict = new ArrayList(2);
		drawnPict = "character"+charID+"_0"+".png";
		pict.add(drawnPict);

		// Create new bufferedImage for canvas and save to file @ drawnPict
		createImage(size);
	}
	
	/**
	 * Get character's thumbnail image.
	 * 
	 * @return Thumbnail
	 */
	public ImageIcon getThumbnail() {
		return thumbnail;
	}
	
	/**
	 * Set character's thumbnail image (this is not saved between sessions).
	 * 
	 * @param i Image of thumbnail
	 */
	public void setThumbnail(ImageIcon i) {
		thumbnail = i;
	}
	
	/**
	 * Create new bufferedImage for canvas and save to file @ drawnPict.
	 * 
	 * @param imgSize Size of character drawing canvas
	 */
	public void createImage(Dimension imgSize){
		if(charID < 0) return;

		big = new BufferedImage(imgSize.width,
								imgSize.height,
								BufferedImage.TYPE_INT_ARGB);
		
		BackgroundSurface.clearCanvas(big, Constants.COLOR_CLEAR);
		saveImage(big);		
	}
	
	/**
	 * 	Save bi to file at drawnPict. The thumbnail is NOT updated during save.
	 * 
	 * @param bi BufferedImage to save in drawnPict
	 */
	public void saveImage(BufferedImage bi){
		try{
			boolean success = ImageIO.write(
						  bi,
						  "png",
						  new File(Clover.TEMP_DIR+drawnPict));
		}
		catch(Exception e){
			System.err.println("Error saving image of character "+charID);
			e.printStackTrace();
		}
	}
	
	/**
	 * Open image at drawnPict.
	 * 
	 * The body of the function could be replaced with:
	 *  openImage(0);
	 * 
	 * @return BufferedImage of file.
	 */
	public BufferedImage openImage(){
		try {
			return ImageIO.read(new File(Clover.TEMP_DIR+drawnPict));
		}
		catch(Exception e){
			System.err.println("Error opening "+Clover.TEMP_DIR+drawnPict+". (" +
					"This is normal for a new story.");
			//e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Open image at pict[index].
	 * 
	 * @param index Image index to open.
	 * @return BufferedImage of file.
	 */
	public BufferedImage openImage(int index){
		try {
			String filename = (String)pict.get(index);
			return ImageIO.read(new File(Clover.TEMP_DIR+drawnPict));
		} catch(Exception e) {
			System.err.println("Error opening "+Clover.TEMP_DIR+drawnPict+".");
			e.printStackTrace();
		}
		return null;		
	}
	
	/**
	 * Creates a thumbnail icon based on the current
	 * drawnPict (bufferedImage).  This should be called in
	 * a separate thread, especially if scaleHint is 
	 * AREA_AVERAGING or SMOOTH
	 * 
	 * @param scaleHint
	 */
	public void makeThumbnail(BufferedImage bi, final int scaleHint) {
		if( drawnPict == null || bi == null )
			return;

		//BufferedImage 
		// This function replaces transparent pixels in the images with
		// the color of the canvas, which is currently pale green.  For
		// thumbnails, just could just as easily use white (replace
		// the second arg with Color.white).  In fact...
		big =
			BackgroundSurface.changeColor(Constants.COLOR_CLEAR.getRGB(),
										  //Constants.COLOR_CANVAS.getRGB(),
										  Color.white.getRGB(),
										  bi);
		 
		thumbnail = new ImageIcon(big.getScaledInstance(Constants.CHAR_THUMB_WIDTH, 
			Constants.CHAR_THUMB_HEIGHT, scaleHint));

		// next call is believed to be worthless.
		big = null;
	}
	
	/**
	 * Creates a thumbnail icon based on the current
	 * drawnPict (bufferedImage).  This should be called in
	 * a separate thread, especially if scaleHint is 
	 * AREA_AVERAGING or SMOOTH.  This verison of the function
	 * first loads the full sized image from the disk.
	 * 
	 * @param scaleHint
	 */
	public void makeThumbnail(final int scaleHint){
		// Open the image.
		big = openImage();
		
		// Make the thumbnail
		makeThumbnail(big, scaleHint);
	}
	

	/**
	 * Legacy function, replaced with saveImage(BufferedImage).
	 * 
	 * Saves passed-in image.
	 * 
	 * @param d Drawing image to save.
	 * @deprecated
	 */
	public void setDrawnImage(BufferedImage d) {
		saveImage(d);
	}
	
	/**
	 * Legacy function; loads image.  Replaced by openImage().
	 * 
	 * @return Drawing image
	 * @deprecated
	 */
	public BufferedImage getDrawnImage() {
		return openImage();
	}
	
	/**
	 * Legacy function; loads image.  Replaced by openImage(index).
	 * 
	 * @param index Index of image
	 * @return Drawing image
	 * @deprecated
	 */
	public ImageIcon getImage(int index) {
		return new ImageIcon(openImage(index));
	}
	
	/**
	 * Returns the character's name
	 * 
	 * @return Name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns the character's gender
	 * 
	 * @return Gender
	 */
	public boolean getGender() {
		return gender;
	}	
	
	/**
	 * Returns the character's vocal recording status
	 * 
	 * @return Record status
	 */
	public boolean getRecord(){
		return record;
	}

	/**
	 * Returns the character's vocal pitch
	 * 
	 * @return Pitch
	 */
	public float getPitch() {
		return pitch;
	}

	/**
	 * Returns the character's vocal volume
	 * 
	 * @return Volume
	 */
	public float getVolume() {
		return volume;
	}
	
	/**
	 * Returns the character's vocal tone
	 * 
	 * @return Tone
	 */
	public float getTone(){
		return tone;
	}
	
	/**
	 * Returns the character's vocal rate
	 * 
	 * @return Rate
	 */
	public float getRate(){
		return rate;
	}
	
	/**
	 * Returns the character's voice index for TTS
	 * 
	 * @return TTS index
	 */
	public int getVoiceIndex(){
		return voiceIndex;
	}
	
	/**
	 * Returns the character's unique ID
	 * 
	 * @return ID
	 */
	public int getID() {
		return charID;
	}
	
	/**
	 * Returns the next action id
	 * 
	 * @return id
	 */
	public long getNextActionID(){
		return nextActionID++;
	}
	
	/**
	 * Returns the character's right bubble location
	 * 
	 * @return Right bubble location
	 */
	public Point getRightBubbleLocation(){
		return rightBubbleLocation;
	}
	
	/**
	 * Returns the character's left bubble location
	 * 
	 * @return Left bubble location
	 */
	public Point getLeftBubbleLocation(){
		return leftBubbleLocation;
	}
	
	/**
	 * Toggles the character's recording status.
	 * This does not effect the voiceIndex.
	 *
	 */
	public void toggleVoice(){
		record = !record;
	}

	/**
	 * Adds a the location of another character image to pict array.
	 * NOT CURRENTLY USED.
	 * 
	 * @param i String giving image location.
	 */
	public void addImage(String i) {
		pict.add(i);
	}
	
	/**
	 * Removes an image from the character.  This does not remove the image
	 * from the disk, nor update the story in any way.
	 * NOT CURRENTLY IN USE.
	 * 
	 * @param index Index of image to remove.
	 */
	public void removeImage(int index) {
		// Not currently supported - would need to update at the story level.
		pict.remove(index);
	}
	
	/**
	 * Set character's name
	 * 
	 * @param n Name
	 */
	public void setName(String n) {
		name = n;
	}
	
	/**
	 * Set character's gender
	 * 
	 * @param g Gender
	 */
	public void setGender(boolean g) {
		gender = g;
	}
	
	/**
	 * Set character's recording status
	 * 
	 * @param r Recording status (false = TTS)
	 */
	public void setRecord(boolean r){
		record = r;
	}
	
	/**
	 * Set character's pitch
	 * 
	 * @param p Pitch
	 */
	public void setPitch(float p) {
		pitch = p;
	}
	
	/**
	 * Set character's volume
	 * 
	 * @param v Volume
	 */
	public void setVolume(float v) {
		volume = v;
	}
	
	/**
	 * Set character's Tone
	 * 
	 * @param t Tone
	 */
	public void setTone(float t){
		tone = t;
	}
	
	/**
	 * Set character's Rate
	 * 
	 * @param r Rate
	 */
	public void setRate(float r){
		rate = r;
	}
	
	/**
	 * Sets the character's globalID.
	 * <b>THIS ERASES ANY PRIOR IMAGES FOR THE CHARACTER!</b> 
	 * 
	 * Actually, this function erases links to images for the 
	 * character.  It would be wise to erase any images that appear
	 * at these locations, but currently this is unnecessary.
	 * 
	 * @param id
	 */
	public void setCharID(int id, Dimension size) {
		charID = id;
		drawnPict = "character"+charID+"_0"+".png";
		pict.clear();
		pict.add(drawnPict);
		createImage(size);
	}
	
	/**
	 * Set character's voice index
	 * 
	 * @param vi Voice Index
	 */
	public void setVoiceIndex(int vi){
		voiceIndex = vi;
	}
	
	/**
	 * Set character's left bubble location
	 * 
	 * @param p Left Bubble location
	 */
	public void setLeftBubbleLoc(Point p){
		leftBubbleLocation = p;
	}
	
	/**
	 * Set character's right bubble location
	 * 
	 * @param p Right bubble location
	 */
	public void setRightBubbleLoc(Point p){
		rightBubbleLocation = p;
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(rightBubbleLocation);
		oos.writeObject(leftBubbleLocation);
		oos.writeObject(pict);
		oos.writeObject(drawnPict);
		oos.writeObject(name);
		oos.writeInt(charID);
		oos.writeInt(voiceIndex);	
		oos.writeLong(nextActionID);
		oos.writeFloat(pitch);
		oos.writeFloat(volume);
		oos.writeFloat(tone);
		oos.writeFloat(rate);
		oos.writeBoolean(gender);
		oos.writeBoolean(record);

	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			rightBubbleLocation = (Point)ois.readObject();
			leftBubbleLocation	= (Point)ois.readObject();
			pict				= (ArrayList)ois.readObject();
			drawnPict			= (String)ois.readObject();
			name				= (String)ois.readObject();
			charID				= ois.readInt();
			voiceIndex			= ois.readInt();
			nextActionID		= ois.readLong();
			pitch				= ois.readFloat();
			volume				= ois.readFloat();	
			tone				= ois.readFloat();
			rate				= ois.readFloat();
			gender				= ois.readBoolean();
			record				= ois.readBoolean();

		} catch (ClassNotFoundException ce){
			System.err.println("GlobalCharacter failed to read "+ce);
		}		
	}
	
}