/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Data;

import java.awt.*;			
import java.util.*;				
import java.io.*;

import edu.uiuc.orchid.clover.*;

/**
 * Story - This class contains all the objects that define a story, including
 * grid elements, scenes, global characters (images, etc.), and specific
 * story properties, like scene dimensions and spacing information.
 * 
 * When saving a Clover story, this is the ONLY data structure that is saved.
 * However, saving a Story ends up saving all other Data classes and several
 * Actions.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 10/18/03
 *
 */

public class Story implements Constants, Serializable {
	
///////////////////////////////////////////////////////////////////////////////
//
//			VARIABLES
//
///////////////////////////////////////////////////////////////////////////////

	protected ArrayList characters;		// Collection of global characters
	private int nextCharID;				// Next (Unique) Global Char ID
	private int nextSceneID;			// Next (Unique) Scene ID
	
	transient protected String narrative;	
										// Narrative text.
										// Mostly replaced with narrative.txt;
										// still offered for undo/redo
	transient protected String m_oldNarrative;	
										// Old Narrative (for reverting)
	
	private HashMap scenes;				// Semi-ordered collection of scenes;
										//   of form (Integer, Scene)
										// Includes responses

	private ArrayList responses;		// Array of Responses to this story
	
	protected ArrayList m_script;		// Ordered array of script elements
										
	transient private CloverProperties props;	
										// Stores STORY properties, both data
										//   AND UI.

	protected String m_storyPath;		// Path to *.clover (jar) file
										// Could (should?) be made transient
	
	protected Dimension m_sceneSize;	// Size of scene drawing surface
	protected Dimension m_charSize;		// Size of character drawing surface


// CONSTANTS	
	private final String PROP_FILENAME = ".story";
	private final String DEFAULT_PROP_FILENAME = ".defaults";
	
	// Prevent InvalidClassExceptions by hardcoding a version id.
	// The id shown below is "Clover0" with an obvious translation 
	// (C = 3, etc.)
	static final long serialVersionUID = 31215225180l;
	
	
///////////////////////////////////////////////////////////////////////////////
//
//			CONSTRUCTOR(S)
//
///////////////////////////////////////////////////////////////////////////////	

	/**
	 * Constructor - Complete version, defines most variables and initializes
	 * the properties.
	 */
	public Story(ArrayList chars, String narr, HashMap scene,
				 CloverProperties p, int cID, int sID)
	{
		scenes = scene;
		responses = new ArrayList();
		m_script = new ArrayList();
		characters = chars;
		narrative = narr;
		props = p;
		nextCharID = cID;
		nextSceneID = sID;
		
		// Load application properties.
		initProps();
	}
	
	/**
	 * Constructor adds a response with the given title.  This
	 * response represents the main story arc.
	 * 
	 * @param title Title of Story (main response)
	 */
	public Story(String title) {
		this(new ArrayList(), "", new HashMap(), null, 0, 0);
		Response mainStory = new Response(title);
		responses.add(mainStory);
		
		// Load Application Properties [CHECK]
		initProps();
	}
	
	/**
	 * Constructor helper - initializes application properties.
	 * 
	 * TODO: [core-2] Story Properties should be stored with the story,
	 * not in separate files (like .story)
	 */
	private void initProps() {
		CloverProperties defaultProps = new CloverProperties();
		FileInputStream in;
		try {
			in = new FileInputStream(DEFAULT_PROP_FILENAME);
			defaultProps.load(in);
			in.close();
		}
		catch (FileNotFoundException e) {}
		catch (IOException f) {}
		
		props = new CloverProperties(defaultProps);

		// now load properties for this story
		try {
			in = new FileInputStream(PROP_FILENAME);
			props.load(in);
			in.close();
		}
		catch (FileNotFoundException e) { return; }
		catch (IOException f) { return; }
	}
	
	
///////////////////////////////////////////////////////////////////////////////
//
//			METHODS
//
///////////////////////////////////////////////////////////////////////////////	

// GET METHODS:
	
	/**
	 * @deprecated Narratives are now stored in 'narrative.txt'
	 */
	public String getNarrative() {
		return narrative;
	}
	
	
	/**
	 * Returns the scene at position pos.  if responseIndex is
	 * 0, scene is from main story.  higher responseIndices = responses.
	 * 
	 * @param pos Position in story, starting at 0 for the first scene.
	 * @param responseIndex Response Index
	 * @return Corresponding Scene object
	 */ 
	public Scene getScene(int pos, int responseIndex) {
		Integer id = null;
		try {
			id = ((Response)responses.get(responseIndex)).getSceneAtPosition(pos);
		}
		catch (Exception e) { 
			return null; 
		}
		return getScene(id);
	}
	
	/**
	 * Returns the scene w/ given sceneID
	 * 
	 * @param id SceneID of Scene to be returned
	 * @return Corresponding Scene object
	 */
	public Scene getScene(Integer id){
		Scene result;
		try {
			result = ((Scene)scenes.get(id));
		}
		catch (Exception e){ result = null; }
		return result;
	}
	
	/**
	 * Returns a hashmap of all scenes (possibly out of order)
	 * 
	 * @return
	 */
	public Set getAllSceneIDs(){
		return scenes.keySet();
	}
	

	/**
	 * 	Takes a GlobalID and return the Character corresponding to that ID.
	 * 
	 * @param id Character ID
	 * @return Character corresponding to ID
	 */
	public GlobalCharacter getCharacter(int id) {
		GlobalCharacter tmp = null;
		for( int i = 0; i < characters.size(); i++ ){
			try {
				tmp = (GlobalCharacter)characters.get(i);
			}
			catch (Exception e) { return null; }
			if( tmp.getID() == id )
				break;
		}
		return tmp;
	}
		
	/**
	 * Returns the number of scenes in one response of the Story.
	 * 
	 * @param responseIndex ResponseIndex to look at
	 * @return Number of scenes in that response
	 */
	public int getSceneCount(int responseIndex) {
		try {
			Response r = (Response)responses.get(responseIndex);
			return r.getSceneCount();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return 0;
	}
	
	
	/**
	 * Gets properties
	 * 
	 * @return Properties
	 */
	public CloverProperties getProps() {
		return props;
	}
	

// SCENE METHODS: Add, remove, move, select scenes
	
	/**
	 * Adds the supplied Scene object as the last
	 * scene in the story
	 * 
	 * @param s Scene to add
	 * @return Added scene's sceneID (NOT position!)
	 */
	public int addSceneToEnd(Scene s, int responseIndex) {
		int sceneID = nextSceneID;
		s.setSceneID(sceneID);
		Integer sid = new Integer(sceneID);
		scenes.put(sid, s);
		//sceneOrder.add(new Integer(sceneID));
		try {
			Response r = (Response)responses.get(responseIndex);
			r.addScene(sid);
		}
		catch (Exception e){
			System.err.println("Scene not added to response - invalid index");
		}
		
		nextSceneID++;
		return sceneID;
	}
	
	/**
	 * Deletes scene with given sceneID
	 * 
	 * @param index sceneID of scene to delete
	 * @return True iff successful
	 */
	public boolean deleteSceneWithID(Integer index) {
		boolean success = true;
		try {
			Scene tmp = (Scene)scenes.remove(index);
			for(int i=0; i<responses.size(); i++){
				Response r = (Response)responses.get(i);
				for(int j=r.getSceneCount(); j>=0; j--){
					Integer k = (Integer)r.getSceneAtPosition(j);
					if( k.intValue() == index.intValue() )
						r.removeSceneAtPosition(j);
				}
			}
		}
		catch (Exception e) {
			success = false;
		}
		return success;
	}
	

	/**
	 * 	Moves scene at source to destination.  SELECTABILITY AND
	 *  OTHER ERROR CHECKS SHOULD OCCUR AT GUI LEVEL (VaEUI class).
	 * 
	 * @param responseIndex Response Index
	 * @param sourceID SceneID of source
	 * @param destinationID SceneID of destination 
	 * @return True iff successful
	 */
	public boolean moveScene(int responseIndex, Integer sourceID,
							 Integer destinationID)
	{
		int sourcePos = getPositionOfScene(sourceID, responseIndex);
		int destPos = getPositionOfScene(destinationID, responseIndex);
		
		Response r = (Response)responses.get(responseIndex);
		Integer i = (Integer)r.removeSceneAtPosition(sourcePos);
		r.addScene(destPos, i);
				
		//System.out.println("New Scene Order="+r.getResponseOrder().toString());
		
		return true;
	}
	
	/**
	 * Returns the position, from 0 to sceneCount - 1,
	 * where the scene with ID == sid appears in the Story
	 * 
	 * @param sid SceneID
	 * @return Position in story, from 0 to sceneCount - 1
	 */
	public int getPositionOfScene(Integer sid, int responseIndex){
		Response r = (Response)responses.get(responseIndex);
		for( int i=0; i<r.getSceneCount(); i++){
			if( sid.intValue() == r.getSceneAtPosition(i).intValue() )
				return i;
		}
		return -1;
	}
	
	/**
	 * Returns an arrayList of sceneIDs for given response.
	 * 
	 * @param responseIndex Response Index
	 * @return ArrayList of Integers (SceneIDs)
	 */
	public ArrayList getSceneList(int responseIndex){
		ArrayList al = new ArrayList();
		Response r = (Response)responses.get(responseIndex);
		for( int i=0; i<r.getSceneCount(); i++){
			Integer id = r.getSceneAtPosition(i);
			al.add(scenes.get(id));
		}
		return al;
	}
	
	/**
	 * Adds a response to the story.
	 * 
	 * NOTE: THIS FUNCTION WILL BREAK IF RESPONSES ARE DELETEABLE.
	 * 
	 * @param r Response
	 * @return responseIndex
	 */
	public int addResponse(Response r){
		responses.add(r);
		return responses.size() - 1;
	}
	
	/**
	 * Returns given response
	 * 
	 * @param index Response Index
	 * @return Response at index
	 */
	public Response getResponse(int index){
		try {
			return (Response)responses.get(index);
		}
		catch (Exception e){
			System.err.println("Error retreiving a response.");
			return null;
		}
	}
	
	/**
	 * Returns array of response titles.
	 * 
	 * @return Array of titles (strings)
	 */
	public String[] getResponseTitles(){
		String[] s = new String[responses.size()-1];
		for(int i=1; i<responses.size(); i++){
			s[i-1] = ((Response)responses.get(i)).getResponseTitle();
		}
		return s;
	}
	
	/**
	 * Returns the number of responses (including main story
	 * 
	 * @return Number of responses.
	 */
	public int getResponseCount(){
		return responses.size();
	}
	
	/**
	 * Returns the order in which characters should appear.  This differs
	 * from the scene level characterOrder, which deals with local instances.
	 * 
	 * @return ArrayList of Integers (Character IDs)
	 */
	public ArrayList getCharacterOrder(){
		ArrayList al = new ArrayList();
		for( int i=0; i<characters.size(); i++){
			int id = ((GlobalCharacter)characters.get(i)).getID();
			al.add(new Integer(id));
		}
		return al;
	}
	
	/**
	 * Returns number of characters
	 * 
	 * @return Number of characters
	 */
	public int getCharacterCount(){
		return characters.size();
	}
	
	/**
	 * Returns sceneOrder of given response
	 * 
	 * @param responseIndex Response Index
	 * @return ArrayList of Integers (SceneIDs)
	 */
	public ArrayList getSceneOrder(int responseIndex){
		try{
			return ((Response)responses.get(responseIndex)).getResponseOrder();
		}
		catch(Exception e){
			System.err.println("Null array in class Story, function getSceneOrder.");
			return null;
		}
	}
	
	/**
	 * Gets next unique scene id.  Does NOT increment nextSceneID.
	 * 
	 * @return Next scene ID
	 */
	public int getNextSceneID(){
		return nextSceneID;
	}
	
//	CHARACTER METHODS: Add, remove (move?)

	/**
	 * Adds a character to the story.
	 * 
	 * @param c Added character
	 * @return That character's id.
	 */
	public int addCharacter(GlobalCharacter c) {
		int gid = nextCharID;
		c.setCharID(gid, m_charSize);
		characters.add(c);		
		nextCharID++;
		return gid;	 
	 }
	 
	/**
	 * Adds a new character to the story.
	 * 
	 * @return New character's ID
	 */
	 public int addCharacter() {
	 	return addCharacter(new GlobalCharacter(nextCharID, m_charSize));
	 }
	 
	 /**
	  * Removes character from the story.  Character images are NOT deleted and
	  * will stay with the story indefinitely.
	  * 
	  * @param gid Character's ID
	  * @return True iff successful
	  */
	 public boolean removeCharacter(int gid) {
	 	boolean result = false;
	 	for(int i=0; i<characters.size(); i++){
	 		if( ((GlobalCharacter)characters.get(i)).charID == gid ) {
	 			characters.remove(i);
	 			result = true;
	 		}
	 	}
	 	return result;
	 }
	 
	 /**
	  * Replaces character with same GID.
	  * 
	  * @param gc Updated character
	  * @return True iff successful.
	  */
	 public boolean replaceCharacter(GlobalCharacter gc) {
	 	int gid = gc.getID();
	 	
	 	// Changed to keep characters in order, making character
	 	//   editing less confusing.
	 	// Get index
	 	int index = -1;
	 	for(int i=0; i<characters.size(); i++){
	 		if( ((GlobalCharacter)characters.get(i)).charID == gid ) {
	 			index = i;
	 			break;
	 		}
	 	}
	 	// Replace character
	 	characters.set(index, gc);
	 	
	 	return true;
	 }
	 
	 /**
	  * Returns next character ID.  nextCharID is NOT incremented.
	  * 
	  * @return Next character ID
	  */
	 public int getNextCharacterID() {
	 	return nextCharID;
	 }
	 
	 /**
	  * Returns character size.
	  * 
	  * @return Character size
	  */
	 public Dimension getCharSize(){
	 	return m_charSize;
	 }
	 
	 /**
	  * Set character size
	  * 
	  * @param size Character size
	  */
	 public void setCharSize(Dimension size){
	 	m_charSize = size;
	 }
	 
	 /**
	  * Returns scene background size.
	  * 
	  * @return Scene background size
	  */
	 public Dimension getSceneSize(){
	 	return m_sceneSize;
	 }
	 
	 /**
	  * Sets scene background size.
	  * 
	  * @param size Scene background size
	  */
	 public void setSceneSize(Dimension size){
	 	m_sceneSize = size;
	 }
	 
	 /**
	  * Gets path to story.
	  * 
	  * @return Story path
	  */
	 public String getStoryPath(){
	 	return m_storyPath;
	 }
	 
	 /**
	  * Sets path to story.
	  * 
	  * @param path Path to story
	  */
	 public void setStoryPath(String path){
	 	m_storyPath = path;
	 }
	
//	NARRATIVE METHODS:

	 /**
	  * Sets narrative text.
	  * 
	  * @param s Narrative text
	  */
	 public void setNarrative(String s) {
	 	m_oldNarrative = narrative;
	 	narrative = s;
	 }
	
	 /**
	  * Backs up narrative text.  Mostly useless with alternate
	  * undo/redo functionality.
	  */
	 public void backupNarrative() {
		m_oldNarrative = narrative;
	 }
	
	 /**
	  * Reverts narrative to old version.  Unused and rather pointless
	  * with Java's undo/redo functionality.
	  */
	 public void revertNarrative() {
		narrative = m_oldNarrative;
	 }
	
// Import/Export

	 /**
	  * Stores narrative to given file.  Used to save story, could be
	  * used to export narrative to other file.
	  * 
	  * @param file Destination of narrative
	  * @param text Text to write
	  */
	 public boolean writeNarrative(File file, String text) {
	 	FileOutputStream out;
		boolean result = true;
		try {
			out = new FileOutputStream(file);
			out.write(text.getBytes());
			out.close();
			result = true;
		}
		catch (Exception e) { result = false; }
		return result;
	}
	
	 /**
	  * Reads narrative from given file.  Used to open story, could be
	  * used to import narrative from other text file.
	  * 
	  * @param file Source of narrative
	  * @param oldText Previous text in narrative
	  * @return True iff successful
	  */
	public boolean readNarrative(File file, String oldText) {
		FileReader in;
		String text = "";
		char c;
		int i;
		boolean result = true;
		
		try {
			in = new FileReader(file);
			while( (i = in.read()) != -1 ) {
				c = (char)i;
				text = text + Character.toString(c);
			}
			in.close();
		}
		catch (Exception e) { 
			text = oldText;
			result = false;
		}
		setNarrative(text);
		backupNarrative();
		return result;
		
	}
	
//  EDITOR METHODS:
	
	/**
	 * Sets story title.
	 * 
	 * @param s Title
	 */
	public void setTitle(String s) {
		try {		
			Response r = (Response)responses.get(0);
			if(r != null)
				r.setResponseTitle(s);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Gets story title.
	 * 
	 * @return Title
	 */
	public String getTitle() {
		try {		
			Response r = (Response)responses.get(0);
			if(r != null)
				return r.getResponseTitle();
		} catch (Exception e){
			e.printStackTrace();
		}
		return "";
	}

//	DIALOG METHODS:	
	
	/**
	 * Add script dialog to the story.
	 * 
	 * @param row Row of added dialog
	 * @param sd ScriptElement added
	 */
	public void addDialog(int row, ScriptElement sd){
		m_script.add(row, sd);
	}
	
	/**
	 * Updates the dialog in a given row
	 * 
	 * @param row Row of updated dialog
	 * @param se ScriptElement updated
	 */
	public void updateDialog(int row, ScriptElement se){
		m_script.set(row, se);
	}
	
	/**
	 * Deletes dialog in given row
	 * 
	 * @param row Row of deleted dialog
	 */
	public void deleteDialog(int row){
		m_script.remove(row);
	}
	
	/**
	 * Moves source dialog to destination row
	 * 
	 * @param source Row of moving dialog
	 * @param destination Destination of moving dialog
	 */
	public void moveDialog(int source, int destination){
		Object obj = m_script.remove(source);
		m_script.add(destination, obj);
	}
	
	/**
	 * Returns this story's dialog
	 * 
	 * @return Dialog
	 */
	public ArrayList getDialogs(){
		return m_script;
	}

	
//	PROPERTY METHODS:

	/**
	 * Sets the properties
	 * 
	 * @param p Properties
	 */
	public void setProps(CloverProperties p) {
		props = p;
	}
	
	/**
	 * Stores the properties to disk
	 */
	public void writeProps() {
		try {
			FileOutputStream out = new FileOutputStream(PROP_FILENAME);
			props.store(out, "Header");
			out.close();
		}
		catch (FileNotFoundException e) {}
		catch (IOException f) {}		
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();

		oos.writeObject(characters);
		oos.writeObject(scenes);
		oos.writeObject(responses);
		oos.writeObject(narrative);
		oos.writeObject(m_storyPath);
		oos.writeInt(nextCharID);
		oos.writeInt(nextSceneID);
		oos.writeObject(m_charSize);
		oos.writeObject(m_sceneSize);			
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			characters	= (ArrayList)ois.readObject();
			scenes		= (HashMap)ois.readObject();
			responses	= (ArrayList)ois.readObject();
			narrative	= (String)ois.readObject();
			m_storyPath	= (String)ois.readObject();
			nextCharID	= ois.readInt();
			nextSceneID	= ois.readInt();
			m_charSize 	= (Dimension)ois.readObject();
			m_sceneSize	= (Dimension)ois.readObject();			 
		} catch (ClassNotFoundException ce){
			System.err.println("Story failed to read "+ce);
		}		
	}
	
}