/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.AffineTransform;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;


/**
 * DrawingSurface - LayeredPane that contains Background surface
 * for sketching and supports the addition of character instances.
 * 
 * @author Terry Bradley
 * @version 04.7.29 
 * @date 9/23/03
 *
 */
public class DrawingSurface extends JLayeredPane {

	protected CloverWindow m_parent;			// Parent
	protected BackgroundSurface m_background;	// Actual canvas
	private JLabel outline;						// Outline for characters
	protected int m_tab;						// Tab index (determines
												// owner of surface)
	protected boolean m_changed = false;		// Could be used to keep
												// track of changes.
	
	private static BufferedImage buffer;		// Temp BufferedImage for
												// getThumbnail()
	
	protected int MODE;							// Drawing mode, determined
												// and set by Toolbox
	
	protected Cursor m_cursor;					// Cursor

	
	/**
	 * Constructor - Creates drawing Surface with given size, background
	 * color, parents, and transparency.
	 * 
	 * @param d Size of canvas
	 * @param bg_color Background color
	 * @param vae CloverWindow - main frame
	 * @param tab Tab within CloverWindow
	 * @param transparent True for characters, false for scenes.
	 */
	public DrawingSurface(Dimension d, Color bg_color, CloverWindow vae, 
							int tab, boolean transparent) {
		super();
		m_parent = vae;
		m_cursor = Cursor.getDefaultCursor();
		m_tab = tab;
		
		setBackground(Constants.COLOR_CANVAS);	
		setBorder(new MatteBorder(1,1,1,1,Color.black));
		setPreferredSize(d);
		setCursor(m_cursor);
		
		m_background = new BackgroundSurface(d, bg_color, m_parent,
											this, transparent);
		add(m_background, JLayeredPane.DEFAULT_LAYER);
		
		outline = new JLabel();
		outline.setVisible(false);
		outline.setBorder(new MatteBorder(2,2,2,2,Color.GRAY));
		add(outline, JLayeredPane.DRAG_LAYER);
		
		// Currently, all mouse cursor actions are handled by BackgroundSurface
		/*
		MouseAdapter ma = new MouseAdapter() {
			public void mouseExited(MouseEvent e) {
				setCursor(Cursor.getDefaultCursor());
			}
			public void mouseEntered(MouseEvent e) {
				if(!CloverWindow.CURSOR_LOCK)
					setCursor(m_cursor);
			}
		};
		this.addMouseListener(ma);
		*/
	}
	
	/**
	 * Changes the size of the canvas.
	 * 
	 * @param d New size
	 */
	public void setSize(Dimension d){
		super.setSize(d);
		m_background = new BackgroundSurface(d, 
											 m_background.m_backColor,
											 m_parent,
											 this,
											 m_background.m_transparentImage);
		add(m_background, JLayeredPane.DEFAULT_LAYER);
	}
	
	/**
	 * Sets the tab in which this drawing surface appears.
	 * 
	 * @param tab Tab index
	 */
	public void setTab(int tab){
		m_tab = tab;
	}
	
	/**
	 * Hides all characters in the drawing surface (if any)
	 * 
	 * @param refresh True will refresh the thumbnail
	 */
	public void unselectCLabels(boolean refresh){
		if( m_tab == Constants.VIEW_AND_EDIT_INDEX ){
			((ViewAndEditUI)m_parent.getInterface(
					Constants.VIEW_AND_EDIT_INDEX)).unselectCLabels(refresh);
		}
		else if( m_tab == Constants.RESPONSE_INDEX ){
			((ResponseUI)m_parent.getInterface(Constants.RESPONSE_INDEX))
					.unselectCLabels(refresh);
		}
	}
	
	/**
	 * Resets the surface by removing all characters and other
	 * non-background images.
	 *
	 */
	public void resetSurface(){
		Component[] todo = this.getComponents();
		for(int i=0; i<todo.length; i++) {
			if(!todo[i].equals(m_background) && !todo[i].equals(outline))
				this.remove(todo[i]);
		}
	}
	
	/**
	 * Updates thumbnail.  Actual updating occurs in appropriate UI
	 * function.
	 *
	 */
	public void updateThumb(){
		switch (m_tab) {
			case Constants.VIEW_AND_EDIT_INDEX:
				((ViewAndEditUI)m_parent.
						getInterface(Constants.VIEW_AND_EDIT_INDEX)).
						updateThumbnail(new Integer(
							((ViewAndEditUI)m_parent.getInterface
									(Constants.VIEW_AND_EDIT_INDEX))
									.getSelectedSceneID()));
			break;
			
			case Constants.CHARACTER_INDEX:
				((CharacterUI)m_parent.getInterface(Constants.CHARACTER_INDEX))
						.updateCharacter();
			break;
			
			case Constants.RESPONSE_INDEX:
				((ResponseUI)m_parent.getInterface(Constants.RESPONSE_INDEX))
					.updateThumbnail(new Integer(
				((ResponseUI)m_parent.getInterface(Constants.RESPONSE_INDEX))
					.getSelectedSceneID()));
			break;
		} 
			
	}
	
	/**
	 * Cancels update, which is always controlled by a timer.  This allows
	 * multiple changes to propogate into a single call to updateThumbnail
	 * in the UI.
	 *
	 */
	public void stopUpdate(){
		switch (m_tab) {
			case Constants.VIEW_AND_EDIT_INDEX:
				((ViewAndEditUI)m_parent.
						getInterface(Constants.VIEW_AND_EDIT_INDEX)).stopTimer();
			break;
			
			case Constants.CHARACTER_INDEX:
				((CharacterUI)m_parent.
						getInterface(Constants.CHARACTER_INDEX)).stopTimer();
			break;
			
			case Constants.RESPONSE_INDEX:
				((ResponseUI)m_parent.
						getInterface(Constants.RESPONSE_INDEX)).stopTimer();
			break;
		} 
	}
	
	/**
	 * Displays the outline but does not move it.
	 * 
	 * @param b Visibility
	 */
	public void displayOutline(boolean b) {
		outline.setVisible(b);
		
	}
	
	/**
	 * Resizes the outline to match a character.
	 * 
	 * @param w Width
	 * @param h Height
	 */
	public void resizeOutline(int w, int h) {
		outline.setSize(w,h);
	}
	
	/**
	 * Resizes and moves an outline to match the 
	 * two corner points provided.
	 * 
	 * @param p1 First point
	 * @param p2 Point in opposite corner.
	 */
	public void resizeOutline(Point p1, Point p2){
		int x, y, w, h;
		x = (p1.x < p2.x) ? p1.x : p2.x;
		y = (p1.y < p2.y) ? p1.y : p2.y;
		w = Math.abs(p1.x - p2.x);
		h = Math.abs(p1.y - p2.y);
		outline.setLocation(x, y);
		outline.setSize(w, h);
	}
	
	/**
	 * Moves the outline.  Width and height are needed to check for off
	 * canvas results.
	 * 
	 * @param x X
	 * @param y Y
	 * @param w Width
	 * @param h Height
	 */
	public void moveOutline(int x, int y, int w, int h){
		outline.setLocation(x, y);
		// Check if any edge is off canvas
		if(x < 0 || x+w > getWidth() || y < 0 || y+h > getHeight()) {
			if(!CloverWindow.CURSOR_LOCK){
				m_parent.setCursor(CloverWindow.MOVE_CURSOR);
			}
		}
	}
	
	/**
	 * Sets bounds of the outline box to given rectangle.
	 * 
	 * @param r Bounding rectangle
	 */
	public void setOutlineBounds(Rectangle r) {
		outline.setBounds(r);
	}
	
	/**
	 * Creates a thumbnail image by taking a screen shot.  Clumsy and
	 * error prone, replaced by a function that draws scaled characters
	 * as neeeded.
	 * 
	 * @deprecated
	 * 
	 * @return Thumbnail image
	 */
	public Image renderImage() {		
		Rectangle rect = getBounds();
		Image fileImage = 
			createImage(rect.width,rect.height);
		
		// create screen shot
		Robot robot = null;
		Point p = new Point();
		try {
			robot = new Robot();
			p = this.getLocationOnScreen();
		}
		catch (Exception e) {}
		
		rect.x = p.x;
		rect.y = p.y;
		BufferedImage image = robot.createScreenCapture(rect);
 		fileImage = image.getScaledInstance(rect.width,rect.height,BufferedImage.SCALE_FAST);
 			  
	  	return fileImage;
	}
	
	/**
	 * Slightly better thumbnail producer that unfortunately does not include
	 * characters in scene thumbnails.  This is solved in UI.updateThumbnail
	 * methods.
	 * 
	 * @return Thumbnail image
	 */
	public Image renderImage2(){
		return m_background.m_buffImg.getScaledInstance
						(getWidth(),getHeight(),BufferedImage.SCALE_FAST);
	}
	
	/**
	 * Gets the background image
	 * 
	 * @return Background image
	 */
	public BufferedImage getBackgroundImage() {
		return m_background.m_buffImg;
	}
	
	/**
	 * Returns the thumbnail image, based SOLELY on the background image.
	 * 
	 * @param width Width of thumbnail
	 * @param height Height of thumbnail
	 * @param scaleHint Scale hints (SMOOTH or FAST)
	 * @return Thumbnail image
	 */
	public Image getThumbnailImage(int width, int height, int scaleHint){
		buffer = BackgroundSurface.changeColor(
							//Constants.COLOR_CLEAR.getRGB(),
							0,
							Constants.COLOR_CANVAS.getRGB(),
							m_background.m_buffImg);
		Image result = buffer.getScaledInstance(width, height, scaleHint);
		buffer = null;
		return result;
	}
	
	/**
	 * Sets the background image
	 * 
	 * @param d New background image
	 */
	public void setBackgroundImage(BufferedImage d) {
		m_background.m_buffImg = new BufferedImage(d.getWidth(),
												   d.getHeight(),
												   BufferedImage.TYPE_INT_ARGB);

		Graphics g = m_background.m_buffImg.createGraphics();
		g.drawImage(d, 0, 0, null);
		m_changed = false;
	}
	
	/**
	 * Restores old image, as in undo functionality.
	 */
	public void restoreImage(){
		m_background.restoreImage();
	}
	
	/**
	 * Returns the portion of the main image in the outline.
	 * Previously useful when characters could be defined in 
	 * the drawingsurface.  Currently used only by next function.
	 * 
	 * @return Portion of image in outline.
	 */
	public BufferedImage getOutlineImage(){
		return (BufferedImage)m_background.m_buffImg.getSubimage(
										outline.getLocation().x,
										outline.getLocation().y,
										outline.getWidth(),
										outline.getHeight() );
	}
	
	/**
	 * Gets a scaled version of the outline image.
	 * 
	 * @param width Width
	 * @param height Height
	 * @param hints Scale hints
	 * @return Outline image, scaled.
	 */
	public BufferedImage getScaledOutlineImage(int width, int height, int hints){ 
		BufferedImage bi = getOutlineImage();
		  					
		double scaleX =  width * 1.0 / bi.getWidth();
		double scaleY =  height * 1.0 / bi.getHeight();
		  
		if (scaleX == 1 && scaleY == 1)  
		   	return bi; 
		  
		AffineTransformOp op = 
			 new AffineTransformOp(
				AffineTransform.getScaleInstance(
							scaleX, scaleY), null); 

		return op.filter(bi, null); 
	}
	
	/**
	 * Clears areas under outline, i.e. for select / cut.
	 * 
	 * @param color New color for all pixels in outline.
	 */
	public void clearAreaUnderOutline(int color) {
		for( int i=outline.getLocation().x; 
					i<outline.getLocation().x+outline.getWidth(); i++){
			for( int j=outline.getLocation().y;
					 j<outline.getLocation().y+outline.getHeight(); j++){
			 	m_background.m_buffImg.setRGB(i,j,color);
			}
		}
	}
	
	/**
	 * Generates either right or left bubble location based on
	 * passed-in value.  Algorithm searches for topmost, rightmost
	 * pixel for rightside and topmost, leftmost for left bubble.
	 * 
	 * @param right True computes right bubble location
	 * @return Bubble location
	 */
	public Point generateBubbleOffset(boolean right){
		// Scan each line from right to left (left to right if !right)
		// Halt on first non-clear pixel and return location
		int halfway = (m_background.m_buffImg.getWidth() - 1) / 2;
		for(int i=0; i<m_background.m_buffImg.getHeight(); i++){
			if(right){
				for(int j=m_background.m_buffImg.getWidth()-1; j>=halfway; j--){
					if(m_background.m_buffImg.getRGB(j,i) != 
								Constants.COLOR_CLEAR.getRGB()	)
						return new Point(j, i);					
				}
				
			}
			else{
				for(int j=0; j<halfway; j++){
					if(m_background.m_buffImg.getRGB(j,i) != 
								Constants.COLOR_CLEAR.getRGB()	)
						return new Point(j, i);		
				}
			}
		}
		if(right)
			return new Point(m_background.m_buffImg.getWidth()-1, 0);
		else
			return new Point(0,0);
	}
	
	/**
	 * Returns changed status
	 * 
	 * @return Changed status
	 */
	public boolean isChanged(){
		return m_changed;
	}
	
	/**
	 * Sets changed status 
	 * 
	 * @param b Changed status
	 */
	public void setChanged(boolean b){
		m_changed = b;
	}
	
}

