/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * FillTool - Drawing tool for filling closed figures.  Fill occurs
 * using a scanline algorithm, defined in initScanFill, propScanFill,
 * and getStartOfSpan functions.  Anti-aliasing is not recommended for
 * strokes because it will break this fill algorithm - a tint scanline
 * algorithm might allow a more natural fill.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 *
 */
public class FillTool extends Tool {
	
	private static int oldColor;
	private static Stack m_stack;
	
	/**
	 * Constructor - Default tool properties
	 */
	public FillTool(){
		super();
	}
	
	/**
	 * Constructor - Cursor, name, and icon given.
	 * 
	 * @param c Cursor
	 * @param s Name
	 * @param i Icon for toolbox
	 */
	public FillTool(Cursor c, String s, ImageIcon i)
	{
		super(c, s, i);
	}
	
	/**
	 * initScanFill.  This function begins the scanline fill algorithm.
	 * The color at Point start becomes the seed color.  Every adjacent pixel
	 * with matching color will be filled with the new color.
	 * 
	 * A simple fill algorithm would recursively look at every adjacent pixel.
	 * In Java, this results in stack overflow for all but the smallest areas.
	 * The scanline algorithm used below identifies rows recursively, and fills
	 * those rows iteratively.  A Stack is used to keep track of the current row.
	 * 
	 * @param start Initial point with the seed color/location
	 * @param bg Background surface containing image
	 */
	public static void initScanFill(Point start, BackgroundSurface bg) {
		int x = start.x;
		int y = start.y;
		
		int oColor = bg.m_buffImg.getRGB(x,y);
		if( oColor == Tool.getForegroundColor().getRGB() )
			return;
		oldColor = oColor;
		
		m_stack = new Stack();
		
		m_stack.push(getStartOfSpan(start, bg));
		propScanFill(bg);		
	}
	
	/**
	 * 	Pop stack to get current point, run alg, then
	 *  call propScan.  Alg assumes that popped point the rightmost
	 *	in a span, and fills all matching points to the left with
	 *	the new color.
	 *
	 * 	@param bg Background surface with image.
	 */
	private static void propScanFill(BackgroundSurface bg) {
		if( m_stack.isEmpty() ) {
			return;
		}
		
		Point pt = (Point)m_stack.pop();

		boolean prevUpInSpan = false;
		boolean prevDownInSpan = false;
		int color, x = pt.x;
		int newColor = Tool.getForegroundColor().getRGB();

		while( x >= 0 && bg.m_buffImg.getRGB(x, pt.y) == oldColor ) {
			bg.m_buffImg.setRGB(x, pt.y, newColor);
			// Look up.  
			// We are starting with a rightmost point.  If the area above
			// hasn't been scanned, we look for the seed color.  If we
			// find it, we have found a new span and add that span's
			// rightmost point to the stack.
			if( pt.y > 0 ) {
				color = bg.m_buffImg.getRGB(x, pt.y-1);
				if( prevUpInSpan == false && color == oldColor ) {
					m_stack.push(getStartOfSpan(new Point(x, pt.y-1),bg));
					prevUpInSpan = true;
				}
				if( color != oldColor )
					prevUpInSpan = false;
			}
			// Look down
			if( pt.y+1 < bg.m_buffImg.getHeight() ){
				color = bg.m_buffImg.getRGB(x, pt.y+1);
				if( prevDownInSpan == false && color == oldColor ) {
					m_stack.push(getStartOfSpan(new Point(x, pt.y+1),bg));
					prevDownInSpan = true;
				}
				if( color != oldColor )
					prevDownInSpan = false;
			}
			x--;
		}
		propScanFill(bg);
	}
	
	/**
	 * Given a point and an image, this function returns the 
	 * rightmost point in the same row in the image with the same color.
	 * 
	 * @param start Starting point
	 * @param bg Background image
	 * @return Rightmost point in a span of pixels with the same color as
	 * the starting point.
	 */
	private static Point getStartOfSpan(Point start, BackgroundSurface bg){
		int x = start.x;
		int y = start.y;
		
		int oColor = bg.m_buffImg.getRGB(x,y);
		int i=x;

		while(oColor == oldColor && i+1<bg.m_buffImg.getWidth()) {
			i++;
			oColor = bg.m_buffImg.getRGB(i,y);
		}

		return new Point( (i==0) ? 0: i-1,y);
	}
	
}