/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.CharacterLabel;

/**
 * ToolBox class - Implements the front end of the drawing toolkit.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 9/23/03
 *
 */
public class ToolBox extends JDialog implements ActionListener {

	protected ArrayList tools;				// Array of tools
	protected JButton colorButton;			// Custom color chooser
	protected CloverWindow m_parent;		// parent
	protected JToggleButton m_brushButton;	// Brush button
	protected JToggleButton m_fillButton;	// Fill button
	protected ColorButton[] m_colors;		// Color buttons for default
											// colors.

	public static int DRAWING_MODE;			// Drawing mode mirrors selected
											// tool.
	public static Cursor CURSOR_PAINT;		// Cursor for painting
	public static Cursor CURSOR_FILL;		// Cursor for filling
	public static Cursor CURSOR_ERASE;		// Cursor for erasing

	/**
	 * Array of colors, used as default colors.
	 */
	private Color[] COLOR_ARRAY = {
		Color.red, Color.blue, Color.green, Color.black,
		Color.gray, Color.orange, Color.cyan, Color.YELLOW,
		
		Color.magenta, Color.pink, Color.white, new Color(100,100,100),
		new Color(227,201,164),
		new Color(163,115,50), new Color(100,70,30), new Color(0,0,80)
	};
	
	static final int NUM_COLS = 6;			// Number of columns for colors.
	
	/**
	 * Constructor - sets up toolbox and cursors, but does not
	 * attempt to construct the toolbox (needs tools to be populated)
	 * and does not set size (user defined).
	 * 
	 * @param par CloverWindow parent
	 */
	public ToolBox(CloverWindow par) { 
		super(par, "Drawing Toolbox", false);
		m_parent = par;
				
		tools = new ArrayList(5);
		m_colors = new ColorButton[COLOR_ARRAY.length];
		
		CURSOR_ERASE = getToolkit().createCustomCursor(
				Clover.createImageIcon2(Clover.ERASE_CURSOR_LOC).getImage(),
				new Point(16, 16),
				"Erase Cursor");
				
		CURSOR_FILL = getToolkit().createCustomCursor(
				Clover.createImageIcon2(Clover.FILL_CURSOR_LOC).getImage(),
				new Point(12, 12),
				"Fill Cursor");
		
		CURSOR_PAINT = m_parent.getToolkit().createCustomCursor(
				Clover.createImageIcon2(Clover.BRUSH_CURSOR_LOC).getImage(),
				new Point(0, 31),
				"Paint Cursor");
		
		// Toolbox cannot be closed.  It is autmatically hidden when
		// not useful for current tab.
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	}
	
	/**
	 * Colors the brush icon to match the current foreground color.
	 * White pixels in the original icon are replaced with the
	 * current color.  The brush cursor is also updated.
	 * 
	 * @param c Brush color
	 */
	public void colorBrush(Color c) {
		BufferedImage bi = new BufferedImage(
			CloverWindow.PLAIN_BRUSH_ICON.getIconWidth(),
			CloverWindow.PLAIN_BRUSH_ICON.getIconHeight(),
			BufferedImage.TYPE_INT_ARGB);
		Graphics2D big = bi.createGraphics();
		big.drawImage(CloverWindow.PLAIN_BRUSH_ICON.getImage(), 0, 0, this);
		
		int tmp[] = new int[bi.getWidth() * bi.getHeight()];
		bi.getRGB(0,0,bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		for( int i=0; i<tmp.length; i++ )
		{
			// transfrom tmp[i] into a color
			int alpha = (tmp[i] >> 24) & 0xff;
			if(alpha==0)continue;
			Color color = new Color(tmp[i]);
			if( color.equals(Color.white) ) {
				tmp[i] = c.getRGB();				
			}
		}
		bi.setRGB(0, 0, bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		CloverWindow.COLORED_BRUSH_ICON = new ImageIcon(
			bi.getScaledInstance(bi.getWidth(),
			bi.getHeight(), BufferedImage.SCALE_FAST));
			
		for( int i=0; i<tools.size(); i++ ) {
			if( tools.get(i) instanceof BrushTool ) {
				((BrushTool)tools.get(i)).m_toolIcon = 
					CloverWindow.COLORED_BRUSH_ICON;
			}				
		}

		m_brushButton.setIcon(CloverWindow.COLORED_BRUSH_ICON);
		
		// Update cursor
		if( CloverWindow.COLORED_BRUSH_ICON != null )
			ToolBox.CURSOR_PAINT = m_parent.getToolkit().createCustomCursor(
				bi,
				new Point(0, 31),
				"Paint Cursor");
	}
	
	/**
	 * Creates a cursor for the given color, which may
	 * differ from the foreground color.  Used by comment layers.
	 * 
	 * @param c Cursor color.
	 * @return
	 */
	public static Cursor getColoredPencilCursor(Color c){
		BufferedImage bi = new BufferedImage(
			CloverWindow.PLAIN_BRUSH_ICON.getIconWidth(),
			CloverWindow.PLAIN_BRUSH_ICON.getIconHeight(),
			BufferedImage.TYPE_INT_ARGB);
		Graphics2D big = bi.createGraphics();
		big.drawImage(CloverWindow.PLAIN_BRUSH_ICON.getImage(), 0, 0, null);
		
		int tmp[] = new int[bi.getWidth() * bi.getHeight()];
		bi.getRGB(0,0,bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		for( int i=0; i<tmp.length; i++ )
		{
			// transfrom tmp[i] into a color
			int alpha = (tmp[i] >> 24) & 0xff;
			if(alpha==0)continue;
			Color color = new Color(tmp[i]);
			if( color.equals(Color.white) ) {
				tmp[i] = c.getRGB();				
			}
		}
		bi.setRGB(0, 0, bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		return Toolkit.getDefaultToolkit().createCustomCursor(
						bi,
						new Point(0, 31),
						"Paint Cursor");
	}
	
	/**
	 * Colors the bucket icon to match the current foreground color.
	 * White pixels in the original icon are replaced with the
	 * current color.  The fill cursor is not updated - it is
	 * static for all colors.
	 * 
	 * @param c Bucket color
	 */
	public void colorBucket(Color c) {
		BufferedImage bi = new BufferedImage(
			CloverWindow.PLAIN_BUCKET_ICON.getIconWidth(),
			CloverWindow.PLAIN_BUCKET_ICON.getIconHeight(),
			BufferedImage.TYPE_INT_ARGB);
		Graphics2D big = bi.createGraphics();
		big.drawImage(CloverWindow.PLAIN_BUCKET_ICON.getImage(), 0, 0, this);
		
		int tmp[] = new int[bi.getWidth() * bi.getHeight()];
		bi.getRGB(0,0,bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		for( int i=0; i<tmp.length; i++ )
		{
			// transfrom tmp[i] into a color
			int alpha = (tmp[i] >> 24) & 0xff;
			if(alpha==0)continue;
			Color color = new Color(tmp[i]);
			if( color.equals(Color.white) ) {
				tmp[i] = c.getRGB();				
			}
		}
		bi.setRGB(0, 0, bi.getWidth(), bi.getHeight(), tmp, 0, bi.getWidth());
		CloverWindow.COLORED_BRUSH_ICON = new ImageIcon(
			bi.getScaledInstance(bi.getWidth(),
			bi.getHeight(), BufferedImage.SCALE_FAST));
			
		for( int i=0; i<tools.size(); i++ ) {
			if( tools.get(i) instanceof FillTool ) {
				((FillTool)tools.get(i)).m_toolIcon = 
					CloverWindow.COLORED_BRUSH_ICON;
			}				
		}

		m_fillButton.setIcon(CloverWindow.COLORED_BRUSH_ICON);
	}
	
	/**
	 * Returns the cursor appropriate for currently selected
	 * drawing mode.
	 * 
	 * @return Cursor
	 */
	public static Cursor getCurrentToolCursor(){
		switch(DRAWING_MODE){
			case Constants.BRUSH_INDEX:
				return CURSOR_PAINT;

			case Constants.FILL_INDEX:
				return CURSOR_FILL;

			case Constants.ERASE_INDEX:
				return CURSOR_ERASE;
		}
		return Cursor.getDefaultCursor();
	}
	
	/**
	 * Adds a tool to the toolbox.
	 * 
	 * @param t New tool
	 */
	public void addTool(Tool t) {
		tools.add(t);
	}
	
	/**
	 * Sets color of color button.  If the color is changed, this function
	 * will automatically switch back to drawing mode.
	 * @param c
	 */
	public void setColorButton(Color c) {
		//TAKEN CARE OF when first tool's color is set.
		colorButton.repaint();
		if(DRAWING_MODE == Constants.ERASE_INDEX)
			m_brushButton.doClick();
	}
	

	/**
	 * Unselects all colors, allowing a single color to be selected.
	 */
	public void unselectAllColors(){
		for(int i=0; i<m_colors.length; i++){
			m_colors[i].setSelected(false);
		}
	}
	
	/**
	 * Constructs toolbox and colors.  A single toolbox
	 * instance is used throughout the application.
	 * 
	 */
	public void constructToolbox() {
		if( tools.size() <= 0 )
			return;
		
		// First add tools.  They are arranged in a 2 x 2 grid.
		JPanel buttons = new JPanel();
		buttons.setLayout(new GridLayout(2, 2, 5, 5));

		ButtonGroup bg = new ButtonGroup();

		for( int i=0; i<tools.size(); i++ ) {
			if (!(tools.get(i) instanceof Tool))
				break;
			// Handle Brush special (for changing icon)
			if( tools.get(i) instanceof BrushTool ) {
				BrushTool bt = (BrushTool)tools.get(i);
				m_brushButton = new JToggleButton(bt.getIcon());
				m_brushButton.setToolTipText(bt.getName());
				m_brushButton.addActionListener(this);
				m_brushButton.setName(""+i);
				m_brushButton.setMargin(new Insets(0,0,0,0));
				buttons.add(m_brushButton);
				bg.add(m_brushButton);
				m_brushButton.doClick();
			}
			// Handle Fill special (for changing icon)
			else if( tools.get(i) instanceof FillTool ) {
				Tool t = (Tool)tools.get(i);
				m_fillButton = new JToggleButton( t.getIcon() );
				m_fillButton.setToolTipText(t.getName());
				m_fillButton.addActionListener(this);
				m_fillButton.setName(""+i);
				buttons.add(m_fillButton);
				bg.add(m_fillButton);
			}
			// Handle all other tools here
			else {			
				Tool t = (Tool)tools.get(i);
				JToggleButton l = new JToggleButton( t.getIcon() );
				l.setToolTipText(t.getName());
				l.addActionListener(this);
				l.setName(""+i);
				buttons.add(l);
				bg.add(l);
			}
		}
		
		// Add the undo button
		JButton undo = new JButton(
			Clover.createImageIcon2(Clover.ICON_DIR+"Undo24.gif"));
		undo.setToolTipText("Undo");
		undo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				// undo == load previous 
				m_parent.undoImage();
			}
		});
		buttons.add(undo);
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weighty = 0;
		getContentPane().add(buttons, gbc);
		
		// Add color chooser
		colorButton = new JButton("Custom Color...");
		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Get new color (or reuse old color on cancel)
				Color color = JColorChooser.showDialog(m_parent,
					"Color Chooser", Tool.getForegroundColor());
				if( color == null ) 
					color = Tool.getForegroundColor();
				Tool.setForegroundColor(color);

				// Color brush and bucket icons / cursors
				colorBrush(color);
				colorBucket(color);
				
				// Set last colorbutton's color to custom color
				m_colors[m_colors.length-1].m_color = color;
				
				// Select last colorbutton
				unselectAllColors();
				m_colors[m_colors.length-1].setSelected(true);
			}
		});
		colorButton.setMargin(new Insets(12,12,12,12));
	
		JPanel jp_color = new JPanel();
		jp_color.setLayout(new GridLayout(1,1));
		jp_color.add(colorButton);
		
		// Add color palette
		 
		//if( transparent )
		//	COLOR_ARRAY[10] = Constants.COLOR_CLEAR;
		
		JPanel palette = new JPanel();
		palette.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.fill=GridBagConstraints.NONE;
		int i = 0;
		while( i < COLOR_ARRAY.length ) {
			m_colors[i] = new ColorButton(COLOR_ARRAY[i], this);
			palette.add(m_colors[i], c);
			i++;
			c.gridx = i % NUM_COLS;
			c.gridy = i / NUM_COLS;
		}
		
		// Click first color to select it.
		m_colors[0].dispatchEvent(new MouseEvent(m_colors[0],
									MouseEvent.MOUSE_CLICKED,
									0,
									MouseEvent.BUTTON1_DOWN_MASK,
									0,0,
									1,
									false));
		
		gbc.gridy = 1;
		gbc.insets = new Insets(5,5,5,5);
		getContentPane().add(palette, gbc);
		
		gbc.gridy = 2;
		gbc.insets = new Insets(0,0,0,0);
		getContentPane().add(jp_color, gbc);
		
		((JPanel)getContentPane()).setBorder(new EmptyBorder(5,5,5,5));
		repaint();
	}
	
	/**
	 * Action listener for each tool button.  Sets drawing mode
	 * based on button name.
	 */
	public void actionPerformed(ActionEvent e) {
		JToggleButton jb = (JToggleButton)e.getSource();
		int index;
		try {
			index = Integer.parseInt(jb.getName());
		}
		catch (Exception ex) { return; }
		
		if( !(tools.get(index) instanceof Tool) )
			return; 
		
		if( jb.isSelected() ) {
			DRAWING_MODE = index;

			CharacterLabel.setSelectTool(false);
			((ViewAndEditUI)m_parent.getInterface(
					Constants.VIEW_AND_EDIT_INDEX)).unselectCLabels(false);
			((ResponseUI)m_parent.getInterface(
					Constants.RESPONSE_INDEX)).unselectCLabels(false);
		}
	}
		
}


/**
 * ColorButton - Custom "toggle" buttons for choosing foreground
 * color.  Based on JLabel to give more of a "flat" feel than
 * the XP plaf allows.  Also store color data.
 * 
 * @author Terry J Bradley
 * @version 04.7.29
 *
 */
class ColorButton extends JLabel implements MouseListener {
	
	protected Color m_color;			// Color of button
	protected ToolBox m_parent;			// Toolbox parent
	
	private boolean isSelected = false;
	private boolean click = false;		// Used when mouse pressed
	private boolean gone = false;		// Used when mouse pressed
	
	static final Border m_selected = new CompoundBorder(
				new SoftBevelBorder(SoftBevelBorder.LOWERED),
				new EmptyBorder(11,11,11,11));
	static final Border m_normal = new CompoundBorder(
				new SoftBevelBorder(SoftBevelBorder.RAISED), 
				new EmptyBorder(11,11,11,11));
		
	/**
	 * Constructor - Builds a color button based on color for
	 * given toolbox.
	 * 
	 * @param c Color
	 * @param t Toolbox parent.
	 */
	public ColorButton(Color c, ToolBox t) {
		super();
		m_parent = t;
		m_color = c;
		
		setBorder(m_normal);
		addMouseListener(this);
	}
	
	/**
	 * Called when button is clicked.
	 * 
	 * @param value Selection value
	 */
	public void setSelected(boolean value){
		isSelected = value;
		if(value)
			setBorder(m_selected);
		else
			setBorder(m_normal);
	}
	
	/**
	 * Mouse click listener.  Sets up Toolbox for new
	 * color if it differs from old color.
	 */
	public void mouseClicked(MouseEvent e) {

		if( isSelected ) return;
		Tool.setForegroundColor(m_color);

		// If clause handles transpancy, but should no 
		// longer be necessary.
		if( m_color.equals(Constants.COLOR_CLEAR) ) {
			m_parent.setColorButton(Color.white);
			m_parent.colorBrush(Color.white);
			m_parent.colorBucket(Color.white);
		}
		else {
			m_parent.setColorButton(m_color);
			m_parent.colorBrush(m_color);
			m_parent.colorBucket(m_color);
		}
		m_parent.unselectAllColors();
		setSelected(true);
	}
	
	/**
	 * On mouse release, we apply the color iff the mouse
	 * cursor is still over the button.
	 */
	public void mouseReleased(MouseEvent e){
		if( !gone ) {
			Tool.setForegroundColor(m_color);

			if( m_color.equals(Constants.COLOR_CLEAR) ) {
				m_parent.setColorButton(Color.white);
				m_parent.colorBrush(Color.white);
				m_parent.colorBucket(Color.white);
			}
			else {
				m_parent.setColorButton(m_color);
				m_parent.colorBrush(m_color);
				m_parent.colorBucket(m_color);
			}
			m_parent.unselectAllColors();
			setSelected(true);
		}
		click = false;
		gone = false;
	}
	
	/**
	 * Mouse pressed - sets up booleans
	 */
	public void mousePressed(MouseEvent e){
		click = true;
		gone = false;
		setBorder(m_selected);
	}
	
	/**
	 * Mouse entered - allows wandering, clicked mouse
	 * to return.
	 */
	public void mouseEntered(MouseEvent e){
		if( click ) {
			gone = false;
			setBorder(m_selected);
		}
	}
	
	/**
	 * Mouse exited - allows wandering mouse to abort click
	 */
	public void mouseExited(MouseEvent e){
		if( !isSelected ) {
			setBorder(m_normal);
		}
		gone = true;
	}
	

	/**
	 * Custom paint function draws an oval with the given color.
	 * Quick and dirty, could be improved.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		if( m_color.equals(Constants.COLOR_CLEAR) )
			g.setColor(Color.white);
		else		
			g.setColor(m_color);
	
		g.fillOval(2,2,24,24);
	}
}
