/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ActionGrid;

import javax.swing.table.*;
import java.util.ArrayList;

import edu.uiuc.orchid.clover.*;

/**
 * DialogTableData:
 *  TableDataModel for DialogEntry.  A single instance is used for
 *  both Tables in DialogEntry.  Data is stored as an array of arrays.
 *  The first array containts Long (Character IDs).  All subsequent
 *  arrays contain Integers whose value corresponds to an action.
 * 
 * @author Terry J Bradley
 * @version 04.7.31
 *
 */
public class DialogTableData extends AbstractTableModel {
	
	protected ArrayList m_data;		// Replacement for chunk and char,
									// AL of { AL of Long (0)
									//		 { AL of Integer (>0) 
	
	/**
	 * Constructor - Initializes needed arrays
	 */
	public DialogTableData() {		
		m_data = new ArrayList();
		ArrayList al = new ArrayList();
		for( int i=0; i<Constants.NUM_BLANK_ROWS; i++ ){
			al.add(new Long(-1));
		}
		m_data.add(al);
		for( int i=1; i<Constants.NUM_BLANK_COLS; i++ ){
			m_data.add(new ArrayList());
		}
	}
	
	/**
	 * Sets the table data
	 * 
	 * @param data Table data
	 */
	public void setTableData(ArrayList data) {		
		m_data = data;
	}
	
	/**
	 * Gets the number of rows, given by the size of the first
	 * array.
	 * 
	 * @return Row count
	 */
	public int getRowCount() {
		try {
			return ((ArrayList)m_data.get(0)).size();
		}
		catch (Exception e) {}
		return 0;
	}
	
	/**
	 * Gets the number of columns, given by the number of arrays.
	 * Required by AbstractTabelModel
	 * 
	 * @return Column count
	 */
	public int getColumnCount() {
		return m_data==null ? 1 : m_data.size();
	}
	
	/**
	 * Gets the name of the specified column
	 * 
	 * @param column Column index
	 * @return Column's name
	 */
	public String getColumnName(int column) {
		return column==0 ? "Character" : "" + column;
	}
	
	/**
	 * Gets the editable status, required by AbstractTableModel.
	 * No cells are editable - action editing occurs in separate
	 * dialogs instead.
	 * 
	 * @param nRow Row
	 * @param nCol Column
	 * @return Editable status (always false)
	 */
	public boolean isCellEditable(int nRow, int nCol) {
		return false;
	}
	
	/**
	 * Gets the value at the given cell.  Returned as either
	 * an Integer or Long.
	 * 
	 * @param nRow Row
	 * @param nCol Column
	 * @return Value
	 */
	public Object getValueAt(int nRow, int nCol) {
		if( nRow < 0 || nRow >= getRowCount())
			return "";

		ArrayList v;
		try {
			v = (ArrayList)m_data.get(nCol);
		}
		catch (Exception ex) {
			v = new ArrayList();
		}
				
		if( nCol == 0 ) {
			return (Long)v.get(nRow);
		}
						
		Integer value;
		try {
			value = (Integer)v.get(nRow);
		}
		catch (Exception ex) {
			value = new Integer(0);
		}
		
		return value;
	}
	
	/**
	 * Provides direct access to the data.
	 * 
	 * @return Data
	 */
	public ArrayList getData(){
		return m_data;
	}
	
	/**
	 * Gets the number of arrays in the data.  Should be (usually) equivalent
	 * to getColumnCount()
	 * 
	 * @return Number of arrays in data
	 */
	public int getDataSize(){
		return m_data.size();
	}
	
	/**
	 * Gets a single sub-array for the given column
	 * 
	 * @param col Column
	 * @return Sub-array of values
	 */
	public ArrayList getData(int col){
		return (ArrayList)m_data.get(col);
	}
	
	/**
	 * Gets row corresponding to given character.
	 * 
	 * @param gid Character's ID
	 * @return Character's row
	 */
	public int getCharacterRow(int gid){
		ArrayList characters = (ArrayList)m_data.get(0);
		int result = -1;
		for(int i=0; i<characters.size(); i++){
			if(((Long)characters.get(i)).intValue() == gid ){
				result = i;
				break;
			}
		}
		return result;
	}
}