/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;

import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.DrawingTools.ToolBox;

/**
 * CharacterLabel - Display character in top layer of drawingSurface.
 * This class is responsible for resizing and moving character instances.
 * 
 * @author Terry Bradley
 * @version 04.7.30
 *
 */
public class CharacterLabel extends JLabel implements MouseListener,
	MouseMotionListener, KeyListener
{
	protected static boolean SELECT_TOOL = false;	// Previous versions of
													// Clover used a select
													// tool to select characters.
													// This functionality is
													// restored by setting
													// SELECT_TOOL to true.
													// (and adding select tool
													// to toolbox)
	protected ViewAndEditUI m_parent;		// Scene editor parent
	protected boolean m_selected;			// True if this character is selected
	
	private boolean m_resizing;				// True if currently resizing
	private boolean m_dragging;				// True if currently moving
	private Point oldPoint;					// Origin point
	private Point p1, p2;					// Corners of outline
	
	final static Border SELECTED_BORDER = new EtchedBorder(EtchedBorder.RAISED);
	final static Border EMPTY_BORDER = new EmptyBorder(1,1,5,5);
	
	protected int m_GlobalCharacterID;		// This character's ID
	
	private int RESIZE_MODE = -1;			// Which corner is being resized?
	final static int RESIZE_PRECISION = 8;	// Size of outline wrt resizing.
	
///////////////////////  CONSTRUCTOR(S)  //////////////////////////////////////
	
	/**
	 * Constructor - Creates label with given image, character id, and parent
	 * 
	 * @param img Character image
	 * @param id Character ID
	 * @param v Parent scene editor
	 */
	public CharacterLabel(ImageIcon img, int id, ViewAndEditUI v) {
		super(img);
		m_GlobalCharacterID = id;
		m_parent = v;

		p1 = new Point();
		p2 = new Point();
		
		m_selected = false;
		m_resizing = false;
		m_dragging = false;
		
		setBorder(EMPTY_BORDER);
		
		addMouseListener(this);
		addMouseMotionListener(this);
		addKeyListener(this);
	}
	
//////////////////////////  METHODS  //////////////////////////////////////////
	
	/**
	 * Sets label selection status.
	 * 
	 * @param b Selection status
	 */
	public void setSelected(boolean b) {
		m_selected = b;
		if( m_selected ) {
			requestFocus();
			setBorder(SELECTED_BORDER);
			m_parent.selectCellInTable(m_GlobalCharacterID);
		}
		else
			setBorder(EMPTY_BORDER);
	}
	
	/**
	 * Sets select tool usage for all labels
	 * 
	 * @param b Select tool usage
	 */
	public static void setSelectTool(boolean b) {
		SELECT_TOOL = b;
	}
	
	/**
	 * Returns character ID
	 * 
	 * @return Character ID
	 */
	public int getCharacterID(){
		return m_GlobalCharacterID;
	}
	
	/**
	 * Prompts first and then removes this character from the current
	 * scene.  Removal occurs through parent to account for actions
	 * and other scene-level character elements.
	 *
	 */
	public void removeCharacter() {
		// Done: [hci-3] Prompt first
		String gender = 
			m_parent.getStory().getCharacter(m_GlobalCharacterID).getGender() 
				== Constants.MALE ? "his" : "her";
		String name = m_parent.getStory().getCharacter(m_GlobalCharacterID).
						getName();
		if(name == null || name.length() < 1) {
			name = "this character";
			gender = "their";
		}
		if(!m_parent.m_parent.confirmDeletion(
				"<html>Do you want to remove "+name+" from the scene?<p>" +
				"  All of "+gender+" dialog and actions will be lost.",
				"Character"))
			return;
		
		// If Ok, remove character
		setVisible(false);
		
		// remove lc and cl and lines from dialog
		m_parent.removeCharacter(m_GlobalCharacterID);		
	}

	/**
	 * Detects mouse clicks on this label.
	 */
	public void mousePressed(MouseEvent e){
		if( !m_selected ) {
			m_parent.selectCharacter(m_GlobalCharacterID);
			if(!Constants.ONE_CLICK_DRAG){
				setFocusable(true);
				requestFocusInWindow(true);
				e.consume();
			}
		}
			
		// Initiate drag or resize based on cursor location
		Point p = e.getPoint();
		Dimension d = this.getSize();
		if( ( p.x + RESIZE_PRECISION >= d.width && 
			  p.y+RESIZE_PRECISION >= d.height ) ){ //SE
			RESIZE_MODE = SOUTH_EAST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
		}
		else if (p.x + RESIZE_PRECISION >= d.width && 
				p.y-RESIZE_PRECISION <= 0)	{		// NE
			RESIZE_MODE = NORTH_EAST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
		}
		else if(p.x - RESIZE_PRECISION <= 0 &&
				p.y+RESIZE_PRECISION >= d.height ) {	// SW
			RESIZE_MODE = SOUTH_WEST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
		}
		else if (p.x - RESIZE_PRECISION <= 0 && 
				p.y-RESIZE_PRECISION <= 0)	{		// NW
			RESIZE_MODE = NORTH_WEST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
		}
		else if( (p.x + RESIZE_PRECISION >= d.width) ){  //E
			RESIZE_MODE = EAST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
		}
		else if( (p.y + RESIZE_PRECISION >= d.height) ){ //S
			RESIZE_MODE = SOUTH;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
		}
		else if (p.x - RESIZE_PRECISION <= 0 )	{		// W
			RESIZE_MODE = WEST;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
		}
		else if (p.y - RESIZE_PRECISION <= 0) 	{		// N
			RESIZE_MODE = NORTH;
			m_parent.getWindow().getGlassPane().setCursor(
				Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
		}
		else
			RESIZE_MODE = CENTER;	

		if( RESIZE_MODE != CENTER ) {
			// Point in corner
			m_resizing = true;
			m_parent.getWindow().getGlassPane().setVisible(true);
			p1 = getLocation();
			p2.x = p1.x + getSize().width - 1;
			p2.y = p1.y + getSize().height - 1;
			Dimension dummy = getSize();
		}
		else {
			// Point in middle
			m_dragging = true;
		}
		oldPoint = p;
		
		// Show outline
		m_parent.getDrawingSurface().setOutlineBounds(getBounds());
		m_parent.getDrawingSurface().displayOutline(true);
				
		setFocusable(true);
		requestFocusInWindow(true);
		e.consume();
	}
	
	/**
	 * Mouse dragging adjusts outline to show character moving/resizing.
	 */
	public void mouseDragged(MouseEvent e) {
		if( m_dragging ) {
			Point newPoint = e.getPoint();
			// draw outline rectangle on parent (need point relative to parent)
			int x = e.getPoint().x + getLocation().x - oldPoint.x;
			int y = e.getPoint().y + getLocation().y - oldPoint.y;
			m_parent.getDrawingSurface().moveOutline(x,y,getWidth(), getHeight());
		}
		else if( m_resizing ) {
			Point p = SwingUtilities.convertPoint(CharacterLabel.this, 
												e.getPoint(), 
												m_parent.getDrawingSurface());
			switch(RESIZE_MODE){
				case EAST:
					p2.x = p.x;
					break;
				case SOUTH:
					p2.y = p.y;
					break;
				case NORTH:
					p1.y = p.y;
					break;
				case WEST:
					p1.x = p.x;
					break;
				case SOUTH_EAST:
					float ratio = 1.0f * getSize().width / getSize().height;
					float attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, (p1.y - p.y))));
					int width, height;
					if( attempt > ratio ){
						p2.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p2.x = p1.x - width;
					}
					else {
						p2.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p2.y = p1.y - height;
					}
					break;
				case SOUTH_WEST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p2.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p1.x = p2.x + width;
					}
					else {
						p1.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p2.y = p1.y - height;
					}
					break;
				case NORTH_WEST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p1.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p1.x = p2.x + width;
					}
					else {
						p1.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p1.y = p2.y + height;
					}
					break;
				case NORTH_EAST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p1.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p2.x = p1.x - width;
					}
					else {
						p2.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p1.y = p2.y + height;
					}
					break;
			}
			m_parent.getDrawingSurface().resizeOutline(p1,p2);
		}
		e.consume();
	}
	
	/**
	 * On release, actually move/resize/delete character.
	 */
	public void mouseReleased(MouseEvent e) {
		if( m_dragging ) {
			// Move on parent
			Point newLocation = e.getPoint();
			Point oldLocation = getLocation();
			newLocation.x = oldLocation.x + newLocation.x - oldPoint.x;
			newLocation.y = oldLocation.y + newLocation.y - oldPoint.y;
			
			// Check for bad location
			int top = 0;
			int bottom = m_parent.getDrawingSurface().getSize().height;
			int left = 0;
			int right = m_parent.getDrawingSurface().getSize().width;
			Point glassPoint = SwingUtilities.convertPoint(
										this,
										e.getPoint(),
										m_parent.getWindow().getContentPane());
			Component component = SwingUtilities.getDeepestComponentAt(
										m_parent.getWindow().getContentPane(), 
										glassPoint.x, 
										glassPoint.y);
			if( component instanceof TrashLabel ) {
				removeCharacter();
				m_parent.getWindow().setCursor(Cursor.getDefaultCursor());
				CloverWindow.CURSOR_LOCK = false;
				return;
			}
			else if( newLocation.x + getSize().width < left ||
				newLocation.x > right ||
				newLocation.y + getSize().height < top || 
				newLocation.y > bottom )
			{
				// Abort Move
				m_parent.getWindow().setCursor(Cursor.getDefaultCursor());
				CloverWindow.CURSOR_LOCK = false;
				return;
			}
			if( newLocation.equals(oldLocation) ) {
				m_parent.getDrawingSurface().displayOutline(false);
				m_parent.getWindow().setCursor(Cursor.getDefaultCursor());
				CloverWindow.CURSOR_LOCK = false;
				return;	
			}
			
			// Then really move
			m_parent.moveCharacter(newLocation, m_GlobalCharacterID);
			setLocation(newLocation);
			
		}
		else if( m_resizing ) {
			Point p = SwingUtilities.convertPoint(CharacterLabel.this, 
												e.getPoint(), 
												m_parent.getDrawingSurface());
			switch(RESIZE_MODE){
				case EAST:
					p2.x = p.x;
					break;
				case SOUTH:
					p2.y = p.y;
					break;
				case NORTH:
					p1.y = p.y;
					break;
				case WEST:
					p1.x = p.x;
					break;
				case SOUTH_EAST:
					float ratio = 1.0f * getSize().width / getSize().height;
					float attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, (p1.y - p.y))));
					int width, height;
					if( attempt > ratio ){
						p2.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p2.x = p1.x - width;
					}
					else {
						p2.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p2.y = p1.y - height;
					}
					break;
				case SOUTH_WEST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p2.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p1.x = p2.x + width;
					}
					else {
						p1.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p2.y = p1.y - height;
					}
					break;
				case NORTH_WEST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p1.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p1.x = p2.x + width;
					}
					else {
						p1.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p1.y = p2.y + height;
					}
					break;
				case NORTH_EAST:
					ratio = 1.0f * getSize().width / getSize().height;
					attempt = Math.abs(1.0f * (p1.x - p.x) / (Math.max(.01f, p2.y - p.y)));
					if( attempt > ratio ){
						p1.y = p.y;
						width = Math.round(ratio * (p1.y - p2.y));
						p2.x = p1.x - width;
					}
					else {
						p2.x = p.x;
						height = Math.round((p1.x - p2.x) / ratio);
						p1.y = p2.y + height;
					}
					break;
			}

			Dimension d = new Dimension(
				Math.abs(p1.x - p2.x), Math.abs(p1.y - p2.y));

			// Resize character instance
			m_parent.resizeCharacter(p1, p2, m_GlobalCharacterID);
			
			// Display resized character
			Image i = m_parent.getCharacterImage(m_GlobalCharacterID).
				getScaledInstance(d.width, d.height, Image.SCALE_AREA_AVERAGING);
			setIcon(new ImageIcon(i));
			
			setSize(d);
			setLocation(Math.min(p1.x, p2.x), Math.min(p1.y, p2.y));
			
			m_parent.getWindow().setCursor(Cursor.getDefaultCursor());
			m_parent.getWindow().getGlassPane().setVisible(false);
		}
		m_dragging = false;
		m_resizing = false;
		
		// Hide outline
		m_parent.getDrawingSurface().displayOutline(false);
		m_parent.getWindow().setCursor(Cursor.getDefaultCursor());
		CloverWindow.CURSOR_LOCK = false;
		
		// Update Thumbnail 
		e.consume();
	}
	
	public void mouseClicked(MouseEvent e) {}
	
	/**
	 * Update cursor to show movable (HAND) or resizable (RESIZE)
	 * areas of the label.
	 */
	public void mouseMoved(MouseEvent e) {
		// detect rollover if in corner
		Point p = e.getPoint();
		Dimension d = getSize();
		if(!CloverWindow.CURSOR_LOCK){
			if ((p.x + RESIZE_PRECISION >= d.width &&
			     p.y+RESIZE_PRECISION >= d.height))
				setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
			else if (p.x + RESIZE_PRECISION >= d.width && p.y-RESIZE_PRECISION <= 0)
				setCursor(Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
			else if (p.x - RESIZE_PRECISION <= 0 && p.y+RESIZE_PRECISION >= d.height )
				setCursor(Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
			else if (p.x - RESIZE_PRECISION <= 0 && p.y-RESIZE_PRECISION <= 0)
				setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
			else if (p.x + RESIZE_PRECISION >= d.width)
				setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
			else if (p.x - RESIZE_PRECISION <= 0 )
				setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
			else if (p.y + RESIZE_PRECISION >= d.height)
				setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
			else if (p.y - RESIZE_PRECISION <= 0)
				setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
			else
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		}
	}
		
	/**
	 * Will offer to delete selected character when delete key is pressed.
	 */
	public void keyTyped(KeyEvent e) {
		if( e.getKeyChar() == KeyEvent.VK_DELETE ) {
			removeCharacter();
		}
		e.consume();
	}
	
		
	public void mouseExited(MouseEvent e) {}
	
	/**
	 * Updates cursor on entry.
	 */
	public void mouseEntered(MouseEvent e) {
		if(!CloverWindow.CURSOR_LOCK){
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		}
		else
			setCursor(ToolBox.getCurrentToolCursor());
	}
	public void keyPressed(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
}