/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

import edu.uiuc.orchid.clover.*;

/**
 * CloverListButton:
 *  Button used in CloverList widget to represent characters,
 *  scenes, or whatever.  Supports a flat look and feel (label)
 *  as well as drag and drop operations.
 * 
 * @author Terry J Bradley
 * @version 04.8.01
 *
 */
public class CloverListButton extends JLabel {
	
	protected CloverList m_parent;
	protected int m_id;				// gid for characters; scene index for scenes
	protected boolean m_isNew;		// true only for "special" buttons
	protected boolean m_isSelected; // true only for selected button

	protected boolean m_dragging;	// true if this button is being dragged
	protected boolean droppable;	// true if button can be dropped on whatever's
									// currently below it
	protected Point startPoint;		// Starting point for drag operations

	static Border SELECTED_BORDER = new CompoundBorder(
		UIManager.getBorder("List.focusCellHighlightBorder"),
		new EmptyBorder(2,2,2,2));
	static Border NORMAL_BORDER = new EmptyBorder(3,3,3,3);
	
	public static final int LABEL_GAP = 5;		// Needed to size labels
	public static final int CHAR_WRAP = 17;		// Wrapping length for long names

	/**
	 * Constructor - Creates a CloverListButton with the given appearance,
	 * text, id, parent, and 'new' status.  A new or !normal button represented
	 * an item creation/addition mechanism and not a normal item in the list.
	 * 
	 * @param img Item image
	 * @param name Item text
	 * @param id Item ID
	 * @param par CloverList parent
	 * @param normal Normal status
	 */
	public CloverListButton(ImageIcon img, String name, 
							int id, CloverList par, boolean normal) 
	{
		super(name, img, JLabel.CENTER);
		setBackground(Constants.COLOR_CANVAS);
		setOpaque(false);
		m_parent = par;
		m_id = id;
		
		// Wrap text as needed
		setText(name);
	
		m_isNew = !normal;
		m_isSelected = false;
		m_dragging = false;
		startPoint = new Point();
	
		droppable = true;
	
		setHorizontalTextPosition(JButton.CENTER);
		setVerticalTextPosition(JButton.BOTTOM);
			
		setBorder(NORMAL_BORDER);
	}
	
	/**
	 * Overwritten setText uses html to properly format long names (inserting
	 * <\BR> as needed.)
	 */
	public void setText(String name){
		// Wrap text as needed
		String labelText = "<html>";
		String text = name;
		if( text != null && text.length() > 0 ) {
			while( text.length() > CHAR_WRAP ) {
				int lastSpace = getLastSpace((String)text.subSequence(0, CHAR_WRAP));
				labelText += text.subSequence(0, lastSpace) + "<BR>";
				text = text.substring(lastSpace);
			}
			labelText += text;
			super.setText( labelText );
		}
		else
			super.setText( name );
	}
	
	/**
	 * Returns the last space in a string.  Used to help format long names
	 * (word wrapping where possible)
	 * 
	 * @param s String to check
	 * @return Index of last space character
	 */
	private int getLastSpace(String s) {
		for( int i=s.length()-1; i>=0; i-- ){
			if( s.charAt(i) == ' ' )
				return i;
		}
		return s.length()-1;
	}

	/**
	 * Sets selection status and changes border to match.
	 * 
	 * @param b Selection status
	 */
	public void setSelected(boolean b){
		m_isSelected = b;
		setBorder( b ? SELECTED_BORDER : NORMAL_BORDER);
	}
	
	/**
	 * Checks selection status.
	 * 
	 * @return Selection status
	 */
	public boolean isSelected(){
		return m_isSelected;
	}
	
	/**
	 * Sets ID
	 * 
	 * @param id ID
	 */
	public void setID(int id){
		m_id = id;
	}
	
	/**
	 * Gets ID
	 * 
	 * @return ID
	 */
	public int getID(){
		return m_id;
	}

}