/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Tabs.ScriptUI;
import edu.uiuc.orchid.clover.Interface.TrashLabel;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.ScriptPane.*;

/**
 * ScriptListButton - Text-only buttons that display character names in
 * the Script editor.  Buttons only have to respond to drops in the script
 * area.
 * 
 * @author Terry J Bradley
 *
 */
public class ScriptListButton extends CloverListButton 
							  implements MouseListener, MouseMotionListener
{

	/**
	 * Constructor - Simpler than CloverListButton, as no image is required.
	 * 
	 * @param name Character name
	 * @param id Character ID
	 * @param par CloverList parent
	 * @param normal Normal status
	 */
	public ScriptListButton(String name,
			int id, CloverList par, boolean normal)
	{
		super(null, name, id, par, normal);
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	
	/**
	 * Overwritten setText hooks into the Script editor to rename
	 * characters to match the entered text.
	 */
	public void setText(String text){
		super.setText(text);
		setForeground(Color.black);
		if(m_parent == null)
			return;
		try {
			((ScriptUI)m_parent.m_parent.getInterface(Constants.SCRIPT_INDEX)).
				renameCharacter(m_id, text);
		} catch (NullPointerException e){
			System.err.println("Error setting script button text.  This error" +
					" is not an issue during new story / load story.");
			//e.printStackTrace();
		}
	}
	
	/**
	 * Sets editing mode for changing a character's name.  An
	 * undecorated dialog with text box will appear over the character's
	 * current name, and changes will be saved back to the story.
	 *
	 */
	public void setEditingMode(){
		// First, hide label text.
		setForeground(Constants.COLOR_LIST);
		
		// Then, bring up JTextField at proper location (see VaEUI for example)
		// TextField should have the character name, highlighted.
		final CharacterNameDialog cnd = new CharacterNameDialog(this);
		
		Thread th = new Thread(){
			public void run(){
				int time = 0;
				while(getLocation().y == 0 && time < 1000){
					try{
						wait(80);
					} catch(Exception e){
					} finally {
						time += 80;
					}
				}
				SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						cnd.show(ScriptListButton.this);
					}
				});
			}
		};
		th.start();
	}
	
	/**
	 * Empty function - static mode is automatically switched to when 
	 * the character dialog loses focus.
	 *
	 */
	public void setStaticMode(){
		
	}
	
	/**
	 * Gets location of the script editor - used to locate dialog.
	 * 
	 * @return Script editor location
	 */
	public Point getParentLocation(){
		return ((ScriptUI)m_parent.m_parent.getInterface(Constants.SCRIPT_INDEX)).
			getLocation();
	} 
	
	public void mousePressed(MouseEvent e){}
	
	/**
	 * Drag listener support multiple drog tarts for adding
	 * new dialog plus trash labels for character deletion.
	 */
	public void mouseReleased(MouseEvent e){
		if(!m_dragging)
			return;
		
		ScriptListButton jb = (ScriptListButton)e.getSource();
		Component glassPane = m_parent.getCloverGlassPane();
		
		Point glassPoint = SwingUtilities.convertPoint(
									jb,
									e.getPoint(),
									glassPane);
		Component component =
					SwingUtilities.getDeepestComponentAt(
								m_parent.getCloverContentPane(),
								glassPoint.x,
								glassPoint.y);
		
		m_dragging = false;
		m_parent.m_parent.setCursor(Cursor.getDefaultCursor());
		
		if( component == null )
			return;
		
		if( component instanceof TrashLabel)  
			// Delete Character
			m_parent.m_parent.deleteCharacter(m_id);

		if( component instanceof ScriptDialogPane )
			// Add dialog
			((ScriptDialogPane)component).getPane().addNewDialog(
					((ScriptDialogPane)component).getPosition(),
					ScriptListButton.this.m_id);
		
		if( component instanceof JButton ) {
			Component cpar = component.getParent().getParent();
			if(!(cpar instanceof ScriptDialogPane)){
				return;
			}
			// Add dialog
			((ScriptDialogPane)cpar).getPane().addNewDialog(
					((ScriptDialogPane)cpar).getPosition(),
					ScriptListButton.this.m_id);
		}
		
		if( component instanceof JTextArea) {
			Component cpar = component.getParent().getParent().getParent();
			if(!(cpar instanceof ScriptDialogPane)){
				return;
			}
			// Add dialog
			((ScriptDialogPane)cpar).getPane().addNewDialog(
					((ScriptDialogPane)cpar).getPosition(),
					ScriptListButton.this.m_id);
		}
		
		if( component instanceof ScriptText ){
			// Add dialog
			((ScriptText)component).getPane().addNewDialog(
					ScriptListButton.this.m_id);
		}

	}
	
	/**
	 * Character's names are editable on double-click / right-click
	 */
	public void mouseClicked(MouseEvent e){
		if(e.getClickCount() > 1 || e.getButton() == MouseEvent.BUTTON3){
			setEditingMode();
		}
	}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	
	public void mouseDragged(MouseEvent e){
		m_dragging = true;
		m_parent.m_parent.setCursor(CloverWindow.MOVE_CURSOR);
	}
	public void mouseMoved(MouseEvent e){}
}

/**
 * CharacterNameDialog - Displays an undecorated dialog with text box
 * for editing character names.
 * 
 * @author Terry J Bradley
 *
 */
class CharacterNameDialog extends JDialog {
	
	private JTextField jtf;		// Character name text field
	
	/**
	 * Constructor - Creates undecorated dialog for given
	 * button and adds a text box.
	 * 
	 * @param slb Parent button
	 */
	public CharacterNameDialog(final ScriptListButton slb){
		super();
		setUndecorated(true);

		jtf = new JTextField(12);
		jtf.setText(CloverWindow.stripHTML2(slb.getText()));
		jtf.selectAll();
		// Close and update on enter
		jtf.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				slb.setText(((JTextField)e.getSource()).getText());
				CharacterNameDialog.this.dispose();
			}
		});
		getContentPane().add(jtf);
			
		// Close and update on lost focus
		addWindowFocusListener(new WindowFocusListener(){
			public void windowLostFocus(WindowEvent e){
				slb.setText(jtf.getText());
				CharacterNameDialog.this.dispose();
			}
			public void windowGainedFocus(WindowEvent e){}
		});	
					
		pack();
		
		// Moves dialog to character name
		setLocationRelativeTo(slb);
	}
	
	/**
	 * When shown, update location to given button.
	 * 
	 * @param slb Button to cover.
	 */
	public void show(ScriptListButton slb){
		setLocationRelativeTo(slb);
		super.show();
	}
		
}