/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ScriptPane;

import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.Interface.Tabs.*;

/**
 * ScriptPane - Displays the story's script.  This currently consists
 * of a collection of ScriptDialogPane's arranged in a single column.
 * 
 * @author Terry J Bradley
 * @version 04.8.07
 *
 */
public class ScriptPane extends JPanel {
	
	protected JPanel m_spiralPane;		// Left pane could be used to
										// display a decorative spiral
	protected JScrollPane m_scriptPane;	// Main script area
	
	protected ScriptText m_script;		// Actual panel that contains
										// dialog panes and controls
										// a comment layer.
	protected ArrayList m_characters;	// Array of ScriptComboCharacters
	protected GridBagConstraints c;		// Used to arrange dialog panes
	protected ScriptUI m_parent;		// Parent
	
	/**
	 * Dimensions of the images used to define the left spirals.
	 * Currently the images are simple but still used/required.
	 */
	public static final int ICON_TOP_HEIGHT = 48;
	public static final int ICON_MID_HEIGHT = 23;
	public static final int ICON_BOT_HEIGHT = 32;
	public static final int ICON_WIDTH = 54;
	public static final int ICON_SPIRAL_WIDTH = 15;
	
	/**
	 * Setting DISPLAY_SPINE to true will show red spirals instead
	 * of a clean white page.
	 */
	static final boolean DISPLAY_SPINE = false;
	
	/**
	 * Constructor - Creates an empty scriptpane.  setupScriptPane(int)
	 * should be called prior to usage.
	 * 
	 * @param par ScriptUI parent
	 */
	public ScriptPane(ScriptUI par){
		super();
		m_parent = par;
		setLayout(new BoxLayout(ScriptPane.this, BoxLayout.X_AXIS));
		setBorder(new EmptyBorder(0,0,10,10));
		setBackground(Constants.COLOR_BACKDROP);
		
		m_characters = new ArrayList();
		
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e) {
				resizeWindow(ScriptPane.this.getSize().height -
						10);
			}
		});
	}
	
	/**
	 * Adds the spiralPane and main ScriptText / scriptPane
	 * to the ScriptPane.
	 * 
	 * @param height Height in px of the ScriptPane
	 */
	public void setupScriptPane(int height){
		resizeWindow(height);
		add(m_spiralPane);
		
		m_script = new ScriptText(Color.red, this);
		m_script.setLayout(new GridBagLayout());
		m_script.setBackground(Color.white);
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTH;
		m_scriptPane = new JScrollPane(m_script);
		m_scriptPane.setBorder(new MatteBorder(2,0,1,1,Color.BLACK));
		m_scriptPane.setVerticalScrollBarPolicy(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		m_scriptPane.setBackground(Color.white);
		m_scriptPane.setVisible(true);
		add(m_scriptPane);
		
		this.revalidate();
		this.repaint();
	}
	
	/**
	 * Given a set of ScriptElements and an array of characters,
	 * this function creates and adds ScriptDialogPanes for each
	 * element and adds them to the ScriptText.
	 * 
	 * @param data ArrayList of ScriptElement
	 * @param chs ArrayList of ScriptComboCharacters
	 */
	public void populateScriptPane(ArrayList data, ArrayList chs){
		if(m_characters == null)
			m_characters = new ArrayList();
		
		if(chs != null) {
			m_characters.clear();
			for(int i=0; i<chs.size(); i++){
				m_characters.add(chs.get(i));
			}
		}
		
		m_script.removeAll();
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTH;
		c.gridy = 0;
		for(int i=0; i<data.size(); i++){
			if(!(data.get(i) instanceof ScriptElement))
				continue;
			ScriptDialog sd = (ScriptDialog)data.get(i);
			m_script.add(new ScriptDialogPane(
					sd.line,
					sd.gid,
					m_characters,
					c.gridy, this), c);
			c.gridy++;
		}

	}
	
	/**
	 * Given a set of ScriptElements,
	 * this function creates and adds ScriptDialogPanes for each
	 * element and adds them to the ScriptText.  The set of characters
	 * is presumed to have already been defined.
	 * 
	 * @param data ArrayList of ScriptElement
	 */
	public void populateScriptPane(ArrayList data){
		populateScriptPane(data, null);
	}
	
	/**
	 * Updates the script as stored by the Story (a collection
	 * of ScriptElements) to reflect the current script.  This
	 * function MUST be called to successfully store the Script
	 * prior to saving.
	 *
	 */
	public void updateStory(){
		Component[] dialogs = m_script.getComponents();
		int row = 0;
		for(int i=0; i<dialogs.length; i++){
			if(!(dialogs[i] instanceof ScriptDialogPane))
				continue;
			ScriptDialog sd = new ScriptDialog(
					((ScriptComboCharacter)
					((ScriptDialogPane)dialogs[i]).m_name.getSelectedItem()).
					gid.intValue(),
					((ScriptDialogPane)dialogs[i]).m_line.getText());
			m_parent.getStory().updateDialog(row, sd);
			row++;
		}
	}
	
	/**
	 * Adds a character to the end of every ScriptDialogPane's combo box.
	 * 
	 * @param gid Character's ID
	 * @param name Character's name
	 */
	public void addCharacter(int gid, String name){
		m_characters.add(new ScriptComboCharacter(name, new Integer(gid)));
		
		updateCharacterNames();
	}
	
	/**
	 * Updates the character's name in the stored ArrayList.  Then updates
	 * every ScriptDialogPane to reflect the name change.
	 * 
	 * @param gid Character's ID
	 * @param newName Character's new name
	 */
	public void renameCharacter(int gid, String newName){
		// Create a new ScriptComboCharacter[]
		for(int i=0; i<m_characters.size(); i++){
			if(((ScriptComboCharacter)m_characters.get(i)).gid.intValue() == gid){
				((ScriptComboCharacter)m_characters.get(i)).name = newName;
				break;
			}
		}
		
		updateCharacterNames();
	}
	
	/**
	 * Removes every dialog spoken by character, and removes character
	 * from every ScriptDialogPane's combo box.
	 * 
	 * @param gid Character's ID
	 */
	public void deleteCharacter(int gid){
		Component[] dialogs = m_script.getComponents();
		if(dialogs.length < 1)
			return;
		
		for(int i=0; i<m_characters.size(); i++){
			if(((ScriptComboCharacter)m_characters.get(i)).gid.intValue() == gid){
				m_characters.remove(i);
				break;
			}
		}
		
		for(int i=0; i<dialogs.length; i++){
			if(!(dialogs[i] instanceof ScriptDialogPane))
				continue;
			ScriptDialogPane sdp = (ScriptDialogPane)dialogs[i];
			if(sdp.getSelectedCharacter() == gid)
				m_script.remove(sdp);
			else
				sdp.updateCharacterNames(m_characters);
		}
		revalidate();
		repaint();
	}
	
	/**
	 * Updates the combo box names for every ScriptDialogPane in the Script
	 *
	 */
	protected void updateCharacterNames(){
		Component[] dialogs = m_script.getComponents();
		if(dialogs.length < 1)
			return;
		
		for(int i=0; i<dialogs.length; i++){
			if(dialogs[i] instanceof ScriptDialogPane){
				((ScriptDialogPane)dialogs[i]).updateCharacterNames(m_characters);
			}
		}
	}
	
	/**
	 * Returns the number of characters in the Script.
	 * 
	 * @return Number of character
	 */
	public int getCharacterCount(){
		return m_characters.size();
	}
	
	/**
	 * Adds a new dialog at the location specified.  The position
	 * value for any subsequent dialog's is adjusted.
	 * 
	 * @param row Row of new dialog
	 * @param gid Speaker's ID
	 */
	public void addNewDialog(int row, int gid){
		ScriptDialogPane sdp = new ScriptDialogPane("", gid, m_characters, row, this);
		Component[] dialogs = m_script.getComponents();
		c.gridy = row;
		m_script.add(sdp, c);
		for(int i = c.gridy; i < dialogs.length; i++){
			c.gridy++;
			((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
			m_script.add(dialogs[i], c);
		}
		c.gridy++;
		ScriptDialog sd = new ScriptDialog(gid, "");
		m_parent.getStory().addDialog(row, sd);
		revalidate();
		repaint();
	}
	
	/**
	 * Adds a blank dialog at the end of the script.  The given
	 * character is selected by default.
	 * 
	 * @param gid Selected character's ID
	 */
	public void addNewDialog(int gid){
		if(c.gridy == -1)
			c.gridy++;
		ScriptDialogPane sdp = new ScriptDialogPane("", gid, m_characters, c.gridy, this);
		m_script.add(sdp, c);
		ScriptDialog sd = new ScriptDialog(gid, "");
		m_parent.getStory().addDialog(c.gridy, sd);
		c.gridy++;
		revalidate();
		repaint();
	}
	
	/**
	 * Accurately moves a dialog from one location in the Script
	 * to another.  The display is updated, as is the position variable
	 * in every DialogPane.
	 * 
	 * @param source Position of moving dialog
	 * @param destination Position of dialog's destination
	 */
	public void moveDialog(int source, int destination){
		Component[] dialogs = m_script.getComponents();
		
		// Case 1: Moving down.  After source is removed,
		// everything at/below source must be adjusted.
		// And of course source must be reintroduced at dest.
		if(source < destination){
			m_script.removeAll();
			c.gridy = 0;
			for(int i=0; i<source; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}

			for(int i=source+1; i<=destination; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}
			((ScriptDialogPane)dialogs[source]).m_position = c.gridy;
			m_script.add(dialogs[source], c);
			c.gridy++;
			for(int i=destination+1; i<dialogs.length; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}
		}
		
		// Case 2: Moving up.  Must place source at top and 
		// readjust down to and including destination.  
		else if (destination < source){
			c.gridy = 0;
			m_script.removeAll();
			for(int i=0; i<destination; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}

			((ScriptDialogPane)dialogs[source]).m_position = c.gridy;
			m_script.add(dialogs[source], c);
			c.gridy++;
			for(int i=destination; i<source; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}
			for(int i=source+1; i<dialogs.length; i++){
				((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
				m_script.add(dialogs[i], c);
				c.gridy++;
			}
		}
		m_parent.getStory().moveDialog(source, destination);
		revalidate();
		repaint();
	}
	
	/**
	 * Deletes the dialog in the given row.
	 * 
	 * @param row Position of dialog to delete
	 */
	public void deleteDialog(int row){
		Component[] dialogs = m_script.getComponents();
		c.gridy=0;
		m_script.removeAll();
		for(int i=0; i<row; i++){
			((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
			m_script.add(dialogs[i], c);
			c.gridy++;
		}
		for(int i=row+1; i<dialogs.length; i++){
			((ScriptDialogPane)dialogs[i]).m_position = c.gridy;
			m_script.add(dialogs[i], c);
			c.gridy++;
		}
		m_parent.getStory().deleteDialog(row);
		revalidate();
		repaint();
	}
	
	/**
	 * Returns the number of dialog events in the Script
	 * 
	 * @return Number of spoken lines
	 */
	public int getDialogCount(){
		return m_script.getComponentCount();
	}
	
	/**
	 * Returns the ID of the 
	 * speaker of the last dialog in the Script
	 * 
	 * @return Last speaker's ID
	 */
	public int getLastSpeaker(){
		Component[] dialogs = m_script.getComponents();
		if(dialogs.length == 0)
			return 0;
		else
			return ((ScriptComboCharacter)((ScriptDialogPane)
					dialogs[dialogs.length - 1]).m_name.
					getSelectedItem()).gid.intValue();
	}
	
	/**
	 * Returns the ID of the second to last speaker.
	 * 
	 * @return Second-to-last speaker's ID
	 */
	public int getAlternatingSpeaker(){
		Component[] dialogs = m_script.getComponents();
		if(dialogs.length < 2)
			return 0;
		else
			return ((ScriptComboCharacter)((ScriptDialogPane)
					dialogs[dialogs.length - 2]).m_name.
					getSelectedItem()).gid.intValue();
	}
	
	/**
	 * Clears the script.
	 *
	 */
	public void clearScript(){
		m_script.removeAll();
		m_characters.clear();
		
		m_script.resetImage();
	}
	
	/**
	 * Returns the ScriptText commentLayer (which also contains all the
	 * ScriptDialogPanes).
	 * 
	 * @return Comment Layer
	 */
	public ScriptText getCommentLayer(){
		return m_script;
	}
	
	/**
	 * Called when the window is resized.  This adjusts the spiral images
	 * but is still necessary when they are not in use.
	 * 
	 * @param height New height in px of ScriptPane
	 */
	public void resizeWindow(int height){
		if(m_spiralPane == null) {
			m_spiralPane = new JPanel();
			m_spiralPane.setLayout(new BoxLayout(m_spiralPane, BoxLayout.Y_AXIS));
			m_spiralPane.setBackground(Constants.COLOR_BACKDROP);
		}
		m_spiralPane.removeAll();
		
		int h = height - ICON_TOP_HEIGHT - ICON_BOT_HEIGHT - 1;
		if(h < 0) {
			return;
		}
		
		// First add the top label
		ImageIcon img = Clover.createImageIcon2(
				(DISPLAY_SPINE) ? 
						Clover.SCRIPTPANE_DIR+"spine_top_trans.gif" :
				    	Clover.SCRIPTPANE_DIR+"spine_top_trans_ns.gif");
		m_spiralPane.add(new JLabel(img));
		
		img = Clover.createImageIcon2(
				(DISPLAY_SPINE) ?
						Clover.SCRIPTPANE_DIR+"spine_mid_trans_3.gif" :
						Clover.SCRIPTPANE_DIR+"spine_mid_trans_ns.gif");

		while(h > ICON_MID_HEIGHT){
			m_spiralPane.add(new JLabel(img));
			h -= ICON_MID_HEIGHT;
		}
		
		img = Clover.createImageIcon2(
				(DISPLAY_SPINE) ?
						Clover.SCRIPTPANE_DIR+"spine_bottom_trans_5.gif" :
						Clover.SCRIPTPANE_DIR+"spine_bottom_trans_ns.gif" );
		m_spiralPane.add(new JLabel(img));
		
		img = Clover.createImageIcon2(Clover.SCRIPTPANE_DIR+
				"spine_filler_trans.gif");
		while(h > 0) {//11
			m_spiralPane.add(new JLabel(img));
			h--;
		}
		
		img = Clover.createImageIcon2(Clover.SCRIPTPANE_DIR+
				"spine_lastline_trans.gif");
		m_spiralPane.add(new JLabel(img));
		
		m_spiralPane.revalidate();
		m_spiralPane.repaint();
	}
	
	
}

