/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ScriptPane;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.*;
import javax.swing.*;

import edu.uiuc.orchid.clover.Constants;
import edu.uiuc.orchid.clover.DrawingTools.BrushTool;
import edu.uiuc.orchid.clover.DrawingTools.ToolBox;

/**
 * ScriptText - Modified JPanel that contains the ScriptDialogPanes
 * that visually depict the Script.  ScriptText is also responsible 
 * for implementing a comment layer.
 * 
 * @author Terry J Bradley
 * @version 04.8.07
 *
 */
public class ScriptText extends JPanel
							implements MouseListener, MouseMotionListener 
{
	protected ScriptPane m_parent;	// ScriptPane parent
	
	private BufferedImage m_img;	// comments to draw
	private boolean m_commentMode = false;
									// true = mouse drags add comments
	private boolean m_visible = false;
									// true = display comments
	private Cursor m_cursor;		// Comment cursor (red pencil)
	private Color m_color;			// Comments color (usually red)
	private Color m_textColor;		// Text color (black)
	private Color m_backColor;		// Background color (white)
	
	private Point m_oldPoint;		// Previous point (for drawing comments)
	
	private static final Object ANTI_ALIASING = Constants.BRUSH_ANTI_ALIAS;
									// Anti-aliasing flag

	
	/**
	 * Constructor - Creates a ScriptText with the given
	 * comment color and parent.  MUST RUN RESETIMAGE() BEFORE
	 * USING THE SCRIPTTEXT.
	 * 
	 * @param cursorColor Color of comments
	 * @param par ScriptPane parent
	 */
	public ScriptText(Color cursorColor, ScriptPane par){
		super();	
		m_parent = par;
		setLayout(new GridBagLayout());
		setBackground(Color.white);
		
		m_cursor = ToolBox.getColoredPencilCursor(cursorColor);	
		m_color = cursorColor;
		m_textColor = Color.black;
		m_backColor = Color.white;
		
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	/**
	 * Resets the comment layer.
	 *
	 */
	public void resetImage(){
		int width = getWidth();
		int height = getHeight();
		if(width <= 0 || height <= 0){
			width = Toolkit.getDefaultToolkit().getScreenSize().width;
			height = Toolkit.getDefaultToolkit().getScreenSize().height;
		}
		m_img = new BufferedImage(width, height, 
				BufferedImage.TYPE_INT_ARGB);
		repaint();
	}
	
	/**
	 * Returns the parent ScriptPane
	 * 
	 * @return ScriptPane parent
	 */
	public ScriptPane getPane(){
		return m_parent;
	}

	/**
	 * Overwritten paintComponent draws the comment layer
	 * if m_visible is true.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
	
		if(m_visible)
			g.drawImage(m_img, 0, 0, 
					m_img.getWidth(), 
					m_img.getHeight(),
					null);
	}

	/**
	 * If in the comment mode, comments are added on a mouse pressed
	 * event.
	 */
	public void mousePressed(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;

		m_oldPoint = e.getPoint();
	
		// Also draw a spec centered at e.getPoint();
		Graphics g = m_img.createGraphics();
		g.setColor(m_color);

		// Get upperleft corner
		int x = e.getPoint().x - BrushTool.STROKE_THICKNESS/2;
		int y = e.getPoint().y - BrushTool.STROKE_THICKNESS/2;

		g.fillRect(x, y, BrushTool.STROKE_THICKNESS, 
			BrushTool.STROKE_THICKNESS);

		g = null;
		e.consume();
	}

	public void mouseReleased(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;
	
		this.repaint();
		e.consume();
	}

	/**
	 * Always draws when dragging (if in comment mode)
	 */
	public void mouseDragged(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;
		
		Graphics2D g = m_img.createGraphics();
		g.setColor(m_color);
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
				ANTI_ALIASING);
		
		g.setStroke(BrushTool.STROKE);
		g.drawLine(m_oldPoint.x, m_oldPoint.y, e.getX(), e.getY());
		
		g = null;
	
		int dx = m_oldPoint.x - e.getX();
		int dy = m_oldPoint.y - e.getY();

		m_oldPoint = e.getPoint();

		if( (dx < BrushTool.THRESHOLD && dx > -BrushTool.THRESHOLD)
			&& (dy < BrushTool.THRESHOLD && dy > -BrushTool.THRESHOLD) )
			this.repaint();
		e.consume();
	}

	/**
	 * Entered and Exited are cursor functions.
	 */
	public void mouseEntered(MouseEvent e){
		if(m_visible && m_commentMode ){
			setCursor(m_cursor);
		}
	}
	
	public void mouseExited(MouseEvent e){
		if(m_visible && m_commentMode ){
			setCursor(Cursor.getDefaultCursor());
		}
	}


	public void mouseClicked(MouseEvent e){}
	public void mouseMoved(MouseEvent e){}

	/**
	 * Sets teh comment mode by adjusting the comment mode
	 * and visible booleans, changing the foreground color
	 * as needed, and setting the appropriate cursor.
	 * 
	 * @param on Comment mode
	 */
	public void setCommentMode(boolean on){
		m_commentMode = on;
		setEditable(!on);
		if(on) {	// Commenting: Disable text editing
			setCursor(m_cursor);
			setForeground(Color.GRAY);
		}
		else {		// Not commenting: Enable text editing
			setCursor(Cursor.getDefaultCursor());
			setForeground(Color.BLACK);
		}
	}

	/**
	 * Sets the editable status by adjusting the appearance of
	 * every ScriptDialogPane
	 * 
	 * @param on Comment mode
	 */
	protected void setEditable(boolean on){
		Component[] dialogs = getComponents();
		for(int i=0; i<dialogs.length; i++){
			try{
				ScriptDialogPane sdp = (ScriptDialogPane)dialogs[i];
				sdp.m_name.setEnabled(on);
				sdp.m_line.setEditable(on);
				sdp.m_name.setForeground(on ? m_textColor : Color.GRAY);
				sdp.m_line.setForeground(on ? m_textColor: Color.gray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}
	}

	/**
	 * Returns the comment mode.
	 * 
	 * @return Comment mode (true = adding comments)
	 */
	public boolean getCommentMode(){
		return m_commentMode;
	}

	/**
	 * Sets comment visibility.  Comments are shown when set to true
	 * during the paintComponent method.
	 * 
	 * @param vis Comment visibility status
	 */
	public void setCommentVisible(boolean vis){
		m_visible = vis;
	}

	/**
	 * Returns the comment visibility status
	 * 
	 * @return Comment visiblity status
	 */
	public boolean getCommentVisible(){
		return m_visible;
	}
	
	/**
	 * Writes the comment layer to given file, current used for 
	 * debugging purposes only.
	 * 
	 * @param path Location of output file.
	 */
	public void writeImage(String path){
		try{
			boolean success = ImageIO.write(m_img
						  ,"png",
						  new File(path));
		}
		catch(Exception e){
			System.err.println("Error writing file.");
		}
	}

}
