/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.speech.synthesis.Voice;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.CloverList.*;
import edu.uiuc.orchid.clover.DrawingTools.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.Interface.TrashLabel;

/**
 * CharacterUI - User interface for character creation.  This class
 * appears in the Characters tab and provides a drawing canvas, an
 * extra space for character properties, and a Cloverlist of character
 * thumbnails.  Characters created here exist at the Story/Global level.
 * 
 * @author Terry J Bradley
 * @version 04.8.10
 *
 */
public class CharacterUI extends CloverSplitPane implements Constants {
	
	public CloverList m_gallery;			// CloverList of characters
	public CloverWindow m_parent;			// parent component
	protected DrawingSurface m_drawing;		// local canvas
	protected JComboBox jl_voice;			// choose voice through combo
	protected JTextField jtf_name;			// textfield for entering name
	protected JRadioButton jrb_male;		// gender radio buttons
	protected JRadioButton jrb_female;
	protected JRadioButton jrb_record;		// voice selection buttons
	protected JRadioButton jrb_tts;
	protected JSlider js_pitch;				// voice properties
	protected JSlider js_tone;				//
	protected JSlider js_rate;				//
	protected JSlider js_volume;			//
	protected JButton jb_preview;			// TTS voice preview button
	protected GlobalCharacter m_gc;			// global character associated w/
											// dialog.
	public static Voice[] voices;			// Combo box of avail. TTS voices 
	private boolean isNew;					// True iff open char is new char 

	/**
	 * Constructor - Creates a new instance of CharacterUI.
	 *   
	 * @param owner CloverWindow parent
	 */
	public CharacterUI(CloverWindow owner){
		super(owner);
		m_parent = owner;
		
		// Size drawing canvas to match user's display
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension charCanvas;
		if(screenSize.width > 1550)
			charCanvas = new Dimension(CHARCANVAS_1600_WIDTH,
										CHARCANVAS_1600_HEIGHT);
		else if (screenSize.width > 1250)
			charCanvas = new Dimension(CHARCANVAS_1280_WIDTH,
										CHARCANVAS_1280_HEIGHT);
		else
			charCanvas = new Dimension(CHARCANVAS_1024_WIDTH,
										CHARCANVAS_1024_HEIGHT);
										
		layoutUI(charCanvas);
		
		// Set thumbnail update handling
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				resetThread();
				th_updateThumb.start();
			}
		};
		m_timer = new javax.swing.Timer(500, al);
		m_timer.setRepeats(false);
		
		resetThread();
	}
	
	/**
	 * Called when the user opens a (possibly new) story.
	 * Should configure the layout by creating a new gallery,
	 * loading a new m_gc, and loading a character/setting
	 * up the drawing canvas.
	 * 
	 * This function assumes that the previous story has already
	 * been saved (if desired).
	 * 
	 * @param newStory True for new story, false for existing story
	 * @param size Size of character canvas 
	 */
	public void openStory(boolean newStory, Dimension size){
		// Reset layout
		layoutUI(size);
		
		// Empty gallery
		m_gallery.clearList();
		
		// Fill gallery
		ArrayList al = getStory().getCharacterOrder();
		for(int i=0; i<al.size(); i++){
			Integer gid = (Integer)al.get(i);
			GlobalCharacter gc = getStory().getCharacter(gid.intValue());
			
			// Update thumbnail
			gc.makeThumbnail(gc.openImage(), Image.SCALE_SMOOTH);
			
			CharacterButton button = new CharacterButton(
				gc.getThumbnail(), gc.getName(), CharacterUI.this,
				gc.getID(), m_gallery, true);
			m_gallery.addItemToList(button);
		}
		
		// Load first character
		if( m_gallery.isEmpty() ){
			if(newStory)
				m_gc = null;
			m_gallery.clickAddButtton();
			try {
				Thread.sleep(1000);
			} catch (Exception x) {
			}
		}
		else {
			editCharacter(((Integer)al.get(0)).intValue(), false);
		}
		setupUI();
		updateGallery();
	}
	
	
	/**
	 * Called when New Character is clicked or dragged onto
	 * BackgroundSurface 
	 *
	 */
	public void createCharacter(){
		// Steps:
		// 1) Save currently edited character to story (prompt?)
			
		if( m_drawing.isChanged() || !isNew )
			saveCharacter();
		
		// 2) Add blank thumbnail to gallery
		if(getStory() != null)
			m_gc = new GlobalCharacter(getStory().getCharSize());
		else
			m_gc = new GlobalCharacter(getStory().getCharSize());
		
		isNew = true;
		m_gallery.setSelectionIndex(-1);
		m_gallery.selectButton();
		
		// 3) Load GC settings to each field
		setupUI();
	}

	/**
	 * Like createCharacter(), only no saving occurs.  The character
	 * is assumed to be new - this function adds the new character to
	 * the story initially.
	 * 
	 * @return Character's gid
	 */
	public void addCharacter(){
		// Get character data from this
		updateCharacter();
		
		// Add character to story
		m_gc.setCharID(getStory().addCharacter(m_gc), getStory().getCharSize());
		
		// Re-open character and setup GUI
		CharacterButton button = new CharacterButton(
			m_gc.getThumbnail(), m_gc.getName(), this,
			m_gc.getID(), m_gallery, true);
		m_gallery.addItemToList(button);
		m_gallery.selectButton();
		m_gallery.constructList(getStory().getCharacterOrder());
		
		m_drawing.setChanged(false);
		isNew = false;
	}
	

	/**
	 * Adds the given character to this interface.  This is called when
	 * a new character is created, either through this interface or
	 * the ScriptUI.
	 * 
	 * @param gc Character to add
	 */
	public void addCharacter(final GlobalCharacter gc){
		m_gc = gc;
		isNew = false;
		
		Thread th = new Thread(){
			public void run(){
				gc.makeThumbnail(Image.SCALE_AREA_AVERAGING);
				//gc.setThumbnail(
				//	new ImageIcon(
				//		m_drawing.getThumbnailImage(
				//				Constants.CHAR_THUMB_WIDTH,
				//				Constants.CHAR_THUMB_HEIGHT,
				//				Image.SCALE_AREA_AVERAGING)));
				CharacterButton button = new CharacterButton(
					gc.getThumbnail(), gc.getName(), CharacterUI.this,
					gc.getID(), m_gallery, true);
				m_gallery.addItemToList(button);
				SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						m_gallery.constructList(getStory().getCharacterOrder());
						m_gallery.selectButton();
						m_gallery.revalidate();
						setupUI();
					}
				});
			}
		};
		th.start();		
				
	}
	
	/**
	 * Stores character information for the currently open character
	 * in the Story.  m_gc must already have a global ID assigned.
	 * 
	 * Current version of this function does not use
	 * multithreading to ensure that character's are saved
	 * before any data is lost.
	 *
	 */
	public void saveCharacter(){
		if( m_gc == null || m_gc.getID() < 0 ) return;
		
		// TODO: Renew multi-threading???
		m_parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		// Populate gc
		if( m_gc.getID() == -1 )
			addCharacter();
		GlobalCharacter gc = getStory().getCharacter(m_gc.getID());
		
		gc.saveImage(m_drawing.getBackgroundImage());
				
		// Store name, gender
		gc.setName(jtf_name.getText());
		if( jrb_male.isSelected() )
			gc.setGender(Constants.MALE);
		else if( jrb_female.isSelected() )
			gc.setGender(Constants.FEMALE);
				
		// Supply default names if necessary
		if( gc.getName() == null || gc.getName().length() == 0 ) {
			if( gc.getGender() == Constants.MALE )
				gc.setName("John");
			else
				gc.setName("Jane");
		}
		jtf_name.setText(gc.getName());
			
		// Store voice info
		gc.setRecord(jrb_record.isSelected());
		gc.setPitch(js_pitch.getValue());
		gc.setVolume(js_volume.getValue());
		
		// Generate bubble offsets
		gc.setLeftBubbleLoc(m_drawing.generateBubbleOffset(false));
		gc.setRightBubbleLoc(m_drawing.generateBubbleOffset(true));
		
		// Update thumbnail list
		CloverListButton button = m_gallery.getItem(gc.getID());
		button.setIcon(gc.getThumbnail());
		button.setText(gc.getName());
		button.setID(gc.getID());	
		m_gallery.revalidate();
		
		// Update appearances in Scene Editor
		((ViewAndEditUI)m_parent.getInterface(VIEW_AND_EDIT_INDEX)).
			setCLabelIcon(gc.getID());
		((ResponseUI)m_parent.getInterface(RESPONSE_INDEX)).
			setCLabelIcon(gc.getID());
	
		m_drawing.setChanged(false);
				
		m_parent.setCursor(Cursor.getDefaultCursor());
	}
	
	/**
	 * Opens the character with ID == globalID for editing.
	 * The currently open character is saved iff save is true.
	 * 
	 * @param globalID Character ID of character to open
	 * @param save Current character is saved iff this is true
	 */
	public void editCharacter(int globalID, boolean save){
		// Steps:
		// 1) Save currently edited character to story
		if( save && m_gc.getID() >= 0 )
			saveCharacter();
		
		// 2) Load Global Character from Story
		m_gc = getStory().getCharacter(globalID);
		isNew = false;

		m_gallery.setSelectionIndex(globalID);
		m_gallery.selectButton();
		
		setupUI();
	}
	
	/**
	 * Renames the character with the given id.  If the open character's name
	 * is changed, that character is reloaded.
	 * 
	 * @param id ID of character whose name changes
	 * @param name New name of character
	 */
	public void renameCharacter(int id, String name){
		// Make change in gallery
		((CharacterButton)m_gallery.getItem(id)).setCharacterName(name);
		
		// if m_gc.getID == id, reload character from scratch
		if(m_gc.getID() == id){
			editCharacter(id, false);
		}
	}
	
	/**
	 * Restores the current character, disregarding any and all changes.
	 *
	 */
	public void revertCharacter(){
		// Don't save!
		int gid = m_gallery.getSelectedItemIndex();
		if( gid < 0 ) {
			m_gc = new GlobalCharacter(getStory().getCharSize());
			isNew = true;
		}
		else {
			m_gc = getStory().getCharacter(gid);
			isNew = false;	
		}		
		setupUI();
	}
	
	/**
	 * Deletes character from gallery without prompt.
	 * 
	 * @param globalID = Character's global id
	 */
	public void deleteCharacter(int globalID){
		// Get the position of the "next" character, according
		// to the getPreviousItemInList() function
		Integer next = getPreviousItemInList(new Integer(globalID));
		
		// Remove from gallery
		m_gallery.removeItemFromList(globalID);
		
		// Reset UI
		if( next.intValue() != -1 ){
			editCharacter(next.intValue(), false);
		}
		
		// If next == -1, another function takes care of adding
		// a blank character.
	}
	
	/**
	 * Deletes the currently selected character.  
	 * 
	 * (Should m_gc be used instead?)
	 *
	 */
	public void deleteSelectedCharacter(){
		int gid = m_gallery.getSelectedItemIndex();
		if( gid < 0 )
			return;
		else
			m_parent.deleteCharacter(gid);
	}
	
	/**
	 * Returns the character ID of the currently open character
	 * 
	 * @return Open character's ID
	 */
	public int getSelectedCharacter(){
		return m_gc.getID();
	}
	
	/**
	 * Update the current character's thumbnail and other
	 * properties like gender, name, and voice.  This stores
	 * property changes from the interface to the GlobalCharacter 
	 * instance, and is very similar to outright saving.
	 *
	 */
	public void updateCharacter(){
		updateThumbnail(new Integer(m_gc.getID()));
		
		// Store name, gender
		m_gc.setName(jtf_name.getText());
		if( jrb_male.isSelected() )
			m_gc.setGender(Constants.MALE);
		else if( jrb_female.isSelected() )
			m_gc.setGender(Constants.FEMALE);
			
		m_gc.setVolume(js_volume.getValue() / 100.0f);
		m_gc.setPitch((js_pitch.getValue()+40)*2.0f);
		m_gc.setTone(js_tone.getValue()*1.0f);
		m_gc.setRate((js_rate.getValue()+40)*2.0f);
		m_gc.setVoiceIndex(jl_voice.getSelectedIndex());
	}
	
	
	/**
	 * Sets up this UI for the current m_gc character.
	 * Loads canvas and all settings from m_gc to the interface.
	 *
	 */
	public void setupUI(){
		// Load canvas
		m_drawing.setBackgroundImage(m_gc.openImage());
		m_drawing.repaint();
		m_drawing.setChanged(false);
		
		// Load name and gender
		jtf_name.setText(m_gc.getName());
		if (m_gc.getGender() == MALE) 
			jrb_male.setSelected(true);
		else
			jrb_female.setSelected(true);
			
		setupVoiceUI();
	}
	
	/**
	 * Sets all voice properties so that the UI matches the values
	 * associated with m_gc.
	 *
	 */
	public void setupVoiceUI(){
		// If recording voices, disable all the tts stuff.  Otherwise,
		// enable it.
		jl_voice.setEnabled(!m_gc.getRecord());
		js_rate.setEnabled(!m_gc.getRecord());
		js_volume.setEnabled(!m_gc.getRecord());
		jrb_record.setSelected(m_gc.getRecord());
		
		// Preview button is enabled only when TTS is selected and available
		jb_preview.setEnabled(!m_gc.getRecord() && CloverWindow.TTS_ENABLED);
		
		// TTS can only be selected if it has initialized correctly.
		jrb_tts.setEnabled(CloverWindow.TTS_ENABLED);
	}
		

	/**
	 * Resets the voice options and reconstructs the gallery of 
	 * character thumbnails.
	 *
	 */
	public void updateGallery(){
		// Setup voices too
		jl_voice.removeAllItems();
		if(CloverWindow.TTS_ENABLED){
			voices = m_parent.m_speechEngine.getVoices();
			for(int i=0; i<voices.length; i++){
				String shortName = voices[i].getName();
				if(shortName.length() > 25)
					shortName = shortName.substring(0, 25) + "...";
				jl_voice.addItem(shortName);
			}
		}
		
		try{
			m_gallery.constructList(getStory().getCharacterOrder());
			m_gallery.revalidate();
		}
		catch (NullPointerException np){
			System.err.println("Character array == null");
		}
	}
	
	/**
	 * Returns the drawing surface
	 * 
	 * @return Character drawing surface
	 */
	public DrawingSurface getDrawingSurface(){
		return m_drawing;
	}
	
	/**
	 * Resets the updateThumbnail thread.  The CloverSplitPane timer
	 * is actually responsible for launching this thread to update a
	 * thumbnail.
	 *
	 */
	public void resetThread(){
		th_updateThumb = new Thread(){
			public void run(){
				//m_gc.makeThumbnail(Image.SCALE_AREA_AVERAGING);
				m_gc.setThumbnail(
						new ImageIcon(
								m_drawing.getThumbnailImage(
										Constants.CHAR_THUMB_WIDTH,
										Constants.CHAR_THUMB_HEIGHT,
										Image.SCALE_AREA_AVERAGING)));

				// Make this character official if it doesn't yet have
				// an id
				if( m_gc.getID() == -1 )
					addCharacter();

				// Update this character's thumbnail and name in the gallery.
				else {
					final CloverListButton cb = m_gallery.getItem(m_gc.getID());
					SwingUtilities.invokeLater(new Runnable(){
						public void run(){
							cb.setIcon(m_gc.getThumbnail());
							cb.setText(m_gc.getName());
							cb.setID(m_gc.getID());
						}
					});	
				}
			}
		};
		th_updateThumb.setPriority(Thread.MIN_PRIORITY);
		th_updateThumb.setName("Clover-Update Thumbnail");
	}
	
	/**
	 * Returns the previous item in the gallery.  If there is
	 * no previous item, but subsequent items exist, then
	 * 0 is returned.  If this was the only element in the list,
	 * -1 is returned.
	 * 
	 * @return Previous item index
	 */
	public Integer getPreviousItemInList(Integer base){
		ArrayList al = getStory().getCharacterOrder();
		int index = al.size();
		for(int i=0; i<al.size(); i++){
			if( ((Integer)al.get(i)).intValue() == base.intValue() ){
				index = i-1;
				break;
			}
		}
		if( index == -1 && al.size() > 1 )
			index = 0;
		if( index < 0 || index >= al.size() )
			return new Integer(-1);
		else
			return (Integer)al.get(index);
	}
	
	/**
	 * Lays out the interface for the given canvas size.
	 * 
	 * @param charSize Size of character canvas
	 */
	public void layoutUI(Dimension charSize){
		// For the split pane, a left pane holds the gallery and trash
		// can.  The gallery is responsible for implementing a scroll
		// interface as well as the "Add something" button.  This
		// function defines that button's label and action.
		JPanel leftSide = new JPanel();
		leftSide.setLayout(new BorderLayout());
		
		m_gallery = new CloverList(m_parent, true, "Add Character");
		m_gallery.setAddButtonAction(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if( m_drawing.isChanged() || !isNew )
					saveCharacter();

				m_parent.addCharacter();
			}
		});
		leftSide.add(m_gallery, BorderLayout.CENTER);
		
		JPanel trashPane = new JPanel();
		trashPane.setLayout(new BoxLayout(trashPane, BoxLayout.Y_AXIS));
		trashPane.add(Box.createVerticalStrut(5));
		trashPane.add(new TrashLabel(this));
		leftSide.add(trashPane, BorderLayout.SOUTH);
		
		this.setLeftComponent(leftSide);
		
		// The right side consists of the drawing canvas and name, gender,
		// and voice properties.  Everything is arranged within a gridBag
		// layout, allowing for a more precise layout.  HOWEVER,
		// an oversized character canvas will NOT BE DISPLAYED AT ALL, so
		// be careful with canvas sizing.
		JPanel rightSide = new JPanel();
		rightSide.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		JPanel drawingArea = new JPanel();
		m_drawing = new DrawingSurface(charSize,
			Constants.COLOR_CANVAS, m_parent, CHARACTER_INDEX, true);
		drawingArea.add(m_drawing);
		drawingArea.setPreferredSize(charSize);
		c.anchor = GridBagConstraints.NORTHWEST;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 3;
		c.insets = new Insets(3,3,3,3);
		c.weightx = 1;
		c.weighty = 1;
		rightSide.add(drawingArea, c);
		
		c.weightx = 0;
		c.weighty = 0;		
		c.fill = GridBagConstraints.NONE;
		
		JLabel jl_name = new JLabel("Name:  ");
		jtf_name = new JTextField(16);
		// The following listener updates the thumbnail's text
		// while the user is typing.
		jtf_name.getDocument().addDocumentListener(new DocumentListener(){
				public void insertUpdate(DocumentEvent e) {
					if( m_gc.getID() == -1 )
						addCharacter();
					else {
						try {
							m_gallery.getItem(m_gc.getID()).setText(jtf_name.getText());
						} catch(NullPointerException ex){}
					}
						
				}
				public void removeUpdate(DocumentEvent e) {}
				public void changedUpdate(DocumentEvent e) {}
		});

		JPanel jp_name = new JPanel();
		jp_name.setLayout(new BorderLayout());
		jp_name.add(jl_name, BorderLayout.WEST);
		jp_name.add(jtf_name, BorderLayout.CENTER);
		c.gridx = 1;
		c.gridy = 0;
		c.gridheight = 1;
		c.insets = new Insets(9,3,0,3);
		rightSide.add(jp_name, c);		
		
		JLabel jl_sex = new JLabel("Gender:  ");
		ButtonGroup bg = new ButtonGroup();
		jrb_male = new JRadioButton("Male");
		jrb_female = new JRadioButton("Female");
		ChangeListener cl_gender = new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				if(voices==null || voices.length != jl_voice.getItemCount())
					return;
				for(int i=0; i<voices.length; i++){
					if( (jrb_male.isSelected() && 
						(voices[i].getGender()==Voice.GENDER_MALE)) ||
						(jrb_female.isSelected() &&
						(voices[i].getGender()==Voice.GENDER_FEMALE)) ) {
						jl_voice.setSelectedIndex(i);
						break;
					}
				}
			}
		};
		jrb_male.addChangeListener(cl_gender);
		jrb_female.addChangeListener(cl_gender);
		bg.add(jrb_female);
		bg.add(jrb_male);
		JPanel jp_sex = new JPanel();
		jp_sex.setLayout(new BorderLayout());
		jp_sex.add(jl_sex, BorderLayout.WEST);
		JPanel jp_buttons = new JPanel();
		jp_buttons.setLayout(new FlowLayout(FlowLayout.LEFT));
		jp_buttons.add(jrb_male);
		jp_buttons.add(jrb_female);
		jp_sex.add(jp_buttons, BorderLayout.CENTER);
		c.gridx=1;
		c.gridy=1;
		c.insets = new Insets(3,3,3,3);
		rightSide.add(jp_sex, c);
		
		
		JLabel jl_record = new JLabel("Voice Source:  ");
		ButtonGroup bg_record = new ButtonGroup();
		jrb_record = new JRadioButton("Recorded Voice");
		ChangeListener cl = new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				if( m_gc.getRecord() != jrb_record.isSelected() ){
					m_gc.setRecord(jrb_record.isSelected());
					updateCharacter();
					setupVoiceUI();
				}
			}
		};
		jrb_record.addChangeListener(cl);
		jrb_tts = new JRadioButton("Computer Voice");
		jrb_tts.addChangeListener(cl);
		bg_record.add(jrb_record);
		bg_record.add(jrb_tts);
		JPanel jp_record = new JPanel();
		jp_record.setLayout(new BorderLayout());
		jp_record.add(jl_record, BorderLayout.WEST);
		JPanel jp_record_buttons = new JPanel();
		jp_record_buttons.setLayout(new FlowLayout(FlowLayout.LEFT));
		jp_record_buttons.add(jrb_record);
		jp_record_buttons.add(jrb_tts);
		jp_record.add(jp_record_buttons, BorderLayout.CENTER);	
		
		// Regarding Voice properties: Some are not supported by
		// current TTS engines.  Sliders and such are shown for these
		// properties, but they are permanently disabled, and
		// saving/updating funcations do NOT take these values into
		// consideration.
		JLabel label = new JLabel("Voice: ");
		JLabel jl_pitch = new JLabel("Pitch: ");
		JLabel jl_tone = new JLabel("Tone: ");
		JLabel jl_rate = new JLabel("Rate: ");
		JLabel jl_volume = new JLabel("Volume: ");
		String[] dummy = { "Default Voice" };
		jl_voice = new JComboBox(dummy);
		Hashtable sliderLabels = new Hashtable(5);
		sliderLabels.put(new Integer(0), new JLabel("Lo"));
		sliderLabels.put(new Integer(25), new JLabel());
		sliderLabels.put(new Integer(50), new JLabel());
		sliderLabels.put(new Integer(75), new JLabel());
		sliderLabels.put(new Integer(100), new JLabel("Hi"));
		js_pitch = new JSlider(0, 100);
		js_pitch.setLabelTable(sliderLabels);
		js_pitch.setPaintLabels(true);
		js_pitch.setMajorTickSpacing(25);
		js_pitch.setPaintTicks(true);
		js_pitch.setEnabled(false);
		js_tone = new JSlider(0, 100);
		js_tone.setLabelTable(sliderLabels);
		js_tone.setPaintLabels(true);
		js_tone.setMajorTickSpacing(25);
		js_tone.setPaintTicks(true);
		js_tone.setEnabled(false);
		js_volume = new JSlider(0, 100);
		js_volume.setLabelTable(sliderLabels);
		js_volume.setPaintLabels(true);
		js_volume.setMajorTickSpacing(25);
		js_volume.setPaintTicks(true);
		js_volume.setValue(100);
		js_rate = new JSlider(0,100);
		Hashtable rateLabels = new Hashtable(5);
		rateLabels.put(new Integer(0), new JLabel("Slow"));
		rateLabels.put(new Integer(25), new JLabel());
		rateLabels.put(new Integer(50), new JLabel());
		rateLabels.put(new Integer(75), new JLabel());
		rateLabels.put(new Integer(100), new JLabel("Fast"));
		js_rate.setLabelTable(rateLabels);
		js_rate.setPaintLabels(true);
		js_rate.setMajorTickSpacing(25);
		js_rate.setPaintTicks(true);
		jb_preview = new JButton("Test Voice");
		
		// The Preview button says a line with the TTS engine.
		// Currently only a plain text line is sent to the engine,
		// with other properties supplied with the speak() 
		// function.  The JSML line computes a jsml
		// compatible string, but this string is unused.
		jb_preview.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				updateCharacter();
				String line = "Hello";
				if(m_gc.getName().length() > 0)
				 	line += ", my name is "+m_gc.getName();
				String jsml = 
					"<jsml>\n"+
					"<div type=\"sent\">" +
					"<prosody volume=\""+m_gc.getVolume()+"\" pitch=\""+
					m_gc.getPitch()+"\" range=\""+
					m_gc.getTone()+"\">" + line + "</prosody></div>\n"+
					"</jsml>";
				m_parent.m_speechEngine.setVoice(voices[m_gc.getVoiceIndex()]);
				m_parent.m_speechEngine.speak(line, m_gc.getVolume(), 
					m_gc.getRate(), m_gc.getPitch(), m_gc.getTone());				
			}
		});
		JPanel jp_voice = new JPanel();
		jp_voice.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED),
			"Voice"));
		jp_voice.setLayout(new GridBagLayout());
		
		GridBagConstraints cint = new GridBagConstraints();
		cint.insets = new Insets(3,3,3,3);
		cint.gridx = 0;
		cint.gridy = 0;
		cint.gridwidth = 3;
		cint.weightx = 1.0;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.anchor = GridBagConstraints.WEST;
		jp_voice.add(jp_record, cint);
				
		cint.gridx = 0;
		cint.gridy = 1;
		cint.gridwidth = 1;
		cint.weightx = 0.0;
		jp_voice.add(label, cint);
		
		cint.gridx = 1;
		cint.gridy = 1;
		cint.gridwidth = 2;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.weightx = 1;
		jp_voice.add(jl_voice, cint);
		
		cint.gridx = 0;
		cint.gridy = 2;
		cint.gridwidth = 1;
		cint.anchor = GridBagConstraints.WEST;
		cint.fill = GridBagConstraints.NONE;
		cint.weightx = 0;
		jp_voice.add(jl_pitch, cint);
		
		cint.gridx = 1;
		cint.gridwidth = 2;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.weightx = 1;
		cint.anchor = GridBagConstraints.EAST;
		jp_voice.add(js_pitch, cint);
		
		cint.gridx = 0;
		cint.gridy = 3;
		cint.gridwidth = 1;
		cint.fill = GridBagConstraints.NONE;
		cint.weightx = 0;
		cint.anchor = GridBagConstraints.WEST;
		jp_voice.add(jl_tone, cint);
		
		cint.gridx = 1;
		cint.gridy = 3;
		cint.gridwidth = 2;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.weightx = 1;
		cint.anchor = GridBagConstraints.EAST;
		jp_voice.add(js_tone, cint);
		
		cint.gridx = 0;
		cint.gridy = 4;
		cint.gridwidth = 1;
		cint.fill = GridBagConstraints.NONE;
		cint.weightx = 0;
		cint.anchor = GridBagConstraints.WEST;
		jp_voice.add(jl_rate, cint);
		
		
		cint.gridx = 1;
		cint.gridy = 4;
		cint.gridwidth = 2;
		cint.anchor = GridBagConstraints.EAST;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.weightx = 1;
		jp_voice.add(js_rate, cint);
		
		cint.gridx = 0;
		cint.gridy = 5;
		cint.gridwidth = 1;
		cint.anchor = GridBagConstraints.WEST;
		cint.fill = GridBagConstraints.NONE;
		cint.weightx = 0;
		jp_voice.add(jl_volume, cint);
		
		cint.gridx = 1;
		cint.gridy = 5;
		cint.gridwidth = 2;
		cint.anchor = GridBagConstraints.EAST;
		cint.fill = GridBagConstraints.HORIZONTAL;
		cint.weightx = 1;
		jp_voice.add(js_volume, cint);
		
		cint.gridx = 2;
		cint.gridy = 6;
		cint.gridwidth = 1;
		cint.anchor = GridBagConstraints.EAST;
		cint.fill = GridBagConstraints.NONE;
		jp_voice.add(jb_preview, cint);
		
		c.gridx = 1;
		c.gridy = 2;
		rightSide.add(jp_voice, c);
		
		rightSide.setBorder(new LineBorder(Color.black));
		
		JPanel pane = new JPanel();
		pane.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy=0;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.CENTER; 
		pane.setBackground(Constants.COLOR_BACKDROP);
		pane.setBorder(new EmptyBorder(10,10,10,10));
		pane.add(rightSide, c);	
		
		//JPanel rightMega = new JPanel();
		//rightMega.setLayout(new BorderLayout());
		//rightMega.setBackground(COLOR_BACKDROP);
		//rightMega.setBorder(new EmptyBorder(10,10,10,10));
		//rightMega.add(pane, BorderLayout.CENTER);
		
		//this.setRightComponent(rightMega);
		this.setRightComponent(pane);
		this.setBorder(new EmptyBorder(3,3,3,3));
	}
}