/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;
import java.io.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.DrawingTools.*;

/**
 * NarrativeUI - Creates the narrative user interface, which displays
 * a notebook-like interface for entering text.  The text area is
 * very simple and supports a single font, weight, size, etc.
 * The notebook effect is painted as part of the NarrativeText
 * class.  In addition, NarrativeText supports a comment layer
 * for editing.
 * 
 * @author Terry Bradley
 * @date 9/20/03
 * @version 04.8.12
 *
 */
public class NarrativeUI extends JPanel implements Constants {
	
	protected CloverWindow m_parent;	// ClovwerWindow parent
	private JScrollPane scrollPane;		// Scrollpane to hold NarrativeText
	private NarrativeText narrText;		// Custom text area, supports comments
	protected UndoManager m_undoMan;	// For undo/redo support
	
	protected JButton m_undo = new JButton();	// Undo button
	protected JButton m_redo = new JButton();	// Redo button
	protected JButton m_clearComments;			// Clear comments button
	protected JToggleButton m_showComments;		// Show comments button
	protected JToggleButton m_commentMode;		// Comment mode button
	private static String[] fontNames;			// Font names for combo box
	
	protected boolean isChanged;				// True iff a change has
												// been made.
	
	// Constants
	private static String EMPTY_MESSAGE = "";	// Not really needed...
	private final static String[] FONT_SIZES = {
		"8", "10", "12", "14", "16", "18", "24", "36", "54", "72" };
												// Array of font sizes for
												// combo box.
	private final static int MENU_HEIGHT = 40;	// Expected menu height.
	
	/**
	 * Constructor - Builds the Narrative interface.  Most work
	 * appears in createScrollPane().
	 * 
	 * @param par CloverWindow parent
	 */
	public NarrativeUI(CloverWindow par) {
		super();
		m_parent = par;
		isChanged = false;	
		
		// Create ScrollPane w/ Text Pane
		createScrollPane();
		
		// Create MenuBar
		JPanel jp = createMenuPane();
		
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(3,3,3,3));
		
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout(0, 0));
		panel.setBackground(COLOR_BACKDROP);
		scrollPane.setBackground(COLOR_BACKDROP);
		panel.add(jp, BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);
		add(panel, BorderLayout.CENTER);
		
		// Add listener for hiding, showing the NarrUI.
		// When hidden, we write the text to the Story.
		// When shown, we backup the old narrative (for revert functionality.)
		addComponentListener(new ComponentListener() {
			public void componentHidden(ComponentEvent e) {
				if( narrText.getText() != null ) {
					Story sty = m_parent.getStory();
					if( sty != null )
						sty.setNarrative(narrText.getText());
				}
			}
			public void componentMoved(ComponentEvent e) {}
			public void componentResized(ComponentEvent e) {}
			public void componentShown(ComponentEvent e) {
				m_parent.getStory().backupNarrative();
				isChanged = false;
			}
		});
	}
	
	/**
	 * Updates undo/redo buttons based on their usefulness.
	 *
	 */
	private void updateButtons() {
		if( m_undoMan.canUndo() ) {
			if( !isChanged ) {
				isChanged = true;
			}
		}
		m_undo.setEnabled(m_undoMan.canUndo());
		m_redo.setEnabled(m_undoMan.canRedo());
	}
	
	/**
	 * Updates this UI (called when opening a story).
	 * Only functionality now consists of setting the narrative text.
	 * (Font, weight, etc. are not saved)
	 */
	public void updateNarrUI() {
		// Restore text.
		narrText.setText(getNarrative());
	}
	
	/**
	 * Called when creating a new story.  The sample narrative outline is
	 * copied into the working directory and displayed in the text area.
	 *
	 */
	public void newStory(){
		Clover.copyFile("narrative.txt",
				Clover.TEMP_DIR+"narrative.txt");
		
		updateNarrUI();
		
		narrText.resetImage();
	}
	
	/**
	 * Called when opening a story.  Sets narrative text, then
	 * sets the comment layer. 
	 *
	 */
	public void openStory(){
		updateNarrUI();
		narrText.resetImage();
		narrText.setCommentMode(false);
		narrText.setCommentVisible(false);
		m_showComments.setSelected(false);
		m_commentMode.setSelected(false);
		m_clearComments.setEnabled(false);
	}
	
	/**
	 * Creates the scrollPane and all its components (basically just
	 * the Narrative Text.)  Also initializes undo/redo.
	 *
	 */
	private void createScrollPane() {
		narrText = new NarrativeText(EMPTY_MESSAGE, 
				   m_parent.getUserPrefs().
							getColor(Constants.i_COMMENT_COLOR, Color.red));
		
		m_undoMan = new UndoManager();
		
		narrText.getDocument().addUndoableEditListener(
			new UndoableEditListener() {
				public void undoableEditHappened(UndoableEditEvent e) {
					m_undoMan.addEdit(e.getEdit());
					updateButtons();
				}
			});		
		
		// Set initial Font, Text, and Size
		// Update: Set to Arial in NarrativeText constructor
		narrText.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if( narrText.getText().length() == 0 )
					narrText.setText(EMPTY_MESSAGE);
				else
					narrText.selectAll();
			}
			public void focusLost(FocusEvent e) {}		
		});
		
		scrollPane = new JScrollPane(narrText, 
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			
		scrollPane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createEmptyBorder(0,10,10,10),
			BorderFactory.createLineBorder(Color.black)));

	}
	
	/**
	 * Creates a menu Pane, containing all the available editing buttons
	 * and combo boxes, including comment layer buttons.
	 * 
	 * @return JPanel containing menu buttons
	 */
	private JPanel createMenuPane() {
		// Load fonts for combo box
		fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().
			getAvailableFontFamilyNames();
		
		// BOLD BUTTON
		ImageIcon img = Clover.createImageIcon2(Clover.ICON_DIR+"Bold24.gif");
		JToggleButton bold = new JToggleButton(img);
		bold.setToolTipText("Bold");
		bold.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// toggle bold
				Font font = narrText.getFont();
				font = new Font(font.getFamily(), font.getStyle()^Font.BOLD, 
					font.getSize());
				narrText.setFont(font);
			}
		});
		
		// ITALICS BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Italic24.gif");
		JToggleButton italics = new JToggleButton(img);
		italics.setToolTipText("Italics");
		italics.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// toggle italics
				Font font = narrText.getFont();
				font = new Font(font.getFamily(), font.getStyle()^Font.ITALIC, 
					font.getSize());
				narrText.setFont(font);
			}
		});
		
		// UNDO BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Undo24.gif");
		m_undo = new JButton(img);
		m_undo.setToolTipText("Undo");
		m_undo.setEnabled(false);
		m_undo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					m_undoMan.undo();
				}
				catch (CannotUndoException cue) {
					// do nothing
				}
				updateButtons();
			}
		});
		
		// REDO BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Redo24.gif");
		m_redo = new JButton(img);
		m_redo.setToolTipText("Redo");
		m_redo.setEnabled(false);
		m_redo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					m_undoMan.redo();
				}
				catch (CannotRedoException cre) {
					// do nothing
				}
				updateButtons();
			}
		});	
		
		// CLEAR COMMENTS BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"clear_comments_2-24.gif");
		m_clearComments = new JButton(img);
		m_clearComments.setToolTipText("Clear Comments");
		m_clearComments.setEnabled(false);
		m_clearComments.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if ( m_parent.confirmDeletion("Remove all comments?",
						"Comments") )
					narrText.resetImage();
			}
		});	
		
		// COMMENT MODE BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"comment_mode_2-24.gif");
		m_commentMode = new JToggleButton(img);
		m_commentMode.setToolTipText("Add Comments");
		m_commentMode.setSelected(false);
		m_commentMode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JToggleButton jtb = (JToggleButton)e.getSource();
				narrText.setCommentMode(jtb.isSelected());
				if(jtb.isSelected() && !m_showComments.isSelected()) {
					m_showComments.doClick();
				}
				narrText.repaint();
			}
		});	
		
		// COMMENT SHOW/HIDE BUTTON
		img = Clover.createImageIcon2(Clover.ICON_DIR+"show_hide_comments_2-24.gif");
		m_showComments = new JToggleButton(img);
		m_showComments.setToolTipText("Show Comments");
		m_showComments.setSelected(false);
		m_showComments.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JToggleButton jtb = (JToggleButton)e.getSource();
				narrText.setCommentVisible(jtb.isSelected());
				m_clearComments.setEnabled(jtb.isSelected());
				if(!jtb.isSelected()) {
					narrText.setCommentMode(false);
					m_commentMode.setSelected(false);
					m_showComments.setToolTipText("Show Comments");
				}
				else
					m_showComments.setToolTipText("Hide Comments");
				narrText.repaint();
			}
		});	
		
	
		// ComboBoxes
		final JComboBox pickFont = new JComboBox(fontNames);
		int test= getFontIndex();
		pickFont.setSelectedIndex(getFontIndex());
		
		// Make sure CB and NarrText fonts match.
		if( narrText != null ) {
			Font font = narrText.getFont();
			String fontName = fontNames[pickFont.getSelectedIndex()];
		}	
		
		// FONT CHOOSER
		pickFont.setFont(new Font(pickFont.getFont().getFamily(), 
			Font.PLAIN,	16));
		ActionListener acl = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				// set font
				Font font = narrText.getFont();
				String fontName = fontNames[cb.getSelectedIndex()];
				font = new Font(fontName, font.getStyle(), font.getSize());
				narrText.setFont(font);
			}
		};
		pickFont.addActionListener(acl);
		
		// FONT SIZER
		JComboBox pickSize = new JComboBox(FONT_SIZES);
		pickSize.setFont(new Font(pickSize.getFont().getFamily(), 
					Font.PLAIN,	16));
		pickSize.setSelectedIndex(getSizeIndex());
		pickSize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				// set font
				Font font = narrText.getFont();
				int fontSize = Integer.parseInt(FONT_SIZES[cb.getSelectedIndex()]);
				font = new Font(font.getFamily(), font.getStyle(), fontSize);
				narrText.setFont(font);
			}
		});
		
		// JLabels (separators)
		JLabel s1 = new JLabel();
		s1.setBorder(new CompoundBorder(
			new EmptyBorder(0,4,0,4),
			new MatteBorder(15,2,15,1,Constants.COLOR_BACKDROP)));

		// PANEL CONSTRUCTION
		JPanel menuPanel = new JPanel();
		menuPanel.add(Box.createHorizontalGlue());
		menuPanel.add(pickFont);
		menuPanel.add(pickSize);
		menuPanel.add(bold);
		menuPanel.add(italics);
		menuPanel.add(m_undo);
		menuPanel.add(m_redo);
		// Insert vertical separator
		menuPanel.add(s1);
		menuPanel.add(m_showComments);
		menuPanel.add(m_commentMode);
		menuPanel.add(m_clearComments);
		menuPanel.add(Box.createHorizontalGlue());
		
		menuPanel.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createMatteBorder(10,10,0,10,Constants.COLOR_BACKDROP),
			BorderFactory.createMatteBorder(1,1,1,1,Color.black)));
		
		return menuPanel;
	}
	
	/**
	 * Gets index in the Font chooser combo box of the currently used font.
	 * If the font is not found, the first index is returned.
	 * 
	 * @return Index of current font
	 */
	private int getFontIndex(){
		for( int i=0; i<fontNames.length; i++ ){
			if( narrText.getFont().getFamily().compareToIgnoreCase(fontNames[i]) == 0 )
				return i;
		}
		return 0; // default to first font
	}
	
	/**
	 * Gets index of current font size.  Default is 3 (=14pt)
	 * 
	 * @return Font size index
	 */
	private int getSizeIndex(){
		for( int i=0; i<FONT_SIZES.length; i++ ){
			if( narrText.getFont().getSize() == Integer.parseInt(FONT_SIZES[i]) )
				return i;
		}
		return 3; // default to size = 14pt
	}
	
	/**
	 * Requests focus for NarrativeText
	 *
	 */
	public void setNarrTextFocus() {
		narrText.grabFocus();
	}
	
	/**
	 * Returns the narrative text in string form
	 * 
	 * @return Narrative text
	 */
	public String getNarrText() {
		return narrText.getText();
	}
	
	/**
	 * Reads the narrative from the disk.  Initially, the narrative will
	 * be a copy of a preferred narrative outline.
	 * 
	 * @return Narrative text from file
	 */
	public String getNarrative(){
		try{
			StringWriter sw = new StringWriter();
			FileReader fr = new FileReader(
					Clover.TEMP_DIR+"narrative.txt");
			int data;
			while( (data = fr.read()) != -1){
				sw.write(data);
			}
			fr.close();
			return sw.toString();
		} catch(Exception e){}
		return null;
	}
	
	/**
	 * Writes narrative changes to file.
	 *
	 */
	public void storeNarrative(){
		try{
			StringReader sr = new StringReader(getNarrText());
			FileWriter fw = new FileWriter(
					Clover.TEMP_DIR+"narrative.txt");
			int data;
			while( (data = sr.read()) != -1){
				fw.write(data);
			}
			fw.close();
		} catch(Exception e){}
	}
	
}


/**
 * NarrativeText class handles comment layer functionality, as well as
 * notebook appearance.  NOTE: Before using comments, resetImage() 
 * MUST be called.
 * 
 * Known bug: Comment layer does not support scrolling.
 * 
 * @author Terry J Bradley
 * @verison 04.8.12
 *
 */
class NarrativeText extends JTextArea 
					implements MouseListener, MouseMotionListener 
{

	static final int DEFAULT_COLS = 30;		// Defines default text size
	static final int DEFAULT_ROWS = 60;		// Default text size
	static final int INSET = 82;			// Inset for vertical red lines
	
	
	private BufferedImage m_img;			// comments to draw
	private boolean m_commentMode = false;	// comment mode (draw vs. view)
	private boolean m_visible = false;		// comment visiblity
	private Cursor m_cursor;				// add comments cursor
	private Color m_color;					// comment color
	private Color m_textColor;				// Text area color
	private Color m_backColor;				// Text area background

	private Point m_oldPoint;				// Previous point (for drawing)
	private boolean m_drawing;				// Drawing boolean (true when drawing)
	
	// Should probably switch to Constants.BRUSH_AA
	private static final Object ANTI_ALIASING = RenderingHints.VALUE_ANTIALIAS_OFF;
 
	/**
	 * Constructor - Sets text area size and properties.  Also
	 * prepares cursors and colors.
	 * 
	 * @param s Default text string
	 * @param cursorColor Comments color
	 */
	public NarrativeText(String s, Color cursorColor){
		super(s,DEFAULT_COLS,DEFAULT_ROWS);	
		
		setTabSize(4);
		setMargin(new Insets(INSET-20,INSET,5,INSET));
		setWrapStyleWord(true);
		setLineWrap(true);
		setFont(new Font("Arial", Font.PLAIN, 14));
		
		m_cursor = ToolBox.getColoredPencilCursor(cursorColor);	
		m_color = cursorColor;
		m_textColor = getSelectedTextColor();
		m_backColor = getSelectionColor();
		
		// Mouse listeners support drawing comments
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	
	/**
	 * Resets the comment layer image.  Also resizes the image to 
	 * fit the narrative text area.
	 *
	 */
	public void resetImage(){
		int width = getWidth();
		int height = getHeight();
		if(getWidth() <= 0 || getHeight() <= 0){
			width = Toolkit.getDefaultToolkit().getScreenSize().width;
			height = Toolkit.getDefaultToolkit().getScreenSize().height;
		}
		m_img = new BufferedImage(width, height, 
					BufferedImage.TYPE_INT_ARGB);
		repaint();
	}
	
	/**
	 * Overwritten function draws blue horizontal lines and red
	 * vertical lines to mimick notebook paper.  Also draws the
	 * comment layer if it should be visible.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		Font f = g.getFont();
		// Not sure why it would be useful, but below code
		// will only draw blue lines for fonts <= 24pt.
		
		FontMetrics fm = g.getFontMetrics();
		Insets in = getInsets();
		
		g.setColor(new Color(255,0,0,80));	// left red line
		int x = in.left - 2;//+ fm.getMaxAdvance() * getTabSize() / 2;
		g.drawLine(x,0,x,getHeight());
	
		g.setColor(new Color(255,0,0,40));	// right red line
		x = getWidth() - in.right + 2 ;//- fm.getMaxAdvance() * getTabSize() / 2;
		g.drawLine(x,0,x,getHeight());
		
		int y = in.top + fm.getAscent() + fm.getDescent();
		
		g.setColor(new Color(0,0,255,80));	// blue lines
		while( y < getHeight() ) {
			// Done: [hci-5] Skip the first 2 or 3 lines iff scrolled to
			// the top of the textArea.
			// Update: Done - Uses margin instead of below code.
			g.drawLine(0,y,getWidth(),y);
			y += fm.getHeight();
		}

		// Draw the comments if they should be visible.
		if(m_visible)
			g.drawImage(m_img, 0, 0, 
						m_img.getWidth(), 
						m_img.getHeight(),
						null);
	}
	
	/**
	 * Sets the comment mode to drawing (true) or viewing (false)
	 * 
	 * @param on Comment mode
	 */
	public void setCommentMode(boolean on){
		m_commentMode = on;
		setEditable(!on);
		if(on) {	// Commenting: Disable text editing
			setCursor(m_cursor);
			setSelectionColor(Color.WHITE);
			setSelectedTextColor(Color.GRAY);
			setForeground(Color.GRAY);
		}
		else {		// Not commenting: Enable text editing
			setSelectionColor(m_backColor);
			setSelectedTextColor(m_textColor);
			setCursor(Cursor.getDefaultCursor());
			setForeground(Color.BLACK);
		}
	}

	/**
	 * Returns the comment mode - true for drawing, false for viewing
	 * 
	 * @return Comment mode
	 */
	public boolean getCommentMode(){
		return m_commentMode;
	}
	
	/**
	 * Sets the comment visibility
	 * 
	 * @param vis Comment visibility
	 */
	public void setCommentVisible(boolean vis){
		m_visible = vis;
	}
	
	/**
	 * Returns the comment visibility
	 *  
	 * @return Comment visibility
	 */
	public boolean getCommentVisible(){
		return m_visible;
	}

	/**
	 * Mouse events here and below deal only with comments.  The code
	 * is taken from DrawingSurface / BackgroundSurface with very little
	 * change.
	 */
	public void mousePressed(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;
		
		m_drawing = true;
		m_oldPoint = e.getPoint();
							
		// Also draw a spec centered at e.getPoint();
		Graphics g = m_img.createGraphics();
		g.setColor(m_color);
					
		// Get upperleft corner
		int x = e.getPoint().x - BrushTool.STROKE_THICKNESS/2;
		int y = e.getPoint().y - BrushTool.STROKE_THICKNESS/2;
					
		g.fillRect(x, y, BrushTool.STROKE_THICKNESS, 
				BrushTool.STROKE_THICKNESS);
					
		g = null;
		e.consume();
	}
	public void mouseReleased(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;
		
		m_drawing = false;
		this.repaint();
		e.consume();
	}
	public void mouseDragged(MouseEvent e){
		if(!m_visible || !m_commentMode)
			return;
		
		Graphics2D g = m_img.createGraphics();
		g.setColor(m_color);
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			Constants.BRUSH_ANTI_ALIAS);
				
		g.setStroke(BrushTool.STROKE);
		g.drawLine(m_oldPoint.x, m_oldPoint.y, e.getX(), e.getY());
					
		g = null;
		
		int dx = m_oldPoint.x - e.getX();
		int dy = m_oldPoint.y - e.getY();
					
		m_oldPoint = e.getPoint();
		
		if( (dx < BrushTool.THRESHOLD && dx > -BrushTool.THRESHOLD)
			 && (dy < BrushTool.THRESHOLD && dy > -BrushTool.THRESHOLD) )
			this.repaint();
		e.consume();
	}

	public void mouseEntered(MouseEvent e){
		if(m_visible && m_commentMode ){
			setCursor(m_cursor);
		}
	}
	
	public void mouseExited(MouseEvent e){
		if(m_visible && m_commentMode ){
			setCursor(Cursor.getDefaultCursor());
		}
	}


	public void mouseClicked(MouseEvent e){}
	public void mouseMoved(MouseEvent e){}
	
}

