/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Playback.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.Actions.*;

/**
 * PlaybackUI - Interface for the story playback tab.
 * This tab plays each scene by enlarging the scene image and characters
 * and running through the actions as normal.
 * 
 * @author Terry Bradley
 * @version 04.8.15
 *
 */
public class PlaybackUI extends JPanel implements Playable {
	
	// Story
	protected Story 		m_story;			// Direct link to story
												// (m_parent.getStory()
												// would also have worked)
	protected CloverWindow	m_parent;			// Parent
	private   float 		m_scale;  			// Scale to adjust scenes
												// and characters to nearly
												// full screen.
	private	  int			m_yOffset = 0;		// Size of top and
												// bottom black bars
	private   int			m_xOffset = 0;		// Size of left and right
												// black bars.
	
	// UI Elements
	protected JLayeredPane m_ds;		// Container for scene and characters
	protected JPanel  m_contents;		// Container for m_ds
	protected HashMap m_characters;		// Map of PlayLabels (Long(gid), PlayLabel)
	protected JButton m_play;			// Play button
	protected JButton m_stop;			// Stop button
	protected JButton m_skip;			// Next button
	protected JButton m_prev;			// Previous button
	protected JButton m_pause;			// Pause button
	
	// Playable elements
	protected JLabel  m_speakBubble;		// Reusable speak bubble 
	protected JLabel  m_thinkBubble;		// Reusable think bubble
	protected HashMap m_emoteBubbles;		// Map of emote bubbles 
											// (Long(gid), EmotionBubble)
	protected HashMap m_sceneBackgrounds;	// key = Integer sid,
											// val = JLabel background img
	
	// Playable threads
	protected ProcessStory  m_processStory;	// Process story thread
	protected ProcessScene  m_processScene;	// Process scene thread
	protected ProcessAction m_processAction;// Process action thread
	
	private   int			m_response;		// currently playing response (0 == main story)
	
	/**
	 * Constructor - Creates the Playback interface: sets up the 
	 * layeredPane container, adds playback buttons and defines their
	 * functionality
	 * 
	 * @param sty Story datatype
	 * @param par CloverWindow parent
	 */
	public PlaybackUI(Story sty, CloverWindow par){
		super();
		m_story = sty;
		m_parent = par;

		// Set appearance
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(3,3,3,3));
				
		// Initialize primary containers
		m_ds = new JLayeredPane();
		m_ds.setBackground(Constants.COLOR_BACKDROP);
		
		m_contents = new JPanel();
		m_contents.setLayout(new BorderLayout());
		m_contents.setBackground(Color.black);
		m_contents.add(m_ds, BorderLayout.CENTER);
		add(m_contents, BorderLayout.CENTER);	
				
		// Initialize all playable elements
		m_characters = new HashMap();
		if(m_story != null){
			updateCharacters();
		}
		
		m_speakBubble = new SpeakBubble();
		m_thinkBubble = new ThinkBubble();
		m_emoteBubbles = new HashMap();
		m_sceneBackgrounds = new HashMap();
		
		m_speakBubble.setVisible(false);
		m_ds.add(m_speakBubble, new Integer(1000));
		m_thinkBubble.setVisible(false);
		m_ds.add(m_thinkBubble, new Integer(1010));
		
		// Define and add playback controls
		m_prev = new JButton(Clover.createImageIcon2(
				Clover.ICON_DIR+"Back24.gif"));
		m_play = new JButton(Clover.createImageIcon2(
				Clover.ICON_DIR+"Play24.gif"));
		m_stop = new JButton(Clover.createImageIcon2(
				Clover.ICON_DIR+"Stop24.gif"));
		m_skip = new JButton(Clover.createImageIcon2(
				Clover.ICON_DIR+"Forward24.gif"));
		
		m_prev.setToolTipText("Skip to previous scene");
		m_play.setToolTipText("Play story");
		m_stop.setToolTipText("Stop playback");
		m_skip.setToolTipText("Skip to next scene");
		
		m_play.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				updateScenes(0);
				/*
				ActionListener al = new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						if(!m_thread.isAlive())	{
							((javax.swing.Timer)e.getSource()).stop();
							play(0);
						}
					}
				};
				javax.swing.Timer t = new javax.swing.Timer(100, al);
				t.setRepeats(true);
				t.start();
				*/
				//try {
				//	wait();
				//}
				//catch(Exception ex){}
				
				play(0);
			}
		});
		
		m_stop.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				stopPlayback(-1, false);
			}
		});
		
		// TODO: Define action listeners for remaining buttons
		JPanel controls = new JPanel();
		controls.add(m_prev);
		controls.add(m_play);
		controls.add(m_stop);
		controls.add(m_skip);
		
		m_contents.add(controls, BorderLayout.SOUTH);
	}
	
	/**
	 * Sets size of playback area.
	 * 
	 * @param d New size
	 */
	public void setSize(Dimension d){
		m_ds.setSize(d.width - 6, d.height - 6); // Adjusted for borders
		super.setSize(d);
	}
	
	/**
	 * Sets the Story (used for opening a story)
	 * 
	 * @param sty New story
	 */
	public void setStory(Story sty){
		m_story = sty;
		m_response = 0;
				
		updateCharacters();
	}
	
	/**
	 * Preloads the scenes for the given response.  Scene images
	 * are loaded from the disc, resized, and stored for smooth playback.
	 * Called before playback begins (after play button is pressed).
	 * 
	 * @param responseIndex Index of response to be played
	 */
	public void updateScenes(final int responseIndex){
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		//TODO: Would be nice to preload in a separate thread, say
		// when opening the tab.  However, errors could occur if the
		// user began playing the story whilst the scenes were loading.
		Dimension size = m_ds.getSize();
		Dimension sceneSize = m_story.getSceneSize();
		m_response = responseIndex;

		// Set scale and offset values.
		m_scale = Math.min((float)size.width / sceneSize.width,
						   (float)size.height/ sceneSize.height);
				   
		int width = Math.round(sceneSize.width * m_scale);
		int height = Math.round(sceneSize.height * m_scale);
		m_xOffset = (m_ds.getSize().width - width) / 2;
		m_yOffset = (m_ds.getSize().height - height) / 2;

		m_sceneBackgrounds.clear();
		for(int i=0; i<m_story.getSceneOrder(responseIndex).size(); i++){
			Scene scene = m_story.getScene((Integer)m_story.getSceneOrder(responseIndex).get(i));
			Image img = scene.openImage().getScaledInstance
							(width, height, Image.SCALE_AREA_AVERAGING);
			JLabel background = new JLabel(new ImageIcon(img));
			background.setSize(size);
			m_sceneBackgrounds.put(new Integer(scene.getSceneID()), background);
		}
		
		setCursor(Cursor.getDefaultCursor());
	}
	
	/**
	 * Updates characters.  Called when the Playback tab is opened.
	 *
	 */
	public void loadPlaybackTab(){
		updateCharacters();
	}
	
	/**
	 * Removes characters.  Called when leaving the Playback tab.
	 *
	 */
	public void exitPlaybackTab(){
		removeCharacters();
		Component[] back = m_ds.getComponentsInLayer
					(JLayeredPane.DEFAULT_LAYER.intValue());
		for(int i=0; i<back.length; i++){
			m_ds.remove(back[i]);
		}
	}
	
	/**
	 * Updates characters by removing all that are currently known by
	 * this interface and then adding all the characters associated with
	 * the Story.
	 *
	 */
	public void updateCharacters(){
		removeCharacters();
		addCharacters();
	}
	
	/**
	 * Frequently called w/ addCharacters, this removes all characters
	 * and labels from the drawing surface and from the HashMap.
	 *
	 */
	public void removeCharacters(){
		// remove old characters, if needed
		for(Iterator it = m_characters.keySet().iterator(); it.hasNext();){
			Object key = it.next();
			PlayLabel jl = (PlayLabel)m_characters.get(key);
			m_ds.remove(jl);
			it.remove();
		}
	}
	
	/**
	 * This should be called when tab changes to Playback.
	 * A JLabel for each character is added to the m_characters
	 * HashMap.  Nothing is added to the drawing surface.
	 *
	 */
	public void addCharacters(){
		for(int i=0; i<m_story.getCharacterCount(); i++){
			GlobalCharacter gc = m_story.getCharacter
				(((Integer)m_story.getCharacterOrder().get(i)).intValue());
			PlayLabel jl = new PlayLabel(gc.openImage());
			m_characters.put(new Long(gc.getID()), jl);
		}
	}
	
	/**
	 * Called to playback the responses indicated by the given index.
	 * (An index of 0 plays the main story.)  Creates and launches the
	 * processing threads.
	 * 
	 * @param responseIndex Index of response to play.
	 */
	public void play(int responseIndex){
		// Set up bubbles for a new response
		initBubbles();
		
		// Create synch instances (necessary to launch threads)
		ActionSynch as = new ActionSynch(this, 
			m_story.getCharacterCount());
		SceneSynch ss = new SceneSynch();
		
		// Create an array of scene ids for processing threads
		Integer[] scenes = new Integer[m_story.getSceneCount(responseIndex)];
		for(int i=0; i<m_story.getSceneOrder(responseIndex).size(); i++){
			scenes[i] = (Integer)m_story.getSceneOrder(responseIndex).get(i);
		}

		// Create and start process threads
		m_processAction = new ProcessAction(as);
		m_processAction.setName("Clover-Process Action");
		m_processAction.start();
				
		m_processStory = new ProcessStory(ss, this, m_story, scenes);
		m_processStory.setName("Clover-Process Story");
		m_processScene = new ProcessScene(as, ss, this);
		m_processScene.setName("Clover-Process Scene");
		m_processScene.start();
		m_processStory.start();
	}
	

	/**
	 * Sets/Resets emotion bubbles for playback.  One emotion bubble
	 * is created and added to the layeredPane for each character that appears
	 * in the Story.  Calling this function wipes out any previously stored
	 * emotions and generates new emotion bubbles for each character.
	 *
	 */
	public void initBubbles(){
		int index = 0;
		for(Iterator i = m_emoteBubbles.keySet().iterator(); i.hasNext();){
			// remove bubbles from ds
			Object key = i.next();
			EmotionBubble eb = (EmotionBubble)m_emoteBubbles.get(key);
			m_ds.remove(eb);
			i.remove();
		}
		for(int i=0; i<m_story.getCharacterCount(); i++){
			Long gid = new Long(((Integer)m_story.getCharacterOrder().get(i)).intValue());
			EmotionBubble eb = new EmotionBubble();
			eb.setVisible(false);
			m_ds.add(eb, new Integer(300+index));
			index++;
			m_emoteBubbles.put(gid, eb);
		}
		m_ds.revalidate();
		m_ds.repaint();
	}
	
	/**
	 * First of the Playable functions.  This function moves either the
	 * speech or thought bubble to a location relative to the
	 * given character.  The bubble may also be hidden or displayed
	 * 
	 */
	public void moveBubble(int actionType, Scene scene, int gid,
						   Point location, boolean show)
	{
		Point carLocation = scene.getCharacter(gid).getLocation();
		Point adjLocation = new Point(Math.round(carLocation.x*m_scale), 
									  Math.round(carLocation.y*m_scale));
		
		/**
		 * The boolean "right" below determines the location of the callout
		 * relative to the actor.  This is the opposite of the side
		 * of the scene the character is on.  For example, if a character
		 * is on the far left, that character's callout will appear
		 * to the right.  This allows the callouts to fit on the screen with
		 * few exceptions.
		 */
		boolean right =	(adjLocation.x < m_ds.getWidth()/2);
		
		if(right){
			adjLocation.x +=  
				Math.round(m_story.getCharacter(gid).getRightBubbleLocation().x * m_scale *
				((float)scene.getCharacter(gid).getSize().width / 
				(float)m_story.getCharacter(gid).openImage().getWidth())) -
				location.x + m_xOffset;
			adjLocation.y += 
				Math.round(m_story.getCharacter(gid).getRightBubbleLocation().y * m_scale *
				((float)scene.getCharacter(gid).getSize().height / 
				(float)m_story.getCharacter(gid).openImage().getHeight())) -
				location.y + m_yOffset;
		}
		else {
			adjLocation.x += 
				Math.round(m_story.getCharacter(gid).getLeftBubbleLocation().x * m_scale *
				((float)scene.getCharacter(gid).getSize().width / 
				(float)m_story.getCharacter(gid).openImage().getWidth())) -
				location.x + m_xOffset;
			adjLocation.y += 
				Math.round(m_story.getCharacter(gid).getLeftBubbleLocation().y * m_scale *
				((float)scene.getCharacter(gid).getSize().height / 
				(float)m_story.getCharacter(gid).openImage().getHeight())) -
				location.y + m_yOffset;
		}
		adjLocation.x = Math.max(0, adjLocation.x);
		adjLocation.y = Math.max(0, adjLocation.y);
		
		/**
		 * Which bubble moves and how it's displayed depends
		 * upon the type of action we want to show.
		 */
		switch(actionType){
			case Constants.SPEAK_OFFSET:
				// Hide think bubble
				if(m_thinkBubble != null)
					m_thinkBubble.setVisible(false);
						
				// hide bubble before move if hiding anyway
				if(!show)
					m_speakBubble.setVisible(false);
				
				// Move speak bubble to location
				if(m_speakBubble == null) {
					m_speakBubble = new SpeakBubble();
					m_ds.add(m_speakBubble, new Integer(1000));
				}
					
				m_speakBubble.setLocation(adjLocation);
					
				// Display speak bubble
				m_speakBubble.setVisible(show);
				m_ds.revalidate();
			break;
				
			case Constants.THINK_OFFSET:
				// Hide speak bubble
				if(m_speakBubble != null)
					m_speakBubble.setVisible(false);
						
				if(!show)
					m_thinkBubble.setVisible(false);
				
				// Move think bubble to location
				if(m_thinkBubble == null) {
					m_thinkBubble = new ThinkBubble();
					m_ds.add(m_thinkBubble, new Integer(1000));
				}
				
				m_thinkBubble.setLocation(adjLocation);
												
				// Display speak bubble
				m_thinkBubble.setVisible(show);
				m_ds.revalidate();
			break;
				
			case Constants.EMOTE_OFFSET:
				// Move emotion "bubble" to location (hide nothing)
				if(right)
					adjLocation.x -= 131;
	
				adjLocation.y -= 131;
				adjLocation.x = Math.max(0, adjLocation.x);
				adjLocation.y = Math.max(0, adjLocation.y);
	
				if(show){ 
					((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
						.setLocation(adjLocation);
				}
					
				((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
					.setVisible(show);
					
				m_ds.revalidate();
				m_ds.repaint();
			break;
		}
	}

	/**
	 * Playable function sets the size of the bubble to display
	 * and adjust the images accordingly.
	 * 
	 */
	public boolean setBubbleSize(int actionType, String text, 
								 Scene scene, int gid)
	{
		boolean right = (scene.getCharacter(gid).getLocation().x < m_ds.getWidth()/2);
		switch(actionType){
			case Constants.SPEAK_OFFSET:
				// Set text in speak bubble
				((SpeakBubble)m_speakBubble).setBubbleSize(text, right);
			break;
			
			case Constants.THINK_OFFSET:
				// Set text in think bubble
				((ThinkBubble)m_thinkBubble).setBubbleSize(text, right);
			break;
			
			case Constants.EMOTE_OFFSET:
				// Ignore
			break;
		}
		return right;
	}
	
	/**
	 * Playable function - Sets the text in the given bubble to the given String.
	 * 
	 */
	public void setBubbleText(int actionType, String text, int start, int end){
		switch(actionType){
			case Constants.SPEAK_OFFSET:
				// Set text in speak bubble
				((SpeakBubble)m_speakBubble).setBubbleText(text, start, end);
			break;
				
			case Constants.THINK_OFFSET:
				// Set text in think bubble
				((ThinkBubble)m_thinkBubble).setBubbleText(text, start, end);
			break;
				
			case Constants.EMOTE_OFFSET:
				// Ignore
			break;
		}
	}
	
	/**
	 * Playable function - Sets the emotion icon in a specific character's 
	 * emotion icon.
	 * 
	 */
	public void setEmotion(int gid, int emotion, boolean active){
		// Set emotion icon
		int index = 0;
		for(int i=0; i<EmotionBubble.EMOTIONS.length; i++){
			if(emotion == EmotionBubble.EMOTIONS[i].m_emotionIndex)
				index = i;
		}
	
		if(active)
			((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
				.setIcon(EmotionBubble.EMOTIONS[index].m_activeImg);
		else
			((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
				.setIcon(EmotionBubble.EMOTIONS[index].m_staticImg);
	}
	
	/**
	 * Unused Playable function.
	 */
	public void colorCell(CloverAction ca, boolean on){}
	
	/**
	 * Playable function - Sets the canvas background to a resized version
	 * of the background image corresponding to the given scene ID.
	 * 
	 */
	public void setBackgroundImage(int sid){
		m_ds.setIgnoreRepaint(true);

		// Get (and later remove) everything in the default layer.
		Component[] components = m_ds.getComponentsInLayer
			(JLayeredPane.DEFAULT_LAYER.intValue());

		hideCharacters();
		for(int i=0; i<components.length; i++){
			m_ds.remove(components[i]);
		}
		
		// Set the background to the preloaded image from the
		// m_sceneBackgrounds Map.
		m_ds.add((JLabel)m_sceneBackgrounds.get(new Integer(sid)),
				 JLayeredPane.DEFAULT_LAYER);
		
		m_ds.setIgnoreRepaint(false);
		m_ds.revalidate();
	}
	
	/**
	 * Playable function - Displays (or hides) a given character, which
	 * should also be moved to the given location.
	 * 
	 */
	public void setCharacter(Point location, Dimension size, 
							 int gid, boolean visible)
	{
		// Get the PlayLabel in question
		PlayLabel jl = (PlayLabel)m_characters.get(new Long(gid));
		
		// Case 1: Hide character through removal from LayeredPane
		if(!visible) {
			m_ds.remove(jl);
			return;
		}
		
		// Case 2: Add character to ds	
		Point newLoc = new Point();
		newLoc.x = Math.round(location.x * m_scale) + m_xOffset;
		newLoc.y = Math.round(location.y * m_scale) + m_yOffset;
		jl.setLocation(newLoc);
		
		int width = Math.round(size.width * m_scale);
		int height = Math.round(size.height * m_scale);
		jl.setIcon(width, height);
		jl.setSize(width, height);
		
		m_ds.add(jl, JLayeredPane.DRAG_LAYER);		
		// TODO: Character depth/layer should be preserved here.
	}
	
	/**
	 * Playable function - Hides every character by removing
	 * each element in from the LayeredPane's Drag Layer.
	 * 
	 */
	public void hideCharacters(){
		
		Component[] chars = m_ds.getComponentsInLayer
							(JLayeredPane.DRAG_LAYER.intValue());
					
		for( int i=0; i<chars.length; i++){
			m_ds.remove(chars[i]);		
		}
	}
	
	/**
	 * Playable function - Stops playback. Transitions are not supported,
	 * and we can only play the current scene, so cleanup simply involves
	 * stopping any audio that is currently playing and hiding all bubbles.
	 * 
	 */
	public void stopPlayback(int sid, boolean displayMenu){
		// Stop audio
		if(m_parent.isAudioPlaying())
			m_parent.playAudio("");
				
		// Hide bubbles
		m_speakBubble.setVisible(false);
		m_thinkBubble.setVisible(false);
		initBubbles();
		
		// Offer menu options, i.e. create response, etc.
		if(displayMenu) {
			if(m_story.getResponseCount() <= 1 || m_response > 0)
				displayEndMenu();
			else
				displayMiddleMenu(m_story.getResponseTitles());
		}
			
		// Try to stop all threads.  This section is known to cause
		// warnings in the Java compiler, but failure to stop the threads
		// in question will not cause any problems.
		try {
			if(m_processAction.isAlive())
				m_processAction.stop();
		}catch(Exception e){}
		try {
			if(m_processScene.isAlive())
				m_processScene.stop();
		}catch(Exception e){}
		try{
			if(m_processStory.isAlive())
				m_processStory.stop();
		}catch(Exception e){}
		
	}
	
	/**
	 * Creates the "Middle" dialog, which is shown after watching the 
	 * main story when responses exist.  The user may cancel the dialog
	 * or choose a response to play.  The responses are specified by the
	 * supplied array.
	 * 
	 * @param responses Array of Response titles
	 */
	public void displayMiddleMenu(String[] responses){
		// TODO: Pre-load navigator for performance
		final JDialog navigator = new JDialog(m_parent, true);
		navigator.setUndecorated(true);
		navigator.setLocationRelativeTo(m_parent);
		navigator.getContentPane().setLayout(
			new BoxLayout(navigator.getContentPane(), BoxLayout.Y_AXIS));
		
		// Top = Choices
		JPanel top = new JPanel();
		top.setBackground(Color.BLACK);
		JLabel text = new JLabel("What would you like to do?");
		text.setBackground(Color.black);
		text.setForeground(Color.white);
		top.add(text);
		
		JPanel chooser = new JPanel();
		chooser.setBackground(Color.BLACK);
		chooser.setBorder(new CompoundBorder(new EmptyBorder(5,5,5,5),
						new MatteBorder(1, 0, 1, 0, Color.white)));
		chooser.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		JLabel response = new JLabel("Play a Response:");
		response.setBackground(Color.black);
		response.setForeground(Color.white);
		c.insets = new Insets(3,0,3,3);
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		chooser.add(response, c);
		
		final JList re = new JList(responses);
		re.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		re.setFixedCellWidth(150);
		re.setSize(150, 100);
		JScrollPane jsp = new JScrollPane(re);

		c.gridy = 1;
		c.gridwidth = 1;
		c.insets = new Insets(3,3,3,3);
		c.anchor = GridBagConstraints.CENTER;
		chooser.add(jsp, c);
		
		JButton play = new JButton(Clover.createImageIcon2(
				Clover.ICON_DIR+"Play24.gif"));
		play.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
				int index = re.getSelectedIndex();
				updateScenes(index+1);
				play(index+1);
			}
		});
		c.gridx = 1;
		c.anchor = GridBagConstraints.EAST;
		chooser.add(play, c);
		
		JPanel bottom = new JPanel();
		bottom.setBackground(Color.black);
		
		ActionListener cancelAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
			}
		};
		
		ActionListener editAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
				m_parent.moveToTab(
					Constants.VIEW_AND_EDIT_INDEX);
			}
		};
		
		ActionListener respondAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
				m_parent.moveToTab(
					Constants.RESPONSE_INDEX);	
			}
		};
		
		JButton edit = new JButton("Edit Story");
		edit.addActionListener(editAction);
		bottom.add(edit);
		
		JButton respond = new JButton("Edit Response");
		respond.addActionListener(respondAction);
		bottom.add(respond);
		
		JButton replay = new JButton("Cancel");
		replay.addActionListener(cancelAction);
		bottom.add(replay);
		
		navigator.getContentPane().add(top);
		navigator.getContentPane().add(chooser);
		navigator.getContentPane().add(bottom);
		
		navigator.pack();
		navigator.setLocation(
			navigator.getLocation().x - navigator.getSize().width / 2,
			navigator.getLocation().y - navigator.getSize().height / 2);
		navigator.show();
		
	}
	
	/**
	 * Creates the "End" dialog, which is shown after watching the 
	 * main story when no responses exist, or after watching any other
	 * response.  The user may cancel the dialog or choose to create a
	 * response or edit the story. 
	 * 
	 * Note: The similarity between the End and Middle menus suggests
	 * that a single menu may be sufficient.
	 */
	public void displayEndMenu(){
		// TODO: Pre-load navigator for performance
		final JDialog navigator = new JDialog(m_parent, true);
		navigator.setUndecorated(true);
		navigator.setLocationRelativeTo(m_parent);
		navigator.getContentPane().setLayout(
			new BoxLayout(navigator.getContentPane(), BoxLayout.Y_AXIS));
		
		// Top = THE END (FIN)
		JPanel top = new JPanel();
		top.setBackground(Color.BLACK);
		top.setBorder(new EmptyBorder(20,20,10,20));
		JLabel end = new JLabel("FIN");
		end.setFont(new Font(end.getFont().getFamily(), Font.BOLD, 72));
		end.setBackground(Color.black);
		end.setForeground(Color.white);
		end.setAlignmentX(Component.CENTER_ALIGNMENT);
		top.add(end);
		navigator.getContentPane().add(top);
		
		// Bottom = Options (w/ line border)
		JPanel bottom = new JPanel();
		bottom.setBackground(Color.BLACK);
		bottom.setBorder(new MatteBorder(1,0,0,0,Color.white));
		bottom.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		JLabel text = new JLabel("What would you like to do?");
		text.setBackground(Color.black);
		text.setForeground(Color.white);
		c.gridwidth = 4;
		c.insets = new Insets(2,2,2,2);
		bottom.add(text, c);
		
		ActionListener cancelAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
			}
		};
		
		ActionListener editAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
				m_parent.moveToTab
					(Constants.VIEW_AND_EDIT_INDEX);
			}
		};
		
		ActionListener respondAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				navigator.dispose();
				m_parent.moveToTab(
					Constants.RESPONSE_INDEX);	
			}
		};
		
		JButton replay = new JButton("Cancel");
		replay.addActionListener(cancelAction);
		c.gridwidth = 1;
		c.gridy = 1;
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 2;
		bottom.add(replay, c);
		
		JButton edit = new JButton("Edit Story");
		edit.addActionListener(editAction);
		c.gridx = 0;
		bottom.add(edit, c);
		
		JButton respond = new JButton("Create Response");
		respond.addActionListener(respondAction);
		c.gridx = 1;
		bottom.add(respond, c);
		
		navigator.getContentPane().add(bottom);
		
		navigator.pack();
		navigator.setLocation(
			navigator.getLocation().x - navigator.getSize().width / 2,
			navigator.getLocation().y - navigator.getSize().height / 2);
		navigator.show();
		
	}
}

/**
 * PlayLabel - Light version of the CharacterLabel used in the Scene Editor.
 * This label allows a character to be displayed in the Playback interface.
 * 
 * @author Terry J Bradley
 * @version 04.8.16
 *
 */
class PlayLabel extends JLabel{

	private Image m_fullSizeImg;	// The full-sized version of the character
									// image is stored separately to make
									// resizing easier.
									// NOTE: This is probably a bad idea.
	
	/**
	 * Constructor - Sets the fullSizedImage WITHOUT actually assigning
	 * any image to the Label.
	 * 
	 * @param img Character image
	 */
	public PlayLabel(Image img){
		super();
		m_fullSizeImg = img;	
	}
	
	/**
	 * Sets and resizes the full sized image, and adjust the Label.
	 * 
	 * @param width Resized width
	 * @param height Resized height
	 */
	public void setIcon(int width, int height){
		super.setIcon(new ImageIcon(m_fullSizeImg.getScaledInstance
						 (width, height, Image.SCALE_AREA_AVERAGING)));
	}
	
}