/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import java.util.*;
import java.util.jar.*;
import java.util.zip.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.DrawingTools.*;
import edu.uiuc.orchid.clover.Playback.*;

/**
 * CloverWindow - Contains Clover's UI
 * 
 * @author Terry Bradley
 * @version 04.8.18
 * @date 9/20/03
 *
 */
public class CloverWindow extends JFrame implements Constants {

	// Story Variables
	private ArrayList stories;				// At some point, allow >1 story
	private int storyIndex = -1;			// Index for >1 story
	private boolean everSaved = false;		// True once a new story has
											// been saved.  If false on exit,
											// empty story is removed.
	
	// UI Variables
	private FileUI fileUI;					// Story UI
	private NarrativeUI narrUI;				// Narrative UI
	private ScriptUI scriptUI;				// Script UI
	private CharacterUI charUI;				// Character UI
	private ViewAndEditUI veUI;				// Scenes UI
	private PlaybackUI 	playUI;				// Playback UI
	private ResponseUI	responseUI;			// ResponseUI
	private int LAST_TAB = FILE_INDEX;		// Stores the current tab, which
											// becomes the prev tab during
											// tab change events
	
	// Dialogs - Most dialogs are kept in memory in this class
	// to improve responsiveness.
	public JFileChooser m_file;				// File chooser for open and save
	public SpeakDialog m_speakDialog;		// Dialog for editing speak actions
	public SpeakDialog m_thinkDialog;		// Dialog for editing think actions
	public ToolBox m_toolBox;				// Drawing toolbox
	public GalleryDialog m_gallery;			// Character gallery
	public EmotionDialog m_emotionDialog;	// Emoticon toolbox
	public ResponseChooser m_responseDialog;
											// Choose a response for ResponseUI
	
	// Other resources
	public AudioCapture m_capture;			// Enables audio recording from mic
	public AudioPlayback m_playback;		// Enables audio playback
	public SpeechEngine m_speechEngine;		// Enables TTS audio playback
	
	// Containers for this
	protected JTabbedPane mainPane;			// Primary container
	
	// Preferences
	private CloverProperties userPrefs;		// Stores user preferences
	
	private boolean m_promptSave = true;	// True iff user should be prompted
											// to save changes on story close.
	
	// Icons (Public)
	public static ImageIcon PLAIN_BRUSH_ICON;		// White pencil icon
	public static ImageIcon COLORED_BRUSH_ICON;		// Pencil icon in current
													// color
	public static ImageIcon PLAIN_BUCKET_ICON;		// While bucket icon
	public static ImageIcon COLORED_BUCKET_ICON;	// Bucket in cur color
	public static ImageIcon ERASER_ICON;			// Eraser icon
	
	// Cursors
	public static Cursor MOVE_CURSOR;		// Cursor denotes dnd in progress

	/**
	 * Sometimes used CURSOR_LOCK prevents cursor changes during
	 * extended operations, like drawing, erasing, or dragging.
	 */
	public static boolean CURSOR_LOCK = false;	
												
	public static boolean TTS_ENABLED = true;	// True iff TTS is available.

	
///////////////////////  CONSTRUCTOR(S)  //////////////////////////////////////
	/**
	 * Constructor - Creates a CloverWindow instance with the given 
	 * properties file.  Sets sizes based on prefs and screen size.
	 * Initializes a CloverSplash screen before initializes all
	 * interface components.
	 * 
	 * @param p Properties, as found in Clover.main()
	 */
	public CloverWindow(CloverProperties p) {
		// Create frame
		super("Clover - Build " + CLOVER_VERSION);
		ImageIcon img = Clover.createImageIcon2(Clover.CLOVER_TITLE_ICON_LOC);
		setIconImage(img.getImage());
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setGlassPane(new DragGlassPane());
		
		// Setup the Preview directory as needed
		File newDir = new File(Clover.PREVIEW_DIR);
		try{
			newDir.mkdir();
		}
		catch(Exception e){}

		// Create/display the splash screen
		CloverSplash splash = new CloverSplash(CLOVER_VERSION, img.getImage());
		
		// Initialize Preferences
		splash.setStatusText("Loading user preferences...");
		userPrefs = p;
		
		// Get (but do not yet apply) screen dimensions
		Dimension size = new Dimension((int)userPrefs.getFloat(f_SCREEN_WIDTH,1024),
			(int)userPrefs.getFloat(f_SCREEN_HEIGHT,768));
		Dimension location = new Dimension((int)userPrefs.getDouble(d_SCREEN_X,0),
			(int)userPrefs.getDouble(d_SCREEN_Y,0));
		
		Insets insets = getInsets();
		size.width += insets.left + insets.right;
		size.height += insets.bottom + insets.top;
		
		// Load icons
		splash.setStatusText("Loading icons...");
		PLAIN_BRUSH_ICON = Clover.createImageIcon2(Clover.BRUSH_ICON_LOC);
		COLORED_BRUSH_ICON = Clover.createImageIcon2(Clover.BRUSH_ICON_LOC);
		PLAIN_BUCKET_ICON = Clover.createImageIcon2(Clover.FILL_ICON_LOC);
		COLORED_BUCKET_ICON = Clover.createImageIcon2(Clover.FILL_ICON_LOC);
		ERASER_ICON = Clover.createImageIcon2(Clover.ERASER_ICON_LOC);
		
		// Initialize interface
		splash.setStatusText("Initializing interface...");
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sceneCanvas;
		
		// Dynamically size scene canvas to fit screen
		if(screenSize.width > 1550)
			sceneCanvas = new Dimension(SCENECANVAS_1600_WIDTH,
										SCENECANVAS_1600_HEIGHT);
		else if (screenSize.width > 1250)
			sceneCanvas = new Dimension(SCENECANVAS_1280_WIDTH,
										SCENECANVAS_1280_HEIGHT);
		else
			sceneCanvas = new Dimension(SCENECANVAS_1024_WIDTH,
										SCENECANVAS_1024_HEIGHT);
		
		// Create UI instances for each tab
		fileUI = new FileUI(this);
		narrUI = new NarrativeUI(this);
		scriptUI = new ScriptUI(this);
		charUI = new CharacterUI(this);
		veUI = new ViewAndEditUI(this, sceneCanvas, "Scene Editor");
		playUI = new PlaybackUI(null, this);
		responseUI = new ResponseUI(this, sceneCanvas, "Response Editor");
		
		// Initialize Dialogs
		splash.setStatusText("Building dialogs...");
		m_file = new JFileChooser();
		m_thinkDialog = new SpeakDialog(CloverWindow.this, THINK_OFFSET);
		m_thinkDialog.setLocationRelativeTo(CloverWindow.this);
		m_gallery = new GalleryDialog(CloverWindow.this);
		m_emotionDialog = new EmotionDialog(CloverWindow.this);
		m_responseDialog = new ResponseChooser(CloverWindow.this);
		
		// Initialize audio system
		splash.setStatusText("Initializing audio system...");
		m_capture = new AudioCapture();
		m_playback = new AudioPlayback(CloverWindow.this);	
		try{
			m_speechEngine = new SpeechEngine();
		}
		catch(Error e){
			m_speechEngine = null;
			TTS_ENABLED = false;			
		}
		// One dialog done here cause it needs TTS_ENABLED
		m_speakDialog = new SpeakDialog(CloverWindow.this, SPEAK_OFFSET);
		m_speakDialog.setLocationRelativeTo(CloverWindow.this);
		
		// Restore toolbox and gallery size/location
		splash.setStatusText("Restoring toolbox preferences...");
		Dimension tbSize = new Dimension(
			(int)userPrefs.getFloat(f_TOOLBOX_WIDTH,250),
			(int)userPrefs.getFloat(f_TOOLBOX_HEIGHT,400));
		Point tbLocation = new Point(
							userPrefs.getInt(i_TOOLBOX_X,0),
							userPrefs.getInt(i_TOOLBOX_Y,0));
		// Check rightmost position.
		if(tbLocation.x > size.width - tbSize.width)
			tbLocation.x = size.width - tbSize.width;

		if(tbLocation.x < 0)
			tbLocation.x = 0;
		
		// Check bottom most position
		if(tbLocation.y > size.height - tbSize.height)
			tbLocation.y = size.height - tbSize.height;
		
		if(tbLocation.y < 0)
			tbLocation.y = 0;
							
		Dimension gallSize = new Dimension(
			(int)userPrefs.getFloat(f_GALLERY_WIDTH,
							Constants.GALLERY_DIALOG_WIDTH) ,
			(int)userPrefs.getFloat(f_GALLERY_HEIGHT,
							Constants.GALLERY_DIALOG_HEIGHT));
		Point gallLocation = new Point(
							userPrefs.getInt(i_GALLERY_X,0),
							userPrefs.getInt(i_GALLERY_Y,0));	
		// Check rightmost position.
		if(gallLocation.x > size.width - gallSize.width)
			gallLocation.x = size.width - gallSize.width;

		if(gallLocation.x < 0)
			gallLocation.x = 0;
		
		// Check bottom most position
		if(gallLocation.y > size.height - gallSize.height)
			gallLocation.y = size.height - gallSize.height;
		
		if(gallLocation.y < 0)
			gallLocation.y = 0;
							
		// Create drawing tools, then build Toolbox.  The toolbox
		// also contains an Undo tool - see Toolbox.java for its addition
		splash.setStatusText("Loading cursors...");
		BrushTool bt = new BrushTool(Cursor.getDefaultCursor(), "Pencil",
			PLAIN_BRUSH_ICON, BRUSH_SIZE);
		FillTool ft = new FillTool(Cursor.getDefaultCursor(), "Fill",
			PLAIN_BUCKET_ICON);	
		ErasorTool et = new ErasorTool(Cursor.getDefaultCursor(), "Eraser",
			ERASER_ICON, ERASOR_SIZE);
			 
		m_toolBox = new ToolBox(CloverWindow.this);
		m_toolBox.addTool(bt);
		m_toolBox.addTool(ft);
		m_toolBox.addTool(et);
		m_toolBox.constructToolbox();	
		m_toolBox.setVisible(false);
		
		m_toolBox.setSize(tbSize);
		m_toolBox.setLocation(tbLocation);
		m_gallery.setSize(gallSize);
		m_gallery.setLocation(gallLocation);

		// Add interface elements to appropriate tabs
		// TODO: [hci-0] Find/create better tab icons
		splash.setStatusText("Building tabs...");		
		mainPane = new JTabbedPane(JTabbedPane.NORTH, 
			JTabbedPane.SCROLL_TAB_LAYOUT);

		img = Clover.createImageIcon2(Clover.ICON_DIR+"Open24.gif");
		mainPane.addTab("Stories", img, fileUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Edit24.gif");
		mainPane.addTab("Narrative", img, narrUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"script_icon_24.gif");
		mainPane.addTab("Script", img, scriptUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"character_icon2.gif");
		mainPane.addTab("Characters", img, charUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Add24.gif");
		mainPane.addTab("Scenes", img, veUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Play24.gif");
		playUI.setSize(mainPane.getSize());
		mainPane.addTab("Playback", img, playUI);
		
		img = Clover.createImageIcon2(Clover.ICON_DIR+"New24.gif");
		mainPane.addTab("Responses", img, responseUI);
		
		// Randomly create move cursor at this point
		MOVE_CURSOR = getToolkit().createCustomCursor(
			Clover.createImageIcon2(
				Clover.CURSOR_DIR+"win32_MoveDrop32x32.gif").getImage(),
			new Point(0, 0),
			"Move Cursor");
		
		// Add tab change listener and window close listener
		splash.setStatusText("Adding listeners...");
		mainPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				// Get index of new tab
				int tab = ((JTabbedPane)e.getSource()).getSelectedIndex();
				
				CloverWindow.this.setCursor(
						Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				// Exit procedures: handle exiting tab
				switch(LAST_TAB){
					case CHARACTER_INDEX:
						charUI.saveCharacter();
					break;
					case VIEW_AND_EDIT_INDEX:
						veUI.updateScene();
					break;
					case PLAYBACK_INDEX:
						playUI.exitPlaybackTab();
					break;
					case RESPONSE_INDEX:
						responseUI.updateScene();
					break;
					case SCRIPT_INDEX:
						scriptUI.closeTab();
					break;
				}
				
				// Entry procedures: handle new tab
				switch (tab) {
					case FILE_INDEX:
						fileUI.updateFileUI();
						fileUI.openTab();
						m_toolBox.setVisible(false);
						m_gallery.setVisible(false);
					break;
					case SCRIPT_INDEX:
						scriptUI.openTab();
						m_toolBox.setVisible(false);
						m_gallery.setVisible(false);
					break;
					case VIEW_AND_EDIT_INDEX:
						m_toolBox.setVisible(true);
						m_gallery.setVisible(true);	// above should be taken care of
													// in this overridden method
						veUI.reconstructGallery(0);
						CloverWindow.this.requestFocus();
						break;
					case CHARACTER_INDEX:
						m_toolBox.setVisible(true);
						m_gallery.setVisible(false);
						break;
					case RESPONSE_INDEX:
						String[] s = getStory().getResponseTitles();
						m_responseDialog.populateChooser(s);
						m_responseDialog.show();
						
						if(!m_responseDialog.isOK()) {
							mainPane.setSelectedIndex(LAST_TAB);
							tab = LAST_TAB;
							break;
						}
						
						int index = m_responseDialog.getChoiceIndex();
						if(index == 0) { // create new response
							Response r = new Response(m_responseDialog.getResponseTitle());
							index = getStory().addResponse(r);
						}
						responseUI.loadResponse(index);
												
						m_toolBox.setVisible(true);
						m_gallery.setVisible(true);
						break;
					case PLAYBACK_INDEX:	
						playUI.loadPlaybackTab();
					default:
						m_toolBox.setVisible(false);
						m_gallery.setVisible(false);
						break; 
				}
				
				for(int i=0; i<mainPane.getTabCount(); i++){
					String bold = (i==tab) ? "<b>" : "";
					mainPane.setTitleAt(i, "<html>" + 
						bold + stripHTML(mainPane.getTitleAt(i)));
				}
				LAST_TAB = tab;
				System.gc();
				CloverWindow.this.setCursor(Cursor.getDefaultCursor());
				CloverWindow.this.m_promptSave = true;
			}
		});
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(mainPane, BorderLayout.CENTER);
		
		// Handle close event -> prompt to save, save settings
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				// Offer to save current story (if one is open), iff
				// b_SAVE_ON_EXIT is true
				if(getStory() != null) {
					boolean offer = getUserPrefs().getBool(b_SAVE_ON_EXIT, false);
					if(offer && m_promptSave){
						int result = JOptionPane.showConfirmDialog(
							CloverWindow.this, 
							"Save changes to \""+getStory().getTitle()+
							"?\"", "Save Changes?",
							JOptionPane.YES_NO_CANCEL_OPTION);
						switch(result){
							case JOptionPane.YES_OPTION:
								saveStory();
							break;
							case JOptionPane.CANCEL_OPTION:
								return;
						}
					}
					if( !everSaved ){
						removeStory();
					}
				}
				
				// Get current window size and save in prefs
				Dimension winSize = getSize();
				getUserPrefs().setProperty(f_SCREEN_WIDTH, 
							Integer.toString(winSize.width));
				getUserPrefs().setProperty(f_SCREEN_HEIGHT, 
							Integer.toString(winSize.height));
				
				// Repeat for window location
				Point pnt = getLocation();
				getUserPrefs().setProperty(d_SCREEN_X,
							Double.toString(pnt.getX()));
				getUserPrefs().setProperty(d_SCREEN_Y,
							Double.toString(pnt.getY()));
					
				// Update toolbox size and location
				winSize = m_toolBox.getSize();
				getUserPrefs().setProperty(f_TOOLBOX_WIDTH,
							Integer.toString(winSize.width));
				getUserPrefs().setProperty(f_TOOLBOX_HEIGHT, 
							Integer.toString(winSize.height));
							
				pnt = m_toolBox.getLocation();
				getUserPrefs().setProperty(i_TOOLBOX_X,
							Integer.toString(pnt.x));
				getUserPrefs().setProperty(i_TOOLBOX_Y,
							Integer.toString(pnt.y));							
				
				// Update gallery size and location
				winSize = m_gallery.getSize();
				getUserPrefs().setProperty(f_GALLERY_WIDTH,
							Integer.toString(winSize.width));
				getUserPrefs().setProperty(f_GALLERY_HEIGHT, 
							Integer.toString(winSize.height));
							
				pnt = m_gallery.getLocation();
				getUserPrefs().setProperty(i_GALLERY_X,
							Integer.toString(pnt.x));
				getUserPrefs().setProperty(i_GALLERY_Y,
							Integer.toString(pnt.y));
				
				
				// Write all prefs to .userpref
				Clover.writeUserPrefs(getUserPrefs());
				
				// Delete temp dir.  Preview dir handled separately (?)
				emptyDir(Clover.TEMP_DIR, true);
								
				// Exit Clover
				System.exit(0);
			}
		});
		
		// Finish constructor
		splash.setStatusText("Finalizing...");
		
		ScenePlayback.setupPlayback(CloverWindow.this);
		
		// Create new story by default
		storyIndex = -1;
		stories = new ArrayList(2);
		updateTabs();
		
		setSize(size);
		setLocation(location.width, location.height);
		splash.dispose();
	}

///////////////////////  METHODS  /////////////////////////////////////////////
	
	/**
	 * Returns the tab interface for a given index.  This allows public access
	 * to all tab elements.
	 * 
	 * @param Tab index (see Constants.java -> ***_INDEX values)
	 * @return Interface element.  These are all JPanels or JSplitPanes
	 */
	public JComponent getInterface(int tab){
		switch(tab){
			case FILE_INDEX:
				return fileUI;
			case NARRATIVE_INDEX:
				return narrUI;
			case SCRIPT_INDEX:
				return scriptUI;
			case CHARACTER_INDEX:
				return charUI;
			case VIEW_AND_EDIT_INDEX:
				return veUI;
			case PLAYBACK_INDEX:
				return playUI;
			case RESPONSE_INDEX:
				return responseUI;
		}
		return null;
	}
	
	/**
	 * Returns the index of the currently selected tab.
	 * 
	 * Returning LAST_TAB should be identical.
	 * 
	 * @return Index of current tab
	 */
	public int getSelectedTab(){
		return mainPane.getSelectedIndex();
	}
	
	/**
	 * Given a string, remove any html elements from that
	 * string.  Seems like it will only remove leading
	 * HTML markings.  Mostly replaced by stripHTML2.
	 * 
	 * @param input HTML-ified string
	 * @return String with HTML removed
	 */
	public static String stripHTML(String input){
		if(input.length() == 0 || input == null)
			return input;
		String tmp = input;
		while( tmp.charAt(0) == '<' ) {
			// strip character
			int end = tmp.length();
			for( int i=0; i<tmp.length(); i++ ){
				if( tmp.charAt(i) == '>' ) {
					end = i;
					break;
				}
			}
			if( end < tmp.length() )
				end++;
			tmp = tmp.substring(end);
		}
		// tmp = ..no html..unknown. 
		int end = tmp.length();
		for( int i=0; i<tmp.length(); i++ ){
			if( tmp.charAt(i) == '<' ) {
				end = i;
				break;
			}
		}
		return tmp.substring(0, end);
	}
	
	/**
	 * Given a string, remove any html elements from that
	 * string.  PREFERRED OVER stripHTML()
	 * 
	 * @param input HTML-ified string
	 * @return String with HTML removed
	 */
	public static String stripHTML2(String input){
		if(input == null || input.length() == 0 )
			return input;
		String tmp = "";
		int pos = 0;
		while( pos < input.length() ){
			while( input.charAt(pos) == '<' ) {
				// strip character
				int end = input.length();
				for( int i=pos; i<input.length(); i++ ){
					if( input.charAt(i) == '>' ) {
						end = i;
						break;
					}
				}
				if( end < input.length() )
					end++;
				pos = end;
			}
			// tmp = ..no html..unknown. 
			int end = input.length();
			for( int i=pos; i<input.length(); i++ ){
				if( input.charAt(i) == '<' ) {
					end = i;
					break;
				}
			}
			tmp = tmp + input.substring(pos, end);
			if(pos < end)
				pos = end;
		}
		
		return tmp;
	}
	
	/**
	 * Formats and crops text to fit the given column and row size.
	 * HTML formatting is added to enforce multiple rows.  Any
	 * extra text is discarded.
	 * 
	 * @param text Input string
	 * @param columns Maximum line length
	 * @param rows Maximum rows
	 * @return String, cropped and formatted in HTML
	 */
	public static String formatText(String text, int columns, int rows){
		String work = stripHTML(text);	// or strip2?
		String result = "<html>";
		
		// Loop
		int nRows = 0;
		while(work.length() > columns && nRows < rows){
			String sub = work.substring(0, columns);
			int end;
			for(end = columns-1; end>=0; end--){
				if(sub.charAt(end) == ' ' || sub.charAt(end) == '\n')
					break;
			}
			if(end<2)
				end = columns-1;
			work = work.substring(end);
			result += sub.substring(0, end) + "<BR>";
			nRows++;
		}
		if(work.length() > 0 && work.length() < columns)
			result += work;
		return result;
	}
	
	/**
	 * Attempts to play the audio file at the location given by filename.
	 * 
	 * @param filename Location of wav file
	 * @return Playback status (see SpeakDialog.java)
	 */
	public int playAudio(String filename){
		File file = new File(Clover.TEMP_DIR+filename);
		if(!file.canRead()) {
			if( m_playback.isRunning() )
				m_playback.stop();
			return SpeakDialog.WAITING;
		}
		
		m_playback.setPath(Clover.TEMP_DIR+filename);
		
		if(m_capture.isRunning())
			m_capture.stop();
			
		if(!m_playback.isRunning()){
			m_playback.start();
			return SpeakDialog.PLAYING_AUDIO;
		}
		else {
			m_playback.stop();
			return SpeakDialog.WAITING;
		}
	}
	
	/**
	 * Stops audio capture and playback as needed.
	 *
	 */
	public void stopAudio(){
		try {
			if(m_playback.isRunning())
				m_playback.stop();	
		} catch(Exception e){}
		try {
			if(m_capture.isRunning()){
				m_capture.stop();
				m_speakDialog.hasAudio = true;
			}
		} catch(Exception e){}
		
		// TODO: Stop TTS
	}
	
	/**
	 * Uses the TTS engine to speak the given text.  The voice assigned to
	 * the given character is used.
	 * 
	 * @param line Text to speak
	 * @param gid Character speaking
	 */
	public void playTTS(String line, int gid){
		if(!CloverWindow.TTS_ENABLED)
			return;
		GlobalCharacter gc = getStory().getCharacter(gid);
		m_speechEngine.setVoice(
			CharacterUI.voices[gc.getVoiceIndex()]);
		m_speechEngine.speak(line, gc.getVolume(), 
			gc.getRate(), gc.getPitch(), gc.getTone());
	}
	
	/**
	 * Checks whether TTS or regular audio is currently playing
	 * 
	 * @return True iff audio is playing
	 */
	public boolean isAudioPlaying(){
		if(!CloverWindow.TTS_ENABLED)
			return m_playback.isPlaying();
		else
			return (
			m_playback.isPlaying() ||
			(m_speechEngine.isSpeaking()));
	}
	
	/**
	 * Attempts to record audio.  The audio is stored in the file
	 * given by filename.
	 * 
	 * @param filename Location in which to store audio
	 * @return Capture status (See SpeakDialog.java)
	 */
	public int captureAudio(String filename){
		// If recording, stop and return
		if(m_capture.isRunning()){
			m_capture.stop();
			m_speakDialog.hasAudio = true;
			return SpeakDialog.WAITING;
		}
		
		// Create a blank file
		File file = new File(Clover.TEMP_DIR+filename);
		if(file.canRead())
			file.delete();
		try{
			file.createNewFile();
		}
		catch(Exception e){
			return SpeakDialog.WAITING;
		}
		if(!file.canWrite())
			return SpeakDialog.WAITING;
		m_capture.setPath(Clover.TEMP_DIR+filename);
		
		if(m_playback.isRunning())
			m_playback.stop();
			
		if(!m_capture.isRunning()){
			m_capture.start();
			return SpeakDialog.RECORDING_AUDIO;
		}
		else {
			m_capture.stop();
			return SpeakDialog.WAITING;
		}	
	}
	
	/**
	 * Notifies speakDialog that audio playback has finished, allowing
	 * the UI to reset.
	 *
	 */
	public void audioStopped(){
		// If in SpeakDialog, call setUI.
		if(m_speakDialog.isVisible())
			m_speakDialog.setUI(SpeakDialog.WAITING);
		// TODO: Add support for scene/story playback ??
	}
	
	/**
	 * Programmatically move to a different tab.
	 * 
	 * @param index Destination tab index
	 */
	public void moveToTab(int index){
		mainPane.setSelectedIndex(index);
	}
	
	/**
	 * Handles undo calls by restoring the backup image, based on
	 * the current tab.
	 *
	 */
	public void undoImage(){
		switch(mainPane.getSelectedIndex()){
			case CHARACTER_INDEX:
				charUI.getDrawingSurface().restoreImage();
				charUI.getDrawingSurface().updateThumb();
			break;
			case VIEW_AND_EDIT_INDEX:
				veUI.getDrawingSurface().restoreImage();
				veUI.getDrawingSurface().updateThumb();
			break;
			case RESPONSE_INDEX:
				responseUI.getDrawingSurface().restoreImage();
				responseUI.getDrawingSurface().updateThumb();
			break;
		}
	
	}

	/**
	 * True iff a story has been created or loaded.
	 * 
	 * @return True iff a story has been created or loaded
	 */
	public boolean hasStory() {
		if( storyIndex == -1 )
			return false;
		else
			return true;
	}
	
	/**
	 * Creates a new story.  This function displays a dialog, asking
	 * for a story title.  That title is then checked against existing 
	 * story titles as well as forbidden file system characters.
	 * The story is only created if the name is valid.
	 * 
	 * THIS FUNCTION WILL NOT SAVE OR PROMPT TO SAVE ANY OPEN STORY
	 *
	 */
	public void createNewStory() {
		// Get story name
		String title, path = System.getProperty("user.dir") +  
							 "\\Stories\\";
		while(true) {
			title = (String) JOptionPane.showInputDialog(
				this, "Please enter a title for your story:",
				"Create New Story", JOptionPane.INFORMATION_MESSAGE,
				null, null, "New Story");
			if( title == null || title == "" )
				// Assume Cancel
				return;
			
			// Check for forbidden characters and notify the user.
						
			// Create new Story			
			File test = new File(path+title+".clover");
			boolean valid = true;
			
			// Story exists error
			if( test != null && test.exists() ) { 
				JOptionPane.showMessageDialog(
					this, "Error Creating Story - A story called " +
					"\""+title+"\" already exists.",
					"Error Creating Story",
					JOptionPane.ERROR_MESSAGE);
			}
	
			// Invalid characters error
			try{
				valid = test.createNewFile();
			} catch (Exception e){
				valid = false;
			}
			if( !valid ) {  
				JOptionPane.showMessageDialog(
					this, "Error Creating Story - The story name "
					+ "contains illegal characters.  Please choose another name.",
					"Error Creating Story",
					JOptionPane.ERROR_MESSAGE);
			}
			else break;						  
		}
		createNewStory(title);

	}
	
	/**
	 * Creates a new story with the given name.  The name is checked for
	 * validity.
	 * 
	 * NOTE: When called from createNewStory(), incurs a second validity test
	 * which is likely to fail (since the fail is already created).  Fortunately,
	 * prior function isn't called.
	 * 
	 * @param title Desired story title
	 */
	public void createNewStory(String title){
		String path = System.getProperty("user.dir") +  "\\Stories\\";			
		File test = new File(path+title+".clover");
		boolean valid = true;
		
		if( test != null && test.exists() ) { 
			JOptionPane.showMessageDialog(
				this, "Error Creating Story - A story called " +
				"\""+title+"\" already exists.",
				"Error Creating Story",
				JOptionPane.ERROR_MESSAGE);
			CloverWindow.this.requestFocus();
			fileUI.highlightNameTextfield();
			return;
		}

		try{
			valid = test.createNewFile();
		} catch (Exception e){
			valid = false;
		}
		if( !valid ) { 
			JOptionPane.showMessageDialog(
				this, "Error Creating Story - The story name "
				+ "contains illegal characters.  Please choose another name.",
				"Error Creating Story",
				JOptionPane.ERROR_MESSAGE);
			CloverWindow.this.requestFocus();
			fileUI.highlightNameTextfield();
			return;
		}
		
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		Story story = new Story(title);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sceneCanvas;
		Dimension charCanvas;
		if(screenSize.width > 1550) {
			sceneCanvas = new Dimension(SCENECANVAS_1600_WIDTH,
										SCENECANVAS_1600_HEIGHT);
			charCanvas = new Dimension(CHARCANVAS_1600_WIDTH,
										CHARCANVAS_1600_HEIGHT);
		}
		else {
			sceneCanvas = new Dimension(SCENECANVAS_1024_WIDTH,
										SCENECANVAS_1024_HEIGHT);
			charCanvas = new Dimension(CHARCANVAS_1024_WIDTH,
										CHARCANVAS_1024_HEIGHT);
		}
		story.setCharSize(charCanvas);
		story.setSceneSize(sceneCanvas);
		resizeCanvases(charCanvas, sceneCanvas);
				
		story.setStoryPath(path);
		File newDir = new File(Clover.TEMP_DIR);
		File subDir = new File(Clover.PREVIEW_DIR);
		try{
			newDir.mkdir();
			subDir.mkdir();
		}
		catch(Exception e){}
		emptyDir(Clover.TEMP_DIR, false);		

		stories.clear();			// might one day keep >1 story in memory
		stories.add(story);
		storyIndex++;				// value not currently used.
		setTitle("Clover - Build " + CLOVER_VERSION + " - \""+title+"\"");
			
		// Update each UI 
		fileUI.updateFileUI();
		narrUI.newStory();

		scriptUI.newStory();
		charUI.openStory(true, getStory().getCharSize());
		veUI.updateVaEUI(getStory().getSceneSize());
		playUI.setStory(story);
		responseUI.updateVaEUI(getStory().getSceneSize());
		
		// Update global stuff; tabs and status
		updateTabs();
		
		// TODO: [hci-5] Flash tab or something to show movement.
		mainPane.setSelectedIndex(NARRATIVE_INDEX);
		setCursor(Cursor.getDefaultCursor());
		m_promptSave = true;
		everSaved = false;
	}
	
	/**
	 * Resizes the character canvas and scene editor canvases.
	 * 
	 * @param charSize New size of character canvas
	 * @param sceneSize New size of scene background canvases
	 */
	public void resizeCanvases(Dimension charSize, Dimension sceneSize){
		// Need to resize ds in charUI, veUI, responseUI.
		charUI.layoutUI(charSize);
		charUI.revalidate();
		
		veUI.getDrawingSurface().setSize(sceneSize);
		veUI.revalidate();
		
		responseUI.getDrawingSurface().setSize(sceneSize);
		responseUI.revalidate();
	}
	
	/**
	 * Sets enable status of all tabs past the first one.
	 * 
	 * @param on New enable status
	 */
	public void enableOtherTabs(boolean on) {
		for( int i = 1; i < mainPane.getTabCount(); i++ ) {
			mainPane.setEnabledAt(i, on);
		}
	}
	
	/**
	 * Updates the status of the tabs.
	 * 
	 * CHANGES - UNTESTED.
	 *
	 */
	public void updateTabs() {
		int tabCount = mainPane.getTabCount();
		if( tabCount < 1 )
			return;
		enableOtherTabs(hasStory());
	}
	
	/**
	 * Saves the current story in a location chosen by the user.
	 * 
	 * @return File saving status (See FileUI.java)
	 */
	public int saveStoryAs() {
		if(getStory() != null)
			m_file.setCurrentDirectory(new File(getStory().getStoryPath()));
		else
			m_file.setCurrentDirectory(new File(Clover.STORY_DIR));
		m_file.setFileFilter(new CloverFilter(true));
		
		if( m_file.showSaveDialog(this) != JFileChooser.APPROVE_OPTION )
			return FileUI.STATUS_BLANK;
		File file = m_file.getSelectedFile();
		
		// Done: [core-2] Check for exisiting file;
		if( file.exists() ) {
			int reply = JOptionPane.showConfirmDialog(this, "Save over exisiting file?");
			switch (reply) {
				case JOptionPane.YES_OPTION:	// will save below
					break;	
				case JOptionPane.NO_OPTION:		// show dialog again
					return saveStoryAs();
				case JOptionPane.CANCEL_OPTION:	// won't save
					return FileUI.STATUS_BLANK;	
			}
		}
			
		// Done: [core-0] Write Story to file.
		boolean success = true;
		getStory().setStoryPath(Clover.getDirectory(file.getPath()));
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		return saveStory();
	}
	
	/**
	 * Saves the story in a location given by getStory().getStoryPath().
	 * The Story data structure is saved using Serialization.  The
	 * resulting .story file and all images / wav files in the working
	 * directory are then saved in a .jar file.
	 * 
	 * @return File saving status (See FileUI.java)
	 */
	public int saveStory() {
		boolean success = true;
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		narrUI.storeNarrative();
		scriptUI.closeTab();
		try{
			File out = new File(Clover.TEMP_DIR + 
								getStory().getTitle() + ".story");
			if(out.exists()) {
				out.renameTo(new File(Clover.TEMP_DIR +
									  getStory().getTitle() + ".backup"));
				out = new File(Clover.TEMP_DIR + 
							   getStory().getTitle() + ".story");
			}

			FileOutputStream fis =
				new FileOutputStream(out);
			BufferedOutputStream bis =
				new BufferedOutputStream(fis);
			ObjectOutputStream ois =
				new ObjectOutputStream(bis);
			ois.writeObject(getStory());
			ois.close();
		} catch(Exception e){
			success = false;
			System.err.println("Error writing to "+
					Clover.TEMP_DIR + getStory().getTitle() + ".story - "
					+ e);
			setCursor(Cursor.getDefaultCursor());
		} 
		if(success) {
			veUI.savePreviewThumbnail();
			success = buildJAR();
			setCursor(Cursor.getDefaultCursor());
			everSaved = true;
		}
		return (success) ? FileUI.STATUS_SAVED : FileUI.STATUS_FAILED;
	}
	
	/**
	 * Stores all story elements in the working directory in a
	 * JAR container (which is renamed "story title.clover")
	 * 
	 * @return File saving status (See FileUI.java)
	 */
	public boolean buildJAR(){
		boolean success = true;
		try{
			File jar = new File(getStory().getStoryPath()+
							    getStory().getTitle()+
							    ".clover");
			JarOutputStream jos = new JarOutputStream(
				new BufferedOutputStream(
					new FileOutputStream(jar)));
			
			ZipEntry ze;
			FileInputStream fis;
			BufferedInputStream bis;
			int data;
			
			File file = new File(Clover.TEMP_DIR);
			File[] files = file.listFiles();
			for(int i=0; i<files.length; i++){
				if(files[i].isFile()) {
					ze = new ZipEntry(files[i].getName());
					jos.putNextEntry(ze);
					bis = new BufferedInputStream(new FileInputStream(files[i]));
					data = bis.read();
					while(data != -1) {
						jos.write(data);
						data = bis.read();
					}
					bis.close();
					jos.closeEntry();
				}
			}
			jos.close();		
			m_promptSave = false;	
		} catch(Exception e){
			success = false;
		}
		return success;
	}
	
	/**
	 * Opens *.clover files by extracting the JAR contents to the
	 * working directory
	 * 
	 * @param jar *.clover file to open
	 * @param directory Destination dir
	 * @return Path to *.story file containing the saved Story structure
	 */
	public String openJAR(File jar, String directory){
		String storyPath = null;
		int data;
		ZipEntry ze;
		File out;
		BufferedOutputStream bos;
		
		try{
			JarInputStream jis = new JarInputStream(
				new BufferedInputStream(
					new FileInputStream(jar)));
			while( (ze = jis.getNextEntry()) != null ){
				out = new File(directory+ze.getName());
				bos = new BufferedOutputStream(
					new FileOutputStream(out));
				while( (data = jis.read() ) != -1 ) {
					bos.write(data);
				}
				if(out.getName().endsWith(".story"))
					storyPath = out.getPath();
				bos.close();
				jis.closeEntry();
			}
		} catch(Exception e){
			storyPath = null;
		}
		return storyPath;		
	}
	
	/**
	 * Opens a *.clover file and extracts "preview.png" if it exists.
	 * This element should be extracted to the preview directory.
	 * 
	 * @param jar *.clover file
	 * @param directory Destination for preview.png
	 */
	public void openJARForPreview(File jar, String directory){
		int data;
		ZipEntry ze;
		File out;
		BufferedOutputStream bos;
		String fileName = "preview.png";
		
		try{
			JarInputStream jis = new JarInputStream(
				new BufferedInputStream(
					new FileInputStream(jar)));
			while( (ze = jis.getNextEntry()) != null ){
				out = new File(directory+ze.getName());
				if ( out.getName().equals(fileName)) {
					bos = new BufferedOutputStream(
							new FileOutputStream(out));
					while( (data = jis.read() ) != -1 ) {
						bos.write(data);
					}
					bos.close();
					jis.closeEntry();
					jis.close();
					break;
				}
			}
		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Previews a story by emptying the preview dir and extracting
	 * the given story's preview icon.  The use of 
	 * Clover.createPreviewImageIcon() is explained in Clover.java
	 * 
	 * @param path Path to story to preview.
	 */
	public void previewStory(final String path){
		emptyDir(Clover.PREVIEW_DIR, false);
		Thread t = new Thread(){
			public void run() {
				// Extract preview.png
				openJARForPreview(new File(path), Clover.PREVIEW_DIR);
				String imgPath = Clover.PREVIEW_DIR+"preview.png";

				// Create new imageicon
				final ImageIcon ii = Clover.createPreviewImageIcon(
						imgPath, SCENE_THUMB_WIDTH, SCENE_THUMB_HEIGHT);
				
				// Update/repaint the preview label
				SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						fileUI.pb_preview.setIcon(ii);
						fileUI.pb_preview.setEnabled(true);
						fileUI.pb_preview.revalidate();
						fileUI.pb_preview.repaint();
					}
				});
			}
		};
		t.setName("Update Preview Icon");
		t.setPriority(Thread.MIN_PRIORITY);
		t.start();
	}
	
	/** 
	 * Indiscriminately deletes contents of path.  Yikes.
	 * 
	 * @param path Directory to empty
	 * @param recursive If true, delete subdirectories as well
	 */
	public void emptyDir(String path, boolean recursive){
		File file = new File(path);
		File[] files = file.listFiles();
		if (files != null) {
			for(int i=0; i<files.length; i++){
				if(files[i].isFile())
					files[i].delete();
				else if(recursive && files[i].isDirectory())
					emptyDir(files[i].getPath(), true);
			}
		}
	}
	
	/**
	 * Deletes an empty title.clover file, in case a new story was
	 * never saved.
	 * 
	 */ 
	public void removeStory(){
		File del = new File(getStory().getStoryPath()
				+ getStory().getTitle() + ".clover");
		if (del != null && del.exists() && del.length() == 0l )
			del.delete();
	}
	
	/**
	 * Attempts to open a story from the disk.  Sets start directory
	 * for file chooser and applies a *.clover filter.
	 * 
	 * @return File status (See FileUI.java)
	 */
	public int openStory() {
		int result = FileUI.STATUS_BLANK;
		
		if(getStory() != null)
			m_file.setCurrentDirectory(new File(getStory().getStoryPath()));
		else
			m_file.setCurrentDirectory(new File(Clover.STORY_DIR));
		m_file.setFileFilter(new CloverFilter(true));
		
		
		if( m_file.showOpenDialog(this) != JFileChooser.APPROVE_OPTION )
			return result;
		return openStory(m_file.getSelectedFile());
	}
	
	/**
	 * Opens the given file, which should be a *.clover JAR file.
	 * 
	 * @param file *.clover file to open
	 * @return File status (See FileUI.java)
	 */
	public int openStory(File file){
		int result = FileUI.STATUS_SAVED;
		
		if( file == null || !file.isFile() )
			return FileUI.STATUS_FAILED;
			
		// Open file.
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		emptyDir(Clover.TEMP_DIR, false);
		
		// Unjar file
		String path = openJAR(file, Clover.TEMP_DIR);	
		String storyPath = Clover.getDirectory(file.getPath());

		if( path == null ) {
    		setCursor(Cursor.getDefaultCursor());
			return FileUI.STATUS_FAILED;
		}
		else {
			file = new File(path);
		}
			
		try{
			FileInputStream fis = new FileInputStream(file);
			BufferedInputStream bis = new BufferedInputStream(fis);
			ObjectInputStream ois = new ObjectInputStream(bis);
			stories.clear();
			storyIndex = -1;
			stories.add(ois.readObject());
			storyIndex = 0;
			ois.close();
		} catch (Exception e){
			System.err.println(e);
			e.printStackTrace();
			result = FileUI.STATUS_FAILED;
			setCursor(Cursor.getDefaultCursor());			
		}

		try {
			// Update title bar
			setTitle(	"Clover - Build " + CLOVER_VERSION + " - \"" +
						getStory().getTitle()+"\"");
	
			// Update each UI
			fileUI.openStory();
			narrUI.openStory();
			scriptUI.openStory();
			charUI.openStory(true, getStory().getCharSize());
			veUI.openStory(getStory().getSceneSize());
			playUI.setStory(getStory());
			responseUI.openStory(getStory().getSceneSize());
		} catch(Exception e){
			System.err.println(e.toString());
			e.printStackTrace();
			result = FileUI.STATUS_FAILED;
		}

		if(result == FileUI.STATUS_SAVED) {
			// Update global stuff; tabs and status
			updateTabs();
			getStory().setStoryPath(storyPath);
	
			// TODO: [hci-5] Flash tab or something to show movement.
			mainPane.setSelectedIndex(NARRATIVE_INDEX);
			userPrefs.setProperty(str_OPEN_PATH, file.getPath());
			everSaved = true;
		}
		setCursor(Cursor.getDefaultCursor());
		m_promptSave = true;
		return result;
	}
	
	// Preferences:
	/**
	 * Returns the user preferences loaded/defined for this story.
	 * 
	 */
	public CloverProperties getUserPrefs() {
		return userPrefs;
	}
	
	/**
	 * Returns the user preferences loaded/defined for all stories.
	 * 
	 * TODO: [core-2] Maintain separate userPrefs (for this session)
	 * and cloverPrefs (for all sessions)
	 * 
	 */
	public CloverProperties getCloverPrefs() {
		return userPrefs;
	}
	
	// Story:
	/**
	 * Returns the current Story data structure.
	 * 
	 * CloverWindow is designed to enabled multiple stories to be
	 * loaded, with the stories ArrayList.  Currently, however, this
	 * array is assumed to hold only a single element at position 0.
	 * In this future, the current story could be referenced by index,
	 * as stories.get(storyIndex)
	 * 
	 * @return Story
	 */
	public Story getStory() {
		Story tmp = null;
		try {
			tmp = (Story)stories.get(0);
		}
		catch (Exception e) {}
		return tmp;		
	}
	
	/**
	 * Returns the title of the current story.
	 * 
	 * @return Story title
	 */
	public String getStoryName() {
		Story tmp = getStory();
		if( tmp == null )
			return "";
		else
			return tmp.getTitle();
	}

	/**
	 * Adds a blank character ("New Character") to the story.
	 * The character must be added to ScriptUI, CharUI, etc.,
	 * as well as the Story data structure.
	 *
	 */
	public void addCharacter(){
		// Create a new character
		GlobalCharacter gc = new GlobalCharacter(getStory().getCharSize()); 
		gc.setName("New Character");
		gc.makeThumbnail(Image.SCALE_FAST);
		
		// Add the character to the story
		gc.setCharID(getStory().addCharacter(gc), getStory().getCharSize());
		
		// Update UIs
		scriptUI.addCharacter(gc);
		charUI.addCharacter(gc);
		m_gallery.setVisible(m_gallery.isVisible());
	}
	
	/**
	 * Show a generic "Delete (blank)" confirmation dialog.
	 * 
	 * @param message = Message in dialog (Are you sure...)
	 * @param thing = object, e.g. Character or Scene
	 * @return true if user agrees, false otherwise
	 */
	public boolean confirmDeletion(String message, String thing)
	{
		int choice = JOptionPane.showConfirmDialog(
								 CloverWindow.this, 
								 message,
								 "Delete "+thing+"?", 
								 JOptionPane.YES_NO_OPTION);

		return (choice == JOptionPane.YES_OPTION);
	}
	
	/**
	 * Deletes all instance of character, including dialog and actions.
	 * There IS CONFIRMATION in this function, so don't ask!
	 * 
	 * @param gid = Character's global id
	 */
	public void deleteCharacter(int gid){
		// Confirm deletion
		String they = (getStory().getCharacter(gid).getGender()==MALE)
				? "He" : "She";
		String name = "this character";
		if( getStory().getCharacter(gid).getName().length() > 0 )
			name = getStory().getCharacter(gid).getName();
		
		if(!confirmDeletion(
				"<html>Do you want to permanently delete " + 
				name + 
				" from the story?<p> " +
				they + " will be removed from every scene in which "+
				they.toLowerCase() + " appears.</html>",
				"Character"))
			return;
		
		// Delete character from scriptUI
		scriptUI.deleteCharacter(gid);
		
		// Delete character from charUI
		charUI.deleteCharacter(gid);
				
		// Delete character from all scenes.
		// (This includes response scenes)
		veUI.deleteCharacter(gid);
				
		// Delete gc from story
		getStory().removeCharacter(gid);
		
		// Add a character if all have been deleted.
		if(getStory().getCharacterCount() < 1) {
			addCharacter();
		}
		
		// Update UIs
		scriptUI.updateGallery();
		charUI.updateGallery();
		m_gallery.setVisible(m_gallery.isVisible());
		veUI.updateActions();
		veUI.updateGallery(0);
		if(mainPane.getSelectedIndex() == RESPONSE_INDEX){
			responseUI.updateActions();
			responseUI.updateGallery();
		}
	}
	
	/**
	 * Deletes all instance of character, including dialog and actions.
	 * 
	 * @param gid = Character's global id
	 */
	public void deleteCharacter(Integer gid){
		deleteCharacter(gid.intValue());
	}

}

