/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.awt.*;
import java.awt.event.*;

/**
 * ResponseChooser - A simple dialog for choosing a response to load
 * or creating a new response.  This dialog appears whenever the user
 * attempts to open the Response tab.
 * 
 * @author Terry J Bradley
 * @version 04.8.18
 *
 */
public class ResponseChooser extends JDialog {
	
	private JList jl_chooser;		// List of existing responses
	private DefaultListModel dlm;	// Model for list
	private JTextField jtf_title;	// Textfield for naming new response
	private JButton ok;				// OK button
	private boolean isOK = false;	// OK status
	
	/**
	 * Constructor - Creates a new dialog.  All work is delegated to
	 * the buildDialog() helper function.
	 * 
	 * @param owner Parent frame (CloverWindow)
	 */
	public ResponseChooser(JFrame owner){
		super(owner, "Choose a Response...", true);
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		buildDialog();
	}
	
	/**
	 * Populates the chooser with an array of existing response
	 * names.
	 * 
	 * @param strings Array of existing responses
	 */
	public void populateChooser(String[] strings){
		dlm.removeAllElements();
		dlm.addElement("Create New Response");
		for(int i=0; i<strings.length; i++){
			dlm.addElement(strings[i]);
		}
		jl_chooser.setSelectedIndex(0);
		jtf_title.setEnabled(true);
		pack();
	}
	
	/**
	 * Returns the index of the currently selected list item.
	 * 
	 * @return List selection index
	 */
	public int getChoiceIndex(){
		return jl_chooser.getSelectedIndex();
	}
	
	/**
	 * Returns the response title as written in the text field.
	 * 
	 * @return New response title
	 */
	public String getResponseTitle(){
		return jtf_title.getText();
	}
	
	/**
	 * Returns isOK status
	 * 
	 * @return isOK
	 */
	public boolean isOK(){
		return isOK;
	}
	
	/**
	 * Overwritten show() resets isOK and moves the dialog to the
	 * middle of the screen.
	 */
	public void show(){
		isOK = false;
		setLocationRelativeTo(null);
		jtf_title.setText("");
		super.show();
	}
	
	/**
	 * BuildDialog helper sets up the list, text field, and buttons.
	 *
	 * TODO: This function/dialog could also support thumbnails, like
	 * those shown in the FileUI.
	 */
	private void buildDialog(){
		JLabel question = new JLabel("Choose a Response to edit:");
		
		dlm = new DefaultListModel();
		jl_chooser = new JList(dlm);
		jl_chooser.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		jl_chooser.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(!e.getValueIsAdjusting()) {
					jtf_title.setEnabled(jl_chooser.getSelectedIndex() == 0);
				}
			}
		});
		MouseAdapter ma = new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(e.getClickCount() > 1){
					ok.doClick();
				}
			}
		};
		jl_chooser.addMouseListener(ma);
		
		JScrollPane listScroll = new JScrollPane(jl_chooser);
		listScroll.setPreferredSize(new Dimension(200,80));
		
		JLabel text = new JLabel("Response Title:");
		
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(jl_chooser.getSelectedIndex() == 0 &&
				   jtf_title.getText().length() < 1) 
				{
					Toolkit.getDefaultToolkit().beep();
					jtf_title.requestFocus();
					return;
				}
				isOK = true;
				hide();
			}
		};
		
		jtf_title = new JTextField(20);
		jtf_title.addActionListener(al);
		
		ok = new JButton("Ok");
		ok.addActionListener(al);
		
		
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				isOK = false;
				hide();
			}
		});
		
		
		JPanel top = new JPanel();
		top.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.anchor = GridBagConstraints.NORTH;
		c.insets = new Insets(5,5,5,5);
		top.add(question, c);
		
		c.gridx = 1;
		top.add(listScroll, c);
		
		c.anchor = GridBagConstraints.WEST;
		c.gridy = 1;
		c.gridx = 0;
		top.add(text, c);
		c.gridx = 1;
		top.add(jtf_title, c);
		
		JPanel bottom = new JPanel();
		bottom.setLayout(new FlowLayout(FlowLayout.RIGHT));
		bottom.add(ok);
		bottom.add(cancel);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(top, BorderLayout.CENTER);
		getContentPane().add(bottom, BorderLayout.SOUTH);
		
		pack();
	}
}