/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.Interface.ActionGrid.*;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Data.*;

/**
 * SpeakDialog:
 *  Implements the dialog for entering spoken words.  The same class
 *  is used for the thought dialog as well.
 * 
 * @author Terry J Bradley
 * @version 04.8.18
 *
 */
public class SpeakDialog extends JDialog {

	protected CloverWindow m_parent;		// CloverWindow parent
	protected JLabel m_character;			// Character name
	protected ActionButton m_action;		// Inactive button shows action
											// type
	protected JTextArea m_text;				// Text area for speech/thought
	protected JToggleButton m_record;		// Record button
	protected JToggleButton m_play;			// Play button
	protected JButton m_stop;				// Stop button
	protected JButton m_ok;					// OK button
	protected JButton m_cancel;				// Cancel button
	protected JLabel m_voiceStatus;			// Voice status label
	protected JToggleButton[] m_emotions;	// Emotion toggles
	protected String m_dialog;				// Stores dialog
	protected int m_emotion;				// Store emotion
	protected boolean isOK = false;			// OK status

	protected boolean isSpeak;				// True for speech, false for thought
	protected boolean hasAudio = false;		// Indicates whether audio exists
	protected boolean canRecord = false;	// Indicates whether user can
											// record audio
	protected String m_audioFilename;		// Filename for storing audio
	protected int m_gid;					// Character's ID
	
	private boolean go = true;				// Anything more than a temp var?
	
	/**
	 * Audio states, enumerated
	 */
	public final static int PLAYING_AUDIO		= 0;
	public final static int RECORDING_AUDIO 	= 1;
	public final static int WAITING 			= 2;
	
	private static int EMOTION_ROWS 	= 2;	// Layout of emotions
	private static int EMOTION_COLS 	= 5;
	
	/**
	 * Voice status labels
	 */
	private static String VOICE_RECORD 	= "voice is recorded by the author.";
	private static String VOICE_TTS 	= "voice is spoken by the computer.";

	/**
	 * Constructor - Creates a dialog for either speech entry or
	 * thought entry, depending on actionType.
	 * 
	 * @param owner CloverWindow parent
	 * @param actionType Action type: See Constants.java (SPEAK_OFFSET)
	 */
	public SpeakDialog(CloverWindow owner, int actionType) {	
		super(owner, true);
		isSpeak = (actionType == Constants.SPEAK_OFFSET);
		setTitle( (isSpeak) ? "Add Dialog:" : "Add Thought:" );
		m_parent = owner;
		
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		setResizable(false);
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
	
	// Left Pane	
		m_character = new JLabel("NAME");
		m_character.setVerticalTextPosition(JLabel.BOTTOM);
		m_character.setHorizontalTextPosition(JLabel.CENTER);
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(15,15,5,5);
		c.fill = GridBagConstraints.NONE;
		getContentPane().add(m_character, c);
		
		m_action = new ActionButton(actionType, 
				(isSpeak) ? Clover.createImageIcon2(Clover.SPEAK_ICON_LOC) :
							Clover.createImageIcon2(Clover.THINK_ICON_LOC), 
									Constants.EMOTE_OFFSET,
									Emote.EMOTICONS[0].m_image,
									null, true, m_parent );
		MouseListener[] mls = (MouseListener[])
							  (m_action.getListeners(MouseListener.class));
		for(int i=0; i<mls.length; i++)
			m_action.removeMouseListener(mls[i]);
		c.gridy = 1;
		c.insets = new Insets(5,15,5,5);
		c.anchor = GridBagConstraints.SOUTH;
		getContentPane().add(m_action, c);
		
	// Dialog and Emotion Panel
		m_text = new JTextArea(4, 30);
		m_text.setLineWrap(true);
		m_text.setWrapStyleWord(true);
		m_text.setMargin(new Insets(4,4,4,4));
		JScrollPane sp = new JScrollPane(m_text);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		JPanel emotions = new JPanel();
		m_emotions = new JToggleButton[Emote.EMOTICONS.length];
		emotions.setLayout(new GridLayout(EMOTION_ROWS,EMOTION_COLS));
		ButtonGroup bg = new ButtonGroup();
		for( int i=0; i<Emote.EMOTICONS.length; i++ ){
			JToggleButton jtb = new JToggleButton(Emote.EMOTICONS[i].m_image);
			jtb.setToolTipText(Emote.EMOTICONS[i].m_description);
			jtb.setName("" + Emote.EMOTICONS[i].m_index);
			final int tmp = i;
			jtb.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					m_action.setSecondaryIcon(Emote.EMOTICONS[tmp].m_image, 
											  Constants.EMOTE_OFFSET);
				}
			});
			// Try to compensate for 10 trans. pixels on the right of each image
			bg.add(jtb);
			emotions.add(jtb);
			if( Emote.EMOTICONS[i].m_index == 0 )
				jtb.doClick();
			m_emotions[i] = jtb;
		}
		c.gridy = 1;
		getContentPane().add(emotions, c);

		JPanel jp_dialog = new JPanel();
		jp_dialog.setLayout(new GridBagLayout());
		jp_dialog.setBorder(new CompoundBorder(
			new TitledBorder(
				new EtchedBorder(EtchedBorder.LOWERED),
				(isSpeak) ? " Dialog " : " Thought "),
			new EmptyBorder(0,0,0,0)));
		GridBagConstraints cint = new GridBagConstraints();
		cint.gridx = 0;
		cint.gridy = 0;
		cint.insets = new Insets(5,5,5,5);
		jp_dialog.add(sp, cint);

		cint.gridy = 1;
		cint.anchor = GridBagConstraints.WEST;
		jp_dialog.add(emotions, cint);

		c.gridx = 1;
		c.gridy = 0;
		c.gridheight = 2;
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(5,5,5,5);
		getContentPane().add(jp_dialog, c);

	// Sound Pane
		if(isSpeak) {
			JPanel sounds = new JPanel();
			m_voiceStatus = new JLabel("Record Dialog");
			m_voiceStatus.setAlignmentX(Component.LEFT_ALIGNMENT);

			m_record = new JToggleButton(Clover.createImageIcon2(
					Clover.ICON_DIR+"Record24.gif"));
			m_record.setAlignmentX(Component.CENTER_ALIGNMENT);
			m_record.setToolTipText("Record Dialog");
			m_record.addActionListener(createCaptureAction());
			m_play = new JToggleButton(Clover.createImageIcon2(
					Clover.ICON_DIR+"Play24.gif"));
			m_play.setAlignmentX(Component.CENTER_ALIGNMENT);
			m_play.setToolTipText("Play Dialog");
			m_play.addActionListener(createPlaybackAction());
			m_stop = new JButton(Clover.createImageIcon2(
					Clover.ICON_DIR+"Stop24.gif"));
			m_stop.setAlignmentX(Component.CENTER_ALIGNMENT);
			m_stop.setToolTipText("Stop Recording or Playback");
			m_stop.addActionListener(createStopAction());
			sounds.add(m_record);
			sounds.add(m_stop);
			sounds.add(m_play);

			JPanel jp_audio = new JPanel();
			jp_audio.setLayout(new GridBagLayout());
			jp_audio.setBorder(new CompoundBorder(
				new TitledBorder(
					new EtchedBorder(EtchedBorder.LOWERED),	" Audio " ),
				new EmptyBorder(0,0,0,0)));
			cint = new GridBagConstraints();
			cint.anchor = GridBagConstraints.WEST;
			
			jp_audio.add(m_voiceStatus, cint);
			cint.gridx = 1;

			JButton change = new JButton("Change");
			if(!CloverWindow.TTS_ENABLED)
				change.setEnabled(false);
			change.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					m_parent.getStory().getCharacter(m_gid).toggleVoice();
					canRecord = !canRecord;
					setUI(WAITING);					
				}
			});
			jp_audio.add(change, cint);
			cint.gridy = 1;
			cint.gridx = 0;
			cint.anchor = GridBagConstraints.WEST;
			jp_audio.add(sounds, cint);
			
			c.gridheight = 1;
			c.gridy = 2;
			getContentPane().add(jp_audio, c);
		}
		
	// Third Line
		m_ok = new JButton("OK");
		this.getRootPane().setDefaultButton(m_ok);
		m_ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Store action settings
				m_dialog = m_text.getText();
				for( int i=0; i<m_emotions.length; i++ ){
					if( m_emotions[i].isSelected() ) {
						m_emotion = Integer.parseInt(m_emotions[i].getName());
					}
				}
				isOK = true;
				hide();
			}
		});

		m_cancel = new JButton("Cancel");
		m_cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Store action settings
				isOK = false; // should already be false
				hide();
			}
		});
		
		JPanel buttons = new JPanel();
		buttons.add(m_ok);
		buttons.add(m_cancel);
		c.gridy = 3;
		c.gridx = 1;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.EAST;

		getContentPane().add(buttons, c);
		
		pack();
	}
	
	/**
	 * Helper function returns an action listener that
	 * plays any available audio for the given action.
	 * 
	 * @return Action listener for playing audio
	 */
	private ActionListener createPlaybackAction(){
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				// Check for audio
				if(canRecord && hasAudio) {
					setUI( m_parent.playAudio(m_audioFilename));						
				}
				else if(!canRecord) {
					m_parent.playTTS(m_text.getText(), m_gid);
					ActionListener al = new ActionListener(){
						public void actionPerformed(ActionEvent e){
							go = m_parent.isAudioPlaying();
							if(!go) {
								((Timer)e.getSource()).stop();
								setUI(WAITING);
							}
						}
					};
					Timer t = new Timer(100, al);
					t.setRepeats(true);
					t.start();
				}
			}
		};
		return al;
	}
	
	/**
	 * Helper function returns an action listener that
	 * attempts to capture audio for the given action.
	 * 
	 * @return Action listener for capturing audio
	 */
	private ActionListener createCaptureAction(){
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(canRecord){
					int result = m_parent.captureAudio(m_audioFilename);
					setUI(result);
				}
			}
		};
		return al;
	}
	
	/**
	 * Helper function returns an action listener that
	 * halts any audio playback for the given action.
	 * 
	 * @return Action listener for halting audio
	 */
	private ActionListener createStopAction(){
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				m_parent.stopAudio();
				setUI(WAITING);
			}
		};
		return al;
	}
	
	/**
	 * Sets the UI, particularly the audio buttons, for the given
	 * audio condition
	 * 
	 * @param condition Audio state (see PLAYING_AUDIO above)
	 */
	public void setUI(int condition){
		switch(condition){
			case PLAYING_AUDIO:
				// Make the play button become a stop button
				m_play.setEnabled(true);
				m_play.setSelected(true);
				m_record.setEnabled(false);
				m_ok.setEnabled(true);
				m_cancel.setEnabled(true);
			break;
			
			case RECORDING_AUDIO:
				// Make the record button become a stop button
				m_play.setEnabled(false);
				m_record.setEnabled(true);
				m_record.setSelected(true);
				m_ok.setEnabled(false);
				m_cancel.setEnabled(false);
			break;
			
			case WAITING:
				if(isSpeak){
					m_record.setSelected(false);
					m_play.setSelected(false);
					if(canRecord){
						m_voiceStatus.setText(m_character.getText()+"'s "+
											  VOICE_RECORD);
						m_record.setEnabled(true);
						m_play.setEnabled(hasAudio);
					}
					else {
						m_voiceStatus.setText(m_character.getText()+"'s "+
											  VOICE_TTS);
						m_record.setEnabled(false);
						m_play.setEnabled(true);
					}
					m_ok.setEnabled(true);
					m_cancel.setEnabled(true);
				}
			break;
			
		}
	}
	
	/**
	 * Specialized show function sets this interface up for a new
	 * or existing action
	 * 
	 * @param pa Action to open (may be new)
	 * @param filename Filename for audio recording/playback
	 */
	public void show(PrimaryAction pa, String filename) {
		GlobalCharacter gc = m_parent.getStory().getCharacter(pa.getSourceGID());
		m_character.setIcon(gc.getThumbnail());
		m_character.setText(gc.getName());
		m_gid = gc.getID();
		
		String text = pa.getActionText();
		if( pa instanceof Speak ) {
			hasAudio = ( ((Speak)pa).getAudioPath() != null );
		}
		
		m_text.setText(text);
		m_dialog = text;
		
		canRecord = gc.getRecord();
		m_audioFilename = filename;
		File file = new File(Clover.TEMP_DIR+filename);
		hasAudio = file.exists();
		
		if(isSpeak)
			setUI(WAITING);
			
		int emotion = 0;
		if(pa.getSecondaryAction() != null && pa.getSecondaryAction() instanceof Emote)
			emotion = ((Emote)pa.getSecondaryAction()).getEmotion();
		m_emotion = emotion;
		for( int i=0; i<m_emotions.length; i++ ){
			if( emotion == Integer.parseInt(m_emotions[i].getName()) ) {
				m_emotions[i].doClick();
				break;
			}
		}
		m_action.setSecondaryIcon(Emote.getEmoticon(emotion).m_image,
								  Constants.EMOTE_OFFSET);
				
		isOK = false;
		
		pack();
		super.show();
	}
	
	/**
	 * Returns the dialog.  This will be unchanged if cancel is
	 * pressed instead of OK.
	 * 
	 * @return Dialog (speech or thought)
	 */
	public String getDialog(){
		return m_dialog;
	}
	
	/**
	 * Returns the filename where captured audio is stored.
	 * 
	 * @return Audio filename
	 */
	public String getFilename(){
		if(hasAudio)
			return m_audioFilename;
		else
			return null;
	}
	
	/**
	 * Returns associated emotion (which may have changed in this dialog)
	 * 
	 * NOTE: This function is changed and UNTESTED
	 * 
	 * @return New emotion
	 */
	public int getEmotion(){
		return m_emotion;
	}
	
	/**
	 * Returns isOK status
	 * 
	 * @return OK Status
	 */
	public boolean isOKClicked(){
		return isOK;
	}
	
}