/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.awt.*;
import java.awt.font.*;
import java.text.*;
import javax.swing.JLabel;
import java.util.Hashtable;

/**
 * ActionBubble Class
 * 
 * Superclass of the various bubbles (like speak, think, and
 * emotion).  Most important function is the ability to paint
 * text on top of the label (which should have an empty 
 * bubble as icon).
 * 
 * @author Terry Bradley
 * @version 04.8.27
 *
 */
public class ActionBubble extends JLabel {
	
	protected AttributedString m_text;	// Formatted(?) text string
	protected static Hashtable m_map;	// Map of text attributes, i.e.
										// font, size, weight, etc.
	protected LineBreakMeasurer lineMeasurer;
										// Measures paragraph positioning
	protected int paragraphStart;		// Index of start of paragraph
	protected int paragraphEnd;			// Index of end of paragraph
	protected int startLine;			// Index of start of line
	protected int endLine;				// Index of end of line

	protected int bubbleHeight;			// Height of this bubble
	protected boolean rightBubble;		// True for right bubble
	
	static final int LINE_OFFSET_Y = 16;// Default offset from top of bubble 
	
	/**
	 * FontRenderContext is necessary to compute line measurements.
	 */
	private static final FontRenderContext DEFAULT_FRC = 
							new FontRenderContext(null, true, false);
							
	/**
	 * For debugging purposes...
	 */
	private static final String SAMPLE_STRING = 
		"When in the Course of human events, it becomes necessary " +
		"for one people to dissolve the political bands which have " +
		"connected them with another, and to assume among the powers " +
		"of the earth, the separate and equal station to which the " +
		"Laws of Nature and of Nature's God entitle them, a decent " +
		"respect to the opinions of mankind requires that they should " +
		"declare the causes which impel them to the separation.";

	/**
	 * Constructor - Sets up the text attributes, including the hardcoded,
	 * comicbook font.  Size and rightBubble are assigned, but it is assumed
	 * that these values will later be overwritten.
	 *
	 */
	public ActionBubble() {
		m_map = new Hashtable();
		m_map.put(TextAttribute.SIZE, new Float(12.0));
		m_map.put(TextAttribute.FONT, new Font("LetterOMatic!", Font.PLAIN, 12));
		m_map.put(TextAttribute.JUSTIFICATION, new Float(0.5));
	
		rightBubble = true;
		setSize(150,150);
	}
	
	/**
	 * Sets the text in the bubble.  Although a complete text block
	 * is included, only the portions lying between start and end will be 
	 * displayed in the bubble.
	 * 
	 * @param text Block of text
	 * @param start First position to include in bubble
	 * @param end Last position to include
	 */
	public void setBubbleText(String text, int start, int end){
		if(text.length() == 0)
			text = " ";
		m_text = new AttributedString(text, m_map);
		startLine = start;
		endLine = end;
		
		AttributedCharacterIterator paragraph = m_text.getIterator();
		paragraphStart = paragraph.getBeginIndex();
		paragraphEnd = paragraph.getEndIndex();
		lineMeasurer = 
			new LineBreakMeasurer(paragraph, DEFAULT_FRC);
	}

	/**
	 * Paint is responsible for displaying the text in the bubble.  This
	 * is perferred to using a label or other approach because Java2D
	 * allows the text to be anti-aliased here.  This option is not 
	 * available to labels, and thus our comic font would look ugly.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D graphics2D = (Graphics2D) g;
		Dimension size = getSize();
		
		boolean roundedRect = (size.width > 200);
		float formatWidth = size.width - 16;	// Max available width

		float drawPosY = 16.0f;		// Starting height, 16px below top
						
		int line = 0;
		lineMeasurer.setPosition(paragraphStart);
		TextLayout layout = 
					lineMeasurer.nextLayout(formatWidth);
		lineMeasurer.setPosition(paragraphStart);
		while (lineMeasurer.getPosition() < paragraphEnd) {
			// Remove early lines
			while(startLine > line) {
				layout = lineMeasurer.nextLayout(formatWidth);
				line++;
			}
		   	drawPosY += layout.getAscent();
		   	float drawPosX;

		   	if(!roundedRect) {
				drawPosX = size.width/2 - (float)Math.sqrt(
					(size.width/2)*(size.width/2) -
					((size.width/2)*(size.width/2)*
						(bubbleHeight/2 - drawPosY)*(bubbleHeight/2 - drawPosY)
						/ ((bubbleHeight/2)*(bubbleHeight/2)))); 
				formatWidth = size.width - 2*drawPosX - 20;
		   	}
		   	else {
		   		drawPosX = 8;
		   	}
		   	
			layout = lineMeasurer.nextLayout(formatWidth);
			drawPosX += size.width/2 - 
				(2*drawPosX+(float)layout.getBounds().getWidth())/2;
				
		   	layout.draw(graphics2D, drawPosX, drawPosY);
		   	drawPosY += layout.getDescent() + 
						layout.getLeading();
			line++;
			if(line > endLine)
				break;
		 }
	  }	
	
	/**
	 * Uses the sample text string to produce a string of the given
	 * length.
	 * 
	 * @param length Desired length of string
	 * @return Sample string of desired length
	 */
	public static String generateTestString(int length){
		return SAMPLE_STRING.substring(0,length-1);
	}
	
	/**
	 * Counts the number of lines needed to display the given dialog
	 * in a bubble.  The process of determining this number is nearly
	 * identical to printing the text in the paintComponent function.
	 * 
	 * @param dialog String to text
	 * @return Number of lines string would need to fit in this bubble
	 */
	public static int getNumberOfLines(String dialog){
		Dimension size = new Dimension(208,160);
		int bubbleHeight = 144;
		float formatWidth = size.width - 16;
		float drawPosY = (size.height - bubbleHeight + 
							LINE_OFFSET_Y)/2;
		float drawPosX = 8;
		int line = 0;
		if(dialog.length() == 0)
			dialog = " ";
		AttributedString text = new AttributedString(dialog, m_map);
		AttributedCharacterIterator paragraph = text.getIterator();
		LineBreakMeasurer lineMeasurer = 
			new LineBreakMeasurer(paragraph, DEFAULT_FRC);
		
		lineMeasurer.setPosition(paragraph.getBeginIndex());
		TextLayout layout = 
					lineMeasurer.nextLayout(formatWidth);
		lineMeasurer.setPosition(paragraph.getBeginIndex());
		
		while (lineMeasurer.getPosition() < paragraph.getEndIndex()) {

			drawPosY += layout.getAscent();
			layout = lineMeasurer.nextLayout(formatWidth);
			drawPosX = 8;
			drawPosX += size.width/2 - 
				(2*drawPosX+(float)layout.getBounds().getWidth())/2;
				
			drawPosY += layout.getDescent() + 
						layout.getLeading();
			line++;
		 }
		 return line;		
	}
}