/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import javax.sound.sampled.*;
import java.io.*;

/**
 * AudioCapture Class
 * Provides sound capture (i.e. mic) support with a resulting audio file
 * in a hardcoded audio format.
 * 
 * The audio format used is: PCM_SIGNED encoding, 22050.0 Hz, 16 bits, mono,
 * not big endian.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class AudioCapture implements Runnable{
	
	Thread thread;			// Audio capture thread
	TargetDataLine line;	// Audio input line (from mic)
	String path;			// Path to output audio file
	
	/**
	 * Returns threads status
	 * 
	 * @return True iff thread is alive
	 */
	public boolean isRunning(){
		if(thread != null)
			return thread.isAlive();
		else
			return false;
	}
	
	/**
	 * Sets the audio capture path
	 * 
	 * @param p New output path for audio file
	 */
	public void setPath(String p){
		path = p;
	}
	
	/**
	 * Begins recording audio (see run() for actual work)
	 *
	 */
	public void start(){
		thread = new Thread(this);
		thread.setName("Capture");
		thread.start();
	}
	
	/**
	 * Stops recording audio and closes the input line.
	 *
	 */
	public void stop(){
		line.stop();
		line.close();
		line = null;
		thread = null;
	}
	
	/**
	 * The only Runnable function; sets the audio format, records audio
	 * from mic, saves audio to path.
	 */
	public void run(){
		double duration = 0;
		AudioInputStream audioInputStream = null;
		File file = new File(path);
		
		// define the required attributes for our line, 
		// and make sure a compatible line is supported.
		AudioFormat	format = new AudioFormat(
								AudioFormat.Encoding.PCM_SIGNED,
								22050.0F,
								16, 
								1, 
								2,
								22050.0F,
								false);
		DataLine.Info info = new DataLine.Info(TargetDataLine.class, 
			format);
                        
		if (!AudioSystem.isLineSupported(info)) {
			return;
		}

		// get and open the target data line for capture.
		try {
			line = (TargetDataLine) AudioSystem.getLine(info);
			line.open(format, line.getBufferSize());
		} catch (LineUnavailableException ex) { 
			return;
		} catch (SecurityException ex) { 
			return;
		} catch (Exception ex) { 
			return;
		}

		// Unneeded legacy code?  Removal UNTESTED.
		/*
		Control[] controls = line.getControls();
		for(int i=0; i<controls.length; i++){
			System.out.println(controls[i].toString());
		}
		*/
            
		// Begin recording
		line.start();
		audioInputStream = new AudioInputStream(line);
		
		try {
			
			AudioSystem.write(audioInputStream,	AudioFileFormat.Type.WAVE, file);
		}
		catch (Exception e){
			System.err.println(e.toString());
			return;
		}
		
		// Late additions -- Need to test
		try{
			line.flush();
			line.close();
			line = null;
		}catch(Exception e){}
		

	}	
	
	
}