/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Data;

import java.io.*;
import java.util.ArrayList;

/**
 * In Clover, a <b>Response</b> represents a set of scenes.  This could
 * be an actual response to the story, or it could represent the orignal
 * story itself.  Some fields could allow simple file locking mechanisms,
 * but these abilities have not yet been included.
 * 
 * Scenes in a response are stored by SceneID numbers, which allows (in
 * theory) the same scene to appear in multiple responses.
 * 
 * @author Terry J Bradley
 * @version 04.7.29
 *
 */
public class Response implements Serializable{
	
	private ArrayList	m_scenes;		// list of Integer sceneIDs
	private String		m_title;		// Response title
	private String		m_author;		// Author/title
										// NOT CURRENTLY USED
	private String	  	m_password;		// Editing password
										// NOT CURRENTLY USED
	
	// Prevent InvalidClassExceptions by hardcoding a version id.
	//  - "cloverRes"
	static final long serialVersionUID = 312152251818519l;
	
	/**
	 * Constructor - initializes scene array and defines response
	 * as "" by "".
	 *
	 */
	public Response(){
		this("");
	}
	
	/**
	 * Constructor - initializes scene array and defines response
	 * as name by name.
	 *
	 * @param name Used for author's name AND response title
	 */
	public Response(String name){
		m_scenes = new ArrayList();
		m_author = name;
		m_title = name;
	}
	
	/**
	 * Adds a scene to the response.
	 * 
	 * @param sid SceneID
	 */
	public void addScene(Integer sid){
		m_scenes.add(sid);
	}
	
	/**
	 * Adds a scene to the response at given index.
	 * 
	 * @param index Location for added scene.
	 * @param sid SceneID
	 */
	public void addScene(int index, Integer sid){
		m_scenes.add(index, sid);
	}
	
	/**
	 * Returns an ordered list of scene indeces.
	 * 
	 * @return Ordered list of scene IDs (ArrayList of Integers)
	 */
	public ArrayList getResponseOrder(){
		return m_scenes;
	}
	
	/**
	 * Returns the SceneID at the specified location.
	 * 
	 * @param pos Position of scene in response.
	 * @return SceneID of scene.
	 */
	public Integer getSceneAtPosition(int pos){
		try {
			return (Integer)m_scenes.get(pos);
		}
		catch(Exception e){
			return new Integer(-1);
		}
	}
	
	/**
	 * Removes scene at given location.
	 * 
	 * @param pos Position of scene in response.
	 * @return SceneID if successful; null otherwise.
	 */
	public Object removeSceneAtPosition(int pos){
		try {
			return m_scenes.remove(pos);
		}
		catch(Exception e){
			System.err.println("Error removing scene at position "+pos);
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Get number of scenes in response
	 * 
	 * @return Number of scenes
	 */
	public int getSceneCount(){
		return m_scenes.size();
	}
	
	/**
	 * Get response author
	 * 
	 * @return Author
	 */
	public String getAuthor() {
		return m_author;
	}

	/**
	 * Set author
	 * 
	 * @param name Author
	 */
	public void setAuthor(String name) {
		m_author = name;
	}

	/**
	 * Get response title
	 * 
	 * @return Title
	 */
	public String getResponseTitle() {
		return m_title;
	}

	/**
	 * Set response title
	 * 
	 * @param string Title
	 */
	public void setResponseTitle(String string) {
		m_title = string;
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(m_scenes);
		oos.writeObject(m_title);
		oos.writeObject(m_author);
		oos.writeObject(m_password);
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			m_scenes	= (ArrayList)ois.readObject();
			m_title		= (String)ois.readObject();
			m_author	= (String)ois.readObject();
			m_password	= (String)ois.readObject();	
		} catch (ClassNotFoundException ce){
			System.err.println(ce);
		}		
	}

}