/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Data;

import java.util.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import javax.imageio.*;
import java.io.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.DrawingTools.*;

/**
 * Scene - Data type used to store each scene in a story.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 10/18/03
 *
 */
public class Scene implements Serializable{
	
	// Variables
	private HashMap appearances;		// Map of (Integer)GID,LocalCharacter pairs
	private ArrayList characterOrder;	// Array of (Long)GIDs
										// [0] = character in first row, etc.
	protected TreeMap actions;			// Map of (Long)GID, ArrayList of actions 

	private String backgroundLoc;		// Scene's image
	
	private String m_title;				// Scene Title
	private int m_sceneID;				// Unique ID for ea. scene
	protected int m_length = Constants.NUM_BLANK_COLS;	
										// Number of cols in Action Grid 
	
	private Story m_parent;
	
	// Prevent InvalidClassExceptions by hardcoding a version id.
	//  - "cloverScn"
	static final long serialVersionUID = 312152251819314l;
	
	/**
	 * Constructor - Creates a new scene with name "".
	 * 
	 * @param sty Story containing scene.
	 */
	public Scene(Story sty) {
		this("", sty);
	}
	
	/**
	 * Constructor - Creates a new scene with name s.
	 * 
	 * @param s Scene name
	 * @param sty Story containing scene.
	 */
	public Scene(String s, Story sty) {
		m_parent = sty;
		m_title = s;
		init();
	}
	
	/**
	 * Helper for constructors sets sceneID and initializes
	 * the data structures.
	 */
	private void init(){
		m_sceneID = -1;
		appearances = new HashMap();
		characterOrder = new ArrayList();
		actions = new TreeMap();
		backgroundLoc = "scene"+m_sceneID+".png";
	}
	
	/**
	 * Create new bufferedImage for canvas and save to file @ backgroundLoc
	 * 
	 * @param sceneSize Size of image to create
	 */
	public void createImage(Dimension sceneSize){
		if(m_sceneID < 0) return;
		BufferedImage bi = new BufferedImage(sceneSize.width,
									  sceneSize.height,
									  BufferedImage.TYPE_INT_ARGB);
		BackgroundSurface.clearCanvas(bi, Color.white);  //Constants.COLOR_CANVAS
		saveImage(bi);		
	}
	
	/**
	 * Save bi to file at backgroundLoc.
	 * 
	 * @param bi Image to store
	 */
	public void saveImage(BufferedImage bi){
		try{
			boolean success = ImageIO.write(
						  bi,
						  "png",
						  new File(Clover.TEMP_DIR+backgroundLoc));
		}
		catch(Exception e){
			System.err.println("Error writing background for scene"+m_sceneID);
			e.printStackTrace();
		}
	}
	

	/**
	 * Opens background image
	 * 
	 * @return Image
	 */
	public BufferedImage openImage(){
		try {
			return ImageIO.read(new File(
								Clover.TEMP_DIR+backgroundLoc));
		}
		catch(Exception e){
			System.err.println("Error reading background for scene"+m_sceneID);
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Returns an ArrayList formatted for dtd.m_data;
	 * that is, it should be an ArrayList of ArrayList of Long/Integer
	 * 
	 * 	TODO: [stability] Add many error checks
	 *  
	 * @return Formatted action arrayList 
	 */
	public ArrayList getActions() {
		ArrayList data = new ArrayList();
		
		// data should have m_length columns
		for( int i=0; i<m_length; i++ ) {
			ArrayList v = new ArrayList();
			data.add(v);
		}

		int currentRow;
		Set s; 
		for(currentRow = 0; currentRow<characterOrder.size(); currentRow++){	
			// find a key that matches characterOrder[currentRow]
			Long key = null;
			s = actions.keySet();
			for( Iterator e = s.iterator(); e.hasNext(); ){
				key = (Long)e.next();
				if(key.intValue() == ((Long)characterOrder.get(currentRow)).intValue()){
					//e.remove();	// this removes from action treemap.  that's bad.
					break;
				}
				else
					key = null;
			}
			if(key == null){
				System.err.println("Error in Scene.getActions()");
				return null;
			}
			
			// Add current key to first column as Long
			((ArrayList)data.get(0)).add(key);
			
			// Add zeroes to every column > 0
			for( int j=1; j<data.size(); j++ ) {
				((ArrayList)data.get(j)).add(new Integer(0));
			}
			
			// Set value in appropriate column for each action			
			ArrayList acts = (ArrayList)actions.get(key);
			if(acts != null){
				for( int j=0; j<acts.size(); j++){
					CloverAction ca = (CloverAction)acts.get(j);
					if(ca instanceof PrimaryAction) {
						PrimaryAction pa = (PrimaryAction)ca;
						ArrayList tmp = (ArrayList)data.get(pa.getStartingCol());
						int incr = 0;
						if( ca instanceof Speak ) {
							incr = Constants.SPEAK_OFFSET;
							if ( ((Speak)ca).getAudioPath() != null || 
								 !m_parent.getCharacter(pa.getSourceGID()).record )
								incr += Constants.HAS_AUDIO_OFFSET;
						}
						else if( ca instanceof Think )
							incr = Constants.THINK_OFFSET;
						else if( ca instanceof Emote )
							incr = Constants.EMOTE_OFFSET + 
								   Constants.EMOTION_DIVIDER * ((Emote)ca).m_emotion;
						tmp.set(currentRow, new Integer(
							((Integer)tmp.get(currentRow)).intValue()+incr));
						// need to add tmp back to data?
						data.set(pa.getStartingCol(), tmp);
					}// PrimaryAction
				}//for [acts]
			}
			//currentRow++;
		}//for [s]
		
		for( int i=((ArrayList)data.get(0)).size(); i<Constants.NUM_BLANK_ROWS; i++){
			((ArrayList)data.get(0)).add(new Long(-1));
		}
	
		return data;
	}
		
	/**
	 * Returns primary action at time, if it exists.  If
	 * that fails, then returns null.
	 * 
	 * @param gid
	 * @param time
	 * @return Action for character at time
	 */
	public PrimaryAction getPrimaryAction(Long gid, int time){
		ArrayList v = (ArrayList)actions.get(gid);
		if(v==null)
			return null;
		
		for( int i=0; i<v.size(); i++ ) {
			CloverAction act = (CloverAction)v.get(i);
			if( act instanceof PrimaryAction && 
				((PrimaryAction)act).getStartingCol() == time )
				return (PrimaryAction)act;
		}
		return null;
	}
	
	/**
	 * Returns secondary action at time, if it exists.  If
	 * it fail, then returns null.  This functions fails to identify
	 * secondary actions that exist apart from primary actions.
	 * (See commented code for a version that would detect both.
	 * Unfortunately, secondary actions currently do not contain
	 * startingCol information.)
	 * 
	 * @param gid
	 * @param time
	 * @return Action for character at time
	 */
	public SecondaryAction getSecondaryAction(Long gid, int time){
		PrimaryAction pa = getPrimaryAction(gid, time);
		if(pa == null)
			return null;
		else
			return pa.getSecondaryAction();
		
		/*
		ArrayList v = (ArrayList)actions.get(gid);
		if(v==null)
			return null;
		
		for( int i=0; i<v.size(); i++ ) {
			CloverAction act = (CloverAction)v.get(i);
			if( act instanceof PrimaryAction && 
				((PrimaryAction)act).getStartingCol() == time )
				return ((PrimaryAction)act).getSecondaryAction();
			else if( act instanceof SecondaryAction &&
					((SecondaryAction)act).getStartingCol() == time )
				return (SecondaryAction)act;
		}
		return null;
		*/
	}

	/**
	 * Match class of character action to given action type.
	 * 
	 * @param ca CharacterAction
	 * @param actionType Expected/desired action class
	 * @return True is action matches expectation
	 */
	private boolean matchActions(CharacterAction ca, int actionType){
		return (ca instanceof Speak && actionType == Constants.SPEAK_OFFSET) ||
			   (ca instanceof Think && actionType == Constants.THINK_OFFSET) ||
		       (ca instanceof Emote && actionType == Constants.EMOTE_OFFSET);
	}
	
	/**
	 * Match class of character action to given action type.  In this version,
	 * Speak and Think actions are considered equal.
	 * 
	 * @param ca CharacterAction
	 * @param actionType Expected/desired action class
	 * @return True is action matches expectation
	 */
	private boolean matchActions2(CharacterAction ca, int actionType){
		// This time, check for emotions / non-emotions (Speak == Think)
		return ((ca instanceof Speak || ca instanceof Think) && 
									actionType != Constants.EMOTE_OFFSET) ||
				((ca instanceof Emote) && 
									actionType == Constants.EMOTE_OFFSET);
	}
	
	/**
	 * Add (replace if it exists) an action to the scene.
	 * Return true if table needs to be rebuilt
	 * 
	 * @param gid
	 * @param a character action
	 */
	public boolean addAction(CharacterAction a, int gid){
		// Store the  of actions corresponding to gid in v		
		ArrayList v;
		boolean b = false;
		try {
			v = (ArrayList)actions.get(new Long(gid));
		}
		catch(Exception e){
			v = new ArrayList();
		}
		
		// Check for exisiting action of type a
		boolean replace = false;	// don't think this is necessary
		int i = 0;
		if(a instanceof PrimaryAction){
			for( i=0; i<v.size(); i++ ) {
				CloverAction ca = (CloverAction)v.get(i);
				if( ca instanceof PrimaryAction) {
					if( ((PrimaryAction)ca).getStartingCol() == 
						((PrimaryAction)a).getStartingCol() ) {
							replace = true;
							break;
					}
				}
			}
		}
		// Replace or add action as necessary
		if(replace)
			v.set(i, a);
		else
			v.add(a);

		// Adjust num columns if needed		
		if(a instanceof PrimaryAction){	
			if( ((PrimaryAction)a).getStartingCol()+2 >= m_length ) {
				//System.out.println("Adding a column");
				m_length = ((PrimaryAction)a).getStartingCol()+2;
				b = true;
			}
		}
		
		// store v in hashtable
		actions.put(new Long(gid), v);
		
		return b;
	}
	
	/**
	 * Removes ALL actions at time time.
	 * 
	 * @param gid
	 * @param time
	 */
	public void removeAction(int gid, int time) {
		// Store the  of actions corresponding to gid in v		
		ArrayList v;
		try {
			v = (ArrayList)actions.get(new Long(gid));
		}
		catch(Exception e){
			System.err.println("Couldn't access action table");
			return;
		}		
		
		// Remove ALL actions at time time of ANY type.
		for(int i=0; i<v.size(); i++){
			CloverAction ca = (CloverAction)v.get(i);
			if( ca instanceof PrimaryAction ){
				if( ((PrimaryAction)ca).getStartingCol() == time )
				   	v.remove(i);
			}
		}
		
		// Store results
		actions.put(new Long(gid), v);

	}

	/**
	 * Legacy background image function.  Replaced with openImage().
	 * 
	 * @deprecated
	 * @return Image
	 */
	public BufferedImage getBackground() {
		return openImage();
	}
	
	/**
	 * Checks to see if given character has local instances.
	 * 
	 * @param gid Character ID
	 * @return True iff one or more local instances exist.
	 */
	public boolean hasCharacter(int gid){
		LocalCharacter lc = (LocalCharacter)appearances.get(new Integer(gid));
		return !(lc == null);
	}
	
	/**
	 * Gets local version of character
	 * 
	 * @param gid Character ID
	 * @return LocalCharacter instance of character
	 */
	public LocalCharacter getCharacter(int gid) {
		return (LocalCharacter)appearances.get(new Integer(gid));
	}
	
	/**
	 * Gets global version of character
	 * 
	 * @param gid Character ID
	 * @return Global character
	 */
	public GlobalCharacter getCharacter(Integer gid){
		return m_parent.getCharacter(gid.intValue());
	}
	
	/**
	 * Returns ID of character at given position
	 * 
	 * @param pos Position in characterOrder
	 * @return ID of character
	 */
	public Long getCharacterIDAtPosition(int pos){
		Long result = new Long(-1);
		try {
			result = (Long)characterOrder.get(pos);
		}
		catch(Exception e){
			//System.err.println(e.toString());
			//e.printStackTrace();
		}
		return result;
	}
	
	/**
	 * Returns position of character with given ID
	 * 
	 * @param gid Character's ID
	 * @return Position in characterOrder
	 */
	public int getCharacterPosition(Long gid){
		for(int i=0; i<characterOrder.size(); i++){
			if(((Long)characterOrder.get(i)).intValue() ==
				gid.intValue() )
				return i;
		}
		return -1;
	}
	
	/**
	 * Gets number of characters in scene
	 * 
	 * @return Number of characters in scene
	 */
	public int getCharacterCount() {
		return appearances.size();
	}
	
	/**
	 * Gets Set of character IDs
	 * 
	 * @return Set of IDs of characters appearing in this scene
	 */
	public Set getCharacterIDs(){
		return appearances.keySet();
	}
	
	
	/**
	 * Adds character to the scene.
	 * 
	 * @param c LocalCharacter instance
	 * @param gid Character ID
	 */
	public void addCharacter(LocalCharacter c, int gid) {
		appearances.put(new Integer(gid), c);
		characterOrder.add(new Long(gid));
		actions.put(new Long(gid), new ArrayList());
	}
		
	/**
	 * Removes character from the scene.
	 * 
	 * @param globalID Character ID
	 * @return True if remove succeeds
	 */
	public boolean removeCharacter(int globalID) {

		boolean result = true;

		Object v = appearances.remove(new Integer(globalID));
		if( v == null )
			result = false;
		// Remove character's actions from actions
		characterOrder.remove(new Long(globalID));
		v = actions.remove(new Long(globalID));
		if( v == null )
			result = false;
		
		return result;		
	}
	
	/**
	 * Moves the character up or down in the dialog table (and between
	 * layers.)
	 * 
	 * @param oldPos
	 * @param newPos
	 */
	public void reorderCharacter(int oldPos, int newPos){
		Object o = characterOrder.remove(oldPos);
		characterOrder.add(newPos, o);
	}
	
	/**
	 * Moves the local instance of the character to a new Point.
	 * 
	 * @param p
	 * @param gid
	 */
	public void moveCharacter(Point p, int gid) {
		LocalCharacter lc = (LocalCharacter)appearances.get(new Integer(gid));
		lc.setLocation(p);

	}
	
	/**
	 * Resizes instance of character in scene
	 * 
	 * @param dim New size
	 * @param gid Character ID
	 */
	public void resizeCharacter(Dimension dim, int gid) {
		LocalCharacter lc = (LocalCharacter)appearances.get(new Integer(gid));
		lc.setSize(dim);
	}
	
	/**
	 * Legacy background image method.  Replaced by saveImage(img)
	 * 
	 * @param d Image to save
	 * @deprecated
	 */
	public void setBackground(BufferedImage d) {
		saveImage(d);
	}
	
	/**
	 * Sets scene title
	 * 
	 * @param s Title
	 */
	public void setTitle(String s) {
		m_title = s;
	}
	
	/**
	 * Gets scene title
	 * 
	 * @return Title
	 */
	public String getTitle() {
		return m_title;
	}
	
	/**
	 * Gets scene ID
	 * 
	 * @return SceneID
	 */
	public int getSceneID(){
		return m_sceneID;
	}
	
	/**
	 * Sets scene ID.  OLD BACKGROUND IMAGE LOCATION WILL BE LOST!
	 * (Image at that location is untouched - perhaps it should be deleted?)
	 * 
	 * @param id New sceneID
	 */
	public void setSceneID(int id){
		m_sceneID = id;
		backgroundLoc = "scene"+m_sceneID+".png";
		createImage(m_parent.m_sceneSize);
	}

	/**
	 * Sets data structure storing character instances.
	 * 
	 * @param chars HashMap of characters
	 */
	public void setCharacters(HashMap chars){
		appearances = chars;
	}
	
	/**
	 * Gets character instances
	 * 
	 * @return Character instances
	 */
	public HashMap getCharacters(){
		return appearances;
	}
	
	/**
	 * Sets actions.
	 * 
	 * @param acts Actions
	 */
	public void setActions(TreeMap acts){
		actions = acts;
	}
	
	/**
	 * Gets actions.
	 * 
	 * @return Actions
	 */
	public TreeMap getActionMap(){
		return actions;
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
			
		oos.writeObject(appearances);
		oos.writeObject(characterOrder);
		oos.writeObject(actions);
		oos.writeObject(backgroundLoc);
		oos.writeObject(m_title);
		oos.writeInt(m_sceneID);
		oos.writeInt(m_length);			
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			appearances 	= (HashMap)ois.readObject();
			characterOrder	= (ArrayList)ois.readObject();
			actions			= (TreeMap)ois.readObject();
			backgroundLoc	= (String)ois.readObject();
			m_title			= (String)ois.readObject();
			m_sceneID		= ois.readInt();
			m_length		= ois.readInt();
		} catch (ClassNotFoundException ce){
			System.err.println("Scene failed to read "+ce);
		}		
	}
}
