/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.border.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;

/**
 * BackgroundSurface - Primary drawing surface, used in CharacterUI
 * and Scene editors.  Background surface currently uses only pixel
 * data - strokes are not stored in any form.  The mouse listeners in
 * this class get the drawing tool from the Toolbox and perform brush,
 * eraser, and undo actions. Fill actions are passed to the Fill Tool.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 9/23/03
 *
 */

public class BackgroundSurface extends JPanel implements Constants {

	//private static Dimension dim;			// Size of canvas, shouldn't
											// be static or even necessary.
	
	protected CloverWindow m_frame;			// Main frame
	protected DrawingSurface m_parent;		// Parent drawing surface
	protected BufferedImage m_buffImg;		// Primary image
	protected BufferedImage m_undoImg;		// Backup image for undo
	protected Color m_backColor;			// Background color.  This
											// is Clear for characters, white
											// for scenes, and currently cannot
											// be changed by the user.
	protected boolean m_transparentImage;	// True iff image has clear pixels
											// (i.e. characters)
	
	public static boolean cursorLock = false;
											// Simple lock to hold drawing cursor
											// during stroke.
	protected static BufferedImage big;		// Temp bufferedImage, designed to
											// reduce memory usage. It may or
											// may not work.
	
	private boolean m_drawing;				// True while drawing
	private Point m_oldPoint;				// Last point when drawing
	private int minX = 20000;				// Used to detect boundaries
	private int maxX = -1;
	private int minY = 20000;
	private int maxY = -1;

	/**
	 * Constructor - Creates a background with given size, background color,
	 * parent data and transparency.
	 * 
	 * @param d Size of canvas
	 * @param bg_color Background color
	 * @param vae CloverWindow parent
	 * @param ds DrawingSurface parent
	 * @param transparency True for characters (i.e. transparent images)
	 */
	public BackgroundSurface(Dimension d, Color bg_color, CloverWindow vae,
							 DrawingSurface ds, boolean transparency) {
		super();
		m_frame = vae;
		m_parent = ds;
		m_backColor = bg_color;
		m_transparentImage = transparency;
		
		m_drawing = false;
		m_oldPoint = new Point();
		
		setBackground(COLOR_CANVAS);
		setBorder(new MatteBorder(1,1,1,1,Color.black));
		setBounds(0,0,d.width,d.height);
		setOpaque(true);
		
		m_buffImg = new BufferedImage(d.width, d.height, 
						BufferedImage.TYPE_INT_ARGB);
		blankCanvas(bg_color);
		
		m_undoImg = new BufferedImage(d.width, d.height,
						BufferedImage.TYPE_INT_ARGB);
		backupImage();
		
		// Add mouse, motion listeners
		addMouseListener(createMouseAdapter());
		addMouseMotionListener(createMouseMotionAdapter());
	}
	
	/**
	 * Overwritten paintComponent function draws the stored image
	 * to the background surface.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		try {
			g.drawImage(m_buffImg, 0,0,this);
		}
		catch(NullPointerException ex) {
			System.err.println("Buffered Image is null");
		}
	}
	
	/**
	 * Backs up the primary image for undo functionality.
	 */
	public void backupImage(){
		clearCanvas(m_undoImg, Constants.COLOR_CLEAR );
		m_undoImg.createGraphics().drawImage(m_buffImg, 0, 0, null);
	}
	
	/**
	 * Restores previously backed up image.
	 */
	public void restoreImage(){
		big = new BufferedImage(m_buffImg.getWidth(), 
						m_buffImg.getHeight(), 
						BufferedImage.TYPE_INT_ARGB);
		big.createGraphics().drawImage(m_buffImg, 0, 0, null);
		clearCanvas(m_buffImg, Constants.COLOR_CLEAR );
		m_buffImg.createGraphics().drawImage(m_undoImg, 0, 0, null);
		repaint();
		clearCanvas(m_undoImg, Constants.COLOR_CLEAR);
		m_undoImg.createGraphics().drawImage(big, 0, 0, null);
	}
	
	/**
	 * Clears the primary canvas, setting every pixel to given color
	 * 
	 * @param color Color of canvas
	 */
	public void blankCanvas(Color color){
		BackgroundSurface.clearCanvas(m_buffImg, color);
	}
	
	/**
	 * Examines BufferedImage, turning every pixel into color.
	 * 
	 * @param bi
	 * @param color
	 */
	public static void clearCanvas(BufferedImage bi, Color color){
		int[] allWhite = new int[bi.getWidth()*bi.getHeight()];
		int white = color.getRGB();
		for(int i=0; i<allWhite.length; i++)
			allWhite[i]=white;
		
		bi.setRGB(0, 0, bi.getWidth(), 
				bi.getHeight(), allWhite, 0,1);
	}
	
	/**
	 * Changes every instance of oldColor in the bufferedImage to newColor.
	 * 
	 * @param oldColor Old color
	 * @param newColor New color
	 * @param bi BufferedImage
	 * @return New version of the bufferedImage.
	 */
	public static BufferedImage changeColor(int oldColor, int newColor, 
														BufferedImage bi)
	{
		BufferedImage result = new BufferedImage(bi.getWidth(), 
								bi.getHeight(), 
								BufferedImage.TYPE_INT_ARGB);
		for( int i = 0; i<bi.getWidth(); i++ ){
			for( int j=0; j<bi.getHeight(); j++) {
				int value = bi.getRGB(i, j);
				if( value == oldColor )
					result.setRGB(i, j, newColor);
				else
					result.setRGB(i, j, value);
			}
		}
		return result;
	}
	
	/**
	 * Changes every occurance of oldColor in the bounds of the
	 * two Points to newColor.  This is used as a helper for erasing.
	 * 
	 * @param oldColor Old color
	 * @param newColor New color
	 * @param corner1 One corner of rectangle
	 * @param corner2 Opposite corner
	 */
	private void changeColorInBounds(int oldColor, int newColor, 
								Point corner1, Point corner2)
	{
		int leftmost = Math.max(0,Math.min(corner1.x, corner2.x) - ERASOR_SIZE);
		int rightmost = Math.min(m_buffImg.getWidth()-1,
						Math.max(corner1.x, corner2.x) + ERASOR_SIZE);
		int topmost = Math.max(0,Math.min(corner1.y, corner2.y) - ERASOR_SIZE);
		int bottommost = Math.min(m_buffImg.getHeight()-1,
 						 Math.max(corner1.y, corner2.y) + ERASOR_SIZE);
		for( int i = leftmost; i<=rightmost; i++ ){
			for( int j=topmost; j<=bottommost; j++) {
				int value = m_buffImg.getRGB(i, j);
				if( value == oldColor )
					m_buffImg.setRGB(i, j, newColor);
			}
		}		
	}
	
	/**
	 * Creates the mouseAdapter, which supports all drawing operations.
	 * 
	 * @return MouseAdapter
	 */
	public MouseAdapter createMouseAdapter(){
		MouseAdapter ma = new MouseAdapter(){
			/**
			 * Two cases: mouse pressed with Brush selected, or
			 * with eraser selected.  In these two cases, pixels should change
			 * on the mouse press, in addition to during drag and on release.
			 * Plus, oldPoint and m_drawing should be set.
			 */
			public void mousePressed(MouseEvent e){
				CloverWindow.CURSOR_LOCK = true;
				backupImage();
				switch (ToolBox.DRAWING_MODE) {
					case BRUSH_INDEX:
						m_drawing = true;
						m_oldPoint = e.getPoint();
						m_parent.m_changed = true;
						
						// Also draw a spec centered at e.getPoint();
						Graphics g = m_buffImg.createGraphics();
						g.setColor(Tool.getForegroundColor());
						
						// Get upperleft corner
						int x = e.getPoint().x - BrushTool.STROKE_THICKNESS/2;
						int y = e.getPoint().y - BrushTool.STROKE_THICKNESS/2;
						
						g.fillRect(x, y, BrushTool.STROKE_THICKNESS, 
								BrushTool.STROKE_THICKNESS);
						
						g = null;		
					break;
				
					case ERASE_INDEX:
						m_drawing = true;
						m_oldPoint = e.getPoint();
						
						// Also erase a block centered at e.getPoint();
						g = m_buffImg.createGraphics();
						
						if( m_transparentImage )
							g.setColor(Constants.COLOR_CANVAS);	//Color.white
						else
							g.setColor(m_backColor);
						
							
						// Get upperleft corner
						x = Math.max(0,e.getPoint().x - ErasorTool.ERASE_SIZE/2);
						y = Math.max(0,e.getPoint().y - ErasorTool.ERASE_SIZE/2);
							
						g.fillRect(x, y, ErasorTool.ERASE_SIZE, 
										ErasorTool.ERASE_SIZE);
						
						// Erase paints the canvas color over existing pixels.
						// The canvas color is then made transparent (if
						// transparency is true) over the erased portion of
						// the image.  Hence, the canvas color on a transparent image
						// should be off-white and something that is not likely
						// to be used in the image.
						if( m_transparentImage )
							changeColorInBounds(	//Color.white.getRGB(), 
													Constants.COLOR_CANVAS.getRGB(),
													COLOR_CLEAR.getRGB(),
													new Point(x,y),
								new Point(x+ERASOR_SIZE,y+ERASOR_SIZE));
						BackgroundSurface.this.repaint();
					break;
			
					case FILL_INDEX:
						break;
				
					default:
						break;			
				}
				m_parent.stopUpdate();
			}// mousePressed()
				
			/**
			 * Repaint after drawing/erasing.
			 */
			public void mouseReleased(MouseEvent e){
				CloverWindow.CURSOR_LOCK = false;
				switch (ToolBox.DRAWING_MODE) {
				
					case BRUSH_INDEX:
						m_drawing = false;
						BackgroundSurface.this.repaint();
			
						// And update the thumbnail.
						m_parent.updateThumb();
					break;
				
					case ERASE_INDEX:
						m_drawing = false;
						BackgroundSurface.this.repaint();
			
						// And update the thumbnail.
						m_parent.updateThumb();
					break;
			
					case FILL_INDEX:
						break;
				
					default:
						break;			
				}	
			}// mouseReleased()
			
			/**
			 * Only check for fill tool.  If filling, then start the
			 * process for e.getPoint().
			 */
			public void mouseClicked(MouseEvent e){
				switch (ToolBox.DRAWING_MODE) {
					case FILL_INDEX:
						// Set cursor to timer?  And/or, perform in min thread?
						FillTool.initScanFill(e.getPoint(), 
									BackgroundSurface.this);
						m_parent.m_changed = true;
							
						// Repaint
						BackgroundSurface.this.repaint();
					
						// And update the thumbnail.
						m_parent.updateThumb();
					break;
				
					default:
						break;			
				}
			}// mouseClicked()
			
			/**
			 * Sets the cursor
			 */
			public void mouseEntered(MouseEvent e){
				if(cursorLock) {
					BackgroundSurface.this.setCursor(null);
					return;
				}
				switch (ToolBox.DRAWING_MODE) {
				
					// TODO: Unique cursors for ea. mode.  Select is ok...
					case BRUSH_INDEX:
						BackgroundSurface.this.setCursor(ToolBox.CURSOR_PAINT);
					break;
					case FILL_INDEX:
						BackgroundSurface.this.setCursor(ToolBox.CURSOR_FILL);
					break;
					case ERASE_INDEX:
						BackgroundSurface.this.setCursor(ToolBox.CURSOR_ERASE);
					break;		
					default:
						BackgroundSurface.this.setCursor(Cursor.
							getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
					break;			
				}
			}// mouseEntered()
			
			/**
			 * Resets the cursor
			 */
			public void mouseExited(MouseEvent e){
				if(!cursorLock)
					BackgroundSurface.this.setCursor(Cursor.getDefaultCursor());
			}
			
		};
		return ma;
	}
	
	/**
	 * Creates the mouseMotionAdapter, which supports all drawing operations.
	 * 
	 * @return MouseMotionAdapter
	 */
	public MouseMotionAdapter createMouseMotionAdapter(){
		MouseMotionAdapter mma = new MouseMotionAdapter(){
			/**
			 * If Brush is selected, draw a line from oldPoint
			 * to current point and update oldPoint.  If erasing,
			 * do the same with a different stroke, and follow
			 * with color changing.
			 */
			public void mouseDragged(MouseEvent e){
				switch (ToolBox.DRAWING_MODE) {
				
					case BRUSH_INDEX:
						Graphics2D g = m_buffImg.createGraphics();
						g.setColor(Tool.getForegroundColor());
						g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
							BRUSH_ANTI_ALIAS);
					
						g.setStroke(BrushTool.STROKE);
						g.drawLine(m_oldPoint.x, m_oldPoint.y, e.getX(), e.getY());
						
						g = null;
			
						int dx = m_oldPoint.x - e.getX();
						int dy = m_oldPoint.y - e.getY();
						
						if(e.getX() < minX)
							minX = e.getX();
						else if(e.getX() > maxX)
							maxX = e.getX();
							
						if(e.getY() < minY)
							minY = e.getY();
						else if(e.getY() > maxY)
							maxY = e.getY();
							
						m_oldPoint = e.getPoint();
			
						if( (dx < BrushTool.THRESHOLD && dx > -BrushTool.THRESHOLD)
							 && (dy < BrushTool.THRESHOLD && dy > -BrushTool.THRESHOLD) )
							BackgroundSurface.this.repaint();
					break;
				
					case ERASE_INDEX:
						g = m_buffImg.createGraphics();

						g.setColor(m_backColor);

						g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
							RenderingHints.VALUE_ANTIALIAS_OFF);

						g.setStroke(ErasorTool.STROKE);
						g.drawLine(m_oldPoint.x, m_oldPoint.y, e.getX(), e.getY());
						
						if( m_transparentImage )
							changeColorInBounds(
									Constants.COLOR_CANVAS.getRGB(), 
									COLOR_CLEAR.getRGB(),
									m_oldPoint, e.getPoint());
			
						dx = m_oldPoint.x - e.getX();
						dy = m_oldPoint.y - e.getY();
					
						m_oldPoint = e.getPoint();		
			
						if( (dx < ErasorTool.THRESHOLD && dx > -ErasorTool.THRESHOLD) 
							&& (dy < ErasorTool.THRESHOLD && dy > -ErasorTool.THRESHOLD) )
							BackgroundSurface.this.repaint();
					break;
			
					case FILL_INDEX:
						break;
				
					default:
						break;			
				}
			}// mouseDragged()
			
			public void mouseMoved(MouseEvent e){
				if( m_drawing ) {
					BackgroundSurface.this.repaint();
					m_drawing = false;
				}	
			}
		};
		return mma;
	}
	
}