/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ActionGrid;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.*;


/**
 * ActionButton Class:
 *  Implements draggable buttons for dialog table.  These are the
 *  buttons that display emotion, speech or thought bubble, and 
 *  some text.
 * 
 * @author Terry Bradley
 * @version 04.7.30
 *
 */
public class ActionButton extends JPanel {
	
	protected int PRIMARY_ACTION;		// Button's primary action
	protected int SECONDARY_ACTION;		// Button's secondary action
	protected int m_emotion = -1;		// Emotion index
	protected JPanel m_panel;			// Copy of this button that
										// appears on the glass pane
										// for drag and drop.
	protected JLabel m_prim;			// Label to display primary icon
	protected JLabel m_sec;				// Label to display sec. icon
	protected JLabel m_text;			// Label to display text
	protected Rectangle m_labelSize;	// Rect stores size of label
	
	protected boolean m_isNew;		// if true then create new / edit
									// should be true for menubar
									// if false then move w/in table
									// should be false for table icons
	
	protected int m_row	=	-1;		// Table location
	protected int m_col	=	-1;		// Table location
	protected CloverWindow m_gpOwner;
	
	// CURRENTLY IN USE by CharacterDialogButton to show non-targets for dnd.
	public static ImageIcon BIG_X = Clover.createImageIcon2(
			Clover.CURSOR_DIR+"invalid.gif");
	
	public static Border UP 	= new SoftBevelBorder(SoftBevelBorder.RAISED);
	public static Border DOWN	= new EmptyBorder(0,0,0,0);
	
	private Point start = new Point();
	
	private static final int LABEL_GAP = 5;		// Needed to size labels
	
	/**
	 * Constructor - Creates action button with the given parameters.
	 * 
	 * @param primAction Primary action
	 * @param prim Primary icon
	 * @param secAction Secondary action
	 * @param sec Secondary icon
	 * @param text Text
	 * @param isNew True iff this button is non-editable
	 * @param owner CloverWindow main frame
	 */
	public ActionButton(int primAction, ImageIcon prim,
					    int secAction,  ImageIcon sec,
					    String text, boolean isNew,
					    CloverWindow owner)
	{
		super();
		
		setAlignmentX(Component.CENTER_ALIGNMENT);
		m_panel = new JPanel();
		m_panel.setVisible(false);
		m_panel.setOpaque(true);
		m_panel.setAlignmentX(Component.CENTER_ALIGNMENT);
		m_panel.setBorder(UP);
		
		if( isNew )
			setBorder(new EmptyBorder(4,4,4,4));
		else 
			setBorder(new EmptyBorder(0,0,0,0));

		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		if(prim == null)
			m_prim = new JLabel();
		else
			m_prim = new JLabel(prim);
		if(sec == null)
			m_sec = new JLabel();
		else
			m_sec  = new JLabel(sec);
		m_text = new JLabel(text);
		
		c.anchor = GridBagConstraints.SOUTHEAST;
		add(m_prim, c);
		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridy = 1;
		add(m_sec, c);
		c.gridx = 1;
		c.gridy = 0;
		c.gridheight = 2;
		c.anchor = GridBagConstraints.CENTER;
		add(m_text, c);
		
		copy();
		
		((DragGlassPane)owner.getGlassPane()).add(m_panel);
		
		PRIMARY_ACTION = primAction;
		SECONDARY_ACTION = secAction;
		m_isNew = isNew;
		m_gpOwner = owner;
		
		addMouseListener(createMouseAdapter());
		addMouseMotionListener(createMouseMotionAdapter());
	}
	
	/**
	 * Sets the text in the action button (to match dialog, for example) 
	 * 
	 * @param s New text
	 */
	public void setText(String s){
		m_text.setText(s);
	}
	
	/**
	 * Sets the emotion of the action button
	 * 
	 * @param emo Emotion index
	 */
	public void setEmotion(int emo){
		m_emotion = emo;
	}
	
	/**
	 * Sets the primary icon of the action button.
	 * 
	 * @param i Primary icon
	 * @param action Primary action
	 */
	public void setPrimaryIcon(ImageIcon i, int action){
		m_prim.setIcon(i);

		copy();

		PRIMARY_ACTION = action;
	}
	
	/**
	 * Sets the secondary icon of the action button
	 * 
	 * @param i Secondary icon
	 * @param action Secondary action
	 */
	public void setSecondaryIcon(ImageIcon i, int action){
		m_sec.setIcon(i);
		SECONDARY_ACTION = action;
	}
	
	/**
	 * Enables or disables this button - used to disable new buttons
	 * when no characters appear in a scene.
	 * 
	 * @param b Enable status
	 */
	public void setEnabled(boolean b){
		super.setEnabled(b);
		m_prim.setEnabled(b);
		m_sec.setEnabled(b);
		m_text.setEnabled(b);		
	}
	
	/**
	 * Gets action index
	 * 
	 * @return Action
	 */
	public int getAction(){
		return PRIMARY_ACTION + SECONDARY_ACTION;
	}
	
	/**
	 * Alters action button for a picked up / dragging effect
	 *
	 */
	public void pickupButton(){
		m_panel.setBackground(ActionButton.this.getBackground());
		m_panel.setBorder(UP);
	}
	
	/**
	 * Alters action button for a dropped effect
	 *
	 */
	public void dropButton(){
		m_panel.setBackground(Constants.COLOR_TABLE);
		m_panel.setBorder(DOWN);
	}
	
	/**
	 * Sets m_panel to contain everything, rather than this
	 *
	 */
	public void copy(){
		m_panel.removeAll();
			
		m_panel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints(); 
		
		JLabel p = new JLabel();
		p.setIcon(m_prim.getIcon());
		if(p.getIcon() != null)
			p.setSize(p.getIcon().getIconWidth(), p.getIcon().getIconHeight());
		JLabel s = new JLabel();
		s.setIcon(m_sec.getIcon());
		if(s.getIcon() != null)
			s.setSize(s.getIcon().getIconWidth(), s.getIcon().getIconHeight());
		
		c.anchor = GridBagConstraints.SOUTHEAST;
		m_panel.add(p, c);
		c.anchor = GridBagConstraints.NORTHWEST;
		c.gridy = 1;
		m_panel.add(s, c);

		if(p.getIcon() != null && s.getIcon() != null)
			m_panel.setSize(Math.max(p.getIcon().getIconWidth(),
									 s.getIcon().getIconWidth()),
							p.getIcon().getIconHeight() +
							s.getIcon().getIconHeight());
		else if(s.getIcon() != null)
			m_panel.setSize(s.getSize());
						
		m_labelSize = new Rectangle( m_panel.getWidth() + LABEL_GAP, 
					m_panel.getHeight() + LABEL_GAP);
	}
	
	
	public MouseAdapter createMouseAdapter(){
		
		ActionListener act = new ActionListener() {
			public void actionPerformed(ActionEvent e){
				// Edit action when this function runs
				// Only run the timer if !isNew
				// On update, edit action
				if(m_gpOwner.getSelectedTab() == Constants.VIEW_AND_EDIT_INDEX) {
					((ViewAndEditUI)m_gpOwner.getInterface(
							Constants.VIEW_AND_EDIT_INDEX)).
							editAction(m_row, m_col, 
									   PRIMARY_ACTION, start, m_emotion);							
					m_gpOwner.getGlassPane().setVisible(false);
					m_panel.setVisible(false);
					((ViewAndEditUI)m_gpOwner.getInterface(
							Constants.VIEW_AND_EDIT_INDEX)).
							de.m_table.getColumnModel().
								getColumn(m_col).getCellEditor().stopCellEditing();
				}
				else if( m_gpOwner.getSelectedTab() == Constants.RESPONSE_INDEX){
					((ResponseUI)m_gpOwner.getInterface(Constants.RESPONSE_INDEX)).
						editAction(m_row, m_col, PRIMARY_ACTION, start, m_emotion);							
					m_gpOwner.getGlassPane().setVisible(false);
					m_panel.setVisible(false);
					((ResponseUI)m_gpOwner.getInterface(Constants.RESPONSE_INDEX)).
						de.m_table.getColumnModel().getColumn(m_col).
							getCellEditor().stopCellEditing();
				}
					
			}
		};

		
		MouseAdapter ma = new MouseAdapter() {
			
			public void mousePressed(MouseEvent e) {
				if( e.getClickCount() > 1 ) return;
				
				ActionButton jb = (ActionButton)e.getSource();
				DragGlassPane glassPane = (DragGlassPane)m_gpOwner.getGlassPane();
				
				if( !jb.isEnabled() ) {
					return;
				}
				
				// Set this label to be visible
				pickupButton();
				m_panel.setVisible(true);				
				glassPane.setVisible(true);
				if(!CloverWindow.CURSOR_LOCK){
					glassPane.setCursor(
						Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					CloverWindow.CURSOR_LOCK = true;
				}
				
				// Move label to overlap this
				Point glassPoint = SwingUtilities.convertPoint(
											jb,
											e.getPoint(),
											glassPane);
				
				// Center cursor in label
				glassPoint.x -= m_labelSize.width / 2;
				glassPoint.y -= m_labelSize.height / 2;
				
				m_panel.setLocation(glassPoint);
				m_panel.setBounds(glassPoint.x, glassPoint.y, 
					m_labelSize.width, m_labelSize.height);
				glassPane.setPoint(glassPoint);
				
				// Check for action movement.  If move action, need to remember
				// initial row and column.
				if( !m_isNew ) {
					start = glassPoint;
					//clickAndHold.start();
					Component component =
							SwingUtilities.getDeepestComponentAt(
													m_gpOwner.getContentPane(),
													glassPoint.x,
													glassPoint.y);
					if ((component != null)
							&& (component instanceof JTable)) {
						
						// Damn well better be m_table
						JTable table = (JTable)component;		
						// table == m_table OR m_froz in de
						Point tablePoint = SwingUtilities.convertPoint(
															   glassPane,
															   glassPoint,
															   table);
						m_row = table.rowAtPoint(tablePoint);
						m_col = table.columnAtPoint(tablePoint);
												
					}		
				}	
			}//mousePressed
			
			public void mouseReleased(MouseEvent e) {
				//if(clickAndHold.isRunning())
				//	clickAndHold.stop();
				if( e.getClickCount() > 1 ) return;
				
				int m_tab = m_gpOwner.getSelectedTab();
				
				dropButton();
				
				ActionButton jb = (ActionButton)e.getSource();
				Component glassPane = m_gpOwner.getGlassPane();
				glassPane.setCursor(Cursor.getDefaultCursor());
				CloverWindow.CURSOR_LOCK = false;
				Point glassPoint = SwingUtilities.convertPoint(
											jb,
											e.getPoint(),
											glassPane);
				Component component =
							SwingUtilities.getDeepestComponentAt(
													m_gpOwner.getContentPane(),
													glassPoint.x,
													glassPoint.y);
				if ((component != null)
							&& (component instanceof JTable)) {
					
					JTable table = (JTable)component;		
					// table == m_table OR m_froz in de
					Point tablePoint = SwingUtilities.convertPoint(
														   glassPane,
														   glassPoint,
														   table);
					int row = table.rowAtPoint(tablePoint);
					int col = table.columnAtPoint(tablePoint);
					
					//((DialogEntry)table.getParent().getParent()).selectCell(row,col);
					
					if( table.getColumnCount() == 1 && m_isNew ) { 
						// table == m_frozen
						System.out.println("Dialog should appear for next open time");
						// Find next open time...somehow
						col = ((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
							getNextOpenTime(row, PRIMARY_ACTION);
						((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
							editAction(row, col, PRIMARY_ACTION,
									   glassPoint, m_emotion);
					}
					else if ( table.getColumnCount() > 1 && m_isNew ){
						// table == m_table, create / edit action	
						((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
							editAction(row, col,
									   PRIMARY_ACTION, glassPoint, m_emotion);
					}
					else if ( table.getColumnCount() > 1 && !m_isNew ) {
						// table == m_table, move existing action
						if( m_row != row || m_col != col )
							((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
								moveAction(m_row, m_col, row, col);
					}
				}// component is jtable
				
				else if ((component != null) && m_isNew
						&& (component instanceof CharacterLabel)) {
					// Same as table.getColumnCount() == 1 && m_isNew
					CharacterLabel cl = (CharacterLabel)component;
					int row = ((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
						getCharacterRow(cl.getCharacterID());
					int col = ((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
						getNextOpenTime(row, PRIMARY_ACTION);
					((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
						editAction(row, col, PRIMARY_ACTION,glassPoint, m_emotion);
				}
				else if ((component != null) && !m_isNew
						&& (component instanceof TrashLabel)) {	
					// Drag to trash
					// Delete action at m_row, m_col
					((ViewAndEditUI)m_gpOwner.getInterface(m_tab)).
						removeAction(m_row, m_col);
				}
					
				m_gpOwner.getGlassPane().setVisible(false);
				m_panel.setVisible(false);
			}//mouseReleased
		
			public void mouseClicked(MouseEvent e) {
				if( e.getClickCount() < 2 || m_isNew ) return;
				
				// Double clicking existing action.  Need to get row
				// and column and run editAction.
				ActionButton jb = (ActionButton)e.getSource();
				Component glassPane = m_gpOwner.getGlassPane();
				Point glassPoint = SwingUtilities.convertPoint(
											jb,
											e.getPoint(),
											glassPane);


				((ViewAndEditUI)m_gpOwner.getInterface(m_gpOwner.getSelectedTab())).
						editAction(PRIMARY_ACTION, glassPoint, m_emotion);
				
			}// mouseClicked
			
			public void mouseEntered(MouseEvent e) {
				if(!CloverWindow.CURSOR_LOCK) {
					setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
			}
			
			public void mouseExited(MouseEvent e) {
				if(!CloverWindow.CURSOR_LOCK)
					setCursor(Cursor.getDefaultCursor());
			}
			
		};
		return ma;
	}
	
	public MouseMotionAdapter createMouseMotionAdapter(){
		MouseMotionAdapter mma = new MouseMotionAdapter() {
			
			public void mouseDragged(MouseEvent e) {
				DragGlassPane glassPane = (DragGlassPane)m_gpOwner.getGlassPane();
				ActionButton jb = (ActionButton)e.getSource();
				Point glassPoint = SwingUtilities.convertPoint(
											jb,
											e.getPoint(),
											glassPane);
				Component component =
							SwingUtilities.getDeepestComponentAt(
												m_gpOwner.getContentPane(),
												glassPoint.x,
												glassPoint.y);
												
				if( component instanceof JTable || 
					((component instanceof CharacterLabel) && m_isNew) ||
					((component instanceof JLabel) && !m_isNew) ) {
					
				}
				
				// Center cursor in label
				glassPoint.x -= m_panel.getWidth() / 2;
				glassPoint.y -= m_panel.getHeight() / 2;
				
				m_panel.setLocation(glassPoint);
				glassPane.setPoint(glassPoint);
			}
			
		};
		return mma;	
	}
	
}

