/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ActionGrid;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.*;

import edu.uiuc.orchid.clover.Interface.Tabs.ViewAndEditUI;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.*;

/**
 * DialogEntry == Action Grid. This sets up a JTable for dialog 
 * and action entry in the scene editor.  The table is composed of
 * two separate tables - one for the first, frozen column, and the 
 * second for the rest of the table.  The table just deals with
 * Longs and Integers, which are then interpretted to display the
 * appropriate visuals - see DialogTableData and DialogTableRenderer
 * for more details. 
 * 
 * @author Terry Bradley
 * @version 04.7.31
 * @date 9/23/03
 *
 */
public class DialogEntry extends JScrollPane /*implements Scrollable*/ {

	public JTable m_table;				// m_table and m_frozenCol should
	protected JTable m_frozenCol;		// share a common tableColModel (dtd)
	protected ViewAndEditUI m_parent;
	
	private Point m_start;				// Start point on drag and drop
	private boolean m_dragging;			// True when dragging a button
	

	/**
	 * Constructor - Creates a table with the given parent.
	 * 
	 * @param own Editor parent
	 */
	public DialogEntry(ViewAndEditUI own) {
		super();
		m_parent = own;
		m_start = new Point();
		m_dragging = false;
		m_table = new JTable();
		m_frozenCol = new JTable();
		
		// Setup scroll pane
		setPreferredSize(new Dimension(0,Constants.DIALOG_HEIGHT));
		setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		setAutoscrolls(true);
		getViewport().setBackground(Constants.COLOR_LIST);
		
		// Setup main table
		m_table.setAutoCreateColumnsFromModel(false);
		m_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		m_table.setRowHeight(Constants.ROW_HEIGHT);
		m_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// Setup frozen column table
		m_frozenCol.setAutoCreateColumnsFromModel(false);
		m_frozenCol.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		m_frozenCol.setRowHeight(Constants.ROW_HEIGHT);
		m_frozenCol.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// Setup appearances
		m_table.setBackground(Constants.COLOR_TABLE);
		m_frozenCol.setBackground(getBackground());
		m_table.setSelectionBackground(Constants.COLOR_TABLE_SELECTION);
		m_frozenCol.setSelectionBackground(Constants.COLOR_TABLE_SELECTION);
		
		// Setup headers
		JTableHeader jth = m_table.getTableHeader();
		jth.setReorderingAllowed(false);
		jth.setDefaultRenderer(createHeaderRenderer());
		jth = m_frozenCol.getTableHeader();
		jth.setDefaultRenderer(createHeaderRenderer());
		setCorner(JScrollPane.UPPER_LEFT_CORNER, jth);	// Necessary for first header
		
		// Add mouse functionality
		m_table.addMouseListener(createMouseAdapter());
		m_frozenCol.addMouseListener(createSelectionListener());
		
		getViewport().add(m_table);
		setRowHeaderView(m_frozenCol);
		// This is the most important PreferredSize
		getRowHeader().setPreferredSize(new Dimension(Constants.COLUMN_WIDTH, 
			new Double(getPreferredSize().getHeight()).intValue()));
	}
	
	/**
	 * Adds a column to the action grid at given index.  Columns at index 0 are
	 * added to the frozen column.  All others appear in the main table.  Both
	 * use DialogTableRenderer to display their values.
	 * 
	 * @param colIndex Column index for new column
	 * @param sty Scene
	 */
	public void addColumn(int colIndex, Scene sty) {
		TableColumn tc; 
		if( colIndex == 0 ) {
			tc = new TableColumn(colIndex, Constants.COLUMN_WIDTH, 
				new DialogTableRenderer(sty, true), null);
			tc.setMinWidth(Constants.COLUMN_WIDTH);
			tc.setMaxWidth(Constants.COLUMN_WIDTH);
			m_frozenCol.addColumn(tc);
			addEmptyCol();
		}
		else {
			tc = new TableColumn(colIndex, Constants.COLUMN_WIDTH, 
				new DialogTableRenderer(sty, false), null);
			tc.setCellEditor(new DialogTableEditor(m_parent.getWindow()));
			tc.setMinWidth(Constants.COLUMN_WIDTH);
			tc.setMaxWidth(Constants.COLUMN_WIDTH);
			m_table.addColumn(tc);
		}
	}
	
	/**
	 * Adds an empty column with no width to the main table.  This is 
	 * used as column 0, so that the first visible column in the table
	 * matches the first header, "1"
	 *
	 */
	private void addEmptyCol(){
		TableColumn tc = new TableColumn(0, 0, null, null);
		tc.setMinWidth(0);
		tc.setMaxWidth(0);
		m_table.addColumn(tc);
	}
	
	/**
	 * Gets the row in the maintable for given point
	 * 
	 * @param p Point
	 * @return Row containing point
	 */
	public int getRowAtPoint(Point p){
		return m_table.rowAtPoint(p);
	}
	
	/**
	 * Gets the col in the main table for the given point
	 * 
	 * @param p Point
	 * @return Column containing point
	 */
	public int getColAtPoint(Point p){
		return m_table.columnAtPoint(p);
	}
	
	/**
	 * Gets the value at the given location.  If col = 0, the frozen
	 * column table is used - otherwise, the main table is used.
	 * 
	 * @param row Row
	 * @param col Col
	 * @return Value at (row, col)
	 */
	public Object getValueAt(int row, int col){
		if(col > 0)
			return m_table.getValueAt(row, col);
		else
			return m_frozenCol.getValueAt(row, col);
	}
	
	/**
	 * Gets the main table
	 * 
	 * @return Main table
	 */
	public JTable getMainTable(){
		return m_table;
	}
	
	/**
	 * Sets the data model for both tables (they share)
	 * 
	 * @param dtd Data model
	 */
	public void setModel(DialogTableData dtd) {
		m_table.setModel(dtd);
		m_frozenCol.setModel(dtd);
	}
	
	/**
	 * Gets the character corresponding to the given row
	 * 
	 * @param row Row
	 * @return Character's ID
	 */
	public int getGIDByRow(int row){
		Long gid = new Long(-1);
		try {
			gid = (Long)m_frozenCol.getModel().getValueAt(row, 0);
		}
		catch (Exception e){}
		return gid.intValue();		
	}
	
	/**
	 * Selects a cell and adjusts both tables to properly display the cell.
	 * 
	 * @param row Row
	 * @param col Column
	 */
	public void selectCell(int row, int col){
		if( col == 0 ) {
			m_frozenCol.setRowSelectionInterval(row,row);
			m_frozenCol.setColumnSelectionInterval(0,0);
		}
		else {
			m_table.setRowSelectionInterval(row,row);
			m_frozenCol.setRowSelectionInterval(row,row);
			m_table.setColumnSelectionInterval(col,col);
		}
		m_table.scrollRectToVisible(m_table.getCellRect(row, 1, true));
	}
	
	/**
	 * Unselects table
	 *
	 */
	public void clearSelection(){
		m_table.clearSelection();
	}
	
	/**
	 * Unselects frozen column
	 *
	 */
	public void clearCharacterSelection(){
		m_frozenCol.clearSelection();
	}
	
	/**
	 * Removes all columns from the Action Grid.
	 *
	 */
	public void clearTable(){
		int i;
		for( i=m_table.getColumnCount()-1; i>=0; i-- ) {
			m_table.removeColumn(m_table.getColumnModel().getColumn(i));
		}
		for( i=m_frozenCol.getColumnCount()-1; i>=0; i-- ) {
			m_frozenCol.removeColumn(m_frozenCol.getColumnModel().getColumn(i));
		}
	}
	
	/**
	 * Implements, inline, a custom renderer for the header row.
	 * This prints the string value of Value and sets up
	 * borders as needed.
	 * 
	 * @return Instance of renderer
	 */
	private TableCellRenderer createHeaderRenderer(){
		DefaultTableCellRenderer label = new DefaultTableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column)
			{
				if( table != null ) {
					JTableHeader header = table.getTableHeader();
					if( header != null) {
						setForeground(header.getForeground());
						setBackground(header.getBackground());
						setFont(header.getFont());
					}
				}
				setText((value==null) ? "" : value.toString());
				setBorder(UIManager.getBorder("TableHeader.cellBorder"));
				return this;
			}
		};
		label.setHorizontalAlignment(JLabel.CENTER);
		return label;
	}
	
	/**
	 * Creates a MouseAdapter for the main table
	 * 
	 * @return
	 */
	private MouseAdapter createMouseAdapter(){
		MouseAdapter ma = new MouseAdapter() {
			
			int col, row, value;
			
			public void mousePressed(MouseEvent e) {
				col = DialogEntry.this.m_table.columnAtPoint(e.getPoint());
				if( col <= 0 || e.getButton() != MouseEvent.BUTTON1 )
					return;
					
				// get value at point: if 0, exit; otherwise, begin
				// drag and drop
				row = DialogEntry.this.m_table.rowAtPoint(e.getPoint());
				try {
					value = ((Integer)DialogEntry.this.m_table.
						getValueAt(row, col)).intValue();
				}
				catch (Exception ex) {
					value = 0;
				}
				
				if(value > 0){
					m_start.x = e.getPoint().x;
					m_start.y = e.getPoint().y;
					m_dragging = true;
					//m_parent.m_parent.setCursor(DrawingSurface.MOVE_CURSOR);
					if(!CloverWindow.CURSOR_LOCK){
						m_parent.setCursor(CloverWindow.MOVE_CURSOR);
						CloverWindow.CURSOR_LOCK = true;
					}
					((DialogTableRenderer)m_table.getCellRenderer(row, col)).setEnabled(false);
					DialogEntry.this.repaint();
				}
				else
					m_dragging = false;
					
				
			}

			public void mouseReleased(MouseEvent e) {
				m_parent.setCursor(Cursor.getDefaultCursor());
				CloverWindow.CURSOR_LOCK = false;

				if(!m_dragging)
					return;
				// Need to get drop target
				Point cwPoint = SwingUtilities.convertPoint(
												(Component)e.getSource(),
												e.getPoint(),
												m_parent.getWindow().getContentPane());
				
				Component component =
						SwingUtilities.getDeepestComponentAt(
								m_parent.getWindow().getContentPane(),
								cwPoint.x,
								cwPoint.y);
								
				Point loc = DialogEntry.this.m_table.getLocation();
				SwingUtilities.convertPointToScreen(loc, DialogEntry.this);
				//System.out.println("Location of good table :" + loc.toString());
				//System.out.println("Drop Point: " + cwPoint.toString());
								
				if(component != null){
					if(	component instanceof JTable ){
						// Damn well better be m_table
						JTable table = (JTable)component;		
						// table == m_table OR m_froz in de
						
						//Point tablePoint = SwingUtilities.convertPoint(
						//					m_parent.m_parent.getContentPane(),
						//					cwPoint,
						//					table.getParent());
						
						int nrow = table.rowAtPoint(e.getPoint());
						int ncol = table.columnAtPoint(e.getPoint());
						
						if( nrow != row || ncol != col )
							m_parent.moveAction(row, col, nrow, ncol);	
						//else {
						((DialogTableRenderer)m_table.getCellRenderer(row, col)).setEnabled(true);
						DialogEntry.this.repaint();
						//}
							
					}
					else if( component instanceof TrashLabel ){
						m_parent.removeAction(row, col);
					}
				}
			}
			
			public void mouseClicked(MouseEvent e) {
				int nrow = DialogEntry.this.m_table.getSelectedRow();

				// w/ 1 or more clicks, select row header and clabel
				DialogEntry.this.m_frozenCol.
					setRowSelectionInterval(nrow,nrow);
				DialogEntry.this.m_parent.selectCharacter(
					((Long)m_frozenCol.getModel().
					getValueAt(nrow,0)).intValue());
					
				if( e.getClickCount() >= 2 || 
					e.getButton() == MouseEvent.BUTTON3 )
				{
					col = DialogEntry.this.m_table.columnAtPoint(e.getPoint());
					row = DialogEntry.this.m_table.rowAtPoint(e.getPoint());
					Integer val = (Integer)DialogEntry.this.m_table.getValueAt(row, col);
					int action = -1;
					if( (val.intValue() & Constants.SPEAK_OFFSET) != 0 )
						action = Constants.SPEAK_OFFSET;
					else if( (val.intValue() & Constants.THINK_OFFSET) != 0)
						action = Constants.THINK_OFFSET;
					else if( (val.intValue() & Constants.EMOTE_OFFSET) != 0)
						action = Constants.EMOTE_OFFSET;
					m_parent.editAction(row, col, action, new Point(0,0), 
						val.intValue() / Constants.EMOTION_DIVIDER);
					m_dragging = false;
				}
			}
		};
		return ma;
	}
	

	/**
	 * Creates a mouseAdapter for the frozen column
	 * 
	 * @return
	 */
	private MouseAdapter createSelectionListener(){
		final Point start = new Point();
		final DialogEntry de = this;
		
		MouseAdapter ma = new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				// Get row
				int row = de.m_frozenCol.getSelectedRow();
				
				// Adjust m_table as needed
				de.m_table.scrollRectToVisible(de.m_table.getCellRect(row, 1, true));
				
				// Select CLabel, if needed
				de.m_parent.selectCharacter(
					((Long)m_frozenCol.getModel().getValueAt(row,0)).intValue());
			}
		};
		return ma;
	}
	
	/**
	 * Creates a mouseAdapter that uses clickAndHold to initiate editing.
	 * 
	 * @return
	 */
	private MouseAdapter createClickHoldMouseAdapter(){
		
		final Point start = new Point();
		final DialogEntry de = this;
		
		ActionListener act = new ActionListener() {
			public void actionPerformed(ActionEvent e){
				// Make sure mouse hasn't wandered
				
				// Get row and column info
				int row = de.m_table.getSelectedRow();
				int col = de.m_table.getSelectedColumn();
				
				// Check for first column
				if( col <= 0 ) return;
				
				// Get table value
				int value;
				try {
					value = ((Integer)de.m_table.getValueAt(row, col)).
							intValue();
				}
				catch (Exception ex) {
					value = 0;
				}
				
				// Update location
				Point loc = new Point(
							start.x + de.getLocationOnScreen().x,
							start.y + de.getLocationOnScreen().y );	
			}
		};
		
		final Timer t = new Timer(Constants.POPUP_DELAY, act);
		t.setRepeats(false);
				
		MouseAdapter ma = new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if(t.isRunning())
					t.stop();
				start.x = e.getPoint().x;
				start.y = e.getPoint().y;
				if( de.m_table.columnAtPoint(e.getPoint()) <= 0 ) return;
				t.start();
			}

			public void mouseReleased(MouseEvent e) {
				t.stop();
			}
			
			public void mouseClicked(MouseEvent e) {
				int row = de.m_table.getSelectedRow();
				if( e.getClickCount() > 1 )	{
					// Get row and column info
					int col = de.m_table.columnAtPoint(e.getPoint());
					
					// Check for first column
					if( col <= 0 ) return;
				
					// Get table value
					int value;
					try {
						value = ((Integer)de.m_table.getValueAt(row, col)).
								intValue();
					}
					catch (Exception ex) {
						value = 0;
					}
				
					// Update location
					Point loc = new Point(
								start.x + de.getLocationOnScreen().x,
								start.y + de.getLocationOnScreen().y );	
				}
				// w/ 1 or more clicks, select row header and clabel
				de.m_frozenCol.setRowSelectionInterval(row,row);
				de.m_parent.selectCharacter(
					((Long)m_frozenCol.getModel().getValueAt(row,0)).intValue());
			}
		};
		return ma;
	}
}




