/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.DrawingTools.*;
import edu.uiuc.orchid.clover.Interface.TrashLabel;

/**
 * CharacterButton Class
 * Another draggable button used in CloverLists.  This particular
 * button is found in the Gallery in CharacterUI.
 * 
 * @author Terry J Bradley
 * @version 04.8.01
 * 
 */
public class CharacterButton extends CloverListButton {
	
	protected CharacterUI m_frame;		// Character editor
	protected String m_name;			// Character name
	
	/**
	 * Constructor - Adds character editor to the CloverListButton
	 * constructor.
	 * 
	 * @param img Character thumbnail
	 * @param name Character name
	 * @param frm Character editor
	 * @param id Character ID
	 * @param par CloverList parent
	 * @param normal Normal status
	 */
	public CharacterButton(ImageIcon img, String name, CharacterUI frm,
							int id, CloverList par, boolean normal)
	{
		super(img, name, id, par, normal);
		m_frame = frm;
		m_name = name;
		
		addMouseListener(createMouseAdapter());
		addMouseMotionListener(createMouseMotionAdapter());
	}
	
	/**
	 * Sets the character name, using the parent setText method
	 * to format the new name.
	 * 
	 * @param name New character name.
	 */
	public void setCharacterName(String name){
		m_name = name;
		setText(name);
	}
	
	/**
	 * Creates a mouseAdapter that listens for buttons dropped on
	 * the Character editor or the trash label.  The character is
	 * opened for editing or deleted based on the drop target.
	 * 
	 * @return MouseAdapter
	 */
	private MouseAdapter createMouseAdapter(){
		MouseAdapter ma = new MouseAdapter(){
			public void mousePressed(MouseEvent e){
				startPoint = e.getPoint();

				m_parent.unselectAll();
				setSelected(true);
				m_parent.getCloverGlassPane().
					setCursor(CloverWindow.MOVE_CURSOR);
				setCursor(CloverWindow.MOVE_CURSOR);
				m_parent.getCloverGlassPane().setVisible(true);
				BackgroundSurface.cursorLock = true;
			}
			public void mouseReleased(MouseEvent e){
				CharacterButton jb = (CharacterButton)e.getSource();
				Component glassPane = m_parent.getCloverGlassPane();
				
				if( m_dragging ){
					Point glassPoint = SwingUtilities.convertPoint(
												jb,
												e.getPoint(),
												glassPane);
					Component component =
								SwingUtilities.getDeepestComponentAt(
											m_parent.getCloverContentPane(),
											glassPoint.x,
											glassPoint.y);
					if( component == null )
						return;
					if( !m_isNew ) {
						if( component instanceof BackgroundSurface )
							// Open character
							m_frame.editCharacter(m_id, true);
						else if( component instanceof TrashLabel)  {
							// Delete Character
							m_frame.m_parent.deleteCharacter(m_id);
						}
						else
							// Restore Character
							m_frame.editCharacter(m_id, true);
					}
					else {
						if( component instanceof BackgroundSurface) {
							// Create Character
							m_frame.createCharacter();
							
						}
						else {
							// Restore Character
							m_frame.createCharacter();
						}
					}
				} // was dragging
				else {	// normal click
					if( m_isNew )
						m_frame.createCharacter();
					else
						m_frame.editCharacter(m_id, true);
				}
				m_dragging = false;
				BackgroundSurface.cursorLock = false;
				glassPane.setCursor(Cursor.getDefaultCursor());
				glassPane.setVisible(false);
				setCursor(Cursor.getDefaultCursor());
			}
			public void mouseClicked(MouseEvent e){}
			public void mouseExited(MouseEvent e){
				if(!m_dragging)
					setCursor(Cursor.getDefaultCursor());
			}
			public void mouseEntered(MouseEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
		};
		return ma;
	}
	
	/**
	 * Simple mouseMotionAdapter listens for dragging and sets
	 * the cursor to the MOVE_CURSOR.
	 * 
	 * @return
	 */
	private MouseMotionAdapter createMouseMotionAdapter(){
		MouseMotionAdapter mma = new MouseMotionAdapter(){
			public void mouseDragged(MouseEvent e){
				m_dragging = true;
				m_parent.getCloverGlassPane().setCursor(CloverWindow.MOVE_CURSOR);
			}
		};
		return mma;
	}
	
}