/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.DragGlassPane;

/**
 * CloverList:
 * 	Custom widget, very similar to JList but with fewer restrictions
 *  and slighty more predictable/manageable behavior.  Used with various
 *  CloverListButton extensions to properly handle characters and
 *  scenes.
 * 
 * @author Terry J Bradley
 * @version 04.8.01
 *
 */
public class CloverList extends JScrollPane {
	
	private JPanel m_pane;			// Primary container
	private JPanel top;				// Contains all buttons
	private JButton m_addButton;	// Button used for adding character/scenes
	protected CloverWindow m_parent;// Parent
	protected Hashtable m_data;		// Hashtable of CListButtons - actual
									// contents of the list.
	
	/**
	 * The m_specialButton was used as an alternate way to add characters.
	 * With this enabled, the first item in every CloverList will be
	 * labeled as "New Character" or "Add Character."  Clicking this button
	 * or dropping it on the canvas would create a new character/scene.
	 * The traditional addButton does not appear when using specailButtons.
	 */
	private CloverListButton m_specialButton = null;
	
	protected boolean m_ordered;	// true iff list is ordered
	protected boolean m_editable;	// true iff list is editable.
									// AN EDITABLE LIST WILL USE THE
									// SPECIAL BUTTON ABOVE INSTEAD OF
									// THE ADD BUTTON.
	protected int m_selection = -1; // current selection
		
	static final int DEFAULT_WIDTH = 30;
	static final int DEFAULT_HEIGHT = 60;
	
	private int NUM_COLS = 1;		// Number of columns in use for 
									// multi-column lists.
	
	/**
	 * Constructor - Creates a new CloverList with given parent,
	 * editable status, and label for m_addButton.
	 * 
	 * @param parent CloverWindow parent
	 * @param editable Editable status
	 * @param buttonLabel Label for m_addButton
	 */
	public CloverList(CloverWindow parent, boolean editable, 
					  String buttonLabel) 
	{
		super();
		m_parent = parent;
		m_editable = editable;
		m_data = new Hashtable();
		if(buttonLabel != null)
			m_addButton = new JButton(buttonLabel);
		else
			m_addButton = null;

		setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		setBorder(new EtchedBorder(EtchedBorder.RAISED));
		getViewport().setBackground(Constants.COLOR_LIST);

		m_pane = new JPanel();
		m_pane.setBackground(Constants.COLOR_LIST);
		m_pane.setLayout(new BorderLayout());
		getViewport().add(m_pane);
	}
	
	/**
	 * Creates the m_specialButton for editable lists.  Assigns the button
	 * the given image and text.
	 * 
	 * @param icon Button image
	 * @param caption Button text
	 */
	public void createSpecialButton(ImageIcon icon, String caption){
		if( !m_editable )
			System.err.println("Creating a special button that will never appear.");
		m_specialButton = new CloverListButton
								(icon, caption, -1, this, false);		
	}
	
	/**
	 * Allows the specailButton to be any extension of a CloverListButton.
	 * 
	 * @param clb Special button
	 */
	public void setSpecialButton(CloverListButton clb){
		m_specialButton = clb;
	}
	
	/**
	 * Sets an icon for the add button.
	 * 
	 * @param i Add button icon
	 */
	public void setAddButtonIcon(ImageIcon i){
		m_addButton.setIcon(i);
	}
	
	/**
	 * Sets the functionality of the add button.  This is needed to
	 * actually add a new character or scene.
	 * 
	 * @param al New add button action
	 */
	public void setAddButtonAction(ActionListener al){
		m_addButton.addActionListener(al);
	}
	
	/**
	 * Clicks the add button.
	 *
	 */
	public void clickAddButtton(){
		m_addButton.doClick();
	}
	
	/**
	 * Constructs the CloverList for an array of Integers.  This could
	 * represent scene IDs or character IDs.
	 * 
	 * @param sceneOrder ArrayList of Integers (IDs)
	 */
	public void constructList(ArrayList sceneOrder) {	
		if( m_data.size() < 0 || sceneOrder==null || sceneOrder.size() < 0) 
			return;
		
		m_pane.removeAll();
		
		top = new JPanel();
		top.setLayout(new GridBagLayout());
		top.setBackground(Constants.COLOR_LIST);		
		
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTH;
		c.insets = new Insets(10,0,10,0);
		c.weightx = 1.0;
		c.weighty = 1.0;
		
		NUM_COLS = getViewport().getWidth() / (Constants.CHAR_THUMB_WIDTH + 20);
		if (NUM_COLS < 1)
			NUM_COLS = 1;
		
		// First add the "New" CharacterButton
		if( m_editable && m_specialButton == null )
			;

		int offset = 0;

		CloverListButton clb = null;
		for( int i=0; i<sceneOrder.size(); i++ ) {
			try {
				Integer id = (Integer)sceneOrder.get(i);
				clb = (CloverListButton)m_data.get(id);
			}
			catch(ClassCastException ex) {
				System.err.println("Bad Vector. "+ex);
				return;
			}
			catch(Exception ex){
				System.err.println("Bad Vector. "+ex);
				return;
			}
			// Create CharacterButton
				
			// Add button to galleryPane
			c.gridx = (i+offset) % NUM_COLS;
			c.gridy = (offset + i)/NUM_COLS;
			
			top.add(clb, c);
			
			if( m_selection == i)
				clb.setSelected(true);
		}
			
		m_pane.add(top, BorderLayout.NORTH);
		
		if(m_addButton != null){
			m_addButton.setAlignmentX(Component.CENTER_ALIGNMENT);
			JPanel addButtonPane = new JPanel();
			addButtonPane.setLayout(
				new BoxLayout(addButtonPane, BoxLayout.Y_AXIS));
			addButtonPane.setBackground(m_pane.getBackground());
			addButtonPane.add(m_addButton);
			addButtonPane.add(Box.createVerticalStrut(5));
			m_pane.add(addButtonPane, BorderLayout.SOUTH);
		}
		
		revalidate();			
	}
	
	/**
	 * Adds a CloverListButton to the end of the list.
	 * 
	 * @param clb Button to add
	 */
	public void addItemToList(CloverListButton clb){
		addItemToList(m_data.size(), clb);
	}
	
	/**
	 * Adds a CloverListButton to the given position in the list.
	 * 
	 * @param pos Position in list
	 * @param clb Button to add
	 */
	public void addItemToList(int pos, CloverListButton clb){
		m_data.put(new Integer(clb.m_id), clb);

		m_selection = clb.m_id;
		unselectAll();
	}
	
	/**
	 * Checks for an empty list.
	 * 
	 * @return True iff list is empty
	 */
	public boolean isEmpty(){
		return m_data.isEmpty();
	}
	
	/**
	 * Replaces item at given index in hashtable with new button
	 * 
	 * @param clb New button
	 */
	public void replaceItem(CloverListButton clb){
		m_data.put(new Integer(clb.m_id), clb);
	}
	
	/**
	 * Gets the button associated with the given id.  Use of hashtable
	 * makes lookup trivial.
	 * 
	 * @param id Item ID
	 * @return Corresponding button
	 */
	public CloverListButton getItem(int id){
		try {
			return (CloverListButton)m_data.get(new Integer(id));
		}
		catch(Exception e){
			return null;
		}
	}
	
	/**
	 * Sets the icon for the button with given ID.
	 * 
	 * @param img New icon
	 * @param id Button ID
	 */
	public void setItemIcon(ImageIcon img, Integer id){
		((CloverListButton)m_data.get(id)).setIcon(img);
	}
	
	/**
	 * Sets a new hashtable of buttons.
	 * 
	 * @param v New hashtable
	 */
	public void setListData(Hashtable v){
		m_data = v;
	}
	
	/**
	 * Gets data hashtable
	 * 
	 * @return Data hashtable
	 */
	public Hashtable getListData(){
		return m_data;
	}
	
	/**
	 * Removes button with the given ID from the list.
	 * 
	 * @param id Button ID
	 */
	public void removeItemFromList(int id){
		m_data.remove(new Integer(id));		
	}
	
	/**
	 * Selects the button at the index given by m_selection.
	 *
	 */
	public void selectButton(){
		unselectAll();
		if( m_selection < 0 )
			m_specialButton.setSelected(true);
		else 
			((CloverListButton)m_data.get(new Integer(m_selection))).setSelected(true);		
	}
	
	/**
	 * Sets the selection status of all elements to false.
	 *
	 */
	public void unselectAll(){
		//m_specialButton.setSelected(false); needed if special buttons are used.
		if( m_data != null && m_data.size() > 0 ) {
			Enumeration i=m_data.elements();
			while( i.hasMoreElements() ) {
				((CloverListButton)i.nextElement()).setSelected(false);
			}
		}
		//m_selection = -1;
	}
	
	/**
	 * Sets m_selection so it matches the currently selected
	 * item.
	 *
	 */
	public void autoSetSelectionIndex(){
		Enumeration i=m_data.elements();
		while( i.hasMoreElements() ){
			CloverListButton clb = (CloverListButton)i.nextElement();
			if( clb.m_isSelected )
				m_selection = clb.m_id; 
		}
	}
	
	/**
	 * Returns the selectionIndex (-1), if selection is not in
	 * the button array (i.e. if new char is selected).
	 * Returns a GID for selected character otherwise.
	 * 
	 * @return Selection ID
	 */
	public int getSelectedItemIndex(){
		if( m_selection < 0 )
			return m_selection;
		CloverListButton clb = (CloverListButton)m_data.get(new Integer(m_selection));
		return clb.m_id;
	}
	
	/**
	 * Adds a label to the glass pane for custom drag and drop support.
	 * 
	 * @param lbl JLabel
	 */
	public void addLabelToGlassPane(JLabel lbl){
		((DragGlassPane)m_parent.getGlassPane()).add(lbl);
	}
	
	/**
	 * Gets the clover glass pane.
	 * 
	 * @return Clover glass pane
	 */
	public Component getCloverGlassPane(){
		return m_parent.getGlassPane();
	}
	
	/**
	 * Gets the Clover content pane
	 * 
	 * @return Clover content pane
	 */
	public Container getCloverContentPane(){
		return m_parent.getContentPane();
	}
	
	/**
	 * Sets the selection index
	 * 
	 * @param i New selection index
	 */
	public void setSelectionIndex(int i){
		m_selection = i;
	}
	
	/**
	 * Gets the selection index
	 * 
	 * @return Selection index
	 */
	public int getSelectionIndex(){
		return m_selection;
	}
	
	/**
	 * Sets the ordered status of the list.  Ordered lists respond to 
	 * drag and drop events by rearranging items.
	 * 
	 * @param b Ordered status
	 */
	public void setOrdered(boolean b){
		m_ordered = b;
	}
	
	/**
	 * Clear the list - remove all items.
	 *
	 */
	public void clearList(){
		m_data.clear();
	}
	
}