/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import edu.uiuc.orchid.clover.Interface.Tabs.ViewAndEditUI;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.DrawingTools.BackgroundSurface;
import edu.uiuc.orchid.clover.Interface.TrashLabel;

/**
 * SceneButton - List button for displaying scenes in the scene editors.
 * These are the only buttons that need to support drag and drop within
 * the CloverList (to rearrange items).  The second class included in this 
 * file is used to prompt the user for a scene title when the double-click
 * or right-click on a scene in the list.
 * 
 * @author Terry J Bradley
 * @version 04.8.01
 *
 */
public class SceneButton extends CloverListButton {
	
	protected ViewAndEditUI m_frame;	// Scene editor
	
	/**
	 * Constructor - CloverListButton constructor with added scene editor link
	 * 
	 * @param img Scene image
	 * @param name Scene title
	 * @param id Scene ID
	 * @param par CloverList parent
	 * @param normal Normal status 
	 * @param frm Scene editor
	 */
	public SceneButton(ImageIcon img, String name, 
						int id, CloverList par, boolean normal,
						ViewAndEditUI frm)
	{
		super(img, name, id, par, normal);
		m_frame = frm;
		
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if( m_id >= 0 ){
					SceneTitleDialog std = new SceneTitleDialog(SceneButton.this);
					std.setVisible(true);
				}
				//timer.stop();
			}
		};
		
		addMouseListener(createMouseAdapter());
		addMouseMotionListener(createMouseMotionAdapter());	
	}
	
	/**
	 * MouseAdapter support drag and drop and scene renaming.
	 * On drag, cursors are set.  On drop, open scene, delete scene,
	 * or move scene.  Double-click and right-click offer to rename
	 * the scene.
	 * 
	 * @return
	 */
	private MouseAdapter createMouseAdapter(){
		MouseAdapter ma = new MouseAdapter(){
			public void mousePressed(MouseEvent e){
				if(m_id >= 0 && e.getButton() == MouseEvent.BUTTON3){
					SceneTitleDialog std = 
						new SceneTitleDialog(SceneButton.this);
					std.setVisible(true);
					setCursor(Cursor.getDefaultCursor());
					return;
				}
				
				startPoint = e.getPoint();
				m_parent.unselectAll();
				setSelected(true);
				setCursor(CloverWindow.MOVE_CURSOR);
				m_parent.getCloverGlassPane().
					setCursor(CloverWindow.MOVE_CURSOR);
				m_parent.getCloverGlassPane().setVisible(true);
			}
			public void mouseReleased(MouseEvent e){
				if(e.getButton() == MouseEvent.BUTTON3)
					return;
				
				SceneButton jb = (SceneButton)e.getSource();
				Component glassPane = m_parent.getCloverGlassPane();

				if( m_dragging ){
					Point glassPoint = SwingUtilities.convertPoint(
												jb,
												e.getPoint(),
												glassPane);
					Component component =
								SwingUtilities.getDeepestComponentAt(
											m_parent.getCloverContentPane(),
											glassPoint.x,
											glassPoint.y);
					if( component == null )
						return;
					if( !m_isNew ) {
						if( component instanceof BackgroundSurface ){
							// Open scene
							m_frame.saveCurrentScene();
							m_frame.openSceneWithID(new Integer(m_id));
						}
						else if( component instanceof TrashLabel){  
							// Delete scene
							m_frame.deleteScene(new Integer(m_id));
						}
						else if( component instanceof SceneButton ){
							// Move scene.
							// Get destination
							int dest = ((SceneButton)component).m_id;
							if( dest >= 0 )
								m_frame.moveSceneToIndex(m_id, dest);	
						}						
						else{
							// Open scene
							m_frame.saveCurrentScene();
							m_frame.openSceneWithID(new Integer(m_id));
						}
					}
					else {
						if( component instanceof BackgroundSurface) { 
							// Do nothing
							;
						}
						else if( component instanceof SceneButton ){
							// Insert new scene at location other than end
							int dest = m_frame.getStory().getPositionOfScene(
									new Integer(((SceneButton)component).m_id), 0);
									// 0 should be responseIndex.
									// USING SPECIAL BUTTONS WILL NOT WORK
									// FOR RESPONSES OTHER THAN THE MAIN STORY.
							if( dest != -1 )
								m_frame.addScene();
							else
								m_frame.addScene();
						}
						else {
							// Do nothing
							;
						}
					}
				} // was dragging
				else {	// normal click
					if( m_isNew ) {
						// Open default scene
						m_frame.saveCurrentScene();
						m_frame.openSceneWithID(new Integer(-1));
					}
					else {
						// Open scene
						m_frame.saveCurrentScene();
						m_frame.openSceneWithID(new Integer(m_id));
					}
				}
				m_dragging = false;
				setCursor(Cursor.getDefaultCursor());
				glassPane.setCursor(Cursor.getDefaultCursor());
				glassPane.setVisible(false);
			}
			public void mouseClicked(MouseEvent e){
				if(m_id >= 0 && (e.getClickCount() > 1 ||
								 e.getButton() == MouseEvent.BUTTON3))
				{
					SceneTitleDialog std = 
						new SceneTitleDialog(SceneButton.this);
					std.setVisible(true);
				}
			}
			public void mouseExited(MouseEvent e){
				if(!m_dragging)
					setCursor(Cursor.getDefaultCursor());
			}
			public void mouseEntered(MouseEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
		};
		return ma;
	}
	
	/**
	 * Sets drag cursor.
	 * 
	 * @return
	 */
	private MouseMotionAdapter createMouseMotionAdapter(){
		MouseMotionAdapter mma = new MouseMotionAdapter(){
			public void mouseDragged(MouseEvent e){
				if(e.getButton() == MouseEvent.BUTTON3)
					return;
				m_dragging = true;
				m_parent.getCloverGlassPane().setCursor(CloverWindow.MOVE_CURSOR);
			}
		};
		return mma;
	}
	
}

/**
 * SceneTitleDialog - Displays a text box in an undecorated dialog below scene
 * thumbnail.  Closes itself when focus is lost or user hits enter.
 * 
 * @author Terry J Bradley
 *
 */
class SceneTitleDialog extends JDialog {
	
	private JTextField jtf;		// Text box for scene title
	
	/**
	 * Constructor - Creates an undecorated dialog containing
	 * a text box.
	 * 
	 * @param sb Associated scene button
	 */
	public SceneTitleDialog(final SceneButton sb){
		super();
		setUndecorated(true);

		// Move dialog to just below scene thumbnail.
		Point location = sb.m_frame.getLocation();
		SwingUtilities.convertPointToScreen(location, sb.m_frame);
		setLocation(location.x + 15,
					location.y + sb.getLocation().y + 45); 
						
		// Add text box
		jtf = new JTextField(12);
		jtf.setText(CloverWindow.stripHTML2(sb.getText()));
		// Close on enter
		jtf.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				sb.setText(((JTextField)e.getSource()).getText());
				SceneTitleDialog.this.dispose();
			}
		});
		getContentPane().add(jtf);
			
		// Close on lost focus
		addWindowFocusListener(new WindowFocusListener(){
			public void windowLostFocus(WindowEvent e){
				sb.setText(jtf.getText());
				SceneTitleDialog.this.dispose();
			}
			public void windowGainedFocus(WindowEvent e){}
		});	
					
		pack();
	}
		
}