/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ScriptPane;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.*;
import java.util.*;

import edu.uiuc.orchid.clover.Interface.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;

/**
 * ScriptDialogPane - Primary component of the ScriptPane.  Every
 * action in the Script is represented by a JPanel - every dialog
 * is represented by a ScriptDialogPane.  This displays a combo box
 * for selecting the speaker and a textarea for editing the line.
 * 
 * Information entered in a ScriptDialogPane is later stored in
 * a ScriptDialog instance.
 * 
 * @author Terry J Bradley
 * @version 04.8.07
 *
 */
public class ScriptDialogPane extends JPanel 
					implements MouseListener, MouseMotionListener
{
	protected ScriptPane m_parent;		// Parent
	protected JComboBox m_name;			// Name combo box
	protected JTextArea m_line;			// Dialog text area
	protected int m_position;			// This dialog's position
										// in the script.
	
	private boolean m_dragging;			// True if the user is dragging
										// a dialog (dnd support).
	
	/**
	 * Constructor - Initializes the combo box and text area.
	 * m_position is set to -1 by default.  Mouse events allow
	 * the combo box and text area to appropriately deal with
	 * drag and drop.  Action is also added so that CTRL-Enter
	 * enters a new dialog.
	 * 
	 * @param par Parent
	 */
	public ScriptDialogPane(ScriptPane par){
		super();
		m_parent = par;
		m_position = -1;
		setBackground(Color.white);
		setBorder(
			new CompoundBorder(
				new CompoundBorder(
						new EmptyBorder(3,3,12,3),
						new LineBorder(Color.GRAY)),
				new EmptyBorder(5,5,5,5)));
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		m_name = new JComboBox();
		m_name.setMaximumRowCount(8);
		m_name.setFont(new Font("monospaced", Font.PLAIN, 12));
		m_name.setBackground(Color.white);
		m_name.setBorder(new EmptyBorder(1,1,1,1));
		m_name.addMouseMotionListener(new MouseMotionAdapter(){
			public void mouseMoved(MouseEvent e){
				if(!m_dragging)
					ScriptDialogPane.this.setCursor(Cursor.getDefaultCursor());
			}
		});
		m_name.addMouseListener(new MouseAdapter(){
			public void mouseEntered(MouseEvent e){
				if(!m_dragging)
					ScriptDialogPane.this.setCursor(Cursor.getDefaultCursor());
			}
		});
		add(m_name, c);
		
		m_line = new JTextArea(4, 60);
		m_line.setLineWrap(true);
		m_line.setMargin(new Insets(4,4,4,4));
		m_line.setWrapStyleWord(true);
		m_line.setFont(new Font("monospaced", Font.PLAIN, 12));
		m_line.setBorder(new EmptyBorder(2,2,2,2));

		Action a = new AbstractAction(){
			public void actionPerformed(ActionEvent e){
				m_parent.addNewDialog(m_parent.m_parent.guessNextSpeaker());
			}
		};
		m_line.getKeymap().addActionForKeyStroke(
				KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,KeyEvent.CTRL_DOWN_MASK), a);
		
		c.gridy = 1;
		JScrollPane jsp = new JScrollPane(m_line);
		jsp.setBorder(new EmptyBorder(3,3,3,3));
		jsp.setBackground(Color.white);
		jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		add(jsp, c);
		
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	
	/**
	 * Constructor - Calls default constructor.  In addition, dialog text,
	 * speaker id, list of character names, and current position are set.
	 * 
	 * @param line Dialog text
	 * @param gid Speaker's ID
	 * @param chs ArrayList of ScriptComboCharacter
	 * @param pos Position of this dialog in script
	 * @param par ScriptPane parent
	 */
	public ScriptDialogPane(String line, int gid, ArrayList chs, int pos, ScriptPane par){
		this(par);
		m_position = pos;
		m_line.setText(line);
		
		updateCharacterNames(chs);
		setSelectedCharacter(gid);
	}
	
	/**
	 * Update the character names that appear in the combo box for 
	 * this DialogPane.  The current selection is saved and restored 
	 * if possible.
	 * 
	 * @param chs Updated ArrayList of ScriptComboCharacters.
	 */
	public void updateCharacterNames(ArrayList chs){
		if(chs == null)
			return;
		m_name.setIgnoreRepaint(true);
		int index = m_name.getSelectedIndex();
		m_name.removeAllItems();
		for(int i=0; i<chs.size(); i++){
			m_name.addItem(chs.get(i));
		}
		if(index < chs.size())
			m_name.setSelectedIndex(index);
		m_name.setIgnoreRepaint(false);
		m_name.repaint();
	}
	
	/**
	 * Sets the combo box to display the character with the given ID.
	 * This is useful when loading a story.
	 * 
	 * @param gid New speaker's ID
	 */
	public void setSelectedCharacter(int gid){
		for(int i=0; i<m_name.getItemCount(); i++){
			ScriptComboCharacter scc = (ScriptComboCharacter)m_name.getItemAt(i);
			if(scc.gid.intValue() == gid) {
				m_name.setSelectedIndex(i);
				break;
			}
		}
	}
	
	/**
	 * Gets the selected character's ID
	 * 
	 * @return Selected character's ID
	 */
	public int getSelectedCharacter(){
		ScriptComboCharacter scc = (ScriptComboCharacter)m_name.getSelectedItem();
		return scc.gid.intValue();
	}
	
	/**
	 * Returns the parent ScriptPane
	 * 
	 * @return Parent ScriptPane
	 */
	public ScriptPane getPane(){
		return m_parent;
	}
	
	/**
	 * Returns the position of this dialog pane.
	 * 
	 * @return ScriptDialogPane's position in Script
	 */
	public int getPosition(){
		return m_position;
	}
	
	/**
	 * Mouse events for main panel.  Different events are used 
	 * for the combo box.  Dragging is only supported when the click occurs
	 * away from the combo box and text area.
	 */
	public void mousePressed(MouseEvent e){
		// Set m_dragging iff not in m_line or m_name.
		Component comp = SwingUtilities.getDeepestComponentAt(
				ScriptDialogPane.this, 
				e.getPoint().x,
				e.getPoint().y);
		if( comp instanceof JTextArea || comp instanceof JButton ){
			m_dragging = false;
			return;
		}
		
		m_dragging = true;
		setCursor(CloverWindow.MOVE_CURSOR);
	}
	
	/**
	 * Handles the drop portion of dnd.  Possible drop targets include
	 * the text area, combo box, or blank space in another dialog pane
	 * which results in a move operation, or a TrashLabel which deletes
	 * the current dialog.
	 */
	public void mouseReleased(MouseEvent e){
		if(!m_dragging)
			return;
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		Point convertedPoint = SwingUtilities.convertPoint(
				ScriptDialogPane.this,
				e.getPoint().x,
				e.getPoint().y,
				m_parent.m_parent );
		
		// Need to figure out where we've been dropped.
		Component comp = SwingUtilities.getDeepestComponentAt(
				//ScriptDialogPane.this.getParent(),
				m_parent.m_parent,
				convertedPoint.x,
				convertedPoint.y);
		m_dragging = false;
		setCursor(Cursor.getDefaultCursor());
		
		if(comp == null){
			System.out.println("null");
			return;
		}
		
		if(comp instanceof JTextArea){
			Component cpar = comp.getParent().getParent().getParent();
			if(!(cpar instanceof ScriptDialogPane)){
				System.out.println(cpar.getClass().toString());
				return;
			}
			m_parent.moveDialog(m_position, ((ScriptDialogPane)cpar).m_position);
		}
		else if(comp instanceof JButton){
			Component cpar = comp.getParent().getParent();
			if(!(cpar instanceof ScriptDialogPane)){
				System.out.println(cpar.getClass().toString());
				return;
			}
			m_parent.moveDialog(m_position, ((ScriptDialogPane)cpar).m_position);
		}
		else if(comp instanceof ScriptDialogPane){
			m_parent.moveDialog(m_position, ((ScriptDialogPane)comp).m_position);
		}
		else if(comp instanceof TrashLabel){
			m_parent.deleteDialog(m_position);
		}
		else
			System.out.println(comp.getClass().toString());

	}
	
	public void mouseClicked(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	
	public void mouseDragged(MouseEvent e){}
	
	/**
	 * MouseMoved sets the cursor to make it obvious that the
	 * blank portions of the ScriptDialogPane are draggable. 
	 */
	public void mouseMoved(MouseEvent e){
		if(m_dragging)
			return;
		
		Component comp = SwingUtilities.getDeepestComponentAt(
				ScriptDialogPane.this,
				e.getPoint().x,
				e.getPoint().y);
		if( comp instanceof JTextArea || comp instanceof JButton ){
			setCursor(Cursor.getDefaultCursor());
		}
		else {
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		}
	}
	
}