/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.File;
import java.util.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.ActionGrid.*;
import edu.uiuc.orchid.clover.Interface.CloverList.*;
import edu.uiuc.orchid.clover.Interface.*;
import edu.uiuc.orchid.clover.Playback.*;
import edu.uiuc.orchid.clover.DrawingTools.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.Actions.*;

/**
 * ViewAndEditUI - Interface for the Scene Viewer/Editor. Scene
 * thumbnails appear in the left side gallery.  The currently
 * edited Scene appears to the right, with an ActionGrid below.
 * 
 * @author Terry Bradley
 * @version 04.8.17 
 * @date 9/23/03
 *
 */
public class ViewAndEditUI extends CloverSplitPane 
						   implements Constants, Playable
{

	// UI elements
	public CloverWindow m_parent;			// CloverWindow parent
	protected DrawingSurface ds;			// Drawing surface allows
											// sketching of scene and holds
											// character instances.
	public DialogEntry de;					// Action Grid
	protected DialogTableData dtd;			// Action Grid data
	protected DialogTableEditor dte;		// Action Grid editor, not really used
	protected CloverList m_sceneList;		// Scene thumbnail gallery
	protected ActionButton[] m_actionButtons;
											// Array of available actions, in
											// button form.
	protected JButton m_emoticons;			// Shows/hides emoticon toolbox
	protected TrashLabel m_tl;				// Trash label
	protected JPanel drawingPane;			// Container for ds
	
	protected Scene	m_scene;			// Currently edited scene
	protected int	m_responseIndex;	// Response index - 0 for Scene editor,
										// >0 for ResonpseUI.
	protected HashMap m_CLabels;		// Hashtable of Integer,Vector(Clabel) pairs
	protected Thread th;				// HENCEFORTH, ALL GIDS WILL BE LONGS
	
	// Playable elements
	private SpeakBubble m_speakBubble;	// Speak bubble for preview
	protected JLabel m_thinkBubble;		// Think bubble for preview
	protected HashMap m_emoteBubbles;	// Map of Long(gid), EmotionBubble pairs
	
	// Playable threads
	protected ProcessStory m_processStory;
	protected ProcessScene m_processScene;
	protected ProcessAction m_processAction;
	
	private static ImageIcon blankThumbnail;	// Blank thumbnail for new scenes
	
	/**
	 * Constructor - Creates a Scene Editor instance.  The left side
	 * gallery, right side drawing surface, and right side action grid
	 * are all initialized.
	 * 
	 * @param win CloverWindow parent
	 * @param dss Size of drawing surface
	 * @param name Title of story
	 */
	public ViewAndEditUI(CloverWindow win, Dimension dss, String name)
	{
		// Create super container, which is a splitpane
		super(win);
		m_parent = win;
		m_responseIndex = 0;
		this.setName(name);
		
		// Set up the thumbnail update timer.  This timer allows
		// multiple requests to update a thumbnail to coalesce into
		// a single request.
		ActionListener al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				resetThread();
				th_updateThumb.start();
			}
		};
		m_timer = new javax.swing.Timer(500, al);
		m_timer.setRepeats(false);
		
		// Initialize various elements
		resetThread();
		
		m_scene = null;
		blankThumbnail = Clover.createImageIcon("Resources/images/all_white.gif",
			Constants.SCENE_THUMB_WIDTH, Constants.SCENE_THUMB_HEIGHT);
		
		m_CLabels = new HashMap();
		m_actionButtons = new ActionButton[2];
		m_emoticons = new JButton();
		
		// Init drawing surface with given size
		ds = new DrawingSurface(dss, Color.white, m_parent, 
								VIEW_AND_EDIT_INDEX, false);
		drawingPane = new JPanel();
		drawingPane.setBorder(new EmptyBorder(3,3,3,3) );
		drawingPane.add(ds, BorderLayout.CENTER);
		
		JPanel dialogPane = createDialogPane();
		
		m_speakBubble = new SpeakBubble();
		m_thinkBubble = new ThinkBubble();
		m_emoteBubbles = new HashMap();
		
		// Add speak and thought bubbles (hidden) for previews
		m_speakBubble.setVisible(false);
		ds.add(m_speakBubble, new Integer(1000));
		m_thinkBubble.setVisible(false);
		ds.add(m_thinkBubble, new Integer(1010));

		// Finish right side of split pane
		JPanel rightHalf = new JPanel();
		rightHalf.setLayout(new BorderLayout());
		rightHalf.add(drawingPane, BorderLayout.NORTH);
		rightHalf.add(dialogPane, BorderLayout.CENTER);
		
		JPanel editor = new JPanel();
		editor.setLayout(new BorderLayout());
		editor.add(rightHalf, BorderLayout.CENTER);
		editor.setBorder(new CompoundBorder(
			new MatteBorder(10,10,10,10,COLOR_BACKDROP),
			new MatteBorder(1,1,1,1,Color.black)));
		editor.setMinimumSize(new Dimension(dss.width+30, dss.height+250));
		// TODO: [hci-5] Adjust minimum right size
		
		// Create scene gallery and set "add" button
		m_sceneList = new CloverList(m_parent, true, "Add Scene");
		m_sceneList.setAddButtonAction(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				saveCurrentScene();
				addScene();
			}
		});
		m_sceneList.setOrdered(true);
		
		// Obsolete use of a "special" button.  When in use, this would
		// create a blank thumbnail with the title "New Scene" that, when
		// edited by the user, would add a new scene to the end of the story.
		// See CloverList.java for more info.
		SceneButton special = new SceneButton(blankThumbnail, 
			"New Scene", -1, m_sceneList,
			false, this);
		m_sceneList.setSpecialButton(special);
		m_sceneList.constructList(null);
		
		JScrollPane viewer = new JScrollPane(m_sceneList);
		viewer.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		// Add trash label below gallery
		m_tl = new TrashLabel(this);
		m_tl.setAlignmentX(Component.CENTER_ALIGNMENT);

		JPanel trashPane = new JPanel();
		trashPane.setLayout(new BoxLayout(trashPane, BoxLayout.Y_AXIS));
		trashPane.add(Box.createVerticalStrut(5));
		trashPane.add(m_tl);
		
		// Finish left side
		JPanel jp_viewer = new JPanel();
		jp_viewer.setLayout(new BorderLayout());
		jp_viewer.add(viewer, BorderLayout.CENTER);
		jp_viewer.add(trashPane, BorderLayout.SOUTH);
		jp_viewer.setPreferredSize(new Dimension(SCENE_THUMB_WIDTH+40, 1000));
		// Comment above line to allow complete minimization of left side.
				
		setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		setBorder(new EmptyBorder(3,3,3,3));
		setLeftComponent(jp_viewer);
		setRightComponent(editor);
		setContinuousLayout(false);
		setDividerSize(5);
	}
	
	
////////////////////////// INSTANCE METHODS ///////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//
//	SCENE MAINTENENCE (including list methods)
//
///////////////////////////////////////////////////////////////////////////////	
	
	/**
	 * Add a blank scene to the story at position pos
	 * ASSUMES CURRENT SCENE ALREADY SAVED
	 * 
	 * @param pos Position of Scene in Story, from 0 to n - 1
	 */
	public int addScene(){
		Story sty = getStory();

		if( sty != null ) {

			// Steps involved:
			// 1) Setup m_scene
			// 2) Add scene to story
			// 3) Setup GUI
			// 4) Add CLB to scene list
			
			// 1
			m_scene = new Scene(getStory());	
			// Is ^^^^ necessary?  Why not just call below, then
			// call m_scene = scene at the end?  Or just say
			// m_scene = null first, then rely on opening the new scene
			// at the end?
			
			// Create a new scene at the end of the story,
			// move it to pos in story, and open it.			
			Scene scene = new Scene(getStory());
			
			// 2
			getStory().addSceneToEnd(scene, m_responseIndex);
			// Add an empty clabel vector
			ArrayList v = new ArrayList();
			m_CLabels.put(new Integer(scene.getSceneID()),v);
			
			// 4 - SCENELIST
			// To Implement:
			// First, create a SceneButton identical to the
			// special button, except for m_id which should
			// be sceneIndex.
			// Second, add the SceneButton to m_jl (or equiv.)
			SceneButton sb = new SceneButton(blankThumbnail, "",
									scene.getSceneID(),
									m_sceneList, true, this);
			m_sceneList.addItemToList(sb);
			m_sceneList.constructList(getStory().getSceneOrder(m_responseIndex));
			m_sceneList.selectButton();
			
			// 3
			Integer sid = new Integer(scene.getSceneID());
			m_scene.setSceneID(-1);
			openSceneWithID(sid);
			
			return sid.intValue();
		}
		return -1;
	}
	
	/**
	 * First prompts, then deletes scene with ID == sceneID
	 * 
	 * @param sceneID - ID of scene to delete
	 */ 
	public void deleteScene(Integer sceneID) {
		saveCurrentScene();
		
		Story sty = getStory();
		// Only attempt deletion if a scene exists
		if( sty != null && m_CLabels.size()>0 ) {
			// Prompt first
			String title = getStory().getScene(sceneID).getTitle();

			if( title.length() <= 0 )
				title = "this scene";
			
			if(!m_parent.confirmDeletion(
					"<html>Do you want to permanently delete " + 
					title + 
					" from the story?<p> " +
					"All your work in this scene will be lost.</html>",
					"Scene"))
				return;
			
			// Delete Clabel, scene, thumbnail			
			ArrayList al = (ArrayList)m_CLabels.get(sceneID);
			for( int i=0; i<al.size(); i++ ){
				CharacterLabel cl = (CharacterLabel)al.get(i);
				cl.setVisible(false);
				ds.remove(cl);
			}
			m_CLabels.remove(sceneID);
							
			//Get deleted item's position
			Integer next = getPreviousItemInList(sceneID);	
			sty.deleteSceneWithID(sceneID);
			m_sceneList.removeItemFromList(sceneID.intValue());
			m_sceneList.constructList(getStory().getSceneOrder(m_responseIndex));
			
			// If we deleted the only scene, add a new one
			if( next.intValue()==-1 ) {
				addScene();
			}
			else {
				openSceneWithID(next);
			}
		}
	}
	
	/**
	 * Returns the previous scene in this response/story.  If this is the
	 * first scene, the second scene is returned.  If this is the only
	 * scene, -1 is returned.
	 * 
	 * @return ID of next scene
	 */
	public Integer getPreviousItemInList(Integer base){
		// Identify the index in sceneOrder of the base scene
		ArrayList al = getStory().getSceneOrder(m_responseIndex);
		int index = al.size();
		for(int i=0; i<al.size(); i++){
			if( ((Integer)al.get(i)).intValue() == base.intValue() ){
				index = i;
				break;
			}
		}
		
		// First scene.  Return next scene if more than 1 scene,
		// or return -1 if this is the only scene
		if( index == 0 && al.size() > 1 )
			index++;
		else
			index--;

		// Invalid index -> return -1.  Otherwise, return the SceneID
		if( index < 0 || index >= al.size() )
			return new Integer(-1);
		else
			return (Integer)al.get(index);
	}
	
	/**
	 * Move the source scene to the destination.
	 * 
	 * @param sourceIndex = sourceID
	 * @param index = destinationID
	 */
	public void moveSceneToIndex(int sourceIndex, int index) {
		// Move object at sourceIndex to index
		// Note that the JList is handled by the DND mechanism.
		if( sourceIndex==index ) {
			m_sceneList.constructList(getStory().getSceneOrder(m_responseIndex));	
			return;
		} 

		// Move the scene in the data structure, rebuild the gallery,
		// and reopen the moved scene.
		getStory().moveScene(m_responseIndex, new Integer(sourceIndex), new Integer(index));
		m_sceneList.constructList(getStory().getSceneOrder(m_responseIndex));
		
		openSceneWithID(new Integer(sourceIndex));
	}
	
	/**
	 * Stores scene changes to the Story data structure.
	 * 
	 */
	public void saveCurrentScene() {
		int currentScenePosition = getStory().getPositionOfScene(
								   new Integer(m_scene.getSceneID()), m_responseIndex);
		if( currentScenePosition >= 0 ) {
			// SAVE A SCENE
			// Need to get title, background, characters
			getStory().getScene(currentScenePosition, m_responseIndex).
							setTitle(m_scene.getTitle());
			getStory().getScene(currentScenePosition, m_responseIndex).
							saveImage(ds.getBackgroundImage());
			getStory().getScene(currentScenePosition, m_responseIndex).
							setCharacters(m_scene.getCharacters());
			getStory().getScene(currentScenePosition, m_responseIndex).
							setActions(m_scene.getActionMap());
		}	
		updateCLabels(new Integer(m_scene.getSceneID()), false);
	}
	
	/**
	 * Stores information about the current scene, then
	 * opens the scene at index i.
	 * 
	 * @param id = id of scene to open
	 */
	public void openSceneWithID(Integer sid) {
		// If we aren't changing the scene, do nothing
		if( m_scene != null && m_scene.getSceneID() == sid.intValue() ) {
			updateCLabels(sid, true);
			m_sceneList.selectButton();
			return;			
		}		 

		// Next, load in new stuff
		if ( sid.intValue() == -1 ) {
			m_scene = new Scene(getStory());
		}
		else
			m_scene = getStory().getScene(sid);
		
		// If scene at index is null, revert to previous scene
		if( m_scene == null ) {
			setEditable(false);
			System.err.println("Error loading scene - Null Scene.");
			return;
		}
		
		// Load image and dialog into vae.
		try {
			ds.setBackgroundImage(m_scene.openImage());
			updateCLabels(sid, true);
			ds.repaint();
			
			// Load the dialog:
			updateActions();			
		}
		catch (Exception e) {
			System.err.println("Error loading scene: " + e.getClass());
			return;
		}
		
		// If no scene was loaded, turn on drawing surface and continue
		if( m_scene.getSceneID() == -1 )
			setEditable(true);

		// Selection should be handled on the click, but otherwise
		// select SceneButton at position...
		m_sceneList.setSelectionIndex(sid.intValue());
		m_sceneList.selectButton();
	}
	
	/**
	 * Opens a scene relative to its position rather than its Scene ID
	 * 
	 * @param pos Scene's position in the current response/story
	 */
	public void openSceneAtPosition(int pos){
		// Get sid from position.  Need to look in
		openSceneWithID(new Integer(getStory().getScene(pos, m_responseIndex).getSceneID()));
	}
	
	/**
	 * Stores the current scene's background in the backend.
	 * LIKELY NEVER USED
	 *
	 */
	public void updateScene() {
		m_scene.saveImage(ds.getBackgroundImage());
	}
	

///////////////////////////////////////////////////////////////////////////////
//
//	Character and CharaterLabel Functions
//
///////////////////////////////////////////////////////////////////////////////	
	
	/**
	 * Add a localcharacter to this scene.  Should
	 * handle both CLabel and Scene update.
	 * 
	 * @param gid = GlobalID of added character
	 * @param loc Character's initial location in the scene
	 */ 
	public void addExistingCharacter(int gid, Point loc) {
		// Check for gid in scene
		if( m_scene.hasCharacter(gid) ){
			// Complain - Style 1: Vocal
			/*
			String name = getStory().getCharacter(gid).getName();
			if( name == "" )
				name = "This character";
			JOptionPane.showMessageDialog(m_parent, name + " already appears "+
			"in this scene.", "Error adding character", JOptionPane.ERROR_MESSAGE);
			*/
			// Complain - Style 2: Beep
			Toolkit.getDefaultToolkit().beep();
			return;
		}
		
		// Create a new character instance
		LocalCharacter lc = new LocalCharacter();
		lc.setLocation(loc);
		if( m_scene.getSceneID() == -1 )
			addScene();
		m_scene.addCharacter(lc, gid);
		
		// Create a new character label and move to loc.
		Image i = m_parent.getStory().getCharacter(gid).
			openImage().getScaledInstance(CLABEL_DEFAULT_WIDTH,
			CLABEL_DEFAULT_HEIGHT, BufferedImage.SCALE_AREA_AVERAGING);
		ImageIcon img = new ImageIcon(i);
		CharacterLabel cl = new CharacterLabel(img, gid, this);
		cl.setBounds(lc.getLocation().x, lc.getLocation().y,
						lc.getSize().width, lc.getSize().height);
		ArrayList v = (ArrayList)m_CLabels.get(new Integer(
						m_scene.getSceneID()));
		v.add(cl);

		cl.setVisible(true);
		ds.add(cl, JLayeredPane.DRAG_LAYER);
		
		// Add character to dtd
		updateActions();	

		selectCharacter(gid);
	}
	
	
	/**
	 * Updates the CLabels at oldSceneID with the given
	 * visibility status.  No other scenes will be modified.
	 * 
	 * @param oldSceneID = ID of old scene
	 * @param show New visibility status
	 */
	public void updateCLabels(Integer oldSceneID, boolean show) {
		if( m_CLabels == null ) return;
		
		// TODO: [perf-2] Disable repaint (save 'til end)?
		m_parent.setIgnoreRepaint(true);	// UNTESTED

		// Only update currentScene and newScene
		// First, hide CLabels for the current Scene
		if( oldSceneID.intValue() >= 0 ) {
			ArrayList labels = (ArrayList)m_CLabels.get(oldSceneID);
			for( int j=0; j<labels.size(); j++ ) 
				((CharacterLabel)labels.get(j)).setVisible(show);
		}
	
		m_parent.setIgnoreRepaint(false);
	}
	
	/**
	 * Updates CLabels, in the event that a character's appearance is
	 * changed globally.  RUNS IN A MIN PRIORITY THREAD.
	 * 
	 * @param gid Character's GlobalID
	 */
	public void setCLabelIcon(final int gid){
		// Look through all CLabels and update the icons
		// of those w/ id == gid
		Thread th = new Thread(){
			public void run(){
				Story sty = getStory();
				final BufferedImage bi = getStory().getCharacter(gid).openImage();
				Set s = m_CLabels.keySet();
				for( Iterator i=s.iterator(); i.hasNext(); ) {
					ArrayList al = (ArrayList)m_CLabels.get((Integer)i.next());
					for( int j=0; j<al.size(); j++){
						final CharacterLabel cl = (CharacterLabel)al.get(j);
						if( cl.getCharacterID() == gid )
							SwingUtilities.invokeLater(new Runnable(){
								public void run(){
									cl.setIcon(new ImageIcon(
										bi.getScaledInstance(
											cl.getIcon().getIconWidth(),
											cl.getIcon().getIconHeight(),
											BufferedImage.SCALE_AREA_AVERAGING)));
								}
							});
					}
				}
			}
		};
		th.setPriority(Thread.MIN_PRIORITY);
		th.start();
	}
	
	/**
	 * Selects a specific character in the current scene.  This
	 * includes the character image in the drawing surface and in
	 * the action table
	 * 
	 * @param gid Character's GlobalID
	 */
	public void selectCharacter(int gid){
		// Called when either DialogEntry is clicked or CLabel is clicked
		// Find the right CLabel and set it selected.
		if( gid == -1 ) return;
		ArrayList al = (ArrayList)m_CLabels.get(new Integer(m_scene.getSceneID()));
		for( int i=0; i<al.size(); i++ ) {
			CharacterLabel cl = (CharacterLabel)al.get(i);
			if( cl.getCharacterID() == gid )
				cl.setSelected(true);
			else
				cl.setSelected(false);
		}
	}
	
	/**
	 * Given a row (== character position), select that character
	 * in the drawing surface.
	 * 
	 * @param row Row in action grid.
	 */
	public void selectCharacterByRow(int row) {
		// Find the gid
		int gid = ((Long)dtd.getData(0).get(row)).intValue();
		
		// Use selectCharacter
		selectCharacter(gid);
	}
	
	/**
	 * Removes the specified character from the current Scene.
	 * CHARACTER IS REMOVED WITHOUT A PROMPT.
	 *  
	 * @param gid Character's GlobalID
	 */
	public void removeCharacter(int gid) {
		// Remove local character
		m_scene.removeCharacter(gid);
		
		// Remove clabel from array and from drawing surface
		ArrayList v = null;
		try {
			v = (ArrayList)m_CLabels.get(new Integer(
				m_scene.getSceneID()));
		}
		catch (Exception e){ 
			return;
		}
		finally {
			if( v == null )
				return;
		}
		
		for( int i=v.size()-1; i>=0; i-- ) {
			CharacterLabel cl = (CharacterLabel)v.get(i);
			if( cl.getCharacterID() == gid ) {
				ds.remove(cl);
				v.remove(i);
			}
		}
		
		// Update dialog table
		updateActions();
	
		// Update the scene's thumbnail
		updateThumbnail(new Integer(m_scene.getSceneID()));
	}
	
	/**
	 * Deletes all instances of character in ALL scenes.
	 * (Only need to call once for VaE & Responses.)
	 * CHARACTER IS DELETED WITHOUT PROMPT/CONFIRMATION.
	 * 
	 * @param gid= Character's global id 
	 */
	public void deleteCharacter(int gid){
		// Look for character in ALL scenes
		Set keys = getStory().getAllSceneIDs(); // keys = Integers	
		for(Iterator i = keys.iterator(); i.hasNext();){
			Integer nextKey = (Integer)i.next();
			Scene scene = getStory().getScene(nextKey);
			if(scene.hasCharacter(gid)){
				scene.removeCharacter(gid);
				ArrayList v = null;
				try {
					v = (ArrayList)m_CLabels.get(new Integer(
						scene.getSceneID()));
				} catch (Exception e){ 
					return;
				} finally {
					if( v == null )
						continue;
				}
				
				for( int j=v.size()-1; j>=0; j-- ) {
					CharacterLabel cl = (CharacterLabel)v.get(j);
					if( cl.getCharacterID() == gid ) {
						ds.remove(cl);
						v.remove(j);
					}
				}
				
			}//if
		}//for
	}
	
	/**
	 * Changes the character's location within the scene
	 * 
	 * @param p New upper-left corner
	 * @param gid Character's GlobalID
	 */
	public void moveCharacter(Point p, int gid) {
		// First move the LC (handled by scene)
		m_scene.moveCharacter(p, gid);
		
		// Moving the CL is handled by CL mouselisteners
		
		// Update the thumbnail
		updateThumbnail(new Integer(m_scene.getSceneID()));
	}
	
	/**
	 * Changes the character's dimensions within the scene
	 * 
	 * @param dim New width and height
	 * @param gid Character's GlobalID
	 */
	public void resizeCharacter(Dimension dim, int gid) {
		m_scene.resizeCharacter(dim, gid);	

		updateThumbnail(new Integer(m_scene.getSceneID())); 		
	}
	
	/**
	 * Simultaneously move and resize the character.  The two provided
	 * points can be any two opposite corners of the containing
	 * rectangle.
	 * 
	 * @param p1 One corner of containing rectangle
	 * @param p2 Opposite corner
	 * @param gid Character's ID
	 */
	public void resizeCharacter(Point p1, Point p2, int gid){
		int x, y, w, h;
		x = (p1.x < p2.x) ? p1.x : p2.x;
		y = (p1.y < p2.y) ? p1.y : p2.y;
		w = Math.abs(p1.x - p2.x);
		h = Math.abs(p1.y - p2.y);
		
		m_scene.resizeCharacter(new Dimension(w, h), gid);
		m_scene.moveCharacter(new Point(x,y), gid);
		
		updateThumbnail(new Integer(m_scene.getSceneID())); 
	}
	
	/**
	 * Returns the given character's drawn image.
	 * 
	 * @param gid Character's ID
	 * @return Character's image
	 */
	public BufferedImage getCharacterImage(int gid) {
		return getStory().getCharacter(gid).openImage();
	}
	
	/**
	 * Sets all CLabels in this scene as not selected.
	 * 
	 * @param refresh Update thumbnail iff refresh == true
	 */
	public void unselectCLabels(boolean refresh) {
		if(m_scene == null) return;
		
		// Unselect all clabels
		ArrayList v = (ArrayList)m_CLabels.get(new Integer(m_scene.getSceneID()));
		if( v==null ) return;
		for( int i=0; i<v.size(); i++ ) {
			((CharacterLabel)v.get(i)).setSelected(false);
		}
		
		// Should also deselect table
		de.clearSelection();
		de.clearCharacterSelection();
		
		// Finally, update the thumbnail.  updateThumbnail runs
		// in its own thread by default.
		if(refresh)
			updateThumbnail(new Integer(m_scene.getSceneID()));
	}
	
	
///////////////////////////////////////////////////////////////////////////////
//
//		Dialog Functions
//
///////////////////////////////////////////////////////////////////////////////	
	
	/**
	 * Returns the row in the Action Grid in which the given character appears.
	 * 
	 * @param gid Character's ID
	 * @return Character's row in action grid
	 */
	public int getCharacterRow(int gid) {
		int row = -1;
		ArrayList al = dtd.getData(0);
		for( int i=0; i<al.size(); i++ ) {
			Long test = (Long)al.get(i);
			if( test.intValue() == gid ) {
				row = i;
				break;
			}
		}
		return row;
	}
	
	/**
	 * Simpler version of editAction takes and converts a point
	 * in the table and passes this value to the full editAction
	 * function.
	 * 
	 * @param action Action type (See Constants.java)
	 * @param loc Location relative to glassPane
	 * @param emotion Emotion index to apply to edited action
	 */
	public void editAction(int action, Point loc, int emotion){
		Point tablePoint = SwingUtilities.convertPoint(
									m_parent.getGlassPane(),
									loc,
									de.getMainTable());
		int row = de.getRowAtPoint(tablePoint);
		int col = de.getColAtPoint(tablePoint);
		editAction(row, col, action, loc, emotion);
	}
	
	/**
	 * Edits the actions at the given row and column.
	 * If action doesn't exist, it will be added to the table.
	 * 
	 * For details on how values are stored in the action grid,
	 * see DialogEntry.java and DialogTableData.java
	 * 
	 * @param row Row in table
	 * @param col Column in table
	 * @param action Type of action
	 * @param loc Point in frame where drop occurred (used for dialogs)
	 */
  	public void editAction(int row, int col, int action, Point loc, int emotion) {
	  	// Get value at (row,col)
	  	int value, emo;
		try {
			value = ((Integer)de.getValueAt(row, col)).intValue();
			emo = value / EMOTION_DIVIDER;
			value = STRIP_ALL_OFFSET & value;
		}
		catch (Exception ex) {
			value = 0;
			emo = 0;
		}
		
	  	// Get gid of character in this row
	  	Long gid = (Long)dtd.getData(0).get(row);
	  	if( gid.intValue() < 0 )
	  		return;
		boolean rebuild = false;
		
		// Handle each action type separately
	  	switch (action) {
			case SPEAK_OFFSET:
			  	// Check value to see if SPEAK exists
				String filename;
				
				// Check for and handle new speak actions
			  	if( (value & SPEAK_OFFSET) == 0 &&
			  		(value & THINK_OFFSET) == 0    ) 
			  	{
				  	Speak spk = new Speak(gid.intValue(), gid.intValue(), col);
				  	SecondaryAction sec = m_scene.getSecondaryAction(gid, col);
				  	if(sec == null)
				  		sec = new Emote();
				  	spk.setSecondaryAction(sec);
				  	
				  	if(spk.getAudioPath() == null)
						filename = "" + gid.intValue() + "_" + 
							getStory().getCharacter(gid.intValue()).getNextActionID() +
							".wav";
					else
						filename = spk.getAudioPath();

				 	m_parent.m_speakDialog.show(spk, filename);
					
				  	// Store dialog.action in Scene iff isOK
				  	if( m_parent.m_speakDialog.isOKClicked() ) {
						spk.setActionText(m_parent.m_speakDialog.getDialog());
						spk.setAudioPath(m_parent.m_speakDialog.getFilename());
						int emoVal = 0;
						if(sec instanceof Emote) {
							((Emote)sec).m_emotion = 
								m_parent.m_speakDialog.getEmotion();
							emoVal = ((Emote)sec).getEmotion();
						}
						
					  	// store spk someplace
					  	rebuild = m_scene.addAction(spk, gid.intValue());
													
					  	// Set value in table
					  	// Values are computed bitwise.
					  	int newVal = (value | SPEAK_OFFSET | EMOTE_OFFSET) 
					 				 +  EMOTION_DIVIDER * emoVal;
					 	if(spk.getAudioPath() != null || 
					 		!getStory().getCharacter(gid.intValue()).getRecord())
					 		newVal = newVal | HAS_AUDIO_OFFSET;
					  	setValueAt(new Integer(newVal), row, col);
				  	}
				  	break; 
			  	}
			  	// Edit an existing speak action
			  	else {
			  		// Two cases: Existing Speak, or Existing Think
			  		if((value & THINK_OFFSET) != 0) {
			  			// Think action exists in this cell.
			  			// Trying to replace w/ a Speak action.
			  			// Beep (flash?) and return
			  			Toolkit.getDefaultToolkit().beep();
			  			break;
			  		}
				  	// edit existing speak action
				  	// get the action
				  	Speak spk = (Speak)m_scene.
					  	getPrimaryAction(gid, col);
					
					if(spk.getAudioPath() == null)
						filename = "" + gid.intValue() + "_" + 
							getStory().getCharacter(gid.intValue()).getNextActionID() +
							".wav";
					else
						filename = spk.getAudioPath();

				  	m_parent.m_speakDialog.show(spk, filename);
					
				  	// Store dialog.action in Scene iff isOK
				  	if( m_parent.m_speakDialog.isOKClicked() ) {
					  	spk.setActionText(m_parent.m_speakDialog.getDialog());
					  	spk.setAudioPath(m_parent.m_speakDialog.getFilename());
					  	int emoVal = 0;
					  	if(spk.getSecondaryAction() instanceof Emote){
					  		((Emote)spk.getSecondaryAction()).m_emotion =
								m_parent.m_speakDialog.getEmotion();
							emoVal = ((Emote)spk.getSecondaryAction()).m_emotion;
					  	}
					  	
					  	// store spk someplace
					  	rebuild = m_scene.addAction(spk, gid.intValue());
						
					  	// Set value in table
					  	int newVal = (value | SPEAK_OFFSET | EMOTE_OFFSET) 
									 +  EMOTION_DIVIDER * emoVal;
						if(spk.getAudioPath() != null)
							newVal = newVal | HAS_AUDIO_OFFSET;
					  	setValueAt(new Integer(newVal), row, col);
				  	}
				  	break; 
			  	}
			case THINK_OFFSET:
				// Check value to see if THINK exists
				if( (value & SPEAK_OFFSET) == 0 &&
					(value & THINK_OFFSET) == 0    )
				{
					// new think action
					Think thk = new Think(gid.intValue(), col);
					SecondaryAction sec = m_scene.getSecondaryAction(gid, col);
					if(sec == null)
						sec = new Emote();
					thk.setSecondaryAction(sec);
										
					m_parent.m_thinkDialog.show(thk, null);
					
					// Store dialog.action in Scene iff isOK
					if( m_parent.m_thinkDialog.isOKClicked() ) {
						thk.setActionText(m_parent.m_thinkDialog.getDialog());
						int emoVal = 0;
						if(thk.getSecondaryAction() instanceof Emote){
							((Emote)thk.getSecondaryAction()).m_emotion =
								m_parent.m_thinkDialog.getEmotion();
							emoVal = ((Emote)thk.getSecondaryAction()).m_emotion;
						}

						// store spk someplace
						rebuild = m_scene.addAction(thk, gid.intValue());
								
						// Set value in table
						int newVal = (value | THINK_OFFSET | EMOTE_OFFSET) 
									 +  EMOTION_DIVIDER * emoVal;
						setValueAt(new Integer(newVal), row, col);
					}
					break; 
				}
				else {
					// Two cases: Existing Speak, or Existing Think
					if((value & SPEAK_OFFSET) != 0) {
						// Speak action exists in this cell.
						// Trying to replace w/ a Think action.
						// Beep (flash?) and return
						Toolkit.getDefaultToolkit().beep();
						break;
					}
					// edit existing think action
					// get the action
					Think thk = (Think)m_scene.getPrimaryAction(gid, col);
					m_parent.m_thinkDialog.show(thk, null);
					
					// Store dialog.action in Scene iff isOK
					if( m_parent.m_thinkDialog.isOKClicked() ) {
						thk.setActionText(m_parent.m_thinkDialog.getDialog());
						int emoVal = 0;
						if(thk.getSecondaryAction() instanceof Emote){
							((Emote)thk.getSecondaryAction()).m_emotion =
								m_parent.m_thinkDialog.getEmotion();
							emoVal = ((Emote)thk.getSecondaryAction()).m_emotion;
						}
						// store spk someplace
						rebuild = m_scene.addAction(thk, gid.intValue());
						
						// Set value in table
						int newVal = (value | THINK_OFFSET | EMOTE_OFFSET) 
									 +  EMOTION_DIVIDER * emoVal;
						setValueAt(new Integer(newVal), row, col);
					}
					break; 
				}
			case EMOTE_OFFSET: case 0:
				// Attach emotion*EMOTION_DIVIDER to value
				/*
				value = value | EMOTE_OFFSET;
				value += emotion*EMOTION_DIVIDER;
				setValueAt(new Integer(value), row, col);
				*/
				if( (value & EMOTE_OFFSET) == 0) {
					// new emote action
					Emote emt = new Emote(emotion);
					
					if( emotion < 0 ) {
						int newVal = (value & ~EMOTE_OFFSET);
						setValueAt(new Integer(newVal), row, col);
					}
					else {
						PrimaryAction pa = m_scene.getPrimaryAction(gid, col);
						if(pa == null)
							pa = new PrimaryAction(gid.intValue(), 0, col);
						int newVal = (value | EMOTE_OFFSET) + 
							EMOTION_DIVIDER*emotion;
						setValueAt(new Integer(newVal), row, col);

						pa.setSecondaryAction(emt);
						rebuild = m_scene.addAction(pa, gid.intValue());
					}

					break; 
				}
				else {
					// edit existing emote action
					// get the action
					Emote emt = (Emote)m_scene.getSecondaryAction(gid, col);
					PrimaryAction pa = m_scene.getPrimaryAction(gid, col);
					
					// New style: Always set action
					emt.m_emotion = emotion;
					if( emotion < 0 ) {
						int newVal = (value & ~EMOTE_OFFSET);
						setValueAt(new Integer(newVal), row, col);
					}
					else {
						int newVal = (value | EMOTE_OFFSET) + 
							EMOTION_DIVIDER*emotion;
						setValueAt(new Integer(newVal), row, col);
					}
					pa.setSecondaryAction(emt);
					rebuild = m_scene.addAction(pa, gid.intValue());
					
					break; 
				}
			// end EMOTE_OFFSET
				
			// HAS_AUDIO_OFFSET was previously used to denote whether a given
			// action had audio associated with it.  A speaker icon would
			// be shown if audio existed.  This functionality was later
			// scrapped.
			case HAS_AUDIO_OFFSET:
				// Playback audio.
				Speak spk = null;
				try {
					spk = (Speak)m_scene.getPrimaryAction(gid, col);
				} catch (Exception e){
					break;
				}
				if(spk == null || (spk.getAudioPath() == null &&
					getStory().getCharacter(gid.intValue()).getRecord()) )
					break;
				
				if( spk.getAudioPath() != null)
					m_parent.playAudio(spk.getAudioPath());
				else {
					// use SpeechEngine
					m_parent.playTTS(spk.getActionText(), gid.intValue());
				}
			break;
			default:
				System.err.println("Invalid action type.");	  	
	  	}
		if( rebuild ) {
			// update dtd, then rebuild
			updateActions();
		}
		else
			de.repaint();
  	}	
  	
  	/**
  	 * Move the primary action, and any attached secondary action,
  	 * from (startRow, startCol) to (destRow, destCol)
  	 * 
  	 * @param startRow
  	 * @param startCol
  	 * @param destRow
  	 * @param destCol
  	 */
  	public void moveAction(int startRow, int startCol, 
  							int destRow, int destCol) 
  	{	

		// Step 0: Get action
		Long gid = (Long)dtd.getData(0).get(startRow);
		if(gid.intValue() == -1)
			return;
		PrimaryAction pa = m_scene.getPrimaryAction(gid, startCol);		
							
		// Step 1: Check for existing action at dest.
		Long destGID = (Long)dtd.getData(0).get(destRow);
		if(destGID.intValue() == -1 )
			return;
		PrimaryAction paDest = m_scene.getPrimaryAction(destGID, destCol);
		
		// Step 1a: If one exists, get action
		// Step 1b: Prompt for overwrite, showing both actions
		if( paDest != null && (
			paDest.getActionText().length() > 0 || 
				(paDest instanceof Speak && 
				((Speak)paDest).getAudioPath() != null)) )
		{

			String message = "";
			if( pa instanceof Speak ) {
				String oldText, dialog = "dialog";
				oldText = paDest.getActionText().length() > 20 ?
							paDest.getActionText().substring(0, 20) :
							paDest.getActionText();
				if( paDest instanceof Think ) {
					dialog = "thought"; 
				}

				String newText = pa.getActionText().length() > 20 ?
							pa.getActionText().substring(0, 20) :
							pa.getActionText();
				message = "Replace "+dialog+" beginning with \""+
						oldText+"\""+"\n with the dialog beginning \""+newText+"\"?";
			}
			else if( pa instanceof Think ) {
				String oldText, dialog = "dialog";
				oldText = paDest.getActionText().length() > 20 ?
							paDest.getActionText().substring(0, 20) :
							paDest.getActionText();
				if( paDest instanceof Think ) {
					dialog = "thought"; 
				}

				String newText = pa.getActionText().length() > 20 ?
							pa.getActionText().substring(0, 20) :
							pa.getActionText();
				message = "Replace "+dialog+" beginning with \""+
								oldText+"\""+"\n with thought beginning \""+newText+"\"?";
			}

			if( JOptionPane.showConfirmDialog(ViewAndEditUI.this,
											message,
											"Replace action?",
											JOptionPane.YES_NO_OPTION,
											JOptionPane.QUESTION_MESSAGE)
						== JOptionPane.NO_OPTION )
			{	
				return;
			}						
		}
			
		// Step 2: Edit action, changing sourceGID, startingCol
		pa.setSourceGID(destGID.intValue());
		pa.setStartingCol(destCol);
		
		// Step 3: Add action to scene
		m_scene.addAction(pa, pa.getSourceGID());
		
		// Step 4: Remove action(s) from scene
		m_scene.removeAction(gid.intValue(), startCol);
		
		// Step 5: Regenerate table data (Optional: Adjust two specific values instead)
		updateActions();  								
  	}
  	
  	/**
  	 * Remove ANY AND ALL actions at (row, col)
  	 * 
  	 */
  	public void removeAction(int row, int col){
  		// Get GID
		Long gid = (Long)dtd.getData(0).get(row);
		
		// Remove action
		m_scene.removeAction(gid.intValue(), col);
	
		// Step 5: Regenerate table data (Optional: Adjust one specific value instead)
		updateActions();
   	}
  	
  	/**
  	 * Given a row in the table and an action type, this function
  	 * looks in the row for the first cell without this action type
  	 * in it.  The column is returned
  	 * 
  	 * @param row Row to search through
  	 * @param action Action type (defined in Constants)
  	 * @return Column
  	 */
  	public int getNextOpenTime(int row, int action){
  		if( dtd.getDataSize() <= 1 )
  			return dtd.getDataSize();
  		for( int i=1; i<dtd.getDataSize(); i++ ) {
  			Integer j = null;
  			try {
  				ArrayList al = dtd.getData(i);
  				j = (Integer)al.get(row);
  			}
  			catch (Exception e){}
  			if( (j.intValue() & action) == 0 )
  				return i;
  		}
  		return dtd.getDataSize();
  	}
  	
	
	/**
	 * Adds value to the dtd - char col if col = 0, chunk col otherwise
	 * 
	 */
	public void setValueAt(Object value, int row, int col) {
		if( col == 0 ) {
			if( value instanceof Long )
				dtd.getData(0).set(row, value);
			else
				System.err.println("Data error - set GID with bad Object Class");
		}
		else {
			if( value instanceof Integer )
				dtd.getData(col).set(row, value);
			else
				System.err.println("Data error - set cell with bad Object Class");
		}
	}

	/**
	 * Selects the cell in the action grid corresponding to the given
	 * character.
	 * 
	 * @param gid Character's ID
	 */
	public void selectCellInTable(int gid){
		// Need to get row; col = 0
		ArrayList al = dtd.getData(0);
		for( int i=0; i<al.size(); i++ ) {
			if( ((Long)al.get(i)).intValue() == gid ) {
				de.selectCell(i,0);
				return;
			}
		}
	}
	
	/**
	 * Gets all action data from the Story data structure and
	 * rebuilds the Action Grid.
	 *
	 */
	public void buildDialogTable() {
		// Only the GUI
		// Clear the table
		de.clearTable();
		
		// Add the first column of GIDs
		// Add a column for each vector in Actions
		for( int j=0; j<dtd.getDataSize(); j++ ) {
			de.addColumn(j, m_scene);
		}
		
		// If there are no characters, disable the actionbuttons
		boolean enableButtons = false;
		try {
			enableButtons = ((Long)dtd.getData(0).get(0)).intValue() >= 0;
		}
		catch (Exception e){}
		for( int j=0; j<m_actionButtons.length; j++ )
			m_actionButtons[j].setEnabled(enableButtons);
		m_emoticons.setEnabled(enableButtons);
					
		// Add blank columns at the end
		// Update: Blanks added to new scene / dtd
			
		// Repaint the table						
		de.repaint();
	}
	
	
///////////////////////////////////////////////////////////////////////////////
//
//			Playback Functions
//
///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Initializes the emotion bubbles by adding one for
	 * each character.
	 * 
	 */
	public void initBubbles(){
		int index = 0;
		for(Iterator i = m_emoteBubbles.keySet().iterator(); i.hasNext();){
			// remove bubbles from ds
			Object key = i.next();
			EmotionBubble eb = (EmotionBubble)m_emoteBubbles.get(key);
			ds.remove(eb);
			i.remove();
		}
		for(int i=0; i<m_scene.getCharacterCount(); i++){
			Long gid = m_scene.getCharacterIDAtPosition(i);
			EmotionBubble eb = new EmotionBubble();
			eb.setVisible(false);
			ds.add(eb, new Integer(300+index));
			index++;
			m_emoteBubbles.put(gid, eb);
		}
		ds.revalidate();
		ds.repaint();
	}
	
	/**
	 * Playable function - Moves a bubble to the given location.
	 * Also sets visibility.
	 * 
	 * Note: This is virtually identical to the PlaybackUI function.
	 * Perhaps these could be combined somehow?
	 */
	public void moveBubble(int actionType, Scene scene, int gid,
						   Point location, boolean show)
	{
		boolean right = 
			(m_scene.getCharacter(gid).getLocation().x < ds.getWidth()/2);
			
		Point loc = new Point();
		if(right){
			loc.x = (m_scene.getCharacter(gid).getLocation().x) + 
				Math.round(getStory().getCharacter(gid).getRightBubbleLocation().x * 
				((float)m_scene.getCharacter(gid).getSize().width / 
				(float)getStory().getCharacter(gid).openImage().getWidth())) -
				location.x;
			loc.y = (m_scene.getCharacter(gid).getLocation().y) + 
				Math.round(getStory().getCharacter(gid).getRightBubbleLocation().y * 
				((float)m_scene.getCharacter(gid).getSize().height / 
				(float)getStory().getCharacter(gid).openImage().getHeight())) -
				location.y;
		}
		else {
			loc.x = (m_scene.getCharacter(gid).getLocation().x) + 
				Math.round(getStory().getCharacter(gid).getLeftBubbleLocation().x * 
				((float)m_scene.getCharacter(gid).getSize().width / 
				(float)getStory().getCharacter(gid).openImage().getWidth())) -
				location.x;
			loc.y = (m_scene.getCharacter(gid).getLocation().y) + 
				Math.round(getStory().getCharacter(gid).getLeftBubbleLocation().y * 
				((float)m_scene.getCharacter(gid).getSize().height / 
				(float)getStory().getCharacter(gid).openImage().getHeight())) -
				location.y;
		}
		loc.x = Math.max(0, loc.x);
		loc.y = Math.max(0, loc.y);
		
		switch(actionType){
			case SPEAK_OFFSET:
				// Hide think bubble
				if(m_thinkBubble != null)
					m_thinkBubble.setVisible(false);
					
				// hide bubble before move if hiding anyway
				ds.add(m_speakBubble, new Integer(1000));	
				if(!show)
					m_speakBubble.setVisible(false);
				// Move speak bubble to location
				if(m_speakBubble == null) {
					m_speakBubble = new SpeakBubble();

					// add to canvas
					ds.add(m_speakBubble, new Integer(1000));
				}
				
				m_speakBubble.setLocation(loc);
				
				// Display speak bubble
				m_speakBubble.setVisible(show);
				ds.revalidate();
			break;
			
			case THINK_OFFSET:
				// Hide speak bubble
				if(m_speakBubble != null)
					m_speakBubble.setVisible(false);
					
				ds.add(m_thinkBubble, new Integer(1010));
				if(!show)
					m_thinkBubble.setVisible(false);
				// Move think bubble to location
				if(m_thinkBubble == null) {
					m_thinkBubble = new ThinkBubble();

					// add to canvas
					ds.add(m_thinkBubble, new Integer(1000));
				}
			
				m_thinkBubble.setLocation(loc);
											
				// Display speak bubble
				m_thinkBubble.setVisible(show);
				ds.revalidate();
			break;
			
			case EMOTE_OFFSET:
				// Move emotion "bubble" to location (hide nothing)
				if(right)
					loc.x -= 131;

				loc.y -= 131;
				loc.x = Math.max(0, loc.x);
				loc.y = Math.max(0, loc.y);

				if(show){ 
					((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
						.setLocation(loc);
				}
				
				((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
					.setVisible(show);
				
				ds.revalidate();
				ds.repaint();
			break;
		}
		ds.revalidate();
		ds.repaint();
	}
	
	/**
	 * Playable function - Sets the bubble size for the given
	 * action type.
	 */
	public boolean setBubbleSize(int actionType, String text, 
								 Scene scene, int gid)
	{
		boolean right = (m_scene.getCharacter(gid).getLocation().x < ds.getWidth()/2);
		switch(actionType){
			case SPEAK_OFFSET:
				// Set text in speak bubble
				((SpeakBubble)m_speakBubble).setBubbleSize(text, right);
			break;
			
			case THINK_OFFSET:
				// Set text in think bubble
				((ThinkBubble)m_thinkBubble).setBubbleSize(text, right);
			break;
			
			case EMOTE_OFFSET:
				// Ignore
			break;
		}
		return right;
	}
	
	/**
	 * Playable function - Sets the text for one of the bubbles
	 */
	public void setBubbleText(int actionType, String text, int start, int end){
		switch(actionType){
			case SPEAK_OFFSET:
				// Set text in speak bubble
				((SpeakBubble)m_speakBubble).setBubbleText(text, start, end);
			break;
			
			case THINK_OFFSET:
				// Set text in think bubble
				((ThinkBubble)m_thinkBubble).setBubbleText(text, start, end);
			break;
			
			case EMOTE_OFFSET:
				// Ignore
			break;
		}
	}
	
	/**
	 * Playable function - Sets the emotion for a given character.
	 */
	public void setEmotion(int gid, int emotion, boolean active){
		// Set emotion icon
		int index = 0;
		for(int i=0; i<EmotionBubble.EMOTIONS.length; i++){
			if(emotion == EmotionBubble.EMOTIONS[i].m_emotionIndex)
				index = i;
		}

		if(active)
			((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
				.setIcon(EmotionBubble.EMOTIONS[index].m_activeImg);
		else
			((EmotionBubble)m_emoteBubbles.get(new Long(gid)))
				.setIcon(EmotionBubble.EMOTIONS[index].m_staticImg);
	}
	
	/**
	 * Playable functions - Not necessary for the Scene editor
	 */
	public void setBackgroundImage(int bi){
		// Not supported for VaE - shouldn't ever need to change ds
		// through the Playable interface
	}
	public void setCharacter(Point location, Dimension size, int bi, boolean b)
	{
		// Not supported for VaE - shouldn't ever need to change the characters
		// through the Playable interface
	}
	public void hideCharacters(){}

	/**
	 * Playable function - Outlines the cell in the action grid
	 * corresponding to the current action during playback.
	 */
	public void colorCell(CloverAction ca, boolean on){
		int row = 0;
		int col = 1;
		if(ca instanceof PrimaryAction) {
			PrimaryAction act = (PrimaryAction)ca;
			row = dtd.getCharacterRow(act.getSourceGID());
			col = act.getStartingCol();
		}
		
		de.m_table.scrollRectToVisible(
			de.m_table.getCellRect(row, col, true));
		Rectangle rect = de.m_table.getCellRect(row, col, true);
		Point p = SwingUtilities.convertPoint(de.m_table, 
								rect.x, rect.y, m_parent.getGlassPane());
		rect.x = p.x;
		rect.y = p.y;
		
		// Draw a rectangle around this cell in the table
		if(on) 
			((DragGlassPane)m_parent.getGlassPane()).setRectangle(rect);
		else
			((DragGlassPane)m_parent.getGlassPane()).setRectangle(null);
			
		((DragGlassPane)m_parent.getGlassPane()).setVisible(on);
		de.repaint();
	}
	
	/**
	 * Playable function - Halts playback, hides bubbles
	 */
	public void stopPlayback(int sid, boolean display){
		// Transitions are not supported, and we can only play the
		// current scene, so cleanup simply involves stopping any audio
		// that is currently playing and hiding all bubbles.
		
		// Stop audio
		if(m_parent.isAudioPlaying())
			m_parent.playAudio("");
			
		// Hide bubbles
		m_speakBubble.setVisible(false);
		m_thinkBubble.setVisible(false);
		initBubbles();
		
		// Hide/turn off painting on glasspane
		((DragGlassPane)m_parent.getGlassPane()).setRectangle(null);
		m_parent.getGlassPane().repaint();
	}


///////////////////////////////////////////////////////////////////////////////
//
//		  Misc Functions
//
///////////////////////////////////////////////////////////////////////////////
		
	/**
	 * Called when creating a story.  Resets the drawing surface, adds
	 * a blank scene, and populates all elements.
	 * 
	 */
	public void updateVaEUI(Dimension size) {
		// update ds
		resetDrawingSurface(size);
		
		addScene();
		
		populateVAE();
		
		m_sceneList.selectButton();
	}
	
	/**
	 * Reacquires the data for the Action Grid from the Story data
	 * structure and rebuilds the Action Grid.
	 *
	 */
	public void updateActions(){
		dtd.setTableData(m_scene.getActions());
		de.setModel(dtd);	// necessary?
		
		buildDialogTable();
	}
	
	/**
	 * Updates the scene thumbnail gallery by rebuilding it for this
	 * response/story.
	 * 
	 */
	public void updateGallery(int responseIndex){
		try{
			ArrayList sceneOrder = getStory().getSceneOrder(responseIndex);
			for(int i=0; i<sceneOrder.size(); i++){
				// Get background image
				Scene s = getStory().getScene((Integer)sceneOrder.get(i));
				BufferedImage img = s.openImage();
				
				// Add characters to background
				Graphics2D g = img.createGraphics();
				Set characters = s.getCharacterIDs();
		
				for(Iterator it = characters.iterator(); it.hasNext();){
					Integer gid = (Integer)it.next();
					LocalCharacter lc = s.getCharacter(gid.intValue());
			
					g.drawImage(	getStory().getCharacter(gid.intValue()).openImage(),
									lc.getLocation().x, lc.getLocation().y,
									lc.getSize().width, lc.getSize().height,
									ViewAndEditUI.this);
				}
		
				final ImageIcon ii = new ImageIcon(img.
					getScaledInstance(SCENE_THUMB_WIDTH, 
										SCENE_THUMB_HEIGHT,
										Image.SCALE_SMOOTH));

				SceneButton sb = new SceneButton(ii, s.getTitle(), 
					s.getSceneID(), m_sceneList, true, this);
				m_sceneList.addItemToList(sb);
			}
			m_sceneList.constructList(getStory().getSceneOrder(responseIndex));
			m_sceneList.revalidate();
		}
		catch (NullPointerException np){
			System.err.println("Character array == null");
		}
	}
	
	/**
	 * Constructs the gallery for this response/story.
	 *
	 */
	public void reconstructGallery(int responseIndex){
		m_sceneList.constructList(getStory().getSceneOrder(responseIndex));
	}
	
	/**
	 * Called when opening a story, this sets up the drawing surface
	 * and action grid and loads the first scene.
	 * 
	 * @param size Size of drawing surface
	 */
	public void openStory(Dimension size){
		// set up ds
		resetDrawingSurface(size);
		
		populateVAE();
				
		// Setup gallery
		updateGallery(0);
		
		// Load first scene
		m_scene = null;
		openSceneAtPosition(0);
	}

	/**
	 * Returns the ID of the currently loaded scene
	 * 
	 * @return Current scene's ID
	 */
	public int getSelectedSceneID() {
		return m_scene.getSceneID();
	}
	
	/**
	 * Sets the enabled status of the drawing surface and action grid.
	 * 
	 * @param b Enabled status
	 */
	public void setEditable(boolean b) {
		ds.setEnabled(b);
		de.setEnabled(b);
	}
	
	/**
	 * Returns the current scene
	 * 
	 * @return Current scene
	 */
	public Scene getCurrentScene() {
		return m_scene;
	}
		
//// Display and layout methods

	/**
	 * Sets up the m_CLables vector and m_List.  Only needs to be
	 * called when creating/opening a story.
	 * 
	 */
	public void populateVAE() {
		Story sty = m_parent.getStory();

		m_CLabels.clear();
		ds.resetSurface();
		
		if( sty != null ){
			for( int i=0; i<sty.getSceneCount(0); i++ ) {
				ArrayList clabels = new ArrayList();
				Scene scene = sty.getScene(i, m_responseIndex);
				Set s = scene.getCharacterIDs();
				for( Iterator j = s.iterator(); j.hasNext(); ){
					int gid = ((Integer)j.next()).intValue();
					LocalCharacter lc = scene.getCharacter(gid);
					
					ImageIcon img = new ImageIcon(
						sty.getCharacter(gid).openImage().
							getScaledInstance(lc.getSize().width,
											  lc.getSize().height,
											  Image.SCALE_AREA_AVERAGING));
					CharacterLabel cl = new CharacterLabel(
						img, gid, this);
											
					cl.setBounds(lc.getLocation().x, lc.getLocation().y,
								 lc.getSize().width, lc.getSize().height);
					clabels.add(cl);
					
					cl.setVisible(false);
					ds.add(cl, JLayeredPane.DRAG_LAYER);
				}
				m_CLabels.put(new Integer(scene.getSceneID()), clabels);	// was m_scene
			}
		}
	}
	
	/**
	 * Returns the drawing surface.
	 * 
	 * @return Drawing surface
	 */
	public DrawingSurface getDrawingSurface(){
		return ds;
	}
	
	/**
	 * Write the thumbnail for the first scene to \\Temp\\preview.png
	 *
	 */
	public void savePreviewThumbnail(){
		try{
			BufferedImage bi = new BufferedImage(
					SCENE_THUMB_WIDTH, SCENE_THUMB_HEIGHT,
					BufferedImage.TYPE_INT_ARGB);
			bi.createGraphics().drawImage(
					((ImageIcon)((CloverListButton)m_sceneList.getItem(0)).
							getIcon()).getImage(), null, null);
			boolean success = ImageIO.write(
						  bi,
						  "png",
						  new File(Clover.TEMP_DIR+"preview.png"));
		}
		catch(Exception e){
			System.err.println("Error writing file.");
		}
	}
	
	/**
	 * Updates the thumbnail in the SceneList.
	 * This function should ALWAYS be called in a
	 * low priority thread.
	 * 
	 * Each call to this function creates a BufferedImage, which
	 * may or may not be (permanently?) cached.
	 *
	 * @param sceneID = ID of scene to update
	 */
	public void buildThumbnail(final Integer sceneID){
		Scene scene = getStory().getScene(sceneID);
		BufferedImage img = scene.openImage();
		
		Graphics2D g = img.createGraphics();
		ArrayList labels = (ArrayList)m_CLabels.get(sceneID);
		Set characters = scene.getCharacterIDs();
		
		for(Iterator it = characters.iterator(); it.hasNext();){
			Integer gid = (Integer)it.next();
			LocalCharacter lc = scene.getCharacter(gid.intValue());
			
			g.drawImage(	getStory().getCharacter(gid.intValue()).openImage(),
							lc.getLocation().x, lc.getLocation().y,
							lc.getSize().width, lc.getSize().height,
							ViewAndEditUI.this);
		}
		
		final ImageIcon ii = new ImageIcon(img.
			getScaledInstance(SCENE_THUMB_WIDTH, 
								SCENE_THUMB_HEIGHT,
								BufferedImage.SCALE_SMOOTH));

		if( sceneID.intValue() > -1 )
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					m_sceneList.setItemIcon(ii, sceneID);
				}
			});

		img = null;
	}
	
	/**
	 * Resets the thumbnail capture thread.
	 *
	 */
	public void resetThread(){
		th_updateThumb = new Thread() {
			public void run() { 				
				BufferedImage img = BackgroundSurface.
					changeColor(0,0,ds.getBackgroundImage());
				
				Graphics2D g = img.createGraphics();
				ArrayList labels = (ArrayList)m_CLabels.get(
					new Integer(m_scene.getSceneID()));
				for(int i = 0; i<labels.size(); i++){
					CharacterLabel cl = (CharacterLabel)labels.get(i);
					if(cl.isVisible()){
						g.drawImage(((ImageIcon)cl.getIcon()).getImage(),
								cl.getLocation().x, cl.getLocation().y,
								ViewAndEditUI.this);
					}
				}
		
				// SCENELIST
				// Need to update the icon for one element of 
				// the scene list's m_data, comparing currentSceneIndex
				// to m_id's
				if( m_scene.getSceneID() == -1 )
					addScene();

				final ImageIcon ii = new ImageIcon(img.
					getScaledInstance(SCENE_THUMB_WIDTH, 
										SCENE_THUMB_HEIGHT,
										BufferedImage.SCALE_SMOOTH));
				final Integer id = new Integer(m_scene.getSceneID());
				if( id.intValue() > -1 )
					SwingUtilities.invokeLater(new Runnable(){
						public void run(){
							m_sceneList.setItemIcon(ii, id);
						}
					});
				img.flush();
				img = null;
			}
		};
		th_updateThumb.setPriority(Thread.MIN_PRIORITY);
		th_updateThumb.setName("Clover-Update Scene Thumbnail");
	}
	
	/**
	 * Resets the drawing surface, possibly changing the size.
	 * 
	 * @param d New size
	 */
	public void resetDrawingSurface(Dimension d){
		ds = new DrawingSurface(d, Color.white, m_parent, 
								VIEW_AND_EDIT_INDEX, false);

		drawingPane.removeAll();
		drawingPane.add(ds, BorderLayout.CENTER);
	}
	
	/**
	 * Function responsible for most of the layout of the interface.
	 * Creates action grid, data, action buttons, preview button...
	 * 
	 * @return DialogPane containing right side elements
	 */
	private JPanel createDialogPane() {
		de = new DialogEntry(this);
		dtd = new DialogTableData();
		
		DialogTableRenderer.loadImages();
		de.setModel(dtd);
		
		// Set up MenuPanel
		GridBagConstraints c = new GridBagConstraints();
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new GridBagLayout());
		menuPanel.setBorder(new CompoundBorder(
			new MatteBorder(0,0,0,1,Color.white),
			new EmptyBorder(2,5,2,5)));
		menuPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		
		// Create and add action Buttons
		JPanel actionButtons = new JPanel();		
		actionButtons.setLayout(new BoxLayout(actionButtons, BoxLayout.Y_AXIS));
		m_actionButtons[0] = new ActionButton(SPEAK_OFFSET, 
			Clover.createImageIcon2(Clover.SPEAK_ICON_LOC), 
			Constants.EMOTE_OFFSET,
			Emote.EMOTICONS[0].m_image, "", true, m_parent);
		m_actionButtons[0].setToolTipText("Drag to add dialog");
		m_actionButtons[1] = new ActionButton(THINK_OFFSET,
			Clover.createImageIcon2(Clover.THINK_ICON_LOC), 
			Constants.EMOTE_OFFSET,
			Emote.EMOTICONS[0].m_image, "", true, m_parent);
		m_actionButtons[1].setToolTipText("Drag to add thoughts");

		m_emoticons = new JButton(Clover.createImageIcon2(Clover.EMOTE_ICON_LOC));
		m_emoticons.setToolTipText("Open Emotion Toolkit");
		m_emoticons.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Point location = SwingUtilities.convertPoint(m_emoticons,
															 0, 0,
															 m_parent);
				m_parent.m_emotionDialog.show(location);
			}
		});
		m_emoticons.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		actionButtons.add(m_actionButtons[0]);
		actionButtons.add(m_actionButtons[1]);
		actionButtons.add(m_emoticons);

		actionButtons.setBorder(new CompoundBorder(
			new LineBorder(new Color(150,150,150), 1),
			new EmptyBorder(2,2,2,2)));
		c.gridy = 0;
		c.gridheight = 3;
		menuPanel.add(actionButtons, c);

		// Create and add preview button
		JButton testButton = new JButton(
				Clover.createImageIcon2(Clover.ICON_DIR+"Play24.gif"));
		testButton.setToolTipText("Preview Scene");
		
		ActionListener al = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Preview Button
				JButton stb = (JButton) e.getSource();

				initBubbles();
				unselectCLabels(false);
				
				ActionSynch as = new ActionSynch(ViewAndEditUI.this, 
					m_scene.getCharacterCount());
				SceneSynch ss = new SceneSynch();
				Integer[] scenes = new Integer[1];
				scenes[0] = new Integer(m_scene.getSceneID());
				
				// Start action thread
				m_processAction = new ProcessAction(as);
				m_processAction.setName("Clover-Process Action");
				m_processAction.start();
				
				// Start remaining threads
				m_processStory = new ProcessStory(ss, ViewAndEditUI.this, 
					getStory(), scenes);
				m_processStory.setName("Clover-Process Story");
				m_processScene = new ProcessScene(as, ss, 
					ViewAndEditUI.this);
				m_processScene.setName("Clover-Process Scene");
				m_processScene.start();
				m_processStory.start();

			}
		};
		testButton.addActionListener(al);
		JPanel testPanel = new JPanel();
		testPanel.add(testButton);
		c.gridy=3;
		c.gridheight=1;
		menuPanel.add(testPanel, c);

		JPanel dialogPane = new JPanel();
		dialogPane.setLayout(new BorderLayout());
		dialogPane.setBackground(COLOR_BACKDROP);

		JPanel tmp = new JPanel();
		tmp.setLayout(new BoxLayout(tmp, BoxLayout.Y_AXIS));
		tmp.add(menuPanel);
		tmp.add(Box.createVerticalStrut(1000));
		
		dialogPane.add(tmp, BorderLayout.WEST);
		dialogPane.add(de, BorderLayout.CENTER);
		
		return dialogPane;
	}
}


