/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

/**
 * ActionSynch Class
 * Provides synchronized access to actions.  This way,
 * only one action is performed at a time.
 * 
 * @author Terry Bradley
 * @version 04.8.19
 *
 */
public class ActionSynch{
	
	private Playable play;			// Playable interface
	private int m_sceneID;			// ID of containing scene
	private int m_row;				// Row in which current action appears
	private int m_col;				// Col in which current action appears
	private int[] m_previousEmos;	// Array of previously expressed emotions.
									// These may (but are not currently) be
									// expressed as lingering "static"
									// emotions in future versions.
	private boolean busy = false;	// True when processing an action, false
									// while waiting for an action
	private boolean stop = false;	// Set to true on the last action, passed
									// to processing threads to finalize them.
	
	/**
	 * Constructor - Builds a simple ActionSynch container with
	 * a Playable interface and the number of characters in the Story.
	 * 
	 * @param p Playable interface
	 * @param numChar Number of characters in the Story
	 */
	public ActionSynch(Playable p, int numChar){
		play = p;
		m_previousEmos = new int[numChar];
	}

	/**
	 * Synchronized method consumes actions by calling static methods
	 * in ScenePlayback.
	 * 
	 * @return True iff stop is true
	 */
	public synchronized boolean runCell() {
		if(stop) {
			ScenePlayback.cleanupScene(play, m_sceneID);
			stop = false;
			return true;
		}
			
		while (busy == false) {
			try {
				// wait for Producer to put value
				wait();
			} catch (InterruptedException e) {
			}
		}
		int lastEmo = m_previousEmos[m_row];
		m_previousEmos[m_row] = 
			ScenePlayback.processCell(play, m_sceneID, m_row, m_col, lastEmo);
		busy = false;
		// notify Producer that value has been retrieved
		notifyAll();
		return false;
	}
	
	/**
	 * Synchronized method produces actions for runCell()
	 * 
	 * @param sceneID Current scene's ID
	 * @param row Action's row
	 * @param col Action's column
	 * @param stop True iff this is the last action in a scene
	 */
	public synchronized void putCell(int sceneID, int row, int col, 
									boolean stop) 
	{
		while (busy == true) {
			try {
				// wait for Consumer to get value
				wait();
			} catch (InterruptedException e) {
			}
		}
		m_sceneID = sceneID;
		m_row = row;
		m_col = col;
		this.stop = stop;
		busy = true;
		// notify Consumer that value has been set
		notifyAll();
	} 	
	
}