/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import javax.sound.sampled.*;
import java.io.*;

import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;

/**
 * AudioPlayback Class
 * Provides support for playing user-recorded audio, as
 * well as prerecorded audio (for emotions).  Currently
 * the format is hardcoded - see AudioCapture for
 * supported format.
 * 
 * @author Terry Bradley
 * @version 04.8.19
 *
 */
public class AudioPlayback implements Runnable{

	SourceDataLine line;	// Audio output line
	Thread thread;			// Playback thread
	String path;			// Path to audio file
	boolean m_playing;		// True during playback
	CloverWindow m_parent;	// CloverWindow parent
	
	/**
	 * Constructor - Creates a new playback instance with the 
	 * assigned parent
	 * 
	 * @param par CloverWindow parent
	 */
	public AudioPlayback(CloverWindow par){
		super();
		m_parent = par;
		m_playing = false;
	}
	
	/**
	 * Checks whether the playback thread is alive
	 * 
	 * @return True iff thread is alive
	 */
	public boolean isRunning(){
		if(thread != null)
			return thread.isAlive();
		else
			return false;
	}
	
	/**
	 * Sets audio path
	 * 
	 * @param p New audio path
	 */
	public void setPath(String p){
		path = p;
	}
	
	/**
	 * Starts playback thread.
	 *
	 */
	public void start(){
		thread = new Thread(this);
		thread.setName("Playback");
		thread.start();
	}
	
	/**
	 * Attempts to stop playback thread.  This function is known
	 * to call a deprecated method (Thread.stop()).
	 *
	 */
	public void stop(){
		thread.stop();
		thread = null;
	}
	
	/**
	 * Only required function of the Runnable interface; this function
	 * is called as part of the start() function and does all the actual
	 * playback work.
	 */
	public void run() {
		m_playing = true;
		AudioInputStream inputStream = null;
		File file = new File(path);
		int totalFramesRead = 0;

		// Try to read audio file
		try {
			inputStream = 
				AudioSystem.getAudioInputStream(file);
		}
		catch(Exception ex){
			System.out.println("Error w/ input stream");
			ex.printStackTrace();
			m_playing = false;
			return;
		}
		
		// Hardcoded to read a specific audio format. See AudioCapture.java
		int bytesPerFrame = inputStream.getFormat().getFrameSize();
		int numBytes = 1024 * bytesPerFrame;
		byte[] audioBytes = new byte[numBytes];
		DataLine.Info info = new DataLine.Info(SourceDataLine.class,
			inputStream.getFormat(), 8820);
		if (!AudioSystem.isLineSupported(info)) {
			// TODO: Handle the error.
		}
		// Obtain and open the line.
		try {
			line = (SourceDataLine) AudioSystem.getLine(info);
			line.open(inputStream.getFormat());
		} 
		catch (LineUnavailableException ex) {
			// TODO: Handle the error.
			System.out.println("Error opening line");
		}

		// Set volume to max
		FloatControl vol = (FloatControl)line.
			getControl(FloatControl.Type.MASTER_GAIN);
		vol.setValue(vol.getMaximum());
				
		// Begin reading the audio file
		int numBytesRead = 0;
		int numFramesRead = 0;
		line.start();
		
		try {
			while ((numBytesRead =
				inputStream.read(audioBytes)) != -1) {
				//  Calculate the number of frames actually read.
				numFramesRead = numBytesRead / bytesPerFrame;
				totalFramesRead += numFramesRead;
				// "Write" the audio data to the output line, i.e.
				// play the audio in the audioBytes array...
				line.write(audioBytes, 0, numBytesRead);
			}
		}
		catch(Exception exp){
			System.out.println("Error reading stream");
		}
		// we reached the end of the stream.  let the data play out, then
		// stop and close the line.
		if (thread != null) {
			try {
				line.drain();
			} catch (Exception e){
				System.err.println(e);
			}
		}
		line.stop();
		line.close();
		line = null;
		m_playing = false;
		// Notify either CloverWindow (which notifies SpeakDialog)
		m_parent.audioStopped();
	}
	
	/**
	 * Checks playback status
	 * 
	 * @return True iff audio is playing
	 */
	public boolean isPlaying(){
		return m_playing;
	}
		
}