/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.awt.*;

import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.Data.*;

/**
 * Playable interface - Interface which must be implemented by any
 * interface that wishes to play back a scene or story.  A large set of
 * supported functions are included, though some implemenations may
 * use trivial versions of some functions.  Playable interfaces in Clover
 * include ViewAndEditUI and PlaybackUI.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public interface Playable {
	
	/**
	 * This function moves either the speech or thought bubble to a 
	 * location relative to the given character.  The bubble may 
	 * also be hidden or displayed.
	 * 
	 * @param actionType Action index (see Constants.java, SPEAK_OFFSET)
	 * @param scene Scene containing the action
	 * @param gid Character performing the action
	 * @param location New location of bubble
	 * @param show New visibility of bubble
	 */
	public void moveBubble(int actionType, Scene scene, 
						   int gid, Point location, boolean show);
	
	/**
	 * Sets the bubble size for the given action type.
	 * 
	 * @param actionType Action index (see Constants.java, SPEAK_OFFSET)
	 * @param text New text for bubble
	 * @param scene Scene containing the action
	 * @param gid Character performing the action
	 * @return Bubble's location relative to the character - true for
	 * right of character, false for left.
	 */
	public boolean setBubbleSize(int actionType, String text, 
								 Scene scene, int gid); 
	
	/**
	 * Sets the text in the given bubble to the given String.
	 * 
	 * @param actionType Action index (see Constants.java, SPEAK_OFFSET)
	 * @param text New text for bubble
	 * @param startLine First line to display
	 * @param endLine Last line to display
	 */
	public void setBubbleText(int actionType, String text, 
							  int startLine, int endLine);
	
	/**
	 * Sets the emotion for a given character.
	 * 
	 * @param gid Character whose emotion is changing
	 * @param emotion New emotion (see Constants.java for emotion indeces)
	 * @param active True for active (animated) emotions
	 */
	public void setEmotion(int gid, int emotion, boolean active);
	
	/**
	 * Sets the background image in the interface to the given
	 * Scene's background.
	 * 
	 * @param sid Scene ID
	 */
	public void setBackgroundImage(int sid);
	
	/**
	 * Moves and resizes given character.
	 * 
	 * @param location New character location
	 * @param size New character size
	 * @param gid Character's ID
	 * @param visible Character's visibility
	 */
	public void setCharacter(Point location, Dimension size, 
							 int gid, boolean visible);
	
	/**
	 * Hides all characters.
	 *
	 */
	public void hideCharacters();
	
	/**
	 * Outlines the cell in the Action Grid containing the given action.
	 * 
	 * @param ca Current action
	 * @param on Outline status
	 */
	public void colorCell(CloverAction ca, boolean on);
	
	/**
	 * Stops playback for given scene.
	 * 
	 * @param sceneID Scene's ID
	 * @param display If true, display an option menu
	 */
	public void stopPlayback(int sceneID, boolean display);
	
}