/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.*;

/**
 * ScenePlayback - A collection of static functions that manipulates
 * a Playable interface in response to user-defined actions.  This class
 * is primarily concerned with actions, not scene events.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class ScenePlayback {
	
	protected static ActionSynch m_synch;	// Action synch instance
											// used to 'grab' an action
	protected static CloverWindow m_parent;	// CloverWindow parent to
											// Playable interfaces
	
	static final int LINE_DELAY 		= 1500;	// Minimum time to display
												// a line of scrolling text
	static final int CHAR_DELAY_DISP 	= 90;	// Time to display a single char
	static final int CHAR_DELAY			= 60;	// Time to read a single char
	static final int SPEAKER_PAUSE 		= 750;	// Conversational pause between
												// actions.
	
	/**
	 * Processes the dialog table cell at (row, col).  Takes
	 * the last emotion for row as an arg and returns the current
	 * emotion for that character (0 or less for no emotion).
	 * 
	 * @param play Playable interface
	 * @param sceneID Scene's ID
	 * @param row Action's row
	 * @param col Action's col
	 * @param lastEmo Index for last emotion
	 */
	public static int processCell(Playable play, int sceneID, 
								   int row, int col, int lastEmo)
	{
		// Only called if some action exists at (row,col)
		// 1) Get scene 
		Scene scene = m_parent.getStory().getScene(new Integer(sceneID));
		
		// 2) Get characterID
		Long gid = scene.getCharacterIDAtPosition(row);
		
		// 3) get primary action (if any)
		PrimaryAction primary = scene.getPrimaryAction(gid, col);
		
		// 4) get secondary
		SecondaryAction secondary = null;
		if(primary != null)
		 	secondary = primary.getSecondaryAction(); 
		if(secondary == null)
			secondary = new Emote(Constants.EMOTION_NONE);
		
		// Process primary action
		if(primary != null){
			if(primary instanceof Speak) {
				processSpeak(play, scene, (Speak)primary, secondary, 
							 lastEmo, false);
				return (secondary instanceof Emote) ? 
					((Emote)secondary).m_emotion : 0;
			}
			else if(primary instanceof Think){
				processThink(play, scene, (Think)primary, secondary, 
					lastEmo, false);
				return (secondary instanceof Emote) ? 
					((Emote)secondary).m_emotion : 0;
			}
			else { // dummy primary action == process secondary only
				if(secondary instanceof Emote) {
					int time = 
						(((Emote)secondary).m_emotion == lastEmo) ? 500 : 2000;
					processEmotion(play, scene, (Emote)secondary, 
						gid.intValue(), lastEmo, time);
					return ((Emote)secondary).m_emotion;
				}
			}
		}
		
		return Constants.EMOTION_NONE;
	}
	
	/**
	 * Process a speak action.  This function takes a speak action
	 * and a Playable interface.  The action is used to display and
	 * update a speech bubble while audio is played.
	 * (Audio and text very likely WON'T line up.)
	 * If animatedText is true, then the text will be displayed
	 * one character at a time.  If false, the text will only change
	 * if it doesn't fit in its bubble.
	 * 
	 * @param play Playable interface
	 * @param scene Scene containing speak action
	 * @param spk Speak action
	 * @param emo Emotion (Shouldn't this be part of Speak??)
	 * @param prev Previous emotion index
	 * @param animatedText If true, display animated text
	 */
	public static void processSpeak(final Playable play, Scene scene,
		final Speak spk, SecondaryAction emo, int prev, 
		boolean animatedText)
	{
		// Step 1: Move the bubble into position
		boolean right = play.setBubbleSize(Constants.SPEAK_OFFSET, 
						spk.getActionText(), scene, spk.getSourceGID());
		
		// Step 1.1: Get offset based on dialog length
		Point offset = SpeakBubble.getDialogPoint(spk.getActionText().length(),
												  right);
				
		play.moveBubble(Constants.SPEAK_OFFSET, scene, spk.getSourceGID(), 
			offset, true);
			
		// Color table cell
		play.colorCell(spk, true);
		
		// Step 2: Begin playing audio
		if ( spk.getAudioPath() != null ) 
			m_parent.playAudio(spk.getAudioPath());
		else
			m_parent.playTTS(spk.getActionText(), spk.getSourceGID());
			
		// Step 3: Begin displaying text
		if(animatedText){
			TimerAction al = new TimerAction(play, spk);
			Timer t = new Timer(CHAR_DELAY, al);
			t.setRepeats(true);
			t.setInitialDelay(0);
			t.setCoalesce(false);
			t.start();
		}
		else if (spk.getActionText().length() > 130){
			// Need to get number of lines first
			int lines = ActionBubble.getNumberOfLines(spk.getActionText());
			
			// get a timer going
			LineTimer lt = new LineTimer(play, spk, lines);
			Timer t = new Timer(LINE_DELAY, lt);
			t.setRepeats(true);
			t.setInitialDelay(0);
			t.start();
		}
		else{
			play.setBubbleText(Constants.SPEAK_OFFSET,
								spk.getActionText(), 0, 
								ActionBubble.getNumberOfLines(spk.getActionText()));
		}

		int delay = 500+CHAR_DELAY*spk.getActionText().length();
		
		if(emo != null && emo instanceof Emote)
			processEmotion(play, scene, 
				(Emote)emo, spk.getSourceGID(), prev, delay);
					
		if(emo == null){
			try {
				Thread.sleep(delay);
			} catch(Exception e){}
		}
		
		while(m_parent.isAudioPlaying()) {
			try {
				Thread.sleep(120);
			} catch(Exception e){}
		}
		
		// Step 5: Clean up - Hide bubble, etc.
		play.moveBubble(Constants.SPEAK_OFFSET, scene, spk.getSourceGID(),
			new Point(0,0), false);
		
		try {
			Thread.sleep(SPEAKER_PAUSE);
		} catch(Exception e){}
		
		play.colorCell(spk, false);
	}
	
	/**
	 * Process a think action.  This function takes a think action
	 * and a Playable interface.  The action is used to display and
	 * update a thought bubble.
	 * 
	 * If animatedText is true, then the text will be displayed
	 * one character at a time.  If false, the text will only change
	 * if it doesn't fit in its bubble.
	 * 
	 * NOTE: This function and processSpeak() are practically identical and
	 * could likely be combined into a single function with little effort.
	 * 
	 * @param play Playable interface
	 * @param scene Scene containing speak action
	 * @param think Think action
	 * @param emo Emotion (Shouldn't this be part of Speak??)
	 * @param prev Previous emotion index
	 * @param animatedText If true, display animated text
	 */
	public static void processThink(final Playable play, Scene scene,
		final Think think, SecondaryAction emo, int prev, 
		boolean animatedText)
	{
		// Step 1: Move the bubble into position
		boolean right = play.setBubbleSize(Constants.THINK_OFFSET,
								 think.getActionText(), scene, think.getSourceGID());
		
		// Step 1.1: Get offset based on dialog length
		Point offset = ThinkBubble.getDialogPoint(think.getActionText().length(),
												  right);

		play.moveBubble(Constants.THINK_OFFSET, scene, think.getSourceGID(), 
			offset, true);
			
		// Color table cell.
		play.colorCell(think, true);
		
		// Step 3: Begin displaying text
		if(animatedText){
			TimerAction al = new TimerAction(play, think);
			Timer t = new Timer(CHAR_DELAY_DISP, al);
			t.setRepeats(true);
			t.setInitialDelay(0);
			t.setCoalesce(false);
			t.start();
		}
		else if (think.getActionText().length() > 130){
			// Need to get number of lines first
			int lines = ActionBubble.getNumberOfLines(think.getActionText());
			
			// get a timer going
			LineTimer lt = new LineTimer(play, think, lines);
			Timer t = new Timer(LINE_DELAY, lt);
			t.setRepeats(true);
			t.setInitialDelay(0);
			t.start();
		}
		else{
			play.setBubbleText(Constants.THINK_OFFSET,
								think.getActionText(), 0, 
								ActionBubble.getNumberOfLines(think.getActionText()));
		}
		// Step 4: Display emotion if attached
		int time = 500+CHAR_DELAY*think.getActionText().length();
		
		if(emo != null && emo instanceof Emote)
			processEmotion(play, scene, (Emote)emo, think.getSourceGID(), 
				prev, time);
			
		if(emo == null || (emo instanceof Emote && 
			((Emote)emo).m_emotion == Constants.EMOTION_NONE) )
		{
			try {
				Thread.sleep(time);
			} catch(Exception e){}
		}
				
		// Step 5: Clean up - Hide bubble, etc.
		play.moveBubble(Constants.THINK_OFFSET, scene, think.getSourceGID(),
			new Point(0,0), false);
			
		try {
			Thread.sleep(SPEAKER_PAUSE);
		} catch(Exception e){}
		
		play.colorCell(think, false);
	}
	
	/**
	 * Processes an emotion by setting up and displaying an appropriate
	 * emotion bubble.
	 * 
	 * @param play Playable interface
	 * @param scene Containing scene
	 * @param emo Emotion action
	 * @param gid Character
	 * @param prev Previous emotion index
	 * @param time Starting time for action
	 */
	public static void processEmotion(Playable play, Scene scene, Emote emo, 
									int gid, int prev, int time)
	{
		// Hide emotion and exit if no emotion
		if(emo == null || emo.m_emotion == Constants.EMOTION_NONE){
			play.moveBubble(Constants.EMOTE_OFFSET, scene, gid,
							new Point(0,0), false);
			return;
		}
		
		play.setEmotion(gid, emo.m_emotion, 
			(prev != emo.m_emotion));
		
		play.moveBubble(Constants.EMOTE_OFFSET, scene, gid, 
			new Point(0,0), true);
			
		play.colorCell(emo, true);
			
		// TODO:  Play audio here, if any is associated with emotion
		if(time < 0)
			time = 2000;
		try {
			Thread.sleep(time);
		} catch(Exception e){}
		
		// Step 5: Clean up - Switch to static bubble, etc.
		play.setEmotion(gid, emo.m_emotion, false);
		
	}
	
// Setup functions
	/**
	 * Sets CloverWindow parent.
	 * 
	 * @param par New CloverWindow parent
	 */
	public static void setupPlayback(CloverWindow par){
		m_parent = par;
	}
	
	/**
	 * Cleans up scene by stopping playback in interface.
	 * 
	 * @param play Playable interface
	 * @param m_sceneID Scene's ID
	 */
	public static void cleanupScene(Playable play, int m_sceneID){
		play.stopPlayback(m_sceneID, true);
	}

}

/**
 * TimerAction - Animates text for Speak or Think actions.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
class TimerAction implements ActionListener {
	
	int startParagraph = 0;		// Start of text to animate
	int endParagraph = 1;		// End of displayed text
	Playable play;				// Playable interface
	CharacterAction spk;		// Action to animate
	
	/**
	 * Constructor - Assigns the playable interface and action.
	 * 
	 * @param p Playable interface
	 * @param s Action to animate
	 */
	public TimerAction(Playable p, CharacterAction s){
		play = p;
		spk = s;
	}
	
	/**
	 * Action to take when timer fires.  Responsible for animating text.
	 */
	public void actionPerformed(ActionEvent e){
		if(spk instanceof Speak){
			if(((Speak)spk).getActionText() == null || 
				((Speak)spk).getActionText().length() < endParagraph)
			{
				((Timer)e.getSource()).stop();
				return;
			}
			play.setBubbleText(Constants.SPEAK_OFFSET,
			((Speak)spk).getActionText().substring(startParagraph, endParagraph),
				0,8);
			endParagraph++;
			if(endParagraph > ((Speak)spk).getActionText().length())
				((Timer)e.getSource()).stop();
		}
		else if(spk instanceof Think){
			if(((Think)spk).getActionText() == null || 
				((Think)spk).getActionText().length() < endParagraph)
			{
				((Timer)e.getSource()).stop();
				return;
			}
			play.setBubbleText(Constants.SPEAK_OFFSET,
			((Think)spk).getActionText().substring(startParagraph, endParagraph),
				0,8);
			endParagraph++;
			if(endParagraph > ((Think)spk).getActionText().length())
				((Timer)e.getSource()).stop();
		}
	}
	
}

/**
 * LineTimer - Similar to TimerAction, this custom timer is used when the 
 * number of lines to display exceeds the size of the bubble.  An entire line
 * is animated, rather than a single character.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
class LineTimer implements ActionListener {
	
	int start, end, numLines;	// Endpoints of text to display
	Playable play;				// Playable interface
	CharacterAction spk;		// Action to display
	
	/**
	 * Constructor - Assigns key values.
	 * 
	 * @param p Playable interface
	 * @param s Action to animate
	 * @param lines Maximum lines for bubble
	 */
	public LineTimer(Playable p, CharacterAction s, int lines){
		play = p;
		spk = s;
		start = 0;
		end = 8;
		numLines = lines;
	}
	
	/**
	 * Action to perform when timer fires.
	 */
	public void actionPerformed(ActionEvent e){
		if(spk instanceof Speak){
			play.setBubbleText(Constants.SPEAK_OFFSET,
				((Speak)spk).getActionText(), start, end);
			start++;
			end++;
			if(end >= numLines) {
				((Timer)e.getSource()).stop();
			}
		}
		else if(spk instanceof Think){
			play.setBubbleText(Constants.THINK_OFFSET,
				((Think)spk).getActionText(), start, end);
			start++;
			end++;
			if(end >= numLines) {
				((Timer)e.getSource()).stop();
			}
		}
	}
	
}