/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import javax.speech.*;
import javax.speech.synthesis.*;

/**
 * SpeechEngine - Attempts to initialize a SAPI engine through
 * the CloudGarden JSAPI implemenation.  Most code here
 * is derived from CloudGarden's examples.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class SpeechEngine {
	
	Synthesizer synth = null;	// Synthesizer instance
	Voice voice = null;			// Voice to use
	Voice[] voices;				// Available voices
    
	/** Creates new from HelloSapi5 */
	public SpeechEngine() throws Error {
		initComponents();
		EngineList list = null;
		try {
			list = Central.availableSynthesizers(null);
		}
		catch(Error e){
			throw e;
		}
					
		try {	
			SynthesizerModeDesc desc = null;
			for(int i=0;i<list.size();i++) {
				desc = (SynthesizerModeDesc)list.elementAt(i);
				voices = desc.getVoices();
				for(int j=0;j<voices.length;j++) {
					if(voices[j].getName().indexOf("SAPI5") > 0) {
						voice = voices[j];
						break;
					}
				}
			}
	    
			if(voice == null) {
				System.out.println("Unable to find a SAPI5 voice - quitting");
				return;
			}
	    
			synth = Central.createSynthesizer(desc);
			((com.cloudgarden.speech.CGEngineProperties)
				synth.getSynthesizerProperties()).setEventsInNewThread(false);
			synth.allocate();
			synth.resume();
			synth.waitEngineState(Synthesizer.ALLOCATED);
			System.out.println("Using voice "+voice);
			SynthesizerProperties props = synth.getSynthesizerProperties();
			props.setVoice(voice);
	    
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents() {}

	/**
	 * Speak given text using the TTS synthesizer. Should
	 * work with JSAPI markup
	 * 
	 * @param words Words to speak
	 */
	public void speak(String words){
		try {
			synth.waitEngineState(Synthesizer.QUEUE_EMPTY);
			synth.speak(words,null);
		}
		catch (Exception e){
			System.err.println("Formatting error.");
		}
	}
	
	/**
	 * Speak given text using the TTS synthesizer
	 * 
	 * @param words Text to speak
	 * @param vol Volume of speech
	 * @param rate Rate of speech
	 * @param pitch Speech pitch
	 * @param range Range of speech
	 */
	public void speak(String words, float vol, float rate, float pitch, float range){
		SynthesizerProperties props = synth.getSynthesizerProperties(); 
		props.setPitch(props.getPitch()+20.0f);
		props.setPitchRange(props.getPitchRange()+10.0f);
		props.setSpeakingRate(rate);
		props.setVolume(vol);
		try {
			synth.waitEngineState(Synthesizer.QUEUE_EMPTY);
			synth.speak(words,null);
		}
		catch (Exception e){
			System.err.println("Formatting error.");
		}
	}
	
	/**
	 * Returns true iff TTS engine is speaking
	 * 
	 * @return TTS engine status
	 */
	public boolean isSpeaking(){
		return synth.testEngineState(
				javax.speech.synthesis.Synthesizer.QUEUE_NOT_EMPTY);
	}
	
	/**
	 * Returns the array of available voices (generated by constructor)
	 * 
	 * @return Array of available voices
	 */
	public Voice[] getVoices(){
		return voices;
	}
	
	/**
	 * Sets the TTS voice to use.
	 * 
	 * @param v Voice to use in TTS playback
	 */
	public void setVoice(Voice v){
		try {
			synth.waitEngineState(Synthesizer.ALLOCATED);
			SynthesizerProperties props = synth.getSynthesizerProperties();
			props.setVoice(v);
			voice = v;
		}
		catch (Exception e){}
	}
	
	/**
	 * Empty function.
	 * 
	 * @param e SpeakableEvent
	 */
	public void speakableStarted(SpeakableEvent e) {}

	/**
	 * Shuts down the TTS engine.
	 *
	 */
	public void shutdown(){
		try {
			synth.waitEngineState(Synthesizer.QUEUE_EMPTY);
			synth.deallocate();
			synth.waitEngineState(Synthesizer.DEALLOCATED);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

}